@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -17,6 +17,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include "scripting/abc.h"
 #include "asobject.h"
 #include "scripting/class.h"
 #include <algorithm>
@@ -91,12 +92,37 @@ TRISTATE ASObject::isLess(ASObject* r)
 	return toPrimitive()->isLess(r);
 }
 
+int variables_map::getNextEnumerable(unsigned int start) const
+{
+	if(start>=Variables.size())
+		return -1;
+
+	const_var_iterator it=Variables.begin();
+
+	unsigned int i=0;
+	while(i<start)
+	{
+		++i;
+		++it;
+	}
+
+	while(it->second.kind!=DYNAMIC_TRAIT)
+	{
+		++i;
+		++it;
+		if(it==Variables.end())
+			return -1;
+	}
+
+	return i;
+}
+
 uint32_t ASObject::nextNameIndex(uint32_t cur_index)
 {
-	if(cur_index < numVariables())
-		return cur_index+1;
-	else
-		return 0;
+	// First -1 converts one-base cur_index to zero-based, +1
+	// moves to the next position (first position to be
+	// considered). Final +1 converts back to one-based.
+	return Variables.getNextEnumerable(cur_index-1+1)+1;
 }
 
 _R<ASObject> ASObject::nextName(uint32_t index)
@@ -177,6 +203,12 @@ unsigned int ASObject::toUInt()
 	return toInt();
 }
 
+uint16_t ASObject::toUInt16()
+{
+	unsigned int val32=toUInt();
+	return val32 & 0xFFFF;
+}
+
 int ASObject::toInt()
 {
 	return 0;
@@ -227,9 +259,9 @@ _R<ASObject> ASObject::toPrimitive(TP_HINT hint)
 
 bool ASObject::has_valueOf()
 {
-	multiname valueOfName;
+	multiname valueOfName(NULL);
 	valueOfName.name_type=multiname::NAME_STRING;
-	valueOfName.name_s="valueOf";
+	valueOfName.name_s_id=getSys()->getUniqueStringId("valueOf");
 	valueOfName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	valueOfName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
 	valueOfName.isAttribute = false;
@@ -241,16 +273,17 @@ bool ASObject::has_valueOf()
  */
 _R<ASObject> ASObject::call_valueOf()
 {
-	multiname valueOfName;
+	multiname valueOfName(NULL);
 	valueOfName.name_type=multiname::NAME_STRING;
-	valueOfName.name_s="valueOf";
+	valueOfName.name_s_id=getSys()->getUniqueStringId("valueOf");
 	valueOfName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	valueOfName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
 	valueOfName.isAttribute = false;
 	assert_and_throw(hasPropertyByMultiname(valueOfName, true));
 
 	_NR<ASObject> o=getVariableByMultiname(valueOfName,SKIP_IMPL);
-	assert_and_throw(o->is<IFunction>());
+	if (!o->is<IFunction>())
+		throw Class<TypeError>::getInstanceS("Error #1006: Call attempted on an object that is not a function.");
 	IFunction* f=o->as<IFunction>();
 
 	incRef();
@@ -260,9 +293,9 @@ _R<ASObject> ASObject::call_valueOf()
 
 bool ASObject::has_toString()
 {
-	multiname toStringName;
+	multiname toStringName(NULL);
 	toStringName.name_type=multiname::NAME_STRING;
-	toStringName.name_s="toString";
+	toStringName.name_s_id=getSys()->getUniqueStringId("toString");
 	toStringName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	toStringName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
 	toStringName.isAttribute = false;
@@ -274,9 +307,9 @@ bool ASObject::has_toString()
  */
 _R<ASObject> ASObject::call_toString()
 {
-	multiname toStringName;
+	multiname toStringName(NULL);
 	toStringName.name_type=multiname::NAME_STRING;
-	toStringName.name_s="toString";
+	toStringName.name_s_id=getSys()->getUniqueStringId("toString");
 	toStringName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	toStringName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
 	toStringName.isAttribute = false;
@@ -300,20 +333,21 @@ bool ASObject::isPrimitive() const
 		type==T_UINTEGER;
 }
 
-variable* variables_map::findObjVar(const tiny_string& n, const nsNameAndKind& ns, TRAIT_KIND createKind, uint32_t traitKinds)
+variables_map::variables_map(MemoryAccount* m):
+	Variables(std::less<mapType::key_type>(), reporter_allocator<mapType::value_type>(m)),slots_vars(m)
 {
-	const var_iterator ret_begin=Variables.lower_bound(n);
-	//This actually look for the first different name, if we accept also previous levels
-	//Otherwise we are just doing equal_range
-	const var_iterator ret_end=Variables.upper_bound(n);
+}
 
-	var_iterator ret=ret_begin;
-	for(;ret!=ret_end;++ret)
+variable* variables_map::findObjVar(uint32_t nameId, const nsNameAndKind& ns, TRAIT_KIND createKind, uint32_t traitKinds)
+{
+	pair<var_iterator, var_iterator> var_range=Variables.equal_range(nameId);
+	var_iterator ret=var_range.first;
+	for(;ret!=var_range.second;++ret)
 	{
 		if(!(ret->second.kind & traitKinds))
 			continue;
 
-		if(ret->second.ns.count(ns))
+		if(ret->second.ns==ns)
 			return &ret->second;
 	}
 
@@ -321,7 +355,7 @@ variable* variables_map::findObjVar(const tiny_string& n, const nsNameAndKind& n
 	if(createKind==NO_CREATE_TRAIT)
 		return NULL;
 
-	var_iterator inserted=Variables.insert(ret_begin,make_pair(n, variable(ns, createKind)) );
+	var_iterator inserted=Variables.insert(var_range.first,make_pair(nameId, variable(ns, createKind)) );
 	return &inserted->second;
 }
 
@@ -360,6 +394,11 @@ void ASObject::setDeclaredMethodByQName(const tiny_string& name, const tiny_stri
 }
 
 void ASObject::setDeclaredMethodByQName(const tiny_string& name, const nsNameAndKind& ns, IFunction* o, METHOD_TYPE type, bool isBorrowed)
+{
+	setDeclaredMethodByQName(getSys()->getUniqueStringId(name), ns, o, type, isBorrowed);
+}
+
+void ASObject::setDeclaredMethodByQName(uint32_t nameId, const nsNameAndKind& ns, IFunction* o, METHOD_TYPE type, bool isBorrowed)
 {
 	check();
 #ifndef NDEBUG
@@ -379,7 +418,7 @@ void ASObject::setDeclaredMethodByQName(const tiny_string& name, const nsNameAnd
 	if(isBorrowed && o->inClass == NULL)
 		o->inClass = this->as<Class_base>();
 
-	variable* obj=Variables.findObjVar(name,ns, (isBorrowed)?BORROWED_TRAIT:DECLARED_TRAIT, (isBorrowed)?BORROWED_TRAIT:DECLARED_TRAIT);
+	variable* obj=Variables.findObjVar(nameId,ns,(isBorrowed)?BORROWED_TRAIT:DECLARED_TRAIT, (isBorrowed)?BORROWED_TRAIT:DECLARED_TRAIT);
 	switch(type)
 	{
 		case NORMAL_METHOD:
@@ -438,7 +477,7 @@ bool ASObject::deleteVariableByMultiname(const multiname& name)
 void ASObject::setVariableByMultiname_i(const multiname& name, int32_t value)
 {
 	check();
-	setVariableByMultiname(name,abstract_i(value));
+	setVariableByMultiname(name,abstract_i(value),CONST_NOT_ALLOWED);
 }
 
 variable* ASObject::findSettable(const multiname& name, bool borrowedMode, bool* has_getter)
@@ -459,7 +498,7 @@ variable* ASObject::findSettable(const multiname& name, bool borrowedMode, bool*
 }
 
 
-void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, Class_base* cls)
+void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst, Class_base* cls)
 {
 	check();
 	assert(!cls || classdef->isSubClass(cls));
@@ -467,6 +506,13 @@ void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, Class_
 	bool has_getter=false;
 	variable* obj=findSettable(name, false, &has_getter);
 
+	if (obj && (obj->kind == CONSTANT_TRAIT && allowConst==CONST_NOT_ALLOWED))
+	{
+		tiny_string err=tiny_string("Error #1074: Illegal write to read-only property ")+name.normalizedName();
+		if(classdef)
+			err+=tiny_string(" on type ")+classdef->as<Class_base>()->getQualifiedClassName();
+		throw Class<ReferenceError>::getInstanceS(err);
+	}
 	if(!obj && cls)
 	{
 		//Look for borrowed traits before
@@ -474,6 +520,11 @@ void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, Class_
 		//looking for a settable even if a super class sets
 		//has_getter to true.
 		obj=cls->findSettable(name,true,&has_getter);
+		if(obj && cls->isFinal && !obj->setter)
+		{
+			tiny_string err=tiny_string("Error #1037: Cannot assign to a method ")+name.normalizedName()+tiny_string(" on ")+cls->getQualifiedClassName();
+			throw Class<ReferenceError>::getInstanceS(err);
+		}
 	}
 
 	if(!obj && cls)
@@ -486,6 +537,11 @@ void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, Class_
 			
 			if(tmp)
 			{
+				if(cls->isFinal && !tmp->setter)
+				{
+					tiny_string err=tiny_string("Error #1037: Cannot assign to a method ")+name.normalizedName()+tiny_string(" on ")+cls->getQualifiedClassName();
+					throw Class<ReferenceError>::getInstanceS(err);
+				}
 				if (tmp->kind != DYNAMIC_TRAIT) // dynamic prototype properties can be overridden 
 					obj = tmp;
 				break;
@@ -493,7 +549,7 @@ void ASObject::setVariableByMultiname(const multiname& name, ASObject* o, Class_
 			proto = proto->getprop_prototype();
 		}
 	}
-
+	
 	if(!obj)
 	{
 		if(has_getter)
@@ -535,12 +591,18 @@ void ASObject::setVariableByQName(const tiny_string& name, const tiny_string& ns
 
 void ASObject::setVariableByQName(const tiny_string& name, const nsNameAndKind& ns, ASObject* o, TRAIT_KIND traitKind)
 {
-	assert_and_throw(Variables.findObjVar(name,ns,NO_CREATE_TRAIT,traitKind)==NULL);
-	variable* obj=Variables.findObjVar(name,ns,traitKind,traitKind);
+	setVariableByQName(getSys()->getUniqueStringId(name), ns, o, traitKind);
+}
+
+void ASObject::setVariableByQName(uint32_t nameId, const nsNameAndKind& ns, ASObject* o, TRAIT_KIND traitKind)
+{
+	assert_and_throw(Variables.findObjVar(nameId,ns,NO_CREATE_TRAIT,traitKind)==NULL);
+	variable* obj=Variables.findObjVar(nameId,ns,traitKind,traitKind);
 	obj->setVar(o);
 }
 
-void ASObject::initializeVariableByMultiname(const multiname& name, ASObject* o, multiname* typemname)
+void ASObject::initializeVariableByMultiname(const multiname& name, ASObject* o, multiname* typemname,
+		ABCContext* context, TRAIT_KIND traitKind)
 {
 	check();
 
@@ -550,18 +612,37 @@ void ASObject::initializeVariableByMultiname(const multiname& name, ASObject* o,
 		//Initializing an already existing variable
 		LOG(LOG_NOT_IMPLEMENTED,"Variable " << name << "already initialized");
 		o->decRef();
-		assert_and_throw(obj->typemname->qualifiedString()==typemname->qualifiedString());
 		return;
 	}
 
-	Variables.initializeVar(name, o, typemname);
+	Variables.initializeVar(name, o, typemname, context, traitKind);
+}
+
+variable::variable(const nsNameAndKind& _ns, TRAIT_KIND _k, ASObject* _v, multiname* _t, const Type* _type)
+		: ns(_ns),var(_v),typeUnion(NULL),setter(NULL),getter(NULL),kind(_k),traitState(NO_STATE)
+{
+	if(_type)
+	{
+		//The type is known, use it instead of the typemname
+		type=_type;
+		traitState=TYPE_RESOLVED;
+	}
+	else
+	{
+		traitTypemname=_t;
+	}
 }
 
 void variable::setVar(ASObject* v)
 {
 	//Resolve the typename if we have one
-	if(!type && typemname)
-		type = Type::getTypeFromMultiname(typemname);
+	if(!(traitState&TYPE_RESOLVED) && traitTypemname)
+	{
+		type = Type::getTypeFromMultiname(traitTypemname, getVm()->currentCallContext->context);
+		assert(type);
+		if(type)
+			traitState=TYPE_RESOLVED;
+	}
 
 	if(type)
 		v = type->coerce(v);
@@ -571,31 +652,9 @@ void variable::setVar(ASObject* v)
 	var=v;
 }
 
-template<class Set1, class Set2>
-bool is_disjoint(const Set1 &set1, const Set2 &set2)
-{
-	if(set1.empty() || set2.empty()) return true;
-
-	typename Set1::const_iterator
-		it1 = set1.begin(),
-		it1End = set1.end();
-	typename Set2::const_iterator
-		it2 = set2.begin(),
-		it2End = set2.end();
-	if(*it1 > *set2.rbegin() || *it2 > *set1.rbegin()) return true;
-
-	while(it1 != it1End && it2 != it2End)
-	{
-		if(*it1 == *it2) return false;
-		if(*it1 < *it2) { it1++; }
-		else { it2++; }
-	}
-	return true;
-}
-
 void variables_map::killObjVar(const multiname& mname)
 {
-	tiny_string name=mname.normalizedName();
+	uint32_t name=mname.normalizedNameId();
 	const pair<var_iterator, var_iterator> ret=Variables.equal_range(name);
 	assert_and_throw(ret.first!=ret.second);
 
@@ -603,7 +662,7 @@ void variables_map::killObjVar(const multiname& mname)
 	var_iterator start=ret.first;
 	for(;start!=ret.second;++start)
 	{
-		if(!is_disjoint(mname.ns,start->second.ns))
+		if(binary_search(mname.ns.begin(),mname.ns.end(),start->second.ns))
 		{
 			Variables.erase(start);
 			return;
@@ -615,35 +674,28 @@ void variables_map::killObjVar(const multiname& mname)
 
 variable* variables_map::findObjVar(const multiname& mname, TRAIT_KIND createKind, uint32_t traitKinds)
 {
-	tiny_string name=mname.normalizedName();
-
-	const var_iterator ret_begin=Variables.lower_bound(name);
-	//This actually look for the first different name, if we accept also previous levels
-	//Otherwise we are just doing equal_range
-	const var_iterator ret_end=Variables.upper_bound(name);
-
+	uint32_t name=mname.normalizedNameId();
+	pair<var_iterator, var_iterator> var_range=Variables.equal_range(name);
 	assert(!mname.ns.empty());
-	var_iterator ret=ret_begin;
-	for(;ret!=ret_end;++ret)
+	var_iterator ret=var_range.first;
+	for(;ret!=var_range.second;++ret)
 	{
 		if(!(ret->second.kind & traitKinds))
 			continue;
 		//Check if one the namespace is already present
 		//We can use binary search, as the namespace are ordered
-		if(!is_disjoint(mname.ns,ret->second.ns))
+		if(binary_search(mname.ns.begin(),mname.ns.end(),ret->second.ns))
 			return &ret->second;
 	}
 
 	//Name not present, insert it, if the multiname has a single ns and if we have to insert it
 	if(createKind==NO_CREATE_TRAIT)
 		return NULL;
-
 	if(createKind == DYNAMIC_TRAIT)
 	{
-		if(mname.ns.begin()->name != "")
+		if(!mname.ns.begin()->hasEmptyName())
 			throw Class<ReferenceError>::getInstanceS("Error #1056: Trying to create a dynamic variable with namespace != \"\"");
-		var_iterator inserted=Variables.insert(ret,make_pair(name,
-					variable(nsNameAndKind("",NAMESPACE), createKind)));
+		var_iterator inserted=Variables.insert(ret,make_pair(name, variable(mname.ns[0], createKind)));
 		return &inserted->second;
 	}
 	assert(mname.ns.size() == 1);
@@ -651,10 +703,8 @@ variable* variables_map::findObjVar(const multiname& mname, TRAIT_KIND createKin
 	return &inserted->second;
 }
 
-void variables_map::initializeVar(const multiname& mname, ASObject* obj, multiname* typemname)
+void variables_map::initializeVar(const multiname& mname, ASObject* obj, multiname* typemname, ABCContext* context, TRAIT_KIND traitKind)
 {
-	tiny_string name=mname.normalizedName();
-
 	const Type* type = NULL;
 	 /* If typename is resolvable right now, we coerce obj.
 	  * It it's not resolvable, then it must be a user defined class,
@@ -667,16 +717,18 @@ void variables_map::initializeVar(const multiname& mname, ASObject* obj, multina
 			//Casting undefined to an object (of unknown class)
 			//results in Null
 			obj->decRef();
-			obj = new Null;
+			obj = getSys()->getNullRef();
 		}
 	}
 	else
 	{
-		type = Type::getTypeFromMultiname(typemname);
+		type = Type::getTypeFromMultiname(typemname, context);
 		obj = type->coerce(obj);
 	}
+	assert(traitKind==DECLARED_TRAIT || traitKind==CONSTANT_TRAIT);
 
-	Variables.insert(make_pair(name, variable(mname.ns[0], DECLARED_TRAIT, obj, typemname, type)));
+	uint32_t name=mname.normalizedNameId();
+	Variables.insert(make_pair(name, variable(mname.ns[0], traitKind, obj, typemname, type)));
 }
 
 ASFUNCTIONBODY(ASObject,generator)
@@ -716,9 +768,9 @@ ASFUNCTIONBODY(ASObject,_toString)
 ASFUNCTIONBODY(ASObject,hasOwnProperty)
 {
 	assert_and_throw(argslen==1);
-	multiname name;
+	multiname name(NULL);
 	name.name_type=multiname::NAME_STRING;
-	name.name_s=args[0]->toString();
+	name.name_s_id=getSys()->getUniqueStringId(args[0]->toString());
 	name.ns.push_back(nsNameAndKind("",NAMESPACE));
 	name.isAttribute=false;
 	if(obj->getClass())
@@ -761,9 +813,9 @@ ASFUNCTIONBODY(ASObject,isPrototypeOf)
 ASFUNCTIONBODY(ASObject,propertyIsEnumerable)
 {
 	assert_and_throw(argslen==1);
-	multiname name;
+	multiname name(NULL);
 	name.name_type=multiname::NAME_STRING;
-	name.name_s=args[0]->toString();
+	name.name_s_id=getSys()->getUniqueStringId(args[0]->toString());
 	name.ns.push_back(nsNameAndKind("",NAMESPACE));
 	name.isAttribute=false;
 	unsigned int index = 0;
@@ -793,7 +845,7 @@ ASFUNCTIONBODY(ASObject,_constructor)
 
 void ASObject::initSlot(unsigned int n, const multiname& name)
 {
-	Variables.initSlot(n,name.name_s,name.ns[0]);
+	Variables.initSlot(n,name.name_s_id,name.ns[0]);
 }
 
 int32_t ASObject::getVariableByMultiname_i(const multiname& name)
@@ -942,6 +994,7 @@ void variables_map::dumpVariables()
 		switch(it->second.kind)
 		{
 			case DECLARED_TRAIT:
+			case CONSTANT_TRAIT:
 				kind="Declared: ";
 				break;
 			case BORROWED_TRAIT:
@@ -979,8 +1032,8 @@ void variables_map::destroyContents()
 	Variables.clear();
 }
 
-ASObject::ASObject():type(T_OBJECT),ref_count(1),manager(NULL),classdef(NULL),constructed(false),
-		implEnable(true)
+ASObject::ASObject(MemoryAccount* m):type(T_OBJECT),Variables(m),ref_count(1),
+	manager(NULL),classdef(NULL),constructed(false),implEnable(true)
 {
 #ifndef NDEBUG
 	//Stuff only used in debugging
@@ -988,13 +1041,21 @@ ASObject::ASObject():type(T_OBJECT),ref_count(1),manager(NULL),classdef(NULL),co
 #endif
 }
 
-ASObject::ASObject(const ASObject& o):type(o.type),ref_count(1),manager(NULL),classdef(o.classdef),
-		constructed(false),implEnable(true)
+ASObject::ASObject(Class_base* c):type(T_OBJECT),Variables((c)?c->memoryAccount:NULL),ref_count(1),
+	manager(NULL),classdef(NULL),constructed(false),implEnable(true)
 {
-	if(classdef)
-	{
-		classdef->incRef();
-	}
+	setClass(c);
+#ifndef NDEBUG
+	//Stuff only used in debugging
+	initialized=false;
+#endif
+}
+
+ASObject::ASObject(const ASObject& o):type(o.type),Variables((o.classdef)?o.classdef->memoryAccount:NULL),ref_count(1),
+	manager(NULL),classdef(NULL),constructed(false),implEnable(true)
+{
+	if(o.classdef)
+		setClass(o.classdef);
 
 #ifndef NDEBUG
 	//Stuff only used in debugging
@@ -1034,16 +1095,16 @@ ASObject::~ASObject()
 	}
 }
 
-void variables_map::initSlot(unsigned int n, const tiny_string& name, const nsNameAndKind& ns)
+void variables_map::initSlot(unsigned int n, uint32_t nameId, const nsNameAndKind& ns)
 {
 	if(n>slots_vars.size())
 		slots_vars.resize(n,Variables.end());
 
-	pair<var_iterator, var_iterator> ret=Variables.equal_range(name);
+	pair<var_iterator, var_iterator> ret=Variables.equal_range(nameId);
 	var_iterator start=ret.first;
 	for(;start!=ret.second;++start)
 	{
-		if(start->second.ns.count(ns))
+		if(start->second.ns==ns)
 		{
 			slots_vars[n-1]=start;
 			return;
@@ -1114,7 +1175,7 @@ tiny_string variables_map::getNameAt(unsigned int index) const
 		for(unsigned int i=0;i<index;i++)
 			++it;
 
-		return it->first;
+		return getSys()->getStringFromUniqueId(it->first);
 	}
 	else
 		throw RunTimeException("getNameAt out of bounds");
@@ -1127,58 +1188,170 @@ unsigned int ASObject::numVariables() const
 
 void ASObject::constructionComplete()
 {
-	//Nothing to be done now
+	Mutex::Lock lock(constructionMutex);
+	constructionSignal.broadcast();
+}
+
+bool ASObject::waitUntilConstructed(unsigned long maxwait_ms)
+{
+	Mutex::Lock lock(constructionMutex);
+
+	if(isConstructed())
+		return true;
+
+	// No need to loop over wait() because the construction state
+	// will change only once from false to true.
+	if(maxwait_ms==0)
+	{
+		constructionSignal.wait(constructionMutex);
+		return true;
+	}
+	else
+	{
+		CondTime wakeUp(maxwait_ms);
+		return wakeUp.wait(constructionMutex, constructionSignal);
+	}
 }
 
 void ASObject::serializeDynamicProperties(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t> traitsMap) const
 {
-	Variables.serialize(out, stringMap, objMap);
+	Variables.serialize(out, stringMap, objMap, traitsMap);
 }
 
 void variables_map::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap) const
 {
 	//Pairs of name, value
 	auto it=Variables.begin();
 	for(;it!=Variables.end();it++)
 	{
-		assert_and_throw(it->second.ns.size() == 1)
-		assert_and_throw(it->second.ns.begin()->name=="");
-		out->writeStringVR(stringMap,it->first);
-		it->second.var->serialize(out, stringMap, objMap);
+		if(it->second.kind!=DYNAMIC_TRAIT)
+			continue;
+		//Dynamic traits always have empty namespace
+		assert(it->second.ns.hasEmptyName());
+		out->writeStringVR(stringMap,getSys()->getStringFromUniqueId(it->first));
+		it->second.var->serialize(out, stringMap, objMap, traitsMap);
 	}
 	//The empty string closes the object
 	out->writeStringVR(stringMap, "");
 }
 
 void ASObject::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap)
 {
-	Class_base* type=getClass();
-	if(type!=Class<ASObject>::getClass())
-		throw UnsupportedException("ASObject::serialize not completely implemented");
-
 	//0x0A -> object marker
-	out->writeByte(amf3::object_marker);
-	//Check if the object has been already serialized
+	out->writeByte(object_marker);
+	//Check if the object has been already serialized to send it by reference
 	auto it=objMap.find(this);
 	if(it!=objMap.end())
 	{
 		//The least significant bit is 0 to signal a reference
 		out->writeU29(it->second << 1);
+		return;
 	}
+
+	Class_base* type=getClass();
+	assert_and_throw(type);
+
+	//Check if an alias is registered
+	auto aliasIt=getSys()->aliasMap.begin();
+	const auto aliasEnd=getSys()->aliasMap.end();
+	//Linear search for alias
+	tiny_string alias;
+	for(;aliasIt!=aliasEnd;++aliasIt)
+	{
+		if(aliasIt->second==type)
+		{
+			alias=aliasIt->first;
+			break;
+		}
+	}
+	bool serializeTraits = alias.empty()==false;
+
+	if(type->isSubClass(InterfaceClass<IExternalizable>::getClass()))
+	{
+		//Custom serialization necessary
+		if(!serializeTraits)
+			throw Class<TypeError>::getInstanceS("#2004: IExternalizable class must have an alias registered");
+		out->writeU29(0x7);
+		out->writeStringVR(stringMap, alias);
+
+		//Invoke writeExternal
+		multiname writeExternalName(NULL);
+		writeExternalName.name_type=multiname::NAME_STRING;
+		writeExternalName.name_s_id=getSys()->getUniqueStringId("writeExternal");
+		writeExternalName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		writeExternalName.isAttribute = false;
+
+		_NR<ASObject> o=getVariableByMultiname(writeExternalName,SKIP_IMPL);
+		assert_and_throw(!o.isNull() && o->getObjectType()==T_FUNCTION);
+		IFunction* f=o->as<IFunction>();
+		this->incRef();
+		out->incRef();
+		ASObject* const tmpArg[1] = {out};
+		f->call(this, tmpArg, 1);
+		return;
+	}
+
+	//Add the object to the map
+	objMap.insert(make_pair(this, objMap.size()));
+
+	uint32_t traitsCount=0;
+	const variables_map::const_var_iterator beginIt = Variables.Variables.begin();
+	const variables_map::const_var_iterator endIt = Variables.Variables.end();
+	//Check if the class traits has been already serialized to send it by reference
+	auto it2=traitsMap.find(type);
+	if(it2!=traitsMap.end())
+		out->writeU29((it2->second << 2) | 1);
 	else
 	{
-		//Add the object to the map
-		objMap.insert(make_pair(this, objMap.size()));
-		//0x0B -> a dynamic instance follows
-		out->writeByte(0x0B);
-		//The class name, empty if no alias is registered
-		//TODO: support aliases
-		out->writeStringVR(stringMap, "");
-		serializeDynamicProperties(out, stringMap, objMap);
+		traitsMap.insert(make_pair(type, traitsMap.size()));
+		for(variables_map::const_var_iterator varIt=beginIt; varIt != endIt; ++varIt)
+		{
+			if(varIt->second.kind==DECLARED_TRAIT)
+			{
+				if(!varIt->second.ns.hasEmptyName())
+				{
+					//Skip variable with a namespace, like protected ones
+					continue;
+				}
+				traitsCount++;
+			}
+		}
+		uint32_t dynamicFlag=(type->isSealed)?0:(1 << 3);
+		out->writeU29((traitsCount << 4) | dynamicFlag | 0x03);
+		out->writeStringVR(stringMap, alias);
+		for(variables_map::const_var_iterator varIt=beginIt; varIt != endIt; ++varIt)
+		{
+			if(varIt->second.kind==DECLARED_TRAIT)
+			{
+				if(!varIt->second.ns.hasEmptyName())
+				{
+					//Skip variable with a namespace, like protected ones
+					continue;
+				}
+				out->writeStringVR(stringMap, getSys()->getStringFromUniqueId(varIt->first));
+			}
+		}
 	}
+	for(variables_map::const_var_iterator varIt=beginIt; varIt != endIt; ++varIt)
+	{
+		if(varIt->second.kind==DECLARED_TRAIT)
+		{
+			if(!varIt->second.ns.hasEmptyName())
+			{
+				//Skip variable with a namespace, like protected ones
+				continue;
+			}
+			varIt->second.var->serialize(out, stringMap, objMap, traitsMap);
+		}
+	}
+	if(!type->isSealed)
+		serializeDynamicProperties(out, stringMap, objMap, traitsMap);
 }
 
 ASObject *ASObject::describeType() const
@@ -1210,22 +1383,15 @@ ASObject *ASObject::describeType() const
 
 bool ASObject::hasprop_prototype()
 {
-	multiname prototypeName;
-	prototypeName.name_type=multiname::NAME_STRING;
-	prototypeName.name_s="prototype";
-	prototypeName.ns.push_back(nsNameAndKind("",NAMESPACE));
-	prototypeName.isAttribute = false;
-	return findGettable(prototypeName, false) != NULL;
+	variable* var=Variables.findObjVar(BUILTIN_STRINGS::PROTOTYPE,nsNameAndKind(BUILTIN_NAMESPACES::EMPTY_NS),
+			NO_CREATE_TRAIT,(DECLARED_TRAIT|DYNAMIC_TRAIT));
+	return (var && var->var);
 }
 
 ASObject* ASObject::getprop_prototype()
 {
-	multiname prototypeName;
-	prototypeName.name_type=multiname::NAME_STRING;
-	prototypeName.name_s="prototype";
-	prototypeName.ns.push_back(nsNameAndKind("",NAMESPACE));
-	prototypeName.isAttribute = false;
-	variable* var = findGettable(prototypeName, false);
+	variable* var=Variables.findObjVar(BUILTIN_STRINGS::PROTOTYPE,nsNameAndKind(BUILTIN_NAMESPACES::EMPTY_NS),
+			NO_CREATE_TRAIT,(DECLARED_TRAIT|DYNAMIC_TRAIT));
 	return var ? var->var : NULL;
 }
 
@@ -1239,9 +1405,9 @@ void ASObject::setprop_prototype(_NR<ASObject>& o)
 	ASObject* obj = o.getPtr();
 	obj->incRef();
 
-	multiname prototypeName;
+	multiname prototypeName(NULL);
 	prototypeName.name_type=multiname::NAME_STRING;
-	prototypeName.name_s="prototype";
+	prototypeName.name_s_id=getSys()->getUniqueStringId("prototype");
 	prototypeName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	bool has_getter = false;
 	variable* ret=findSettable(prototypeName,false, &has_getter);
@@ -1257,4 +1423,3 @@ void ASObject::setprop_prototype(_NR<ASObject>& o)
 	else
 		ret->setVar(obj);
 }
-
