@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -129,7 +129,7 @@ Config::Config():
 		{
 			create_directories(cacheDirectoryP);
 		}
-		catch(const basic_filesystem_error<path>& e)
+		catch(const filesystem_error& e)
 		{
 			LOG(LOG_INFO, _("Could not create cache directory, falling back to default cache directory: ") <<
 					defaultCacheDirectory);
@@ -159,11 +159,15 @@ Config::Config():
 		else
 		{
 			path gnash_exec_path = s;
-			gnash_exec_path /= "sdl-gnash.exe";
-			if(is_regular_file(gnash_exec_path))
+			if(is_regular_file(gnash_exec_path / "gtk-gnash.exe"))
 			{
-				LOG(LOG_INFO,"Found gnash at " << gnash_exec_path);
-				gnashPath = gnash_exec_path.string();
+				LOG(LOG_INFO,"Found gnash at " << (gnash_exec_path / "gtk-gnash.exe"));
+				gnashPath = (gnash_exec_path / "gtk-gnash.exe").string();
+			}
+			else if(is_regular_file(gnash_exec_path / "sdl-gnash.exe"))
+			{
+				LOG(LOG_INFO,"Found gnash at " << (gnash_exec_path / "sdl-gnash.exe"));
+				gnashPath = (gnash_exec_path / "sdl-gnash.exe").string();
 			}
 			else
 				LOG(LOG_ERROR, "Could not find gnash in " << gnash_exec_path);
