@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
 **************************************************************************/
 
 #include "compat.h"
-#include <assert.h>
+#include <cassert>
 
 #include "decoder.h"
 #include "platforms/fastpaths.h"
@@ -466,6 +466,9 @@ FFMpegAudioDecoder::FFMpegAudioDecoder(LS_AUDIO_CODEC audioCodec, uint8_t* initd
 		status=VALID;
 	else
 		status=INIT;
+#if HAVE_AVCODEC_DECODE_AUDIO4
+	frameIn=avcodec_alloc_frame();
+#endif
 }
 
 FFMpegAudioDecoder::FFMpegAudioDecoder(AVCodecContext* _c):codecContext(_c)
@@ -479,6 +482,9 @@ FFMpegAudioDecoder::FFMpegAudioDecoder(AVCodecContext* _c):codecContext(_c)
 
 	if(fillDataAndCheckValidity())
 		status=VALID;
+#if HAVE_AVCODEC_DECODE_AUDIO4
+	frameIn=avcodec_alloc_frame();
+#endif
 }
 
 FFMpegAudioDecoder::~FFMpegAudioDecoder()
@@ -486,6 +492,9 @@ FFMpegAudioDecoder::~FFMpegAudioDecoder()
 	avcodec_close(codecContext);
 	if(ownedContext)
 		av_free(codecContext);
+#if HAVE_AVCODEC_DECODE_AUDIO4
+	av_free(frameIn);
+#endif
 }
 
 bool FFMpegAudioDecoder::fillDataAndCheckValidity()
@@ -521,14 +530,29 @@ uint32_t FFMpegAudioDecoder::decodeData(uint8_t* data, uint32_t datalen, uint32_
 {
 	FrameSamples& curTail=samplesBuffer.acquireLast();
 	int maxLen=AVCODEC_MAX_AUDIO_FRAME_SIZE;
-#if HAVE_AVCODEC_DECODE_AUDIO3
+#if HAVE_AVCODEC_DECODE_AUDIO3 || HAVE_AVCODEC_DECODE_AUDIO4
 	AVPacket pkt;
 	av_init_packet(&pkt);
 	pkt.data=data;
 	pkt.size=datalen;
-	uint32_t ret=avcodec_decode_audio3(codecContext, curTail.samples, &maxLen, &pkt);
+#if HAVE_AVCODEC_DECODE_AUDIO4
+	avcodec_get_frame_defaults(frameIn);
+	int frameOk=0;
+	int32_t ret=avcodec_decode_audio4(codecContext, frameIn, &frameOk, &pkt);
+	if(frameOk==0)
+		ret=-1;
+	else
+	{
+		//This is suboptimal but equivalent to what libavcodec
+		//does for the compatibility version of avcodec_decode_audio3
+		memcpy(curTail.samples, frameIn->extended_data[0], frameIn->linesize[0]);
+		maxLen=frameIn->linesize[0];
+	}
 #else
-	uint32_t ret=avcodec_decode_audio2(codecContext, curTail.samples, &maxLen, data, datalen);
+	int32_t ret=avcodec_decode_audio3(codecContext, curTail.samples, &maxLen, &pkt);
+#endif
+#else
+	int32_t ret=avcodec_decode_audio2(codecContext, curTail.samples, &maxLen, data, datalen);
 #endif
 	assert_and_throw(ret==datalen);
 
@@ -549,7 +573,20 @@ uint32_t FFMpegAudioDecoder::decodePacket(AVPacket* pkt, uint32_t time)
 	FrameSamples& curTail=samplesBuffer.acquireLast();
 	int maxLen=AVCODEC_MAX_AUDIO_FRAME_SIZE;
 
-#if HAVE_AVCODEC_DECODE_AUDIO3
+#if HAVE_AVCODEC_DECODE_AUDIO4
+	avcodec_get_frame_defaults(frameIn);
+	int frameOk=0;
+	int ret=avcodec_decode_audio4(codecContext, frameIn, &frameOk, pkt);
+	if(frameOk==0)
+		ret=-1;
+	else
+	{
+		//This is suboptimal but equivalent to what libavcodec
+		//does for the compatibility version of avcodec_decode_audio3
+		memcpy(curTail.samples, frameIn->extended_data[0], frameIn->linesize[0]);
+		maxLen=frameIn->linesize[0];
+	}
+#elif HAVE_AVCODEC_DECODE_AUDIO3
 	int ret=avcodec_decode_audio3(codecContext, curTail.samples, &maxLen, pkt);
 #else
 	int ret=avcodec_decode_audio2(codecContext, curTail.samples, &maxLen, pkt->data, pkt->size);
@@ -587,11 +624,15 @@ StreamDecoder::~StreamDecoder()
 	delete videoDecoder;
 }
 
+#ifdef ENABLE_LIBAVCODEC
 FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NULL),audioFound(false),videoFound(false),avioContext(NULL)
 {
 	valid=false;
-	//NOTE: this will become avio_alloc_context in FFMpeg 0.7
+#ifdef HAVE_AVIO_ALLOC_CONTEXT
+	avioContext=avio_alloc_context(avioBuffer,4096,0,this,avioReadPacket,NULL,NULL);
+#else
 	avioContext=av_alloc_put_byte(avioBuffer,4096,0,this,avioReadPacket,NULL,NULL);
+#endif
 	if(avioContext==NULL)
 		return;
 
@@ -620,7 +661,13 @@ FFMpegStreamDecoder::FFMpegStreamDecoder(std::istream& s):stream(s),formatCtx(NU
 	if(fmt==NULL)
 		return;
 
+#ifdef HAVE_AVIO_ALLOC_CONTEXT
+	formatCtx=avformat_alloc_context();
+	formatCtx->pb = avioContext;
+	int ret=avformat_open_input(&formatCtx, "lightspark_stream", fmt, NULL);
+#else
 	int ret=av_open_input_stream(&formatCtx, avioContext, "lightspark_stream", fmt, NULL);
+#endif
 	if(ret<0)
 		return;
 	
@@ -668,7 +715,13 @@ FFMpegStreamDecoder::~FFMpegStreamDecoder()
 	audioDecoder=NULL;
 	videoDecoder=NULL;
 	if(formatCtx)
+	{
+#ifdef HAVE_AVIO_ALLOC_CONTEXT
+		av_close_input_file(formatCtx);
+#else
 		av_close_input_stream(formatCtx);
+#endif
+	}
 	if(avioContext)
 		av_free(avioContext);
 }
@@ -713,3 +766,4 @@ int FFMpegStreamDecoder::avioReadPacket(void* t, uint8_t* buf, int buf_size)
 	int ret=th->stream.gcount();
 	return ret;
 }
+#endif //ENABLE_LIBAVCODEC
