@@ -1,8 +1,8 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2010-2011  Alessandro Pignotti (a.pignotti@sssup.it)
-    Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -125,7 +125,7 @@ public:
 	ExtVariant(double value);
 	ExtVariant(bool value);
 	ExtVariant(const ExtVariant& other);
-	ExtVariant(ASObject* other);
+	ExtVariant(_R<ASObject> other);
 
 	// Since this class is used as value in property maps
 	// it must implement a proper copy operator that must
