@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -24,6 +24,8 @@
 #include "logger.h"
 #include "geometry.h"
 #include "compat.h"
+#include "flash/display/flashdisplay.h"
+#include "flash/display/BitmapData.h"
 
 using namespace std;
 using namespace lightspark;
@@ -166,7 +168,7 @@ const Vector2& ShapesBuilder::getVertex(unsigned int index)
 	::abort();
 }
 
-void ShapesBuilder::outputTokens(const std::list<FILLSTYLE>& styles, vector< GeomToken >& tokens)
+void ShapesBuilder::outputTokens(const std::list<FILLSTYLE>& styles, tokensVector& tokens)
 {
 	joinOutlines();
 	//Try to greedily condense as much as possible the output
@@ -184,7 +186,7 @@ void ShapesBuilder::outputTokens(const std::list<FILLSTYLE>& styles, vector< Geo
 			assert(stylesIt!=styles.end());
 		}
 		//Set the fill style
-		tokens.push_back(GeomToken(SET_FILL,*stylesIt));
+		tokens.emplace_back(GeomToken(SET_FILL,*stylesIt));
 		vector<vector<ShapePathSegment> >& outlinesForColor=it->second;
 		for(unsigned int i=0;i<outlinesForColor.size();i++)
 		{
