@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -17,7 +17,7 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
-#include <assert.h>
+#include <cassert>
 
 #include "swf.h"
 #include "graphics.h"
@@ -27,8 +27,7 @@
 #include "backends/config.h"
 #include "compat.h"
 #include "scripting/flash/text/flashtext.h"
-
-#include <iostream>
+#include "flash/display/BitmapData.h"
 
 using namespace lightspark;
 
@@ -105,7 +104,7 @@ void TextureBuffer::init(uint32_t w, uint32_t h, GLenum f)
 	
 	glBindTexture(GL_TEXTURE_2D,0);
 	
-	if(RenderThread::handleGLErrors())
+	if(GLRenderContext::handleGLErrors())
 	{
 		LOG(LOG_ERROR,_("OpenGL error in TextureBuffer::init"));
 		throw RunTimeException("OpenGL error in TextureBuffer::init");
@@ -122,7 +121,7 @@ void TextureBuffer::resize(uint32_t w, uint32_t h)
 			LOG(LOG_CALLS,_("Reallocating texture to size ") << w << 'x' << h);
 			setAllocSize(w,h);
 			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, allocWidth, allocHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, 0);
-			if(RenderThread::handleGLErrors())
+			if(GLRenderContext::handleGLErrors())
 			{
 				LOG(LOG_ERROR,_("OpenGL error in TextureBuffer::resize"));
 				throw RunTimeException("OpenGL error in TextureBuffer::resize");
@@ -173,44 +172,6 @@ TextureBuffer::~TextureBuffer()
 	shutdown();
 }
 
-
-MatrixApplier::MatrixApplier()
-{
-	//First of all try to preserve current matrix
-	getRenderThread()->lsglPushMatrix();
-
-	//TODO: implement smart stack flush
-	//Save all the current stack, compute using SSE the final matrix and push that one
-	//Maybe mulps, shuffle and parallel add
-	//On unapply the stack will be reset as before
-}
-
-MatrixApplier::MatrixApplier(const MATRIX& m)
-{
-	//First of all try to preserve current matrix
-	getRenderThread()->lsglPushMatrix();
-
-	float matrix[16];
-	m.get4DMatrix(matrix);
-	getRenderThread()->lsglMultMatrixf(matrix);
-	getRenderThread()->setMatrixUniform(LSGL_MODELVIEW);
-}
-
-void MatrixApplier::concat(const MATRIX& m)
-{
-	float matrix[16];
-	m.get4DMatrix(matrix);
-	getRenderThread()->lsglMultMatrixf(matrix);
-	getRenderThread()->setMatrixUniform(LSGL_MODELVIEW);
-}
-
-void MatrixApplier::unapply()
-{
-	getRenderThread()->lsglPopMatrix();
-	getRenderThread()->setMatrixUniform(LSGL_MODELVIEW);
-}
-
-
 TextureChunk::TextureChunk(uint32_t w, uint32_t h)
 {
 	width=w;
@@ -291,75 +252,9 @@ bool TextureChunk::resizeIfLargeEnough(uint32_t w, uint32_t h)
 	return false;
 }
 
-CairoRenderer::CairoRenderer(ASObject* _o, CachedSurface& _t, const MATRIX& _m,
-		int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a)
-	: owner(_o),surface(_t),matrix(_m),xOffset(_x),yOffset(_y),alpha(_a),width(_w),height(_h),
-	surfaceBytes(NULL),scaleFactor(_s),uploadNeeded(true)
+CairoRenderer::CairoRenderer(const MATRIX& _m, int32_t _x, int32_t _y, int32_t _w, int32_t _h, float _s, float _a)
+	: IDrawable(_w, _h, _x, _y, _a), matrix(_m), scaleFactor(_s)
 {
-	owner->incRef();
-}
-
-CairoRenderer::~CairoRenderer()
-{
-	delete[] surfaceBytes;
-	owner->decRef();
-}
-
-void CairoRenderer::sizeNeeded(uint32_t& w, uint32_t& h) const
-{
-	w=width;
-	h=height;
-}
-
-void CairoRenderer::upload(uint8_t* data, uint32_t w, uint32_t h) const
-{
-	if(surfaceBytes)
-		memcpy(data,surfaceBytes,w*h*4);
-}
-
-const TextureChunk& CairoRenderer::getTexture()
-{
-	/* This is called in the render thread,
-	 * so we need no locking for surface */
-	//Verify that the texture is large enough
-	if(!surface.tex.resizeIfLargeEnough(width, height))
-		surface.tex=getSys()->getRenderThread()->allocateTexture(width, height,false);
-	surface.xOffset=xOffset;
-	surface.yOffset=yOffset;
-	surface.alpha=alpha;
-	return surface.tex;
-}
-
-void CairoRenderer::uploadFence()
-{
-	delete this;
-}
-
-cairo_matrix_t CairoRenderer::MATRIXToCairo(const MATRIX& matrix)
-{
-	cairo_matrix_t ret;
-
-	cairo_matrix_init(&ret,
-	                  matrix.ScaleX, matrix.RotateSkew0,
-	                  matrix.RotateSkew1, matrix.ScaleY,
-	                  matrix.TranslateX, matrix.TranslateY);
-
-	return ret;
-}
-
-void CairoRenderer::threadAbort()
-{
-	//Nothing special to be done
-}
-
-void CairoRenderer::jobFence()
-{
-	//If the data must be uploaded (there were no errors) the Job add itself to the upload queue.
-	//Otherwise it destroys itself
-	if(uploadNeeded)
-		getSys()->getRenderThread()->addUploadJob(this);
-	else
-		delete this;
 }
 
 void CairoTokenRenderer::quadraticBezier(cairo_t* cr, double control_x, double control_y, double end_x, double end_y)
@@ -403,8 +298,8 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 			}
 
 			MATRIX tmp=style.Matrix;
-			tmp.TranslateX/=scaleCorrection;
-			tmp.TranslateY/=scaleCorrection;
+			tmp.x0/=scaleCorrection;
+			tmp.y0/=scaleCorrection;
 			// The dimensions of the pattern space are specified in SWF specs
 			// as a 32768x32768 box centered at (0,0)
 			if (style.FillStyleType == LINEAR_GRADIENT)
@@ -447,17 +342,17 @@ cairo_pattern_t* CairoTokenRenderer::FILLSTYLEToCairo(const FILLSTYLE& style, do
 		case REPEATING_BITMAP:
 		case CLIPPED_BITMAP:
 		{
-			if(style.bitmap==NULL)
-				throw RunTimeException("Invalid bitmap");
-
-			cairo_surface_t* surface = cairo_image_surface_create_for_data (style.bitmap->data,
-										CAIRO_FORMAT_ARGB32, style.bitmap->width, style.bitmap->height,
-										cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, style.bitmap->width));
+			//bitmap is always present, it may be empty though
+			//Do an explicit cast, the data will not be modified
+			cairo_surface_t* surface = cairo_image_surface_create_for_data ((uint8_t*)style.bitmap.getData(),
+								CAIRO_FORMAT_ARGB32, style.bitmap.getWidth(), style.bitmap.getHeight(),
+								cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, style.bitmap.getWidth()));
 
 			pattern = cairo_pattern_create_for_surface(surface);
 			cairo_surface_destroy(surface);
 
-			cairo_matrix_t mat = MATRIXToCairo(style.Matrix);
+			//Make a copy to invert it
+			cairo_matrix_t mat=style.Matrix;
 			cairo_status_t st = cairo_matrix_invert(&mat);
 			assert(st == CAIRO_STATUS_SUCCESS);
 			mat.x0 /= scaleCorrection;
@@ -589,13 +484,15 @@ bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<Geom
 			}
 
 			case CLEAR_FILL:
+			case FILL_KEEP_SOURCE:
 				if(skipPaint)
 					break;
 
 				cairo_fill(cr);
 
-				// Clear source.
-				cairo_set_operator(cr, CAIRO_OPERATOR_DEST);
+				if(tokens[i].type==CLEAR_FILL)
+					// Clear source.
+					cairo_set_operator(cr, CAIRO_OPERATOR_DEST);
 				break;
 			case CLEAR_STROKE:
 				if(skipPaint)
@@ -606,6 +503,23 @@ bool CairoTokenRenderer::cairoPathFromTokens(cairo_t* cr, const std::vector<Geom
 				// Clear source.
 				cairo_set_operator(stroke_cr, CAIRO_OPERATOR_DEST);
 				break;
+			case FILL_TRANSFORM_TEXTURE:
+			{
+				if(skipPaint)
+					break;
+
+				cairo_matrix_t origmat;
+				cairo_pattern_t* pattern;
+				pattern=cairo_get_source(cr);
+				cairo_pattern_get_matrix(pattern, &origmat);
+
+				cairo_pattern_set_matrix(pattern, &tokens[i].textureTransform);
+
+				cairo_fill(cr);
+
+				cairo_pattern_set_matrix(pattern, &origmat);
+				break;
+			}
 			default:
 				assert(false);
 		}
@@ -639,13 +553,12 @@ void CairoRenderer::cairoClean(cairo_t* cr)
 	cairo_paint(cr);
 }
 
-cairo_surface_t* CairoRenderer::allocateSurface()
+cairo_surface_t* CairoRenderer::allocateSurface(uint8_t*& buf)
 {
 	int32_t cairoWidthStride=cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	assert(cairoWidthStride==width*4);
-	assert(surfaceBytes==NULL);
-	surfaceBytes=new uint8_t[cairoWidthStride*height];
-	return cairo_image_surface_create_for_data(surfaceBytes, CAIRO_FORMAT_ARGB32, width, height, cairoWidthStride);
+	buf=new uint8_t[cairoWidthStride*height];
+	return cairo_image_surface_create_for_data(buf, CAIRO_FORMAT_ARGB32, width, height, cairoWidthStride);
 }
 
 void CairoTokenRenderer::executeDraw(cairo_t* cr)
@@ -656,17 +569,17 @@ void CairoTokenRenderer::executeDraw(cairo_t* cr)
 	cairoPathFromTokens(cr, tokens, scaleFactor, false);
 }
 
+#ifdef HAVE_NEW_GLIBMM_THREAD_API
+StaticMutex CairoRenderer::cairoMutex;
+#else
 StaticMutex CairoRenderer::cairoMutex = GLIBMM_STATIC_MUTEX_INIT;
+#endif
 
-/* This implements IThreadJob::execute */
-void CairoRenderer::execute()
+uint8_t* CairoRenderer::getPixelBuffer()
 {
 	Mutex::Lock l(cairoMutex);
 	if(width==0 || height==0 || !Config::getConfig()->isRenderingEnabled())
-	{
-		uploadNeeded = false;
-		return;
-	}
+		return NULL;
 
 	int32_t windowWidth=getSys()->getRenderThread()->windowWidth;
 	int32_t windowHeight=getSys()->getRenderThread()->windowHeight;
@@ -674,22 +587,29 @@ void CairoRenderer::execute()
 	if(xOffset >= windowWidth || yOffset >= windowHeight
 		|| xOffset + width <= 0 || yOffset + height <= 0)
 	{
-		uploadNeeded = false;
-		return;
+		width=0;
+		height=0;
+		return NULL;
 	}
 
-	//TODO:clip on the right and bottom also
 	if(xOffset<0)
+	{
 		width+=xOffset;
+		xOffset=0;
+	}
 	if(yOffset<0)
+	{
 		height+=yOffset;
+		yOffset=0;
+	}
 
 	//Clip the size to the screen borders
-	if((xOffset>=0) && (width+xOffset) > windowWidth)
+	if((xOffset>0) && (width+xOffset) > windowWidth)
 		width=windowWidth-xOffset;
 	if((yOffset>0) && (height+yOffset) > windowHeight)
 		height=windowHeight-yOffset;
-	cairo_surface_t* cairoSurface=allocateSurface();
+	uint8_t* ret=NULL;
+	cairo_surface_t* cairoSurface=allocateSurface(ret);
 
 	cairo_t* cr=cairo_create(cairoSurface);
 	cairo_surface_destroy(cairoSurface); /* cr has an reference to it */
@@ -699,15 +619,15 @@ void CairoRenderer::execute()
 	//This also guarantees that all the shape fills in width/height pixels
 	//We don't translate for negative offsets as we don't want to see what's in negative coords
 	if(xOffset >= 0)
-		matrix.TranslateX-=xOffset;
+		matrix.x0-=xOffset;
 	if(yOffset >= 0)
-		matrix.TranslateY-=yOffset;
-	const cairo_matrix_t& mat=MATRIXToCairo(matrix);
-	cairo_transform(cr, &mat);
+		matrix.y0-=yOffset;
+	cairo_transform(cr, &matrix);
 
 	executeDraw(cr);
 
 	cairo_destroy(cr);
+	return ret;
 }
 
 bool CairoTokenRenderer::hitTest(const std::vector<GeomToken>& tokens, float scaleFactor, number_t x, number_t y)
@@ -754,11 +674,13 @@ bool CairoTokenRenderer::isOpaque(const std::vector<GeomToken>& tokens, float sc
 	return pixelBytes[0]!=0x00;
 }
 
-uint8_t* CairoRenderer::convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t width, uint32_t height, size_t* dataSize, size_t* stride)
+void CairoRenderer::convertBitmapWithAlphaToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
+		uint32_t height, size_t* dataSize, size_t* stride)
 {
 	*stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	*dataSize = *stride * height;
-	uint8_t* outData = new uint8_t[*dataSize];
+	data.resize(*dataSize, 0);
+	uint8_t* outData=&data[0];
 	uint32_t* inData32 = (uint32_t*)inData;
 
 	for(uint32_t i = 0; i < height; i++)
@@ -769,14 +691,15 @@ uint8_t* CairoRenderer::convertBitmapWithAlphaToCairo(uint8_t* inData, uint32_t
 			*outDataPos = GINT32_FROM_BE( *(inData32+(i*width+j)) );
 		}
 	}
-	return outData;
 }
 
-uint8_t* CairoRenderer::convertBitmapToCairo(uint8_t* inData, uint32_t width, uint32_t height, size_t* dataSize, size_t* stride)
+void CairoRenderer::convertBitmapToCairo(std::vector<uint8_t, reporter_allocator<uint8_t>>& data, uint8_t* inData, uint32_t width,
+		uint32_t height, size_t* dataSize, size_t* stride)
 {
 	*stride = cairo_format_stride_for_width(CAIRO_FORMAT_ARGB32, width);
 	*dataSize = *stride * height;
-	uint8_t* outData = new uint8_t[*dataSize];
+	data.resize(*dataSize, 0);
+	uint8_t* outData = &data[0];
 	for(uint32_t i = 0; i < height; i++)
 	{
 		for(uint32_t j = 0; j < width; j++)
@@ -791,10 +714,13 @@ uint8_t* CairoRenderer::convertBitmapToCairo(uint8_t* inData, uint32_t width, ui
 			*outDataPos = GINT32_FROM_BE(pdata);
 		}
 	}
-	return outData;
 }
 
+#ifdef HAVE_NEW_GLIBMM_THREAD_API
+StaticMutex CairoPangoRenderer::pangoMutex;
+#else
 StaticMutex CairoPangoRenderer::pangoMutex = GLIBMM_STATIC_MUTEX_INIT;
+#endif
 
 void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData& tData)
 {
@@ -823,6 +749,8 @@ void CairoPangoRenderer::pangoLayoutFromData(PangoLayout* layout, const TextData
 			alignment = PANGO_ALIGN_CENTER;
 			break;
 		}
+		default:
+			assert(false);
 	}
 	pango_layout_set_alignment(layout,alignment);
 
@@ -902,3 +830,73 @@ bool CairoPangoRenderer::getBounds(const TextData& _textData, uint32_t& w, uint3
 
 	return (h!=0) && (w!=0);
 }
+
+AsyncDrawJob::AsyncDrawJob(IDrawable* d, _R<DisplayObject> o):drawable(d),owner(o),surfaceBytes(NULL),uploadNeeded(false)
+{
+}
+
+AsyncDrawJob::~AsyncDrawJob()
+{
+	delete drawable;
+	delete[] surfaceBytes;
+}
+
+void AsyncDrawJob::execute()
+{
+	surfaceBytes=drawable->getPixelBuffer();
+	if(surfaceBytes)
+		uploadNeeded=true;
+}
+
+void AsyncDrawJob::threadAbort()
+{
+	//Nothing special to be done
+}
+
+void AsyncDrawJob::jobFence()
+{
+	//If the data must be uploaded (there were no errors) the Job add itself to the upload queue.
+	//Otherwise it destroys itself
+	if(uploadNeeded)
+		getSys()->getRenderThread()->addUploadJob(this);
+	else
+		delete this;
+}
+
+void AsyncDrawJob::upload(uint8_t* data, uint32_t w, uint32_t h) const
+{
+	assert(surfaceBytes);
+	memcpy(data, surfaceBytes, w*h*4);
+}
+
+void AsyncDrawJob::sizeNeeded(uint32_t& w, uint32_t& h) const
+{
+	w=drawable->getWidth();
+	h=drawable->getHeight();
+}
+
+const TextureChunk& AsyncDrawJob::getTexture()
+{
+	/* This is called in the render thread,
+	 * so we need no locking for surface */
+	CachedSurface& surface=owner->cachedSurface;
+	uint32_t width=drawable->getWidth();
+	uint32_t height=drawable->getHeight();
+	//Verify that the texture is large enough
+	if(!surface.tex.resizeIfLargeEnough(width, height))
+		surface.tex=getSys()->getRenderThread()->allocateTexture(width, height,false);
+	surface.xOffset=drawable->getXOffset();
+	surface.yOffset=drawable->getYOffset();
+	surface.alpha=drawable->getAlpha();
+	return surface.tex;
+}
+
+void AsyncDrawJob::uploadFence()
+{
+	delete this;
+}
+
+void SoftwareInvalidateQueue::addToInvalidateQueue(_R<DisplayObject> d)
+{
+	queue.emplace_back(d);
+}
