@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2011  Matthias Gehre (M.Gehre@gmx.de)
+    Copyright (C) 2011-2012  Matthias Gehre (M.Gehre@gmx.de)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -16,11 +16,13 @@
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
-#include <stdint.h>
+#include <cstdint>
 #include <istream>
 
 extern "C" {
 #include "jpeglib.h"
+#define PNG_SKIP_SETJMP_CHECK
+#include "png.h"
 }
 
 namespace lightspark
@@ -30,6 +32,7 @@ class ImageDecoder
 {
 private:
 	static uint8_t* decodeJPEGImpl(jpeg_source_mgr& src, uint32_t* width, uint32_t* height);
+	static uint8_t* decodePNGImpl(png_structp pngPtr, uint32_t* width, uint32_t* height);
 public:
 	/*
 	 * Returns a new[]'ed array of decompressed data and sets width, height and format
@@ -37,6 +40,8 @@ public:
 	 */
 	static uint8_t* decodeJPEG(uint8_t* inData, int len, uint32_t* width, uint32_t* height);
 	static uint8_t* decodeJPEG(std::istream& str, uint32_t* width, uint32_t* height);
+	static uint8_t* decodePNG(uint8_t* inData, int len, uint32_t* width, uint32_t* height);
+	static uint8_t* decodePNG(std::istream& str, uint32_t* width, uint32_t* height);
 };
 
 }
