@@ -1,8 +1,8 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
-    Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -18,14 +18,16 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **************************************************************************/
 
+#include "abc.h"
 #include "swf.h"
 #include "config.h"
 #include "netutils.h"
 #include "rtmputils.h"
 #include "compat.h"
+#include "security.h"
 #include <string>
 #include <algorithm>
-#include <ctype.h>
+#include <cctype>
 #include <iostream>
 #include <fstream>
 #ifdef ENABLE_CURL
@@ -1301,3 +1303,76 @@ void LocalDownloader::execute()
 	//Notify the downloader no more data should be expected
 	setFinished();
 }
+
+DownloaderThreadBase::DownloaderThreadBase(_NR<URLRequest> request, IDownloaderThreadListener* _listener): listener(_listener), downloader(NULL)
+{
+	assert(listener);
+	if(!request.isNull())
+	{
+		url=request->getRequestURL();
+		request->getPostData(postData);
+	}
+}
+
+bool DownloaderThreadBase::createDownloader(bool cached,
+					    const char* contentType,
+					    _NR<EventDispatcher> dispatcher,
+					    ILoadable* owner,
+					    bool checkPolicyFile)
+{
+	if(checkPolicyFile)
+	{
+		SecurityManager::EVALUATIONRESULT evaluationResult = \
+			getSys()->securityManager->evaluatePoliciesURL(url, true);
+		if(threadAborting)
+			return false;
+		if(evaluationResult == SecurityManager::NA_CROSSDOMAIN_POLICY)
+		{
+			getVm()->addEvent(dispatcher,_MR(Class<SecurityErrorEvent>::getInstanceS("SecurityError: "
+												 "connection to domain not allowed by securityManager")));
+			return false;
+		}
+	}
+
+	if(threadAborting)
+		return false;
+
+	//All the checks passed, create the downloader
+	if(postData.empty())
+	{
+		//This is a GET request
+		downloader=getSys()->downloadManager->download(url, cached, owner);
+	}
+	else
+	{
+		downloader=getSys()->downloadManager->downloadWithData(url, postData, contentType, owner);
+	}
+
+	return true;
+}
+
+void DownloaderThreadBase::jobFence()
+{
+	//Get a copy of the downloader, do hold the lock less time.
+	//It's safe to set this->downloader to NULL, this is the last function that will
+	//be called over this thread job
+	Downloader* d=NULL;
+	{
+		SpinlockLocker l(downloaderLock);
+		d=downloader;
+		downloader=NULL;
+	}
+	if(d)
+		getSys()->downloadManager->destroy(d);
+
+	listener->threadFinished(this);
+}
+
+void DownloaderThreadBase::threadAbort()
+{
+	//We have to stop the downloader
+	SpinlockLocker l(downloaderLock);
+	if(downloader != NULL)
+		downloader->stop();
+	threadAborting=true;
+}
