@@ -1,8 +1,8 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
-    Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +29,7 @@
 #include "swftypes.h"
 #include "thread_pool.h"
 #include "backends/urlutils.h"
+#include "smartrefs.h"
 
 namespace lightspark
 {
@@ -282,5 +283,39 @@ public:
 	LocalDownloader(const tiny_string& _url, bool _cached, ILoadable* o);
 };
 
+class IDownloaderThreadListener
+{
+protected:
+	virtual ~IDownloaderThreadListener() {}
+public:
+	virtual void threadFinished(IThreadJob*)=0;
+};
+
+class URLRequest;
+class EventDispatcher;
+
+// Common functionality for the resource loading classes (Loader,
+// URLStream, etc)
+class DownloaderThreadBase : public IThreadJob
+{
+private:
+	IDownloaderThreadListener* listener;
+protected:
+	URLInfo url;
+	std::vector<uint8_t> postData;
+	Spinlock downloaderLock;
+	Downloader* downloader;
+	bool createDownloader(bool cached,
+			      const char* contentType=NULL,
+			      _NR<EventDispatcher> dispatcher=NullRef,
+			      ILoadable* owner=NULL,
+			      bool checkPolicyFile=true);
+	void jobFence();
+public:
+	DownloaderThreadBase(_NR<URLRequest> request, IDownloaderThreadListener* listener);
+	void execute()=0;
+	void threadAbort();
+};
+
 };
 #endif
