@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
     Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
@@ -22,10 +22,12 @@
 
 #include "swf.h"
 #include "compat.h"
+#include "class.h"
+#include "toplevel/Error.h"
 #include <sstream>
 #include <string>
 #include <algorithm>
-#include <ctype.h>
+#include <cctype>
 
 #include "security.h"
 
@@ -327,6 +329,33 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluateURLStatic(const URLIn
 	return ALLOWED;
 }
 
+/**
+ * \brief Throw SecurityError if URL doesn't satisfy security constrains.
+ */
+void SecurityManager::checkURLStaticAndThrow(const URLInfo& url,
+					     int allowedSandboxesRemote, 
+					     int allowedSandboxesLocal,
+					     bool restrictLocalDirectory)
+{
+	SecurityManager::EVALUATIONRESULT evaluationResult = 
+		getSys()->securityManager->evaluateURLStatic(url, ~(SecurityManager::LOCAL_WITH_FILE),
+			SecurityManager::LOCAL_WITH_FILE | SecurityManager::LOCAL_TRUSTED, true);
+	//Network sandboxes can't access local files (this should be a SecurityErrorEvent)
+	if(evaluationResult == SecurityManager::NA_REMOTE_SANDBOX)
+		throw Class<SecurityError>::getInstanceS("SecurityError: "
+				"connect to network");
+	//Local-with-filesystem sandbox can't access network
+	else if(evaluationResult == SecurityManager::NA_LOCAL_SANDBOX)
+		throw Class<SecurityError>::getInstanceS("SecurityError: "
+				"connect to local file");
+	else if(evaluationResult == SecurityManager::NA_PORT)
+		throw Class<SecurityError>::getInstanceS("SecurityError: "
+				"connect to restricted port");
+	else if(evaluationResult == SecurityManager::NA_RESTRICT_LOCAL_DIRECTORY)
+		throw Class<SecurityError>::getInstanceS("SecurityError: "
+				"not allowed to navigate up for local files");
+}
+
 /**
  * \brief Checks if the current sandbox is allowed access to the given URL.
  * 
@@ -411,9 +440,7 @@ SecurityManager::EVALUATIONRESULT SecurityManager::evaluatePoliciesURL(const URL
 
 	//Streaming from RTMP is always allowed (see
 	//http://forums.adobe.com/thread/422391)
-	if(url.getProtocol() == "rtmp" ||
-	   url.getProtocol() == "rtmpe" ||
-	   url.getProtocol() == "rtmps")
+	if(url.isRTMP())
 		return ALLOWED;
 
 	LOG(LOG_INFO, _("SECURITY: Evaluating URL for cross domain policies:"));
