@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
     Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #include <string>
 #include <list>
 #include <map>
-#include <inttypes.h>
+#include <cinttypes>
 #include "swftypes.h"
 
 namespace lightspark
@@ -134,6 +134,8 @@ public:
 	}
 	EVALUATIONRESULT evaluateURLStatic(const URLInfo& url, int allowedSandboxesRemote, 
 			int allowedSandboxesLocal, bool restrictLocalDirectory=true);
+        static void checkURLStaticAndThrow(const URLInfo& url, int allowedSandboxesRemote, 
+                        int allowedSandboxesLocal, bool restrictLocalDirectory=true);
 	
 	//Evaluates an URL by checking if the type of URL (local/remote) matches the allowed sandboxes
 	EVALUATIONRESULT evaluateSandboxURL(const URLInfo& url,
