@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
     Copyright (C) 2010-2011  Timon Van Overveldt (timonvo@gmail.com)
 
     This program is free software: you can redistribute it and/or modify
@@ -21,9 +21,10 @@
 #include "swf.h"
 #include "urlutils.h"
 #include "compat.h"
+#include "scripting/toplevel/Integer.h"
 #include <string>
 #include <algorithm>
-#include <ctype.h>
+#include <cctype>
 #include <sstream>
 #include <iostream>
 #include <fstream>
@@ -445,3 +446,9 @@ std::string URLInfo::decode(const std::string& u, ENCODING type)
 	}
 	return str;
 }
+
+bool URLInfo::isRTMP() const
+{
+	return protocol == "rtmp" || protocol == "rtmpe" || protocol == "rtmps" ||
+	       protocol == "rtmpt" || protocol == "rtmpte" || protocol == "rtmpts";
+}
