@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +20,9 @@
 
 #include "compat.h"
 #include <string>
-#include <stdlib.h>
+#include <cstdlib>
 #include "logger.h"
+#include <unistd.h>
 
 using namespace std;
 
@@ -73,6 +74,7 @@ uint64_t timespecToUsecs(timespec t)
 uint64_t compat_get_thread_cputime_us()
 {
 	timespec tp;
+
 #ifndef _POSIX_THREAD_CPUTIME
 	#error no thread clock available
 #endif
