@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2008-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -32,13 +32,14 @@
 #include "logger.h"
 #include "compat.h"
 #include "streams.h"
+#include "scripting/flash/display/BitmapData.h"
 
 #undef RGB
 
 using namespace std;
 using namespace lightspark;
 
-_NR<Tag> TagFactory::readTag()
+Tag* TagFactory::readTag(RootMovieClip* root)
 {
 	RECORDHEADER h;
 
@@ -52,7 +53,7 @@ _NR<Tag> TagFactory::readTag()
 			throw e;
 		f.clear();
 		LOG(LOG_INFO,"Simulating EndTag at EOF @ " << f.tellg());
-		return _MR(new EndTag(h,f));
+		return new EndTag(h,f);
 	}
 
 	unsigned int expectedLen=h.getLength();
@@ -68,27 +69,27 @@ _NR<Tag> TagFactory::readTag()
 			ret=new ShowFrameTag(h,f);
 			break;
 		case 2:
-			ret=new DefineShapeTag(h,f);
+			ret=new DefineShapeTag(h,f,root);
 			break;
 	//	case 4:
 	//		ret=new PlaceObjectTag(h,f);
 		case 6:
-			ret=new DefineBitsTag(h,f);
+			ret=new DefineBitsTag(h,f,root);
 			break;
 		case 9:
 			ret=new SetBackgroundColorTag(h,f);
 			break;
 		case 10:
-			ret=new DefineFontTag(h,f);
+			ret=new DefineFontTag(h,f,root);
 			break;
 		case 11:
-			ret=new DefineTextTag(h,f);
+			ret=new DefineTextTag(h,f,root);
 			break;
 		case 13:
 			ret=new DefineFontInfoTag(h,f);
 			break;
 		case 14:
-			ret=new DefineSoundTag(h,f);
+			ret=new DefineSoundTag(h,f,root);
 			break;
 		case 15:
 			ret=new StartSoundTag(h,f);
@@ -100,43 +101,43 @@ _NR<Tag> TagFactory::readTag()
 			ret=new SoundStreamBlockTag(h,f);
 			break;
 		case 20:
-			ret=new DefineBitsLosslessTag(h,f,1);
+			ret=new DefineBitsLosslessTag(h,f,1,root);
 			break;
 		case 21:
-			ret=new DefineBitsJPEG2Tag(h,f);
+			ret=new DefineBitsJPEG2Tag(h,f,root);
 			break;
 		case 22:
-			ret=new DefineShape2Tag(h,f);
+			ret=new DefineShape2Tag(h,f,root);
 			break;
 		case 24:
 			ret=new ProtectTag(h,f);
 			break;
 		case 26:
-			ret=new PlaceObject2Tag(h,f);
+			ret=new PlaceObject2Tag(h,f,root);
 			break;
 		case 28:
 			ret=new RemoveObject2Tag(h,f);
 			break;
 		case 32:
-			ret=new DefineShape3Tag(h,f);
+			ret=new DefineShape3Tag(h,f,root);
 			break;
 		case 33:
-			ret=new DefineText2Tag(h,f);
+			ret=new DefineText2Tag(h,f,root);
 			break;
 		case 34:
-			ret=new DefineButton2Tag(h,f);
+			ret=new DefineButton2Tag(h,f,root);
 			break;
 		case 35:
-			ret=new DefineBitsJPEG3Tag(h,f);
+			ret=new DefineBitsJPEG3Tag(h,f,root);
 			break;
 		case 36:
-			ret=new DefineBitsLosslessTag(h,f,2);
+			ret=new DefineBitsLosslessTag(h,f,2,root);
 			break;
 		case 37:
-			ret=new DefineEditTextTag(h,f);
+			ret=new DefineEditTextTag(h,f,root);
 			break;
 		case 39:
-			ret=new DefineSpriteTag(h,f);
+			ret=new DefineSpriteTag(h,f,root);
 			break;
 		case 41:
 			ret=new ProductInfoTag(h,f);
@@ -148,16 +149,16 @@ _NR<Tag> TagFactory::readTag()
 			ret=new SoundStreamHead2Tag(h,f);
 			break;
 		case 46:
-			ret=new DefineMorphShapeTag(h,f);
+			ret=new DefineMorphShapeTag(h,f,root);
 			break;
 		case 48:
-			ret=new DefineFont2Tag(h,f);
+			ret=new DefineFont2Tag(h,f,root);
 			break;
 		case 58:
 			ret=new EnableDebuggerTag(h,f);
 			break;
 		case 60:
-			ret=new DefineVideoStreamTag(h,f);
+			ret=new DefineVideoStreamTag(h,f,root);
 			break;
 		case 63:
 			ret=new DebugIDTag(h,f);
@@ -175,7 +176,7 @@ _NR<Tag> TagFactory::readTag()
 			ret=new FileAttributesTag(h,f);
 			break;
 		case 70:
-			ret=new PlaceObject3Tag(h,f);
+			ret=new PlaceObject3Tag(h,f,root);
 			break;
 		case 72:
 			ret=new DoABCTag(h,f);
@@ -187,7 +188,7 @@ _NR<Tag> TagFactory::readTag()
 			ret=new CSMTextSettingsTag(h,f);
 			break;
 		case 75:
-			ret=new DefineFont3Tag(h,f);
+			ret=new DefineFont3Tag(h,f,root);
 			break;
 		case 76:
 			ret=new SymbolClassTag(h,f);
@@ -202,19 +203,22 @@ _NR<Tag> TagFactory::readTag()
 			ret=new DoABCDefineTag(h,f);
 			break;
 		case 83:
-			ret=new DefineShape4Tag(h,f);
+			ret=new DefineShape4Tag(h,f,root);
+			break;
+		case 84:
+			ret=new DefineMorphShape2Tag(h,f,root);
 			break;
 		case 86:
 			ret=new DefineSceneAndFrameLabelDataTag(h,f);
 			break;
 		case 87:
-			ret=new DefineBinaryDataTag(h,f);
+			ret=new DefineBinaryDataTag(h,f,root);
 			break;
 		case 88:
 			ret=new DefineFontNameTag(h,f);
 			break;
 		case 91:
-			ret=new DefineFont4Tag(h,f);
+			ret=new DefineFont4Tag(h,f,root);
 			break;
 		default:
 			LOG(LOG_NOT_IMPLEMENTED,_("Unsupported tag type ") << h.getTagType());
@@ -238,7 +242,7 @@ _NR<Tag> TagFactory::readTag()
 		throw ParseException("Malformed SWF file");
 	}
 	
-	return _MNR(ret);
+	return ret;
 }
 
 RemoveObject2Tag::RemoveObject2Tag(RECORDHEADER h, std::istream& in):DisplayListTag(h)
@@ -247,7 +251,7 @@ RemoveObject2Tag::RemoveObject2Tag(RECORDHEADER h, std::istream& in):DisplayList
 	LOG(LOG_TRACE,_("RemoveObject2 Depth: ") << Depth);
 }
 
-void RemoveObject2Tag::execute(DisplayObjectContainer* parent)
+void RemoveObject2Tag::execute(DisplayObjectContainer* parent) const
 {
 	parent->deleteLegacyChildAt(Depth);
 }
@@ -257,7 +261,7 @@ SetBackgroundColorTag::SetBackgroundColorTag(RECORDHEADER h, std::istream& in):C
 	in >> BackgroundColor;
 }
 
-DefineEditTextTag::DefineEditTextTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineEditTextTag::DefineEditTextTag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DictionaryTag(h,root)
 {
 	//setVariableByName("text",SWFObject(new Integer(0)));
 	in >> CharacterID >> Bounds;
@@ -305,28 +309,29 @@ DefineEditTextTag::DefineEditTextTag(RECORDHEADER h, std::istream& in):Dictionar
 	LOG(LOG_NOT_IMPLEMENTED, "DefineEditTextTag does not parse many attributes");
 }
 
-ASObject* DefineEditTextTag::instance() const
+ASObject* DefineEditTextTag::instance(Class_base* c) const
 {
-	TextField* ret=new TextField(textData);
+	if(c==NULL)
+		c=Class<TextField>::getClass();
 	//TODO: check
 	assert_and_throw(bindedTo==NULL);
-	ret->setClass(Class<TextField>::getClass());
+	TextField* ret=new (c->memoryAccount) TextField(c, textData);
 	return ret;
 }
 
-DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DictionaryTag(h,root)
 {
 	in >> SpriteID >> FrameCount;
 
 	LOG(LOG_TRACE,_("DefineSprite ID: ") << SpriteID);
 	//Create a non top level TagFactory
 	TagFactory factory(in, false);
-	_NR<Tag> tag;
+	Tag* tag;
 	bool done=false;
 	bool empty=true;
 	do
 	{
-		tag=factory.readTag();
+		tag=factory.readTag(root);
 		/* We need no locking here, because the vm can only
 		 * access this object after construction
 		 */
@@ -335,7 +340,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 			case DICT_TAG:
 				throw ParseException("Dictionary tag inside a sprite. Should not happen.");
 			case DISPLAY_LIST_TAG:
-				addToFrame(tag.cast<DisplayListTag>());
+				addToFrame(static_cast<DisplayListTag*>(tag));
 				empty=false;
 				break;
 			case SHOW_TAG:
@@ -349,7 +354,7 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 			case CONTROL_TAG:
 				throw ParseException("Control tag inside a sprite. Should not happen.");
 			case FRAMELABEL_TAG:
-				addFrameLabel(frames.size()-1,static_cast<FrameLabelTag*>(tag.getPtr())->Name);
+				addFrameLabel(frames.size()-1,static_cast<FrameLabelTag*>(tag)->Name);
 				empty=false;
 				break;
 			case TAG:
@@ -375,19 +380,17 @@ DefineSpriteTag::DefineSpriteTag(RECORDHEADER h, std::istream& in):DictionaryTag
 	LOG(LOG_TRACE,_("EndDefineSprite ID: ") << SpriteID);
 }
 
-ASObject* DefineSpriteTag::instance() const
+ASObject* DefineSpriteTag::instance(Class_base* c) const
 {
-	DefineSpriteTag* ret=new DefineSpriteTag(*this);
-	//TODO: check
-	if(bindedTo)
-	{
-		//A class is binded to this tag
-		ret->setClass(bindedTo);
-	}
+	Class_base* retClass=NULL;
+	if(c)
+		retClass=c;
+	else if(bindedTo)
+		retClass=bindedTo;
 	else
-		ret->setClass(Class<MovieClip>::getClass());
+		retClass=Class<MovieClip>::getClass();
 
-	return ret;
+	return new (retClass->memoryAccount) MovieClip(retClass, *this);
 }
 
 void lightspark::ignore(istream& i, int count)
@@ -399,7 +402,7 @@ void lightspark::ignore(istream& i, int count)
 	delete[] buf;
 }
 
-DefineFontTag::DefineFontTag(RECORDHEADER h, std::istream& in):FontTag(h, 20)
+DefineFontTag::DefineFontTag(RECORDHEADER h, std::istream& in, RootMovieClip* root):FontTag(h, 20, root)
 {
 	LOG(LOG_TRACE,_("DefineFont"));
 	in >> FontID;
@@ -423,8 +426,24 @@ DefineFontTag::DefineFontTag(RECORDHEADER h, std::istream& in):FontTag(h, 20)
 		GlyphShapeTable.push_back(t);
 	}
 }
+ASObject* DefineFontTag::instance(Class_base* c) const
+{ 
+	tiny_string fontname("");
+	Class_base* retClass=NULL;
+	if(c)
+		retClass=c;
+	else if(bindedTo)
+		retClass=bindedTo;
+	else
+		retClass=Class<ASFont>::getClass();
 
-DefineFont2Tag::DefineFont2Tag(RECORDHEADER h, std::istream& in):FontTag(h, 20)
+	ASFont* ret=new (retClass->memoryAccount) ASFont(retClass);
+	LOG(LOG_NOT_IMPLEMENTED,"DefineFontTag::instance doesn't handle all font properties");
+	ret->SetFont(fontname,false,false,true,false);
+	return ret;
+}
+
+DefineFont2Tag::DefineFont2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):FontTag(h, 20, root)
 {
 	LOG(LOG_TRACE,_("DefineFont2"));
 	in >> FontID;
@@ -512,8 +531,24 @@ DefineFont2Tag::DefineFont2Tag(RECORDHEADER h, std::istream& in):FontTag(h, 20)
 	//TODO: implmented Kerning support
 	ignore(in,KerningCount*4);
 }
+ASObject* DefineFont2Tag::instance(Class_base* c) const
+{ 
+	tiny_string fontname((const char*)FontName.data(),true);
+	Class_base* retClass=NULL;
+	if(c)
+		retClass=c;
+	else if(bindedTo)
+		retClass=bindedTo;
+	else
+		retClass=Class<ASFont>::getClass();
 
-DefineFont3Tag::DefineFont3Tag(RECORDHEADER h, std::istream& in):FontTag(h, 1)
+	ASFont* ret=new (retClass->memoryAccount) ASFont(retClass);
+	LOG(LOG_NOT_IMPLEMENTED,"DefineFont2Tag::instance doesn't handle all font properties");
+	ret->SetFont(fontname,FontFlagsBold,FontFlagsItalic,true,false);
+	return ret;
+}
+
+DefineFont3Tag::DefineFont3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):FontTag(h, 1, root)
 {
 	LOG(LOG_TRACE,_("DefineFont3"));
 	in >> FontID;
@@ -596,8 +631,24 @@ DefineFont3Tag::DefineFont3Tag(RECORDHEADER h, std::istream& in):FontTag(h, 1)
 	//TODO: implment Kerning support
 	ignore(in,KerningCount*4);
 }
+ASObject* DefineFont3Tag::instance(Class_base* c) const
+{ 
+	tiny_string fontname((const char*)FontName.data(),true);
+	Class_base* retClass=NULL;
+	if(c)
+		retClass=c;
+	else if(bindedTo)
+		retClass=bindedTo;
+	else
+		retClass=Class<ASFont>::getClass();
 
-DefineFont4Tag::DefineFont4Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+	ASFont* ret=new (retClass->memoryAccount) ASFont(retClass);
+	LOG(LOG_NOT_IMPLEMENTED,"DefineFont3Tag::instance doesn't handle all font properties");
+	ret->SetFont(fontname,FontFlagsBold,FontFlagsItalic,true,false);
+	return ret;
+}
+
+DefineFont4Tag::DefineFont4Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DictionaryTag(h,root)
 {
 	LOG(LOG_TRACE,_("DefineFont4"));
 	int dest=in.tellg();
@@ -615,8 +666,28 @@ DefineFont4Tag::DefineFont4Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h
 		LOG(LOG_NOT_IMPLEMENTED,"DefineFont4Tag with FontData");
 	ignore(in,dest-in.tellg());
 }
+ASObject* DefineFont4Tag::instance(Class_base* c) const
+{ 
+	tiny_string fontname = FontName;
+	Class_base* retClass=NULL;
+	if(c)
+		retClass=c;
+	else if(bindedTo)
+		retClass=bindedTo;
+	else
+		retClass=Class<ASFont>::getClass();
 
-DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in, int version):DictionaryTag(h)
+	ASFont* ret=new (retClass->memoryAccount) ASFont(retClass);
+	LOG(LOG_NOT_IMPLEMENTED,"DefineFont4Tag::instance doesn't handle all font properties");
+	ret->SetFont(fontname,FontFlagsBold,FontFlagsItalic,FontFlagsHasFontData,false);
+	return ret;
+}
+
+BitmapTag::BitmapTag(RECORDHEADER h,RootMovieClip* root):DictionaryTag(h,root),BitmapContainer(getSys()->tagsMemory)
+{
+}
+
+DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in, int version, RootMovieClip* root):BitmapTag(h,root)
 {
 	int dest=in.tellg();
 	dest+=h.getLength();
@@ -657,28 +728,29 @@ DefineBitsLosslessTag::DefineBitsLosslessTag(RECORDHEADER h, istream& in, int ve
 	fromRGB(inData, BitmapWidth, BitmapHeight, true);
 }
 
-ASObject* DefineBitsLosslessTag::instance() const
+ASObject* BitmapTag::instance(Class_base* c) const
 {
-	DefineBitsLosslessTag* ret=new DefineBitsLosslessTag(*this);
+	Class_base* realClass=(c)?c:bindedTo;
 	//Flex imports bitmaps using BitmapAsset as the base class, which is derived from bitmap
 	//Also BitmapData is used in the wild though, so support both cases
-	if(bindedTo && bindedTo->isSubClass(Class<Bitmap>::getClass()))
+	bool returnBitmap = false;
+	Class_base* classRet = Class<BitmapData>::getClass();
+	if(realClass && realClass->isSubClass(Class<Bitmap>::getClass()))
+		returnBitmap = true;
+	else if(realClass && realClass->isSubClass(Class<BitmapData>::getClass()))
+		classRet = realClass;
+
+	BitmapData* ret=new (classRet->memoryAccount) BitmapData(classRet, *this);
+	if(returnBitmap)
 	{
-		ret->setClass(Class<BitmapData>::getClass());
-		Bitmap* bitmapRet=new Bitmap(_MR((BitmapData*)ret));
-		bitmapRet->setClass(bindedTo);
+		Bitmap* bitmapRet=new (realClass->memoryAccount) Bitmap(realClass,_MR(ret));
 		return bitmapRet;
 	}
-	else if(bindedTo && bindedTo->isSubClass(Class<BitmapData>::getClass()))
-	{
-		ret->setClass(bindedTo);
-	}
-	else
-		ret->setClass(Class<BitmapData>::getClass());
 	return ret;
 }
 
-DefineTextTag::DefineTextTag(RECORDHEADER h, istream& in, int v):DictionaryTag(h),version(v)
+DefineTextTag::DefineTextTag(RECORDHEADER h, istream& in, RootMovieClip* root,int v):DictionaryTag(h,root),
+	tokens(reporter_allocator<GeomToken>(getSys()->tagsMemory)),version(v)
 {
 	in >> CharacterId >> TextBounds >> TextMatrix >> GlyphBits >> AdvanceBits;
 	assert(v==1 || v==2);
@@ -697,7 +769,7 @@ DefineTextTag::DefineTextTag(RECORDHEADER h, istream& in, int v):DictionaryTag(h
 	}
 }
 
-ASObject* DefineTextTag::instance() const
+ASObject* DefineTextTag::instance(Class_base* c) const
 {
 	/* we cannot call computeCached in the constructor
 	 * because loadedFrom is not available there for dictionary lookups
@@ -705,8 +777,10 @@ ASObject* DefineTextTag::instance() const
 	if(tokens.empty())
 		computeCached();
 
-	StaticText* ret=new StaticText(tokens);
-	ret->setClass(Class<StaticText>::getClass());
+	if(c==NULL)
+		c=Class<StaticText>::getClass();
+
+	StaticText* ret=new (c->memoryAccount) StaticText(c, tokens);
 	return ret;
 }
 
@@ -715,7 +789,7 @@ void DefineTextTag::computeCached() const
 	if(!tokens.empty())
 		return;
 
-	FontTag* curFont = NULL;
+	const FontTag* curFont = NULL;
 	std::list<FILLSTYLE> fillStyles;
 	Vector2 curPos;
 	FILLSTYLE fs(1);
@@ -727,8 +801,8 @@ void DefineTextTag::computeCached() const
 	{
 		if(TextRecords[i].StyleFlagsHasFont)
 		{
-			_R<DictionaryTag> it3=loadedFrom->dictionaryLookup(TextRecords[i].FontID);
-			curFont=dynamic_cast<FontTag*>(it3.getPtr());
+			const DictionaryTag* it3=loadedFrom->dictionaryLookup(TextRecords[i].FontID);
+			curFont=dynamic_cast<const FontTag*>(it3);
 			assert_and_throw(curFont);
 		}
 		assert_and_throw(curFont);
@@ -764,30 +838,36 @@ void DefineTextTag::computeCached() const
 	}
 }
 
-DefineShapeTag::DefineShapeTag(RECORDHEADER h, std::istream& in):DictionaryTag(h),Shapes(1)
+DefineShapeTag::DefineShapeTag(RECORDHEADER h,int v,RootMovieClip* root):DictionaryTag(h,root),Shapes(v),
+	tokens(reporter_allocator<GeomToken>(getSys()->tagsMemory))
+{
+}
+
+DefineShapeTag::DefineShapeTag(RECORDHEADER h, std::istream& in,RootMovieClip* root):DictionaryTag(h,root),Shapes(1),
+	tokens(reporter_allocator<GeomToken>(getSys()->tagsMemory))
 {
 	LOG(LOG_TRACE,_("DefineShapeTag"));
 	in >> ShapeId >> ShapeBounds >> Shapes;
 	TokenContainer::FromShaperecordListToShapeVector(Shapes.ShapeRecords,tokens,Shapes.FillStyles.FillStyles);
 }
 
-DefineShape2Tag::DefineShape2Tag(RECORDHEADER h, std::istream& in):DefineShapeTag(h,2)
+DefineShape2Tag::DefineShape2Tag(RECORDHEADER h, std::istream& in,RootMovieClip* root):DefineShapeTag(h,2,root)
 {
 	LOG(LOG_TRACE,_("DefineShape2Tag"));
 	in >> ShapeId >> ShapeBounds >> Shapes;
 	TokenContainer::FromShaperecordListToShapeVector(Shapes.ShapeRecords,tokens,Shapes.FillStyles.FillStyles);
 }
 
-DefineShape3Tag::DefineShape3Tag(RECORDHEADER h, std::istream& in):DefineShape2Tag(h,3)
+DefineShape3Tag::DefineShape3Tag(RECORDHEADER h, std::istream& in,RootMovieClip* root):DefineShape2Tag(h,3,root)
 {
-	LOG(LOG_TRACE,_("DefineShape3Tag"));
+	LOG(LOG_TRACE,"DefineShape3Tag");
 	in >> ShapeId >> ShapeBounds >> Shapes;
 	TokenContainer::FromShaperecordListToShapeVector(Shapes.ShapeRecords,tokens,Shapes.FillStyles.FillStyles);
 }
 
-DefineShape4Tag::DefineShape4Tag(RECORDHEADER h, std::istream& in):DefineShape3Tag(h,4)
+DefineShape4Tag::DefineShape4Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DefineShape3Tag(h,4,root)
 {
-	LOG(LOG_TRACE,_("DefineShape4Tag"));
+	LOG(LOG_TRACE,"DefineShape4Tag");
 	in >> ShapeId >> ShapeBounds >> EdgeBounds;
 	BitStream bs(in);
 	UB(5,bs);
@@ -798,59 +878,49 @@ DefineShape4Tag::DefineShape4Tag(RECORDHEADER h, std::istream& in):DefineShape3T
 	TokenContainer::FromShaperecordListToShapeVector(Shapes.ShapeRecords,tokens,Shapes.FillStyles.FillStyles);
 }
 
-DefineMorphShapeTag::DefineMorphShapeTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineMorphShapeTag::DefineMorphShapeTag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DictionaryTag(h, root),
+	MorphLineStyles(1)
 {
-	int dest=in.tellg();
-	dest+=h.getLength();
-	in >> CharacterId >> StartBounds >> EndBounds >> Offset >> MorphFillStyles >> MorphLineStyles >> StartEdges >> EndEdges;
-	if(in.tellg()<dest)
-		ignore(in,dest-in.tellg());
-}
-
-ASObject* DefineMorphShapeTag::instance() const
-{
-	DefineMorphShapeTag* ret=new DefineMorphShapeTag(*this);
-	assert_and_throw(bindedTo==NULL);
-	ret->setClass(Class<MorphShape>::getClass());
-	return ret;
-}
-
-void DefineMorphShapeTag::renderImpl(RenderContext& ctxt, bool maskEnabled, number_t t1,number_t t2,number_t t3,number_t t4) const
-{
-	if(alpha==0)
-		return;
-	if(!visible)
-		return;
-	//TODO: implement morph shape support
-/*	std::vector < GeomShape > shapes;
-	SHAPERECORD* cur=&(EndEdges.ShapeRecords);
-
-	FromShaperecordListToShapeVector(cur,shapes);
-
-//	for(unsigned int i=0;i<shapes.size();i++)
-//		shapes[i].BuildFromEdges(MorphFillStyles.FillStyles);
-
-	float matrix[16];
-	Matrix.get4DMatrix(matrix);
-	lsglPushMatrix();
-	lsglMultMatrixf(matrix);
-	setMatrixUniform(LSGL_MODELVIEW);
-
-	rt->acquireFramebuffer();
-
-	std::vector < GeomShape >::iterator it=shapes.begin();
-	for(;it!=shapes.end();++it)
-		it->Render();
-
-	rt->blitFramebuffer();
-	if(rt->acquireIdBuffer())
+	LOG(LOG_TRACE,"DefineMorphShapeTag");
+	UI32_SWF Offset;
+	in >> CharacterId >> StartBounds >> EndBounds >> Offset >> MorphFillStyles >> MorphLineStyles;
+	try
 	{
-		std::vector < GeomShape >::iterator it=shapes.begin();
-		for(;it!=shapes.end();++it)
-			it->Render();
-		rt->releaseIdBuffer();
+		in >> StartEdges >> EndEdges;
+	}
+	catch(LightsparkException& e)
+	{
+		LOG(LOG_ERROR,_("Invalid data for morph shape"));
+	}
+}
+
+ASObject* DefineMorphShapeTag::instance(Class_base* c) const
+{
+	assert_and_throw(bindedTo==NULL);
+	if(c==NULL)
+		c=Class<MorphShape>::getClass();
+	LOG(LOG_NOT_IMPLEMENTED, _("MorphShape not really supported"));
+	return Class<MorphShape>::getInstanceS(c);
+}
+
+DefineMorphShape2Tag::DefineMorphShape2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DefineMorphShapeTag(h, root, 2)
+{
+	LOG(LOG_TRACE,"DefineMorphShape2Tag");
+	UI32_SWF Offset;
+	in >> CharacterId >> StartBounds >> EndBounds >> StartEdgeBounds >> EndEdgeBounds;
+	BitStream bs(in);
+	UB(6,bs);
+	UsesNonScalingStrokes=UB(1,bs);
+	UsesScalingStrokes=UB(1,bs);
+	in >> Offset >> MorphFillStyles >> MorphLineStyles;
+	try
+	{
+		in >> StartEdges >> EndEdges;
+	}
+	catch(LightsparkException& e)
+	{
+		LOG(LOG_ERROR,_("Invalid data for morph shape"));
 	}
-	lsglPopMatrix();*/
 }
 
 //void DefineFont3Tag::genGlyphShape(vector<GeomShape>& s, int glyph)
@@ -976,7 +1046,7 @@ void PlaceObject2Tag::setProperties(DisplayObject* obj, DisplayObjectContainer*
 	}
 }
 
-void PlaceObject2Tag::execute(DisplayObjectContainer* parent)
+void PlaceObject2Tag::execute(DisplayObjectContainer* parent) const
 {
 	//TODO: support clipping
 	if(ClipDepth!=0)
@@ -996,21 +1066,15 @@ void PlaceObject2Tag::execute(DisplayObjectContainer* parent)
 		//A new character must be placed
 		LOG(LOG_TRACE,_("Placing ID ") << CharacterId);
 
-		RootMovieClip* localRoot=NULL;
-		DictionaryTag* parentDict=dynamic_cast<DictionaryTag*>(parent);
-		//TODO: clean up this nonsense. Of course the parent is a dictionary tag!
-		if(parentDict)
-			localRoot=parentDict->loadedFrom;
-		else
-			localRoot=parent->getRoot().getPtr();
-		_R<DictionaryTag> dict=localRoot->dictionaryLookup(CharacterId);
+		if(placedTag==NULL)
+			throw RunTimeException("No tag to place");
 
 		//We can create the object right away
-		DisplayObject* toAdd=dynamic_cast<DisplayObject*>(dict->instance());
+		DisplayObject* toAdd=dynamic_cast<DisplayObject*>(placedTag->instance());
 
 		assert_and_throw(toAdd);
 		//The matrix must be set before invoking the constructor
-		toAdd->setMatrix(Matrix);
+		toAdd->setLegacyMatrix(Matrix);
 
 		setProperties(toAdd, parent);
 
@@ -1037,7 +1101,7 @@ void PlaceObject2Tag::execute(DisplayObjectContainer* parent)
 	}
 }
 
-PlaceObject2Tag::PlaceObject2Tag(RECORDHEADER h, std::istream& in):DisplayListTag(h)
+PlaceObject2Tag::PlaceObject2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DisplayListTag(h),placedTag(NULL)
 {
 	LOG(LOG_TRACE,_("PlaceObject2"));
 
@@ -1072,10 +1136,13 @@ PlaceObject2Tag::PlaceObject2Tag(RECORDHEADER h, std::istream& in):DisplayListTa
 	if(PlaceFlagHasClipAction)
 		in >> ClipActions;
 
-	assert_and_throw(!(PlaceFlagHasCharacter && CharacterId==0))
+	assert_and_throw(!(PlaceFlagHasCharacter && CharacterId==0));
+
+	if(PlaceFlagHasCharacter)
+		placedTag=root->dictionaryLookup(CharacterId);
 }
 
-PlaceObject3Tag::PlaceObject3Tag(RECORDHEADER h, std::istream& in):PlaceObject2Tag(h)
+PlaceObject3Tag::PlaceObject3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):PlaceObject2Tag(h)
 {
 	LOG(LOG_TRACE,_("PlaceObject3"));
 
@@ -1130,9 +1197,12 @@ PlaceObject3Tag::PlaceObject3Tag(RECORDHEADER h, std::istream& in):PlaceObject2T
 		in >> ClipActions;
 
 	assert_and_throw(!(PlaceFlagHasCharacter && CharacterId==0))
+
+	if(PlaceFlagHasCharacter)
+		placedTag=root->dictionaryLookup(CharacterId);
 }
 
-void SetBackgroundColorTag::execute(RootMovieClip* root)
+void SetBackgroundColorTag::execute(RootMovieClip* root) const
 {
 	root->setBackground(BackgroundColor);
 }
@@ -1164,7 +1234,7 @@ FrameLabelTag::FrameLabelTag(RECORDHEADER h, std::istream& in):Tag(h)
 		in >> NamedAnchor;
 }
 
-DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in,RootMovieClip* root):DictionaryTag(h,root)
 {
 	in >> ButtonId;
 	BitStream bs(in);
@@ -1187,7 +1257,7 @@ DefineButton2Tag::DefineButton2Tag(RECORDHEADER h, std::istream& in):DictionaryT
 		LOG(LOG_NOT_IMPLEMENTED,"DefineButton2Tag: Actions are not supported");
 }
 
-ASObject* DefineButton2Tag::instance() const
+ASObject* DefineButton2Tag::instance(Class_base* c) const
 {
 	DisplayObject* states[4] = {NULL, NULL, NULL, NULL};
 	bool isSprite[4] = {false, false, false, false};
@@ -1211,13 +1281,13 @@ ASObject* DefineButton2Tag::instance() const
 				continue;
 			if(j==3 && !i->ButtonStateUp)
 				continue;
-			_R<DictionaryTag> dict=loadedFrom->dictionaryLookup(i->CharacterID);
+			const DictionaryTag* dict=loadedFrom->dictionaryLookup(i->CharacterID);
 
 			//We can create the object right away
 			DisplayObject* state=dynamic_cast<DisplayObject*>(dict->instance());
 			assert_and_throw(state);
 			//The matrix must be set before invoking the constructor
-			state->setMatrix(i->PlaceMatrix);
+			state->setLegacyMatrix(i->PlaceMatrix);
 			/*
 			 * TODO: BlendMode, filerList, PlaceDepth, ColorTransfrom
 			 */
@@ -1241,14 +1311,15 @@ ASObject* DefineButton2Tag::instance() const
 		}
 	}
 
-	SimpleButton* ret=new SimpleButton(states[0], states[1], states[2], states[3]);
-	ret->setClass(Class<SimpleButton>::getClass());
+	if(c==NULL)
+		c=Class<SimpleButton>::getClass();
+	SimpleButton* ret=new (c->memoryAccount) SimpleButton(c, states[0], states[1], states[2], states[3]);
 	return ret;
 }
 
-DefineVideoStreamTag::DefineVideoStreamTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineVideoStreamTag::DefineVideoStreamTag(RECORDHEADER h, std::istream& in, RootMovieClip* root):DictionaryTag(h, root)
 {
-	LOG(LOG_INFO,_("DefineVideoStreamTag"));
+	LOG(LOG_INFO,"DefineVideoStreamTag");
 	in >> CharacterID >> NumFrames >> Width >> Height;
 	BitStream bs(in);
 	UB(4,bs);
@@ -1257,20 +1328,21 @@ DefineVideoStreamTag::DefineVideoStreamTag(RECORDHEADER h, std::istream& in):Dic
 	in >> CodecID;
 }
 
-ASObject* DefineVideoStreamTag::instance() const
+ASObject* DefineVideoStreamTag::instance(Class_base* c) const
 {
-	Video* ret=new Video(Width, Height);
-	if(bindedTo)
-	{
-		//A class is binded to this tag
-		ret->setClass(bindedTo);
-	}
+	Class_base* classRet = NULL;
+	if(c)
+		classRet=c;
+	else if(bindedTo)
+		classRet=bindedTo;
 	else
-		ret->setClass(Class<Video>::getClass());
+		classRet=Class<Video>::getClass();
+
+	Video* ret=new (classRet->memoryAccount) Video(classRet, Width, Height);
 	return ret;
 }
 
-DefineBinaryDataTag::DefineBinaryDataTag(RECORDHEADER h,std::istream& s):DictionaryTag(h)
+DefineBinaryDataTag::DefineBinaryDataTag(RECORDHEADER h,std::istream& s,RootMovieClip* root):DictionaryTag(h,root)
 {
 	LOG(LOG_TRACE,_("DefineBinaryDataTag"));
 	int size=h.getLength();
@@ -1281,6 +1353,23 @@ DefineBinaryDataTag::DefineBinaryDataTag(RECORDHEADER h,std::istream& s):Diction
 	s.read((char*)bytes,size);
 }
 
+ASObject* DefineBinaryDataTag::instance(Class_base* c) const
+{
+	uint8_t* b = new uint8_t[len];
+	memcpy(b,bytes,len);
+
+	Class_base* classRet = NULL;
+	if(c)
+		classRet=c;
+	else if(bindedTo)
+		classRet=bindedTo;
+	else
+		classRet=Class<ByteArray>::getClass();
+
+	ByteArray* ret=new (classRet->memoryAccount) ByteArray(classRet, b, len);
+	return ret;
+}
+
 FileAttributesTag::FileAttributesTag(RECORDHEADER h, std::istream& in):Tag(h)
 {
 	LOG(LOG_TRACE,_("FileAttributesTag Tag"));
@@ -1295,7 +1384,7 @@ FileAttributesTag::FileAttributesTag(RECORDHEADER h, std::istream& in):Tag(h)
 	UB(24,bs);
 }
 
-DefineSoundTag::DefineSoundTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineSoundTag::DefineSoundTag(RECORDHEADER h, std::istream& in,RootMovieClip* root):DictionaryTag(h,root)
 {
 	LOG(LOG_TRACE,_("DefineSound Tag"));
 	in >> SoundId;
@@ -1309,10 +1398,10 @@ DefineSoundTag::DefineSoundTag(RECORDHEADER h, std::istream& in):DictionaryTag(h
 	ignore(in,h.getLength()-7);
 }
 
-ASObject* DefineSoundTag::instance() const
+ASObject* DefineSoundTag::instance(Class_base* c) const
 {
 	//TODO: use the tag sound data
-	return Class<Sound>::getInstanceS();
+	return Class<Sound>::getInstanceS(c);
 }
 
 ScriptLimitsTag::ScriptLimitsTag(RECORDHEADER h, std::istream& in):ControlTag(h)
@@ -1384,7 +1473,7 @@ MetadataTag::MetadataTag(RECORDHEADER h, std::istream& in):Tag(h)
 	}
 }
 
-DefineBitsTag::DefineBitsTag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineBitsTag::DefineBitsTag(RECORDHEADER h, std::istream& in,RootMovieClip* root):BitmapTag(h,root)
 {
 	LOG(LOG_TRACE,_("DefineBitsTag Tag"));
 	in >> CharacterId;
@@ -1399,7 +1488,7 @@ DefineBitsTag::~DefineBitsTag()
 	delete[] data;
 }
 
-DefineBitsJPEG2Tag::DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h)
+DefineBitsJPEG2Tag::DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):BitmapTag(h,root)
 {
 	LOG(LOG_TRACE,_("DefineBitsJPEG2Tag Tag"));
 	in >> CharacterId;
@@ -1412,17 +1501,7 @@ DefineBitsJPEG2Tag::DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in):Diction
 	delete[] inData;
 }
 
-ASObject* DefineBitsJPEG2Tag::instance() const
-{
-	DefineBitsJPEG2Tag* ret=new DefineBitsJPEG2Tag(*this);
-	if(bindedTo)
-		ret->setClass(bindedTo);
-	else
-		ret->setClass(Class<BitmapData>::getClass());
-	return ret;
-}
-
-DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in):DictionaryTag(h),alphaData(NULL)
+DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root):BitmapTag(h,root),alphaData(NULL)
 {
 	LOG(LOG_TRACE,_("DefineBitsJPEG3Tag Tag"));
 	UI32_SWF dataSize;
@@ -1439,20 +1518,30 @@ DefineBitsJPEG3Tag::DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in):Diction
 	int alphaSize=Header.getLength()-dataSize-6;
 	if(alphaSize>0) //If less that 0 the consistency check on tag size will stop later
 	{
-		LOG(LOG_NOT_IMPLEMENTED,"DefineBitsJPEG3Tag does not use alpha yet");
-		alphaData=new(nothrow) uint8_t[alphaSize];
-		in.read((char*)alphaData,alphaSize);
-	}
-}
+		//Create a zlib filter
+		string alphaData;
+		alphaData.resize(alphaSize);
+		in.read(&alphaData[0], alphaSize);
+		istringstream alphaStream(alphaData);
+		zlib_filter zf(alphaStream.rdbuf());
+		istream zfstream(&zf);
+		zfstream.exceptions ( istream::eofbit | istream::failbit | istream::badbit );
 
-ASObject* DefineBitsJPEG3Tag::instance() const
-{
-	DefineBitsJPEG3Tag* ret=new DefineBitsJPEG3Tag(*this);
-	if(bindedTo)
-		ret->setClass(bindedTo);
-	else
-		ret->setClass(Class<BitmapData>::getClass());
-	return ret;
+		//Catch the exception if the stream ends
+		try
+		{
+			//Set alpha
+			for(int32_t i=0;i<height;i++)
+			{
+				for(int32_t j=0;j<width;j++)
+					data[i*stride + j*4 + 3]=zfstream.get();
+			}
+		}
+		catch(std::exception& e)
+		{
+			LOG(LOG_ERROR, "Exception while parsing Alpha data in DefineBitsJPEG3");
+		}
+	}
 }
 
 DefineBitsJPEG3Tag::~DefineBitsJPEG3Tag()
@@ -1479,7 +1568,7 @@ DefineSceneAndFrameLabelDataTag::DefineSceneAndFrameLabelDataTag(RECORDHEADER h,
 	}
 }
 
-void DefineSceneAndFrameLabelDataTag::execute(RootMovieClip* root)
+void DefineSceneAndFrameLabelDataTag::execute(RootMovieClip* root) const
 {
 	for(size_t i=0;i<SceneCount;++i)
 	{
