@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -22,13 +22,11 @@
 
 #ifdef LLVM_28
 #define alignof alignOf
-#endif
-#ifdef LLVM_3
-#define LLVMTYPE llvm::Type*
-#define LLVMMAKEARRAYREF(T) makeArrayRef(T)
-#else
 #define LLVMTYPE const llvm::Type*
 #define LLVMMAKEARRAYREF(T) T
+#else
+#define LLVMTYPE llvm::Type*
+#define LLVMMAKEARRAYREF(T) makeArrayRef(T)
 #endif
 
 #include "compat.h"
@@ -47,6 +45,7 @@
 #include "swf.h"
 #include "abcutils.h"
 #include "abctypes.h"
+#include "scripting/flash/system/flashsystem.h"
 
 namespace lightspark
 {
@@ -176,6 +175,8 @@ class ABCContext
 friend class ABCVm;
 friend class method_info;
 public:
+	_R<RootMovieClip> root;
+
 	method_info* get_method(unsigned int m);
 	const tiny_string& getString(unsigned int s) const;
 	//Qname getQname(unsigned int m, call_context* th=NULL) const;
@@ -187,16 +188,18 @@ public:
 	u16 major;
 	cpool_info constant_pool;
 	u30 method_count;
-	std::vector<method_info> methods;
+	std::vector<method_info, reporter_allocator<method_info>> methods;
 	u30 metadata_count;
-	std::vector<metadata_info> metadata;
+	std::vector<metadata_info, reporter_allocator<metadata_info>> metadata;
 	u30 class_count;
-	std::vector<instance_info> instances;
-	std::vector<class_info> classes;
+	std::vector<instance_info, reporter_allocator<instance_info>> instances;
+	std::vector<class_info, reporter_allocator<class_info>> classes;
 	u30 script_count;
-	std::vector<script_info> scripts;
+	std::vector<script_info, reporter_allocator<script_info>> scripts;
 	u30 method_body_count;
-	std::vector<method_body_info> method_body;
+	std::vector<method_body_info, reporter_allocator<method_body_info>> method_body;
+	//Base for namespaces in this context
+	uint32_t namespaceBaseId;
 
 	std::vector<bool> hasRunScriptInit;
 	/**
@@ -215,10 +218,12 @@ public:
 	multiname* getMultiname(unsigned int m, call_context* th);
 	multiname* getMultinameImpl(ASObject* rt1, ASObject* rt2, unsigned int m);
 	void buildInstanceTraits(ASObject* obj, int class_index);
-	ABCContext(std::istream& in) DLL_PUBLIC;
+	ABCContext(_R<RootMovieClip> r, std::istream& in, ABCVm* vm) DLL_PUBLIC;
 	void exec(bool lazy);
 
-	static bool isinstance(ASObject* obj, multiname* name);
+	bool isinstance(ASObject* obj, multiname* name);
+
+	std::map<const multiname*, Class_base*> classesBeingDefined;
 
 #ifdef PROFILING_SUPPORT
 	void dumpProfilingData(std::ostream& f) const;
@@ -250,7 +255,7 @@ private:
 		ASObject* arg1=th->runtime_stack_pop();
 		uint32_t addr=arg1->toUInt();
 		arg1->decRef();
-		_R<ApplicationDomain> appDomain = getSys()->mainApplicationDomain;
+		_R<ApplicationDomain> appDomain = getCurrentApplicationDomain(th);
 		T ret=appDomain->readFromDomainMemory<T>(addr);
 		th->runtime_stack_push(abstract_ui(ret));
 	}
@@ -260,8 +265,10 @@ private:
 		ASObject* arg1=th->runtime_stack_pop();
 		ASObject* arg2=th->runtime_stack_pop();
 		uint32_t addr=arg1->toUInt();
+		arg1->decRef();
 		uint32_t val=arg2->toUInt();
-		_R<ApplicationDomain> appDomain = getSys()->mainApplicationDomain;
+		arg2->decRef();
+		_R<ApplicationDomain> appDomain = getCurrentApplicationDomain(th);
 		appDomain->writeToDomainMemory<T>(addr, val);
 	}
 	static void callSuper(call_context* th, int n, int m, method_info** called_mi, bool keepReturn);
@@ -344,26 +351,26 @@ private:
 	static void pop();
 	static ASObject* typeOf(ASObject*);
 	static void _throw(call_context* th);
-	static ASObject* asType(ASObject* obj, multiname* name);
+	static ASObject* asType(ABCContext* context, ASObject* obj, multiname* name);
 	static ASObject* asTypelate(ASObject* type, ASObject* obj);
 	static bool isTypelate(ASObject* type, ASObject* obj);
-	static bool isType(ASObject* obj, multiname* name);
+	static bool isType(ABCContext* context, ASObject* obj, multiname* name);
 	static void swap();
 	static ASObject* add(ASObject*,ASObject*);
 	static int32_t add_i(ASObject*,ASObject*);
 	static ASObject* add_oi(ASObject*,int32_t);
 	static ASObject* add_od(ASObject*,number_t);
-	static uint32_t bitAnd(ASObject*,ASObject*);
-	static uint32_t bitNot(ASObject*);
-	static uint32_t bitAnd_oi(ASObject* val1, int32_t val2);
-	static uint32_t bitOr(ASObject*,ASObject*);
-	static uint32_t bitOr_oi(ASObject*,uint32_t);
-	static uint32_t bitXor(ASObject*,ASObject*);
-	static uint32_t rShift(ASObject*,ASObject*);
+	static int32_t bitAnd(ASObject*,ASObject*);
+	static int32_t bitNot(ASObject*);
+	static int32_t bitAnd_oi(ASObject* val1, int32_t val2);
+	static int32_t bitOr(ASObject*,ASObject*);
+	static int32_t bitOr_oi(ASObject*,int32_t);
+	static int32_t bitXor(ASObject*,ASObject*);
+	static int32_t rShift(ASObject*,ASObject*);
 	static uint32_t urShift(ASObject*,ASObject*);
 	static uint32_t urShift_io(uint32_t,ASObject*);
-	static uint32_t lShift(ASObject*,ASObject*);
-	static uint32_t lShift_io(uint32_t,ASObject*);
+	static int32_t lShift(ASObject*,ASObject*);
+	static int32_t lShift_io(uint32_t,ASObject*);
 	static number_t multiply(ASObject*,ASObject*);
 	static int32_t multiply_i(ASObject*,ASObject*);
 	static number_t multiply_oi(ASObject*, int32_t);
@@ -410,6 +417,7 @@ private:
 	static void method_reset(method_info* th);
 	static void newClassRecursiveLink(Class_base* target, Class_base* c);
 	static ASObject* constructFunction(call_context* th, IFunction* f, ASObject** args, int argslen);
+	void parseRPCMessage(_R<ByteArray> message, _NR<ASObject> client, _R<Responder> responder);
 
 	//Opcode tables
 	void register_table(LLVMTYPE ret_type,typed_opcode_handler* table, int table_len);
@@ -428,27 +436,31 @@ private:
 
 	//Event handling
 	volatile bool shuttingdown;
-	std::deque<std::pair<_NR<EventDispatcher>,_R<Event> > > events_queue;
+	typedef std::pair<_NR<EventDispatcher>,_R<Event>> eventType;
+	std::deque<eventType, reporter_allocator<eventType>> events_queue;
 	void handleEvent(std::pair<_NR<EventDispatcher>,_R<Event> > e);
 	void signalEventWaiters();
-	void buildClassAndBindTag(const std::string& s, _R<DictionaryTag> t);
+	void buildClassAndBindTag(const std::string& s, DictionaryTag* t);
 	void buildClassAndInjectBase(const std::string& s, _R<RootMovieClip> base);
-	Class_inherit* findClassInherit(const std::string& s);
+	Class_inherit* findClassInherit(const std::string& s, RootMovieClip* r);
 
 	//Profiling support
 	static uint64_t profilingCheckpoint(uint64_t& startTime);
+	// The base to assign to the next loaded context
+	ATOMIC_INT32(nextNamespaceBase);
 public:
 	call_context* currentCallContext;
-	GlobalObject* global;
 	Manager* int_manager;
 	Manager* uint_manager;
 	Manager* number_manager;
 
+	MemoryAccount* vmDataMemory;
+
 	llvm::ExecutionEngine* ex;
 	llvm::FunctionPassManager* FPM;
 	llvm::LLVMContext llvm_context;
 
-	ABCVm(SystemState* s) DLL_PUBLIC;
+	ABCVm(SystemState* s, MemoryAccount* m) DLL_PUBLIC;
 	/**
 		Destroys the VM
 
@@ -469,6 +481,8 @@ public:
 	static Global* getGlobalScope(call_context* th);
 	static bool strictEqualImpl(ASObject*, ASObject*);
 	static void publicHandleEvent(_R<EventDispatcher> dispatcher, _R<Event> event);
+	static _R<ApplicationDomain> getCurrentApplicationDomain(call_context* th);
+	static _R<SecurityDomain> getCurrentSecurityDomain(call_context* th);
 
 	/* The current recursion level. Each call increases this by one,
 	 * each return from a call decreases this. */
@@ -481,6 +495,8 @@ public:
 		uint32_t script_timeout;
 	} limits;
 
+	uint32_t getAndIncreaseNamespaceBase(uint32_t nsNum);
+
 };
 
 class DoABCTag: public ControlTag
@@ -490,7 +506,7 @@ private:
 public:
 	DoABCTag(RECORDHEADER h, std::istream& in);
 	TAGTYPE getType() const{ return ABC_TAG; }
-	void execute(RootMovieClip* root);
+	void execute(RootMovieClip* root) const;
 };
 
 class DoABCDefineTag: public ControlTag
@@ -502,7 +518,7 @@ private:
 public:
 	DoABCDefineTag(RECORDHEADER h, std::istream& in);
 	TAGTYPE getType() const{ return ABC_TAG; }
-	void execute(RootMovieClip* root);
+	void execute(RootMovieClip* root) const;
 };
 
 class SymbolClassTag: public ControlTag
@@ -514,16 +530,11 @@ private:
 public:
 	SymbolClassTag(RECORDHEADER h, std::istream& in);
 	TAGTYPE getType() const{ return SYMBOL_CLASS_TAG; }
-	void execute(RootMovieClip* root);
+	void execute(RootMovieClip* root) const;
 };
 
 ASObject* undefinedFunction(ASObject* obj,ASObject* const* args, const unsigned int argslen);
 
-inline GlobalObject* getGlobal()
-{
-	return getSys()->currentVm->global;
-}
-
 inline ABCVm* getVm()
 {
 	return getSys()->currentVm;
