@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -24,26 +24,27 @@
 #include "compat.h"
 #include "abcutils.h"
 #include "scripting/toplevel/ASString.h"
+#include "scripting/toplevel/RegExp.h"
 #include "toplevel/XML.h"
 #include "toplevel/XMLList.h"
 
 using namespace std;
 using namespace lightspark;
 
-uint32_t ABCVm::bitAnd(ASObject* val2, ASObject* val1)
+int32_t ABCVm::bitAnd(ASObject* val2, ASObject* val1)
 {
-	uint32_t i1=val1->toUInt();
-	uint32_t i2=val2->toUInt();
+	int32_t i1=val1->toInt();
+	int32_t i2=val2->toInt();
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("bitAnd_oo ") << hex << i1 << '&' << i2 << dec);
 	return i1&i2;
 }
 
-uint32_t ABCVm::bitAnd_oi(ASObject* val1, int32_t val2)
+int32_t ABCVm::bitAnd_oi(ASObject* val1, int32_t val2)
 {
-	uint32_t i1=val1->toUInt();
-	uint32_t i2=val2;
+	int32_t i1=val1->toInt();
+	int32_t i2=val2;
 	val1->decRef();
 	LOG(LOG_CALLS,_("bitAnd_oi ") << hex << i1 << '&' << i2 << dec);
 	return i1&i2;
@@ -53,8 +54,8 @@ void ABCVm::setProperty(ASObject* value,ASObject* obj,multiname* name)
 {
 	LOG(LOG_CALLS,_("setProperty ") << *name << ' ' << obj);
 
-	//We have to reset the level before finding a variable
-	obj->setVariableByMultiname(*name,value);
+	//Do not allow to set contant traits
+	obj->setVariableByMultiname(*name,value,ASObject::CONST_NOT_ALLOWED);
 	obj->decRef();
 }
 
@@ -123,13 +124,13 @@ void ABCVm::lookupswitch()
 ASObject* ABCVm::pushUndefined()
 {
 	LOG(LOG_CALLS, _("pushUndefined") );
-	return new Undefined;
+	return getSys()->getUndefinedRef();
 }
 
 ASObject* ABCVm::pushNull()
 {
 	LOG(LOG_CALLS, _("pushNull") );
-	return new Null;
+	return getSys()->getNullRef();
 }
 
 void ABCVm::coerce_a()
@@ -158,7 +159,7 @@ void ABCVm::coerce(call_context* th, int n)
 	multiname* mn = th->context->getMultiname(n,NULL);
 	LOG(LOG_CALLS,_("coerce ") << *mn);
 
-	const Type* type = Type::getTypeFromMultiname(mn);
+	const Type* type = Type::getTypeFromMultiname(mn, th->context);
 
 	ASObject* o=th->runtime_stack_pop();
 	o=type->coerce(o);
@@ -210,7 +211,7 @@ void ABCVm::setSlot(ASObject* value, ASObject* obj, int n)
 {
 	LOG(LOG_CALLS,"setSlot " << n);
 	if(value==NULL)
-		value=new Undefined;
+		value=getSys()->getUndefinedRef();
 	obj->setSlot(n,value);
 	obj->decRef();
 }
@@ -243,37 +244,37 @@ int32_t ABCVm::negate_i(ASObject* o)
 	return -n;
 }
 
-uint32_t ABCVm::bitNot(ASObject* val)
+int32_t ABCVm::bitNot(ASObject* val)
 {
-	uint32_t i1=val->toUInt();
+	int32_t i1=val->toInt();
 	val->decRef();
 	LOG(LOG_CALLS,_("bitNot ") << hex << i1 << dec);
 	return ~i1;
 }
 
-uint32_t ABCVm::bitXor(ASObject* val2, ASObject* val1)
+int32_t ABCVm::bitXor(ASObject* val2, ASObject* val1)
 {
-	int i1=val1->toUInt();
-	int i2=val2->toUInt();
+	int32_t i1=val1->toInt();
+	int32_t i2=val2->toInt();
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("bitXor ") << hex << i1 << '^' << i2 << dec);
 	return i1^i2;
 }
 
-uint32_t ABCVm::bitOr_oi(ASObject* val2, uint32_t val1)
+int32_t ABCVm::bitOr_oi(ASObject* val2, int32_t val1)
 {
-	int i1=val1;
-	int i2=val2->toUInt();
+	int32_t i1=val1;
+	int32_t i2=val2->toInt();
 	val2->decRef();
 	LOG(LOG_CALLS,_("bitOr ") << hex << i1 << '|' << i2 << dec);
 	return i1|i2;
 }
 
-uint32_t ABCVm::bitOr(ASObject* val2, ASObject* val1)
+int32_t ABCVm::bitOr(ASObject* val2, ASObject* val1)
 {
-	int i1=val1->toUInt();
-	int i2=val2->toUInt();
+	int32_t i1=val1->toInt();
+	int32_t i2=val2->toInt();
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("bitOr ") << hex << i1 << '|' << i2 << dec);
@@ -295,6 +296,7 @@ void ABCVm::callProperty(call_context* th, int n, int m, method_info** called_mi
 
 	//We should skip the special implementation of get
 	_NR<ASObject> o=obj->getVariableByMultiname(*name, ASObject::SKIP_IMPL);
+	name->resetNameIfObject();
 
 	if(!o.isNull())
 	{
@@ -307,9 +309,9 @@ void ABCVm::callProperty(call_context* th, int n, int m, method_info** called_mi
 		if(obj->classdef && obj->classdef->isSubClass(Class<Proxy>::getClass()))
 		{
 			//Check if there is a custom caller defined, skipping implementation to avoid recursive calls
-			multiname callPropertyName;
+			multiname callPropertyName(NULL);
 			callPropertyName.name_type=multiname::NAME_STRING;
-			callPropertyName.name_s="callProperty";
+			callPropertyName.name_s_id=getSys()->getUniqueStringId("callProperty");
 			callPropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 			_NR<ASObject> o=obj->getVariableByMultiname(callPropertyName,ASObject::SKIP_IMPL);
 
@@ -321,7 +323,7 @@ void ABCVm::callProperty(call_context* th, int n, int m, method_info** called_mi
 				//Create a new array
 				ASObject** proxyArgs=g_newa(ASObject*, m+1);
 				//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
-				proxyArgs[0]=Class<ASString>::getInstanceS(name->name_s);
+				proxyArgs[0]=Class<ASString>::getInstanceS(getSys()->getStringFromUniqueId(name->name_s_id));
 				for(int i=0;i<m;i++)
 					proxyArgs[i+1]=args[i];
 
@@ -347,7 +349,7 @@ void ABCVm::callProperty(call_context* th, int n, int m, method_info** called_mi
 
 		LOG(LOG_NOT_IMPLEMENTED,"callProperty: " << name->normalizedName() << " not found on " << obj->toDebugString());
 		if(keepReturn)
-			th->runtime_stack_push(new Undefined);
+			th->runtime_stack_push(getSys()->getUndefinedRef());
 
 		obj->decRef();
 		for(int i=0;i<m;i++)
@@ -377,7 +379,7 @@ ASObject* ABCVm::getProperty(ASObject* obj, multiname* name)
 	if(prop.isNull())
 	{
 		LOG(LOG_NOT_IMPLEMENTED,"getProperty: " << name->normalizedName() << " not found on " << obj->toDebugString());
-		ret = new Undefined;
+		ret = getSys()->getUndefinedRef();
 	}
 	else
 	{
@@ -589,18 +591,21 @@ ASObject* ABCVm::constructFunction(call_context* th, IFunction* f, ASObject** ar
 	SyntheticFunction* sf=f->as<SyntheticFunction>();
 
 	ASObject* ret=Class<ASObject>::getInstanceS();
-	assert(sf->mi->body);
 #ifndef NDEBUG
 	ret->initialized=false;
 #endif
-	LOG(LOG_CALLS,_("Building method traits"));
-	for(unsigned int i=0;i<sf->mi->body->trait_count;i++)
-		th->context->buildTrait(ret,&sf->mi->body->traits[i],false);
+	if (sf->isConstructed())
+	{
+		LOG(LOG_CALLS,_("Building method traits"));
+		assert(sf->mi->body);
+		for(unsigned int i=0;i<sf->mi->body->trait_count;i++)
+			th->context->buildTrait(ret,&sf->mi->body->traits[i],false);
+	}
 #ifndef NDEBUG
 	ret->initialized=true;
 #endif
 	//Now add our classdef
-	Class_function *cf=new Class_function();
+	Class_function *cf=new (getSys()->unaccountedMemory) Class_function();
 	ret->setClass(cf);
 	//setClass created a new reference, we can release the local
 	//reference
@@ -611,7 +616,9 @@ ASObject* ABCVm::constructFunction(call_context* th, IFunction* f, ASObject** ar
 	ret->setVariableByQName("constructor","",sf,DYNAMIC_TRAIT);
 
 	ret->incRef();
-	assert_and_throw(!sf->closure_this);
+	if (sf->closure_this)
+		throw Class<TypeError>::getInstanceS("Error #1064: Cannot call method as constructor");
+
 	sf->incRef();
 	ASObject* ret2=sf->call(ret,args,argslen);
 	sf->decRef();
@@ -674,8 +681,7 @@ void ABCVm::construct(call_context* th, int m)
 
 		default:
 		{
-			LOG(LOG_ERROR,_("Object type ") << obj->getObjectType() << _(" not supported in construct"));
-			throw UnsupportedException("This object is not supported in construct");
+			throw Class<TypeError>::getInstanceS("Error #1007: Instantiation attempted on a non-constructor");
 			break;
 		}
 	}
@@ -688,7 +694,8 @@ void ABCVm::construct(call_context* th, int m)
 void ABCVm::constructGenericType(call_context* th, int m)
 {
 	LOG(LOG_CALLS, _("constructGenericType ") << m);
-	assert_and_throw(m==1);
+	if (m != 1)
+			throw Class<TypeError>::getInstanceS("Error #1128");
 	ASObject** args=g_newa(ASObject*, m);
 	for(int i=0;i<m;i++)
 		args[m-i-1]=th->runtime_stack_pop();
@@ -699,7 +706,7 @@ void ABCVm::constructGenericType(call_context* th, int m)
 	{
 		LOG(LOG_NOT_IMPLEMENTED, "constructGenericType of " << obj->getObjectType());
 		obj->decRef();
-		obj = new Undefined();
+		obj = getSys()->getUndefinedRef();
 		th->runtime_stack_push(obj);
 		for(int i=0;i<m;i++)
 			args[i]->decRef();
@@ -748,6 +755,8 @@ ASObject* ABCVm::typeOf(ASObject* obj)
 		case T_NULL:
 		case T_ARRAY:
 		case T_CLASS: //this is not clear from spec, but was tested
+		case T_QNAME:
+		case T_NAMESPACE:
 			ret="object";
 			break;
 		case T_BOOLEAN:
@@ -941,7 +950,10 @@ ASObject* ABCVm::add(ASObject* val2, ASObject* val1)
 		}
 		else
 		{//Convert both to numbers and add
-			number_t result = val2p->toNumber() + val1p->toNumber();
+			number_t num1=val1p->toNumber();
+			number_t num2=val2p->toNumber();
+			LOG(LOG_CALLS,"addN " << num1 << '+' << num2);
+			number_t result = num1 + num2;
 			return abstract_d(result);
 		}
 	}
@@ -1036,33 +1048,33 @@ ASObject* ABCVm::add_od(ASObject* val2, number_t val1)
 
 }
 
-uint32_t ABCVm::lShift(ASObject* val1, ASObject* val2)
+int32_t ABCVm::lShift(ASObject* val1, ASObject* val2)
 {
-	uint32_t i2=val2->toInt();
-	int32_t i1=val1->toInt()&0x1f;
+	int32_t i2=val2->toInt();
+	uint32_t i1=val1->toUInt()&0x1f;
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("lShift ")<<hex<<i2<<_("<<")<<i1<<dec);
 	//Left shift are supposed to always work in 32bit
-	uint32_t ret=i2<<i1;
+	int32_t ret=i2<<i1;
 	return ret;
 }
 
-uint32_t ABCVm::lShift_io(uint32_t val1, ASObject* val2)
+int32_t ABCVm::lShift_io(uint32_t val1, ASObject* val2)
 {
-	uint32_t i2=val2->toInt();
-	int32_t i1=val1&0x1f;
+	int32_t i2=val2->toInt();
+	uint32_t i1=val1&0x1f;
 	val2->decRef();
 	LOG(LOG_CALLS,_("lShift ")<<hex<<i2<<_("<<")<<i1<<dec);
 	//Left shift are supposed to always work in 32bit
-	uint32_t ret=i2<<i1;
+	int32_t ret=i2<<i1;
 	return ret;
 }
 
-uint32_t ABCVm::rShift(ASObject* val1, ASObject* val2)
+int32_t ABCVm::rShift(ASObject* val1, ASObject* val2)
 {
 	int32_t i2=val2->toInt();
-	int32_t i1=val1->toInt()&0x1f;
+	uint32_t i1=val1->toUInt()&0x1f;
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("rShift ")<<hex<<i2<<_(">>")<<i1<<dec);
@@ -1071,8 +1083,8 @@ uint32_t ABCVm::rShift(ASObject* val1, ASObject* val2)
 
 uint32_t ABCVm::urShift(ASObject* val1, ASObject* val2)
 {
-	uint32_t i2=val2->toInt();
-	int32_t i1=val1->toInt()&0x1f;
+	uint32_t i2=val2->toUInt();
+	uint32_t i1=val1->toUInt()&0x1f;
 	val1->decRef();
 	val2->decRef();
 	LOG(LOG_CALLS,_("urShift ")<<hex<<i2<<_(">>")<<i1<<dec);
@@ -1081,8 +1093,8 @@ uint32_t ABCVm::urShift(ASObject* val1, ASObject* val2)
 
 uint32_t ABCVm::urShift_io(uint32_t val1, ASObject* val2)
 {
-	uint32_t i2=val2->toInt();
-	int32_t i1=val1&0x1f;
+	uint32_t i2=val2->toUInt();
+	uint32_t i1=val1&0x1f;
 	val2->decRef();
 	LOG(LOG_CALLS,_("urShift ")<<hex<<i2<<_(">>")<<i1<<dec);
 	return i2>>i1;
@@ -1164,7 +1176,7 @@ ASObject* ABCVm::pushNaN()
 {
 	LOG(LOG_CALLS, _("pushNaN") );
 	//Not completely correct, but mostly ok
-	return new Undefined;
+	return getSys()->getUndefinedRef();
 }
 
 bool ABCVm::ifGT(ASObject* obj2, ASObject* obj1)
@@ -1249,7 +1261,8 @@ void ABCVm::setSuper(call_context* th, int n)
 	assert_and_throw(obj->getClass());
 	assert_and_throw(obj->getClass()->isSubClass(th->inClass));
 
-	obj->setVariableByMultiname(*name,value,th->inClass->super.getPtr());
+	obj->setVariableByMultiname(*name,value,ASObject::CONST_NOT_ALLOWED,th->inClass->super.getPtr());
+	name->resetNameIfObject();
 	obj->decRef();
 }
 
@@ -1266,10 +1279,11 @@ void ABCVm::getSuper(call_context* th, int n)
 	assert_and_throw(obj->getClass()->isSubClass(th->inClass));
 
 	_NR<ASObject> ret = obj->getVariableByMultiname(*name,ASObject::NONE,th->inClass->super.getPtr());
+	name->resetNameIfObject();
 	if(ret.isNull())
 	{
 		LOG(LOG_NOT_IMPLEMENTED,"getSuper: " << name->normalizedName() << " not found on " << obj->toDebugString());
-		ret = _MNR(new Undefined);
+		ret = _MNR(getSys()->getUndefinedRef());
 	}
 
 	obj->decRef();
@@ -1310,16 +1324,18 @@ void ABCVm::getLex(call_context* th, int n)
 	if(o==NULL)
 	{
 		ASObject* target;
-		o=getGlobal()->getVariableAndTargetByMultiname(*name, target);
+		o=getCurrentApplicationDomain(th)->getVariableAndTargetByMultiname(*name, target);
 		if(o==NULL)
 		{
 			LOG(LOG_NOT_IMPLEMENTED,"getLex: " << *name<< " not found, pushing Undefined");
-			th->runtime_stack_push(new Undefined);
+			th->runtime_stack_push(getSys()->getUndefinedRef());
+			name->resetNameIfObject();
 			return;
 		}
 		o->incRef();
 	}
 
+	name->resetNameIfObject();
 	th->runtime_stack_push(o);
 }
 
@@ -1339,6 +1355,7 @@ void ABCVm::constructSuper(call_context* th, int m)
 	LOG(LOG_CALLS,_("Super prototype name ") << th->inClass->super->class_name);
 
 	th->inClass->super->handleConstruction(obj,args, m, false);
+	obj->decRef();
 	LOG(LOG_CALLS,_("End super construct "));
 }
 
@@ -1364,7 +1381,7 @@ ASObject* ABCVm::findProperty(call_context* th, multiname* name)
 	{
 		//try to find a global object where this is defined
 		ASObject* target;
-		ASObject* o=getGlobal()->getVariableAndTargetByMultiname(*name, target);
+		ASObject* o=getCurrentApplicationDomain(th)->getVariableAndTargetByMultiname(*name, target);
 		if(o)
 			ret=target;
 		else //else push the current global object
@@ -1398,13 +1415,13 @@ ASObject* ABCVm::findPropStrict(call_context* th, multiname* name)
 	if(!found)
 	{
 		ASObject* target;
-		ASObject* o=getGlobal()->getVariableAndTargetByMultiname(*name, target);
+		ASObject* o=getCurrentApplicationDomain(th)->getVariableAndTargetByMultiname(*name, target);
 		if(o)
 			ret=target;
 		else
 		{
 			LOG(LOG_NOT_IMPLEMENTED,"findPropStrict: " << *name << " not found, pushing Undefined");
-			return new Undefined;
+			return getSys()->getUndefinedRef();
 		}
 	}
 
@@ -1450,7 +1467,8 @@ void ABCVm::initProperty(ASObject* obj, ASObject* value, multiname* name)
 {
 	LOG(LOG_CALLS, _("initProperty ") << *name << ' ' << obj);
 
-	obj->setVariableByMultiname(*name,value);
+	//Allow to set contant traits
+	obj->setVariableByMultiname(*name,value,ASObject::CONST_ALLOWED);
 
 	obj->decRef();
 }
@@ -1471,6 +1489,7 @@ void ABCVm::callSuper(call_context* th, int n, int m, method_info** called_mi, b
 	assert_and_throw(obj->getClass());
 	assert_and_throw(obj->getClass()->isSubClass(th->inClass));
 	_NR<ASObject> f = obj->getVariableByMultiname(*name,ASObject::NONE,th->inClass->super.getPtr());
+	name->resetNameIfObject();
 	if(!f.isNull())
 	{
 		f->incRef();
@@ -1478,16 +1497,16 @@ void ABCVm::callSuper(call_context* th, int n, int m, method_info** called_mi, b
 	}
 	else
 	{
-		LOG(LOG_ERROR,_("Calling an undefined function ") << name->name_s);
+		LOG(LOG_ERROR,_("Calling an undefined function ") << getSys()->getStringFromUniqueId(name->name_s_id));
 		if(keepReturn)
-			th->runtime_stack_push(new Undefined);
+			th->runtime_stack_push(getSys()->getUndefinedRef());
 	}
 	LOG(LOG_CALLS,_("End of callSuper ") << *name);
 }
 
-bool ABCVm::isType(ASObject* obj, multiname* name)
+bool ABCVm::isType(ABCContext* context, ASObject* obj, multiname* name)
 {
-	bool ret = ABCContext::isinstance(obj, name);
+	bool ret = context->isinstance(obj, name);
 	obj->decRef();
 	return ret;
 }
@@ -1524,9 +1543,16 @@ bool ABCVm::isTypelate(ASObject* type, ASObject* obj)
 	//Special case numeric types
 	if(obj->getObjectType()==T_INTEGER || obj->getObjectType()==T_UINTEGER || obj->getObjectType()==T_NUMBER)
 	{
-		obj->decRef();
-		real_ret=(c==Class<Integer>::getClass() || c==Class<Number>::getClass() || c==Class<UInteger>::getClass());
+		if(c==Class<Number>::getClass() || c==Class<ASObject>::getClass())
+			real_ret=true;
+		else if(c==Class<Integer>::getClass())
+			real_ret=(obj->toNumber()==obj->toInt());
+		else if(c==Class<UInteger>::getClass())
+			real_ret=(obj->toNumber()==obj->toUInt());
+		else
+			real_ret=false;
 		LOG(LOG_CALLS,_("Numeric type is ") << ((real_ret)?"":_("not ")) << _("subclass of ") << c->class_name);
+		obj->decRef();
 		type->decRef();
 		return real_ret;
 	}
@@ -1542,7 +1568,8 @@ bool ABCVm::isTypelate(ASObject* type, ASObject* obj)
 		assert_and_throw(type->getObjectType()==T_CLASS);
 
 		//Special case for Class
-		if(c->class_name.name=="Class" && c->class_name.ns=="")
+		if(c->class_name.ns=="" &&
+			(c->class_name.name=="Class" || c->class_name.name=="Object"))
 		{
 			type->decRef();
 			obj->decRef();
@@ -1572,9 +1599,9 @@ bool ABCVm::isTypelate(ASObject* type, ASObject* obj)
 	return real_ret;
 }
 
-ASObject* ABCVm::asType(ASObject* obj, multiname* name)
+ASObject* ABCVm::asType(ABCContext* context, ASObject* obj, multiname* name)
 {
-	bool ret = ABCContext::isinstance(obj, name);	
+	bool ret = context->isinstance(obj, name);
 	LOG(LOG_CALLS,_("asType"));
 	
 	if(ret)
@@ -1582,7 +1609,7 @@ ASObject* ABCVm::asType(ASObject* obj, multiname* name)
 	else
 	{
 		obj->decRef();
-		return new Null;
+		return getSys()->getNullRef();
 	}
 }
 
@@ -1608,7 +1635,15 @@ ASObject* ABCVm::asTypelate(ASObject* type, ASObject* obj)
 	//Special case numeric types
 	if(obj->getObjectType()==T_INTEGER || obj->getObjectType()==T_UINTEGER || obj->getObjectType()==T_NUMBER)
 	{
-		bool real_ret=(c==Class<Integer>::getClass() || c==Class<Number>::getClass() || c==Class<UInteger>::getClass());
+		bool real_ret;
+		if(c==Class<Number>::getClass() || c==Class<ASObject>::getClass())
+			real_ret=true;
+		else if(c==Class<Integer>::getClass())
+			real_ret=(obj->toNumber()==obj->toInt());
+		else if(c==Class<UInteger>::getClass())
+			real_ret=(obj->toNumber()==obj->toUInt());
+		else
+			real_ret=false;
 		LOG(LOG_CALLS,_("Numeric type is ") << ((real_ret)?"":_("not ")) << _("subclass of ") << c->class_name);
 		type->decRef();
 		if(real_ret)
@@ -1616,7 +1651,7 @@ ASObject* ABCVm::asTypelate(ASObject* type, ASObject* obj)
 		else
 		{
 			obj->decRef();
-			return new Null;
+			return getSys()->getNullRef();
 		}
 	}
 
@@ -1626,20 +1661,21 @@ ASObject* ABCVm::asTypelate(ASObject* type, ASObject* obj)
 	else if(obj->getObjectType()==T_CLASS)
 	{
 		//Special case for Class
-		if(c->class_name.name=="Class" && c->class_name.ns=="")
+		if(c->class_name.ns=="" &&
+			(c->class_name.name=="Class" || c->class_name.name=="Object"))
 		{
 			type->decRef();
 			return obj;
 		}
 		obj->decRef();
 		type->decRef();
-		return new Null;
+		return getSys()->getNullRef();
 	}
 	else
 	{
 		obj->decRef();
 		type->decRef();
-		return new Null;
+		return getSys()->getNullRef();
 	}
 
 	bool real_ret=objc->isSubClass(c);
@@ -1651,7 +1687,7 @@ ASObject* ABCVm::asTypelate(ASObject* type, ASObject* obj)
 	else
 	{
 		obj->decRef();
-		return new Null;
+		return getSys()->getNullRef();
 	}
 }
 
@@ -1687,11 +1723,13 @@ bool ABCVm::ifStrictNE(ASObject* obj2, ASObject* obj1)
 bool ABCVm::in(ASObject* val2, ASObject* val1)
 {
 	LOG(LOG_CALLS, _("in") );
-	multiname name;
-	name.name_type=multiname::NAME_STRING;
-	name.name_s=val1->toString();
+	multiname name(NULL);
+	name.name_type=multiname::NAME_OBJECT;
+	//Acquire the reference
+	name.name_o=val1;
 	name.ns.push_back(nsNameAndKind("",NAMESPACE));
 	bool ret=val2->hasPropertyByMultiname(name, true);
+	name.name_o=NULL;
 	val1->decRef();
 	val2->decRef();
 	return ret;
@@ -1719,6 +1757,7 @@ void ABCVm::constructProp(call_context* th, int n, int m)
 	ASObject* obj=th->runtime_stack_pop();
 
 	_NR<ASObject> o=obj->getVariableByMultiname(*name);
+	name->resetNameIfObject();
 
 	if(o.isNull())
 	{
@@ -1730,17 +1769,28 @@ void ABCVm::constructProp(call_context* th, int n, int m)
 
 	LOG(LOG_CALLS,_("Constructing"));
 	ASObject* ret;
-	if(o->getObjectType()==T_CLASS)
+	try
 	{
-		Class_base* o_class=static_cast<Class_base*>(o.getPtr());
-		ret=o_class->getInstance(true,args,m);
+		if(o->getObjectType()==T_CLASS)
+		{
+			Class_base* o_class=static_cast<Class_base*>(o.getPtr());
+			ret=o_class->getInstance(true,args,m);
+		}
+		else if(o->getObjectType()==T_FUNCTION)
+		{
+			ret = constructFunction(th, o->as<IFunction>(), args, m);
+		}
+		else
+			throw Class<TypeError>::getInstanceS("Error #1007: Cannot construct such an object in constructProp");
 	}
-	else if(o->getObjectType()==T_FUNCTION)
+	catch(ASObject* exc)
 	{
-		ret = constructFunction(th, o->as<IFunction>(), args, m);
+		LOG(LOG_CALLS,_("Exception during object construction. Returning Undefined"));
+		//Handle eventual exceptions from the constructor, to fix the stack
+		th->runtime_stack_push(getSys()->getUndefinedRef());
+		obj->decRef();
+		throw exc;
 	}
-	else
-		throw RunTimeException("Cannot construct such an object in constructProp");
 
 	th->runtime_stack_push(ret);
 	obj->decRef();
@@ -1763,7 +1813,7 @@ bool ABCVm::hasNext2(call_context* th, int n, int m)
 	if(newIndex==0)
 	{
 		th->locals[n]->decRef();
-		th->locals[n]=new Null;
+		th->locals[n]=getSys()->getNullRef();
 		return false;
 	}
 	return true;
@@ -1774,16 +1824,16 @@ void ABCVm::newObject(call_context* th, int n)
 	LOG(LOG_CALLS,_("newObject ") << n);
 	ASObject* ret=Class<ASObject>::getInstanceS();
 	//Duplicated keys overwrite the previous value
-	multiname propertyName;
+	multiname propertyName(NULL);
 	propertyName.name_type=multiname::NAME_STRING;
 	propertyName.ns.push_back(nsNameAndKind("",NAMESPACE));
 	for(int i=0;i<n;i++)
 	{
 		ASObject* value=th->runtime_stack_pop();
 		ASObject* name=th->runtime_stack_pop();
-		propertyName.name_s=name->toString();
+		propertyName.name_s_id=getSys()->getUniqueStringId(name->toString());
 		name->decRef();
-		ret->setVariableByMultiname(propertyName, value);
+		ret->setVariableByMultiname(propertyName, value, ASObject::CONST_NOT_ALLOWED);
 	}
 
 	th->runtime_stack_push(ret);
@@ -1796,17 +1846,58 @@ void ABCVm::getDescendants(call_context* th, int n)
 	ASObject* obj=th->runtime_stack_pop();
 	//The name must be a QName
 	assert_and_throw(name->name_type==multiname::NAME_STRING);
-	vector<_R<XML> > ret;
+	XML::XMLVector ret;
 	//TODO: support multiname and namespaces
 	if(obj->getClass()==Class<XML>::getClass())
 	{
 		XML* xmlObj=Class<XML>::cast(obj);
-		xmlObj->getDescendantsByQName(name->name_s, "", ret);
+		xmlObj->getDescendantsByQName(getSys()->getStringFromUniqueId(name->name_s_id), "", ret);
 	}
 	else if(obj->getClass()==Class<XMLList>::getClass())
 	{
 		XMLList* xmlObj=Class<XMLList>::cast(obj);
-		xmlObj->getDescendantsByQName(name->name_s, "", ret);
+		xmlObj->getDescendantsByQName(getSys()->getStringFromUniqueId(name->name_s_id), "", ret);
+	}
+	else if(obj->getClass()->isSubClass(Class<Proxy>::getClass()))
+	{
+		_NR<ASObject> o=obj->getVariableByMultiname(*name, ASObject::SKIP_IMPL);
+		if(!o.isNull())
+		{
+			o->incRef();
+			multiname callPropertyName(NULL);
+			callPropertyName.name_type=multiname::NAME_STRING;
+			callPropertyName.name_s_id=getSys()->getUniqueStringId("callProperty");
+			callPropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
+			_NR<ASObject> o=obj->getVariableByMultiname(callPropertyName,ASObject::SKIP_IMPL);
+
+			if(!o.isNull())
+			{
+				assert_and_throw(o->getObjectType()==T_FUNCTION);
+
+				IFunction* f=static_cast<IFunction*>(o.getPtr());
+				//Create a new array
+				ASObject** proxyArgs=g_newa(ASObject*, 1);
+				//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
+				proxyArgs[0]=Class<ASString>::getInstanceS(getSys()->getStringFromUniqueId(name->name_s_id));
+
+				//We now suppress special handling
+				LOG(LOG_CALLS,_("Proxy::callProperty"));
+				f->incRef();
+				obj->incRef();
+				ASObject* ret=f->call(obj,proxyArgs,1);
+				f->decRef();
+				th->runtime_stack_push(ret);
+
+				obj->decRef();
+				LOG(LOG_CALLS,_("End of calling ") << *name);
+				return;
+			}
+			else
+			{
+				obj->decRef();
+				throw Class<TypeError>::getInstanceS("Only XML and XMLList objects have descendants");
+			}
+		}
 	}
 	else
 	{
@@ -1885,34 +1976,32 @@ void ABCVm::newClass(call_context* th, int n)
 	ASObject* baseClass=th->runtime_stack_pop();
 
 	assert_and_throw(mname->ns.size()==1);
-	QName className(mname->name_s,mname->ns[0].name);
+	QName className(getSys()->getStringFromUniqueId(mname->name_s_id),mname->ns[0].getImpl().name);
 	//Check if this class has been already defined
-	auto oldDefinition=getSys()->classes.find(className);
-	if(oldDefinition!=getSys()->classes.end())
+	_NR<ApplicationDomain> domain = getCurrentApplicationDomain(th);
+	ASObject* target;
+	ASObject* oldDefinition=domain->getVariableAndTargetByMultiname(*mname, target);
+	if(oldDefinition && oldDefinition->getObjectType()==T_CLASS)
 	{
 		LOG(LOG_CALLS,_("Class ") << className << _(" already defined. Pushing previous definition"));
 		baseClass->decRef();
-		oldDefinition->second->incRef();
-		th->runtime_stack_push(oldDefinition->second);
+		oldDefinition->incRef();
+		th->runtime_stack_push(oldDefinition);
 		return;
 	}
 
-	Class_inherit* ret=new Class_inherit(className);
+	MemoryAccount* memoryAccount = getSys()->allocateMemoryAccount(className.name);
+	Class_inherit* ret=new (getSys()->unaccountedMemory) Class_inherit(className, memoryAccount);
+
+	//Add the class to the ones being currently defined in this context
+	th->context->classesBeingDefined.insert(make_pair(mname, ret));
+
 	ret->isFinal = th->context->instances[n].isFinal();
 	ret->isSealed = th->context->instances[n].isSealed();
 
 	assert_and_throw(th->context);
 	ret->context=th->context;
 
-	//Add protected namespace if needed
-	if(th->context->instances[n].isProtectedNs())
-	{
-		ret->use_protected=true;
-		int ns=th->context->instances[n].protectedNs;
-		const namespace_info& ns_info=th->context->constant_pool.namespaces[ns];
-		ret->protected_ns=nsNameAndKind(th->context->getString(ns_info.name),(NS_KIND)(int)ns_info.kind);
-	}
-
 	//Null is a "valid" base class
 	if(baseClass->getObjectType()!=T_NULL)
 	{
@@ -1922,6 +2011,15 @@ void ABCVm::newClass(call_context* th, int n)
 		ret->setSuper(_MR(base));
 	}
 
+	//Add protected namespace if needed
+	if(th->context->instances[n].isProtectedNs())
+	{
+		ret->use_protected=true;
+		int ns=th->context->instances[n].protectedNs;
+		const namespace_info& ns_info=th->context->constant_pool.namespaces[ns];
+		ret->initializeProtectedNamespace(th->context->getString(ns_info.name),ns_info);
+	}
+
 	ret->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(Class_base::_toString),NORMAL_METHOD,false);
 
 	ret->class_scope=th->scope_stack;
@@ -1977,7 +2075,7 @@ void ABCVm::newClass(call_context* th, int n)
 
 		//Make the class valid if needed
 		ASObject* target;
-		ASObject* obj=getGlobal()->getVariableAndTargetByMultiname(*name, target);
+		ASObject* obj=getCurrentApplicationDomain(th)->getVariableAndTargetByMultiname(*name, target);
 
 		//Named only interfaces seems to be allowed 
 		if(obj==NULL)
@@ -1998,12 +2096,30 @@ void ABCVm::newClass(call_context* th, int n)
 	SyntheticFunction* cinit=Class<IFunction>::getSyntheticFunction(m);
 	//cinit must inherit the current scope
 	cinit->acquireScope(th->scope_stack);
-	ASObject* ret2=cinit->call(ret,NULL,0);
+	ASObject* ret2=NULL;
+	try
+	{
+		ret2=cinit->call(ret,NULL,0);
+	}
+	catch(ASObject* exc)
+	{
+		LOG(LOG_CALLS,_("Exception during class initialization. Returning Undefined"));
+		//Handle eventual exceptions from the constructor, to fix the stack
+		th->runtime_stack_push(getSys()->getUndefinedRef());
+		cinit->decRef();
+
+		//Remove the class to the ones being currently defined in this context
+		th->context->classesBeingDefined.erase(mname);
+		throw exc;
+	}
 	assert_and_throw(ret2->is<Undefined>());
 	ret2->decRef();
 	LOG(LOG_CALLS,_("End of Class init ") << ret);
 	th->runtime_stack_push(ret);
 	cinit->decRef();
+
+	//Remove the class to the ones being currently defined in this context
+	th->context->classesBeingDefined.erase(mname);
 }
 
 void ABCVm::swap()
@@ -2063,7 +2179,6 @@ void ABCVm::callImpl(call_context* th, ASObject* f, ASObject* obj, ASObject** ar
 	if(f->is<Function>())
 	{
 		IFunction* func=f->as<Function>();
-		func->incRef();
 		ASObject* ret=func->call(obj,args,m);
 		//call getMethodInfo only after the call, so it's updated
 		if(called_mi)
@@ -2085,6 +2200,14 @@ void ABCVm::callImpl(call_context* th, ASObject* f, ASObject* obj, ASObject** ar
 		else
 			ret->decRef();
 	}
+	else if(f->is<RegExp>())
+	{
+		ASObject* ret=RegExp::exec(f,args,m);
+		if(keepReturn)
+			th->runtime_stack_push(ret);
+		else
+			ret->decRef();
+	}
 	else
 	{
 		obj->decRef();
@@ -2095,7 +2218,7 @@ void ABCVm::callImpl(call_context* th, ASObject* f, ASObject* obj, ASObject** ar
 		if(f->is<Undefined>())
 		{
 			if(keepReturn)
-				th->runtime_stack_push(new Undefined);
+				th->runtime_stack_push(getSys()->getUndefinedRef());
 		}
 		else
 			throw Class<TypeError>::getInstanceS("Error #1006: Tried to call something that is not a function");
@@ -2144,8 +2267,8 @@ ASObject* ABCVm::newCatch(call_context* th, int n)
 {
 	ASObject* catchScope = Class<ASObject>::getInstanceS();
 	assert_and_throw(n >= 0 && (unsigned int)n < th->mi->body->exception_count);
-	multiname* name = th->context->getMultiname(th->mi->body->exceptions[n].var_name, th);
-	catchScope->setVariableByMultiname(*name, new Undefined);
+	multiname* name = th->context->getMultiname(th->mi->body->exceptions[n].var_name, NULL);
+	catchScope->setVariableByMultiname(*name, getSys()->getUndefinedRef(),ASObject::CONST_NOT_ALLOWED);
 	catchScope->initSlot(1, *name);
 	return catchScope;
 }
@@ -2156,7 +2279,7 @@ void ABCVm::newArray(call_context* th, int n)
 	Array* ret=Class<Array>::getInstanceS();
 	ret->resize(n);
 	for(int i=0;i<n;i++)
-		ret->set(n-i-1,th->runtime_stack_pop());
+		ret->set(n-i-1,_MR(th->runtime_stack_pop()));
 
 	th->runtime_stack_push(ret);
 }
@@ -2191,7 +2314,14 @@ bool ABCVm::instanceOf(ASObject* value, ASObject* type)
 
 	if(type->is<IFunction>())
 	{
-		LOG(LOG_NOT_IMPLEMENTED,"instanceOf opcodes probably does the wrong thing");
+		IFunction* t=static_cast<IFunction*>(type);
+		ASObject* proto = value->getClass()->prototype.getPtr();
+		while(proto)
+		{
+			if (proto == t->prototype.getPtr())
+				return true;
+			proto = proto->getprop_prototype();
+		}
 		return false;
 	}
 
