@@ -20,6 +20,8 @@
 #include "abctypes.h"
 
 using namespace std;
+using namespace lightspark;
+
 istream& lightspark::operator>>(istream& in, s32& v)
 {
 	int i=0;
@@ -127,6 +129,9 @@ istream& lightspark::operator>>(istream& in, namespace_info& v)
 	in >> v.kind >> v.name;
 	if(v.kind!=0x05 && v.kind!=0x08 && v.kind!=0x16 && v.kind!=0x17 && v.kind!=0x18 && v.kind!=0x19 && v.kind!=0x1a)
 		throw UnsupportedException("Unexpected namespace kind");
+	//Collapse PACKAGE_NAMESPACE on NAMESPACE
+	if(v.kind==PACKAGE_NAMESPACE)
+		v.kind=(int)NAMESPACE;
 
 	return in;
 }
@@ -379,3 +384,14 @@ istream& lightspark::operator>>(istream& in, cpool_info& v)
 
 	return in;
 }
+
+cpool_info::cpool_info(MemoryAccount* m):
+	integer(reporter_allocator<s32>(m)),
+	uinteger(reporter_allocator<u32>(m)),
+	doubles(reporter_allocator<d64>(m)),
+	strings(reporter_allocator<string_info>(m)),
+	namespaces(reporter_allocator<namespace_info>(m)),
+	ns_sets(reporter_allocator<ns_set_info>(m)),
+	multinames(reporter_allocator<multiname_info>(m))
+{
+}
