@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -18,6 +18,7 @@
 **************************************************************************/
 
 #include "scripting/abc.h"
+#include "scripting/toplevel/ASString.h"
 #include "class.h"
 #include "parsing/tags.h"
 
@@ -28,29 +29,41 @@ ASObject* lightspark::new_asobject()
 	return Class<ASObject>::getInstanceS();
 }
 
+Class_inherit::Class_inherit(const QName& name, MemoryAccount* m):Class_base(name, m),tag(NULL),bindedToRoot(false)
+{
+	this->incRef(); //create on reference for the classes map
+#ifndef NDEBUG
+	bool ret=
+#endif
+	getSys()->customClasses.insert(this).second;
+	assert(ret);
+}
+
 void Class_inherit::finalize()
 {
 	Class_base::finalize();
 	class_scope.clear();
 }
 
-ASObject* Class_inherit::getInstance(bool construct, ASObject* const* args, const unsigned int argslen)
+ASObject* Class_inherit::getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass)
 {
+	//We override the classdef
+	if(realClass==NULL)
+		realClass=this;
+
 	ASObject* ret=NULL;
 	assert_and_throw(!bindedToRoot);
 	if(tag)
 	{
-		ret=tag->instance();
+		ret=tag->instance(realClass);
 		assert_and_throw(ret);
 	}
 	else
 	{
 		assert_and_throw(super);
 		//Our super should not construct, we are going to do it ourselves
-		ret=super->getInstance(false,NULL,0);
+		ret=super->getInstance(false,NULL,0,realClass);
 	}
-	//We override the classdef
-	ret->setClass(this);
 	if(construct)
 		handleConstruction(ret,args,argslen,true);
 	return ret;
@@ -66,7 +79,7 @@ void Class_inherit::buildInstanceTraits(ASObject* o) const
 }
 
 template<>
-Global* Class<Global>::getInstance(bool construct, ASObject* const* args, const unsigned int argslen)
+Global* Class<Global>::getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass)
 {
 	throw Class<TypeError>::getInstanceS("Error #1007: Cannot construct global object");
 }
@@ -78,7 +91,7 @@ void lightspark::lookupAndLink(Class_base* c, const tiny_string& name, const tin
 	//Find the origin
 	while(cur)
 	{
-		var=cur->Variables.findObjVar(name,nsNameAndKind("",NAMESPACE),NO_CREATE_TRAIT,BORROWED_TRAIT);
+		var=cur->Variables.findObjVar(getSys()->getUniqueStringId(name),nsNameAndKind("",NAMESPACE),NO_CREATE_TRAIT,BORROWED_TRAIT);
 		if(var)
 			break;
 		cur=cur->super.getPtr();
@@ -106,3 +119,36 @@ void lightspark::lookupAndLink(Class_base* c, const tiny_string& name, const tin
 		c->setDeclaredMethodByQName(name,interfaceNs,f,SETTER_METHOD,true);
 	}
 }
+
+ASObject* Class<ASObject>::getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass)
+{
+	if (construct && args && argslen == 1 && this == Class<ASObject>::getClass())
+	{
+		// Construction according to ECMA 15.2.2.1
+		switch(args[0]->getObjectType())
+		{
+		case T_BOOLEAN:
+			return abstract_b(Boolean_concrete(args[0]));
+		case T_NUMBER:
+			return abstract_d(args[0]->toNumber());
+		case T_INTEGER:
+			return abstract_i(args[0]->toInt());
+		case T_UINTEGER:
+			return abstract_ui(args[0]->toUInt());
+		case T_STRING:
+			return Class<ASString>::getInstanceS(args[0]->toString());
+		case T_FUNCTION:
+		case T_OBJECT:
+			args[0]->incRef();
+			return args[0];
+		default:
+			break;
+		}
+	}
+	if(realClass==NULL)
+		realClass=this;
+	ASObject* ret=new (realClass->memoryAccount) ASObject(realClass);
+	if(construct)
+		handleConstruction(ret,args,argslen,true);
+	return ret;
+}
