@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -48,6 +48,7 @@ REGISTER_CLASS_NAME(SecurityErrorEvent);
 REGISTER_CLASS_NAME(AsyncErrorEvent);
 REGISTER_CLASS_NAME(StatusEvent);
 REGISTER_CLASS_NAME(DataEvent);
+REGISTER_CLASS_NAME(InvokeEvent);
 
 void IEventDispatcher::linkTraits(Class_base* c)
 {
@@ -57,6 +58,11 @@ void IEventDispatcher::linkTraits(Class_base* c)
 	lookupAndLink(c,"hasEventListener","flash.events:IEventDispatcher");
 }
 
+Event::Event(Class_base* cb, const tiny_string& t, bool b, bool c):
+	ASObject(cb),type(t),target(),currentTarget(),bubbles(b),cancelable(c),eventPhase(0),defaultPrevented(false)
+{
+}
+
 void Event::finalize()
 {
 	ASObject::finalize();
@@ -87,6 +93,7 @@ void Event::sinit(Class_base* c)
 	c->setVariableByQName("RESIZE","",Class<ASString>::getInstanceS("resize"),DECLARED_TRAIT);
 	c->setVariableByQName("MOUSE_LEAVE","",Class<ASString>::getInstanceS("mouseLeave"),DECLARED_TRAIT);
 	c->setVariableByQName("SELECT","",Class<ASString>::getInstanceS("select"),DECLARED_TRAIT);
+	c->setVariableByQName("SOUND_COMPLETE","",Class<ASString>::getInstanceS("soundComplete"),DECLARED_TRAIT);
 	c->setVariableByQName("FULLSCREEN","",Class<ASString>::getInstanceS("fullScreen"),DECLARED_TRAIT);
 	c->setVariableByQName("TAB_CHILDREN_CHANGE","",Class<ASString>::getInstanceS("tabChildrenChange"),DECLARED_TRAIT);
 	c->setVariableByQName("TAB_ENABLED_CHANGE","",Class<ASString>::getInstanceS("tabEnabledChange"),DECLARED_TRAIT);
@@ -153,10 +160,10 @@ ASFUNCTIONBODY(Event,formatToString)
 		msg += prop;
 		msg += "=";
 
-		multiname propName;
+		multiname propName(NULL);
 		propName.name_type=multiname::NAME_STRING;
-		propName.name_s=prop;
-		propName.ns.push_back(nsNameAndKind("",PACKAGE_NAMESPACE));
+		propName.name_s_id=getSys()->getUniqueStringId(prop);
+		propName.ns.push_back(nsNameAndKind("",NAMESPACE));
 		_NR<ASObject> value=th->getVariableByMultiname(propName);
 		if (!value.isNull())
 			msg += value->toString();
@@ -186,7 +193,7 @@ void EventPhase::sinit(Class_base* c)
 	c->setVariableByQName("AT_TARGET","",abstract_i(AT_TARGET),DECLARED_TRAIT);
 }
 
-FocusEvent::FocusEvent():Event("focusEvent")
+FocusEvent::FocusEvent(Class_base* c):Event(c, "focusEvent")
 {
 }
 
@@ -203,23 +210,30 @@ void FocusEvent::sinit(Class_base* c)
 
 ASFUNCTIONBODY(FocusEvent,_constructor)
 {
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
 	return NULL;
 }
 
-MouseEvent::MouseEvent():Event("mouseEvent"), localX(0), localY(0), stageX(0), stageY(0), relatedObject(NullRef)
+MouseEvent::MouseEvent(Class_base* c):Event(c, "mouseEvent"), localX(0), localY(0), stageX(0), stageY(0), relatedObject(NullRef)
 {
 }
 
-MouseEvent::MouseEvent(const tiny_string& t, number_t lx, number_t ly, bool b, _NR<InteractiveObject> relObj):Event(t,b),
+MouseEvent::MouseEvent(Class_base* c, const tiny_string& t, number_t lx, number_t ly, bool b, _NR<InteractiveObject> relObj):Event(c,t,b),
 	 localX(lx), localY(ly), stageX(0), stageY(0), relatedObject(relObj)
 {
 }
 
-ProgressEvent::ProgressEvent():Event("progress",false),bytesLoaded(0),bytesTotal(0)
+Event* MouseEvent::cloneImpl() const
+{
+	return Class<MouseEvent>::getInstanceS(type,localX,localY,bubbles,relatedObject);
+}
+
+ProgressEvent::ProgressEvent(Class_base* c):Event(c, "progress",false),bytesLoaded(0),bytesTotal(0)
 {
 }
 
-ProgressEvent::ProgressEvent(uint32_t loaded, uint32_t total):Event("progress",false),bytesLoaded(loaded),bytesTotal(total)
+ProgressEvent::ProgressEvent(Class_base* c, uint32_t loaded, uint32_t total):Event(c, "progress",false),bytesLoaded(loaded),bytesTotal(total)
 {
 }
 
@@ -260,7 +274,7 @@ ASFUNCTIONBODY(ProgressEvent,_constructor)
 
 void TimerEvent::sinit(Class_base* c)
 {
-//	c->constructor=Class<IFunction>::getFunction(_constructor);
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("TIMER","",Class<ASString>::getInstanceS("timer"),DECLARED_TRAIT);
@@ -269,7 +283,7 @@ void TimerEvent::sinit(Class_base* c)
 
 void MouseEvent::sinit(Class_base* c)
 {
-//	c->constructor=Class<IFunction>::getFunction(_constructor);
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("CLICK","",Class<ASString>::getInstanceS("click"),DECLARED_TRAIT);
@@ -291,6 +305,20 @@ void MouseEvent::sinit(Class_base* c)
 	REGISTER_GETTER_SETTER(c,localY);
 }
 
+ASFUNCTIONBODY(MouseEvent,_constructor)
+{
+	MouseEvent* th=static_cast<MouseEvent*>(obj);
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
+	if(argslen>=4)
+		th->localX=args[3]->toNumber();
+	if(argslen>=5)
+		th->localY=args[4]->toNumber();
+	if(argslen>=6)
+		th->relatedObject=ArgumentConversion< _NR<InteractiveObject> >::toConcrete(args[5]);
+	return NULL;
+}
+
 ASFUNCTIONBODY_GETTER(MouseEvent,relatedObject);
 ASFUNCTIONBODY_GETTER(MouseEvent,localX);
 ASFUNCTIONBODY_GETTER(MouseEvent,localY);
@@ -301,7 +329,7 @@ ASFUNCTIONBODY(MouseEvent,_setter_localX)
 {
 	MouseEvent* th=static_cast<MouseEvent*>(obj);
 	if(argslen != 1) 
-		throw ArgumentError("Wrong number of arguments in setter"); 
+		throw Class<ArgumentError>::getInstanceS("Wrong number of arguments in setter"); 
 	number_t val=args[0]->toNumber();
 	th->localX = val;
 	//Change StageXY if target!=NULL else don't do anything
@@ -318,7 +346,7 @@ ASFUNCTIONBODY(MouseEvent,_setter_localY)
 {
 	MouseEvent* th=static_cast<MouseEvent*>(obj);
 	if(argslen != 1) 
-		throw ArgumentError("Wrong number of arguments in setter"); 
+		throw Class<ArgumentError>::getInstanceS("Wrong number of arguments in setter"); 
 	number_t val=args[0]->toNumber();
 	th->localY = val;
 	//Change StageXY if target!=NULL else don't do anything	
@@ -358,19 +386,19 @@ void MouseEvent::setTarget(_NR<ASObject> t)
 	}
 }
 
-IOErrorEvent::IOErrorEvent() : ErrorEvent("ioError")
+IOErrorEvent::IOErrorEvent(Class_base* c) : ErrorEvent(c, "ioError")
 {
 }
 
 void IOErrorEvent::sinit(Class_base* c)
 {
-//	c->constructor=Class<IFunction>::getFunction(_constructor);
-	c->setSuper(Class<Event>::getRef());
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
+	c->setSuper(Class<ErrorEvent>::getRef());
 
 	c->setVariableByQName("IO_ERROR","",Class<ASString>::getInstanceS("ioError"),DECLARED_TRAIT);
 }
 
-EventDispatcher::EventDispatcher()
+EventDispatcher::EventDispatcher(Class_base* c):ASObject(c)
 {
 }
 
@@ -378,6 +406,7 @@ void EventDispatcher::finalize()
 {
 	ASObject::finalize();
 	handlers.clear();
+	forcedTarget.reset();
 }
 
 void EventDispatcher::sinit(Class_base* c)
@@ -518,10 +547,10 @@ ASFUNCTIONBODY(EventDispatcher,dispatchEvent)
 	if(!e->target.isNull())
 	{
 		//Object must be cloned, closing is implemented with the clone AS method
-		multiname cloneName;
+		multiname cloneName(NULL);
 		cloneName.name_type=multiname::NAME_STRING;
-		cloneName.name_s="clone";
-		cloneName.ns.push_back(nsNameAndKind("",PACKAGE_NAMESPACE));
+		cloneName.name_s_id=getSys()->getUniqueStringId("clone");
+		cloneName.ns.push_back(nsNameAndKind("",NAMESPACE));
 
 		_NR<ASObject> clone=e->getVariableByMultiname(cloneName);
 		//Clone always exists since it's implemented in Event itself
@@ -539,6 +568,8 @@ ASFUNCTIONBODY(EventDispatcher,dispatchEvent)
 		}
 		e=_MR(newEvent);
 	}
+	if(!th->forcedTarget.isNull())
+		e->setTarget(th->forcedTarget);
 	th->incRef();
 	ABCVm::publicHandleEvent(_MR(th), e);
 	return abstract_b(true);
@@ -546,6 +577,17 @@ ASFUNCTIONBODY(EventDispatcher,dispatchEvent)
 
 ASFUNCTIONBODY(EventDispatcher,_constructor)
 {
+	EventDispatcher* th=Class<EventDispatcher>::cast(obj);
+	_NR<ASObject> forcedTarget;
+	ARG_UNPACK(forcedTarget, NullRef);
+	if(!forcedTarget.isNull())
+	{
+		if(forcedTarget->getObjectType()==T_NULL || forcedTarget->getObjectType()==T_UNDEFINED)
+			forcedTarget=NullRef;
+		else if(!forcedTarget->getClass()->isSubClass(InterfaceClass<IEventDispatcher>::getClass()))
+			throw Class<ArgumentError>::getInstanceS("Wrong argument for EventDispatcher");
+	}
+	th->forcedTarget=forcedTarget;
 	return NULL;
 }
 
@@ -566,7 +608,6 @@ void EventDispatcher::handleEvent(_R<Event> e)
 	//Create a temporary copy of the listeners, as the list can be modified during the calls
 	vector<listener> tmpListener(h->second.begin(),h->second.end());
 	l.release();
-	//TODO: check, ok we should also bind the level
 	for(unsigned int i=0;i<tmpListener.size();i++)
 	{
 		if( (e->eventPhase == EventPhase::BUBBLING_PHASE && tmpListener[i].use_capture)
@@ -598,7 +639,7 @@ bool EventDispatcher::hasEventListener(const tiny_string& eventName)
 		return true;
 }
 
-NetStatusEvent::NetStatusEvent(const tiny_string& l, const tiny_string& c):Event("netStatus"),level(l),code(c)
+NetStatusEvent::NetStatusEvent(Class_base* cb, const tiny_string& l, const tiny_string& c):Event(cb, "netStatus"),level(l),code(c)
 {
 	//The object has been initialized internally
 	ASObject* info=Class<ASObject>::getInstanceS();
@@ -637,13 +678,13 @@ ASFUNCTIONBODY(NetStatusEvent,_constructor)
 	else
 	{
 		//Uninitialized info
-		info=new Null;
+		info=getSys()->getNullRef();
 	}
 	obj->setVariableByQName("info","",info,DECLARED_TRAIT);
 	return NULL;
 }
 
-FullScreenEvent::FullScreenEvent():Event("fullScreenEvent")
+FullScreenEvent::FullScreenEvent(Class_base* c):Event(c, "fullScreenEvent")
 {
 }
 
@@ -657,10 +698,12 @@ void FullScreenEvent::sinit(Class_base* c)
 
 ASFUNCTIONBODY(FullScreenEvent,_constructor)
 {
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
 	return NULL;
 }
 
-KeyboardEvent::KeyboardEvent():Event("keyboardEvent")
+KeyboardEvent::KeyboardEvent(Class_base* c):Event(c, "keyboardEvent")
 {
 }
 
@@ -675,10 +718,12 @@ void KeyboardEvent::sinit(Class_base* c)
 
 ASFUNCTIONBODY(KeyboardEvent,_constructor)
 {
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
 	return NULL;
 }
 
-TextEvent::TextEvent(const tiny_string& t):Event(t)
+TextEvent::TextEvent(Class_base* c,const tiny_string& t):Event(c,t)
 {
 }
 
@@ -688,15 +733,23 @@ void TextEvent::sinit(Class_base* c)
 	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("TEXT_INPUT","",Class<ASString>::getInstanceS("textInput"),DECLARED_TRAIT);
+
+	REGISTER_GETTER_SETTER(c,text);
 }
 
+ASFUNCTIONBODY_GETTER_SETTER(TextEvent,text);
+
 ASFUNCTIONBODY(TextEvent,_constructor)
 {
-	Event::_constructor(obj,NULL,0);
+	TextEvent* th=static_cast<TextEvent*>(obj);
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
+	if(argslen>=4)
+		th->text=args[3]->toString();
 	return NULL;
 }
 
-ErrorEvent::ErrorEvent(const tiny_string& t, const std::string& e): TextEvent(t), errorMsg(e)
+ErrorEvent::ErrorEvent(Class_base* c, const tiny_string& t, const std::string& e): TextEvent(c,t), errorMsg(e)
 {
 }
 
@@ -708,13 +761,18 @@ void ErrorEvent::sinit(Class_base* c)
 	c->setVariableByQName("ERROR","",Class<ASString>::getInstanceS("error"),DECLARED_TRAIT);
 }
 
+Event* ErrorEvent::cloneImpl() const
+{
+	return Class<ErrorEvent>::getInstanceS(text, errorMsg);
+}
+
 ASFUNCTIONBODY(ErrorEvent,_constructor)
 {
-	TextEvent::_constructor(obj,NULL,0);
+	TextEvent::_constructor(obj,args,argslen);
 	return NULL;
 }
 
-SecurityErrorEvent::SecurityErrorEvent(const std::string& e):ErrorEvent("securityError",e)
+SecurityErrorEvent::SecurityErrorEvent(Class_base* c, const std::string& e):ErrorEvent(c, "securityError",e)
 {
 }
 
@@ -726,13 +784,7 @@ void SecurityErrorEvent::sinit(Class_base* c)
 	c->setVariableByQName("SECURITY_ERROR","",Class<ASString>::getInstanceS("securityError"),DECLARED_TRAIT);
 }
 
-ASFUNCTIONBODY(SecurityErrorEvent,_constructor)
-{
-	ErrorEvent::_constructor(obj,NULL,0);
-	return NULL;
-}
-
-AsyncErrorEvent::AsyncErrorEvent():ErrorEvent("asyncError")
+AsyncErrorEvent::AsyncErrorEvent(Class_base* c):ErrorEvent(c, "asyncError")
 {
 }
 
@@ -746,26 +798,34 @@ void AsyncErrorEvent::sinit(Class_base* c)
 
 ASFUNCTIONBODY(AsyncErrorEvent,_constructor)
 {
-	ErrorEvent::_constructor(obj,NULL,0);
+	uint32_t baseClassArgs=imin(argslen,4);
+	ErrorEvent::_constructor(obj,args,baseClassArgs);
 	return NULL;
 }
 
-ABCContextInitEvent::ABCContextInitEvent(ABCContext* c, bool l):Event("ABCContextInitEvent"),context(c),lazy(l)
+ABCContextInitEvent::ABCContextInitEvent(ABCContext* c, bool l):Event(NULL, "ABCContextInitEvent"),context(c),lazy(l)
 {
 }
 
-ShutdownEvent::ShutdownEvent():Event("shutdownEvent")
+ShutdownEvent::ShutdownEvent():Event(NULL, "shutdownEvent")
 {
 }
 
 void HTTPStatusEvent::sinit(Class_base* c)
 {
-//	c->setConstructor(Class<IFunction>::getFunction(_constructor));
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<Event>::getRef());
 
 	c->setVariableByQName("HTTP_STATUS","",Class<ASString>::getInstanceS("httpStatus"),DECLARED_TRAIT);
 }
 
+ASFUNCTIONBODY(HTTPStatusEvent,_constructor)
+{
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
+	return NULL;
+}
+
 FunctionEvent::FunctionEvent(_R<IFunction> _f, _NR<ASObject> _obj, ASObject** _args, uint32_t _numArgs):
 		WaitableEvent("FunctionEvent"),f(_f),obj(_obj),numArgs(_numArgs)
 {
@@ -795,14 +855,28 @@ ExternalCallEvent::~ExternalCallEvent()
 }
 
 BindClassEvent::BindClassEvent(_R<RootMovieClip> b, const tiny_string& c)
-	: Event("bindClass"),base(b),class_name(c)
+	: Event(NULL, "bindClass"),base(b),tag(NULL),class_name(c)
 {
 }
-BindClassEvent::BindClassEvent(_R<DictionaryTag> t, const tiny_string& c)
-	: Event("bindClass"),tag(t),class_name(c)
+
+BindClassEvent::BindClassEvent(DictionaryTag* t, const tiny_string& c)
+	: Event(NULL, "bindClass"),tag(t),class_name(c)
 {
 }
 
+ParseRPCMessageEvent::ParseRPCMessageEvent(_R<ByteArray> ba, _NR<ASObject> c, _R<Responder> r):
+	Event(NULL, "ParseRPCMessageEvent"),message(ba),client(c),responder(r)
+{
+}
+
+void ParseRPCMessageEvent::finalize()
+{
+	Event::finalize();
+	message.reset();
+	client.reset();
+	responder.reset();
+}
+
 void StatusEvent::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -822,3 +896,18 @@ void DataEvent::sinit(Class_base* c)
 	/* TODO: dispatch this event */
 	c->setVariableByQName("UPLOAD_COMPLETE_DATA","",Class<ASString>::getInstanceS("uploadCompleteData"),DECLARED_TRAIT);
 }
+
+void InvokeEvent::sinit(Class_base* c)
+{
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
+	c->setSuper(Class<Event>::getRef());
+
+	c->setVariableByQName("INVOKE","",Class<ASString>::getInstanceS("invoke"),DECLARED_TRAIT);
+}
+
+ASFUNCTIONBODY(InvokeEvent,_constructor)
+{
+	uint32_t baseClassArgs=imin(argslen,3);
+	Event::_constructor(obj,args,baseClassArgs);
+	return NULL;
+}
