@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -33,13 +33,14 @@ namespace lightspark
 
 enum EVENT_TYPE { EVENT=0, BIND_CLASS, SHUTDOWN, SYNC, MOUSE_EVENT,
 	FUNCTION, EXTERNAL_CALL, CONTEXT_INIT, INIT_FRAME,
-	FLUSH_INVALIDATION_QUEUE, ADVANCE_FRAME };
+	FLUSH_INVALIDATION_QUEUE, ADVANCE_FRAME, PARSE_RPC_MESSAGE };
 
 class ABCContext;
 class DictionaryTag;
 class InteractiveObject;
 class PlaceObject2Tag;
 class MovieClip;
+class Responder;
 
 class Event: public ASObject
 {
@@ -49,9 +50,7 @@ private:
 	 */
 	virtual Event* cloneImpl() const;
 public:
-	Event(const tiny_string& t = "Event", bool b=false, bool c=false)
-		: type(t),target(),currentTarget(),bubbles(b),cancelable(c),
-		  eventPhase(0),defaultPrevented(false) {}
+	Event(Class_base* cb, const tiny_string& t = "Event", bool b=false, bool c=false);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
@@ -78,13 +77,14 @@ class WaitableEvent : public Event
 {
 public:
 	WaitableEvent(const tiny_string& t = "Event", bool b=false, bool c=false)
-		: Event(t,b,c), done(0) {}
+		: Event(NULL,t,b,c), done(0) {}
 	Semaphore done;
 };
 
 class EventPhase: public ASObject
 {
 public:
+	EventPhase(Class_base* c):ASObject(c){}
 	enum
 	{
 		CAPTURING_PHASE = 1,
@@ -98,7 +98,7 @@ public:
 class KeyboardEvent: public Event
 {
 public:
-	KeyboardEvent();
+	KeyboardEvent(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -109,7 +109,7 @@ public:
 class FocusEvent: public Event
 {
 public:
-	FocusEvent();
+	FocusEvent(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -120,7 +120,7 @@ public:
 class FullScreenEvent: public Event
 {
 public:
-	FullScreenEvent();
+	FullScreenEvent(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -134,8 +134,8 @@ private:
 	tiny_string level;
 	tiny_string code;
 public:
-	NetStatusEvent():Event("netStatus"){}
-	NetStatusEvent(const tiny_string& l, const tiny_string& c);
+	NetStatusEvent(Class_base* c):Event(c, "netStatus"){}
+	NetStatusEvent(Class_base* cb, const tiny_string& l, const tiny_string& c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -146,16 +146,20 @@ public:
 class HTTPStatusEvent: public Event
 {
 public:
+	HTTPStatusEvent(Class_base* c):Event(c){}
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
 	}
+	ASFUNCTION(_constructor);
 };
 
 class TextEvent: public Event
 {
+protected:
+	ASPROPERTY_GETTER_SETTER(tiny_string,text);
 public:
-	TextEvent(const tiny_string& t = "textEvent");
+	TextEvent(Class_base* c, const tiny_string& t = "textEvent");
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -167,8 +171,9 @@ class ErrorEvent: public TextEvent
 {
 private:
 	std::string errorMsg;
+	Event* cloneImpl() const;
 public:
-	ErrorEvent(const tiny_string& t = "error", const std::string& e = "");
+	ErrorEvent(Class_base* c, const tiny_string& t = "error", const std::string& e = "");
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -179,7 +184,7 @@ public:
 class IOErrorEvent: public ErrorEvent
 {
 public:
-	IOErrorEvent();
+	IOErrorEvent(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -189,18 +194,17 @@ public:
 class SecurityErrorEvent: public ErrorEvent
 {
 public:
-	SecurityErrorEvent(const std::string& e = "");
+	SecurityErrorEvent(Class_base* c, const std::string& e = "");
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
 	}
-	ASFUNCTION(_constructor);
 };
 
 class AsyncErrorEvent: public ErrorEvent
 {
 public:
-	AsyncErrorEvent();
+	AsyncErrorEvent(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -215,8 +219,8 @@ private:
 	ASPROPERTY_GETTER_SETTER(number_t,bytesTotal);
 	Event* cloneImpl() const;
 public:
-	ProgressEvent();
-	ProgressEvent(uint32_t loaded, uint32_t total);
+	ProgressEvent(Class_base* c);
+	ProgressEvent(Class_base* c, uint32_t loaded, uint32_t total);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -225,8 +229,8 @@ public:
 class TimerEvent: public Event
 {
 public:
-	TimerEvent():Event("DEPRECATED"){}
-	TimerEvent(const tiny_string& t):Event(t,true){};
+	TimerEvent(Class_base* c):Event(c, "DEPRECATED"){}
+	TimerEvent(Class_base* c,const tiny_string& t):Event(c,t,true){};
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o)
 	{
@@ -235,13 +239,16 @@ public:
 
 class MouseEvent: public Event
 {
+private:
+	Event* cloneImpl() const;
 public:
-	MouseEvent();
-	MouseEvent(const tiny_string& t, number_t lx, number_t ly, bool b=true, _NR<InteractiveObject> relObj = NullRef);
+	MouseEvent(Class_base* c);
+	MouseEvent(Class_base* c, const tiny_string& t, number_t lx, number_t ly, bool b=true, _NR<InteractiveObject> relObj = NullRef);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	void setTarget(_NR<ASObject> t);
 	EVENT_TYPE getEventType() const { return MOUSE_EVENT;}
+	ASFUNCTION(_constructor);
 	ASPROPERTY_GETTER_SETTER(number_t,localX);
 	ASPROPERTY_GETTER_SETTER(number_t,localY);
 	ASPROPERTY_GETTER(number_t,stageX);
@@ -249,6 +256,15 @@ public:
 	ASPROPERTY_GETTER(_NR<InteractiveObject>,relatedObject);	
 };
 
+class InvokeEvent: public Event
+{
+public:
+	InvokeEvent(Class_base* c) : Event(c, "invoke"){}
+	static void sinit(Class_base* c);
+	static void buildTraits(ASObject* o){}
+	ASFUNCTION(_constructor);
+};
+
 class listener
 {
 friend class EventDispatcher;
@@ -287,8 +303,12 @@ class EventDispatcher: public ASObject, public IEventDispatcher
 private:
 	Mutex handlersMutex;
 	std::map<tiny_string,std::list<listener> > handlers;
+	/*
+	 * This will be used when a target is passed to EventDispatcher constructor
+	 */
+	_NR<ASObject> forcedTarget;
 public:
-	EventDispatcher();
+	EventDispatcher(Class_base* c);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
@@ -303,6 +323,22 @@ public:
 	ASFUNCTION(_hasEventListener);
 };
 
+class StatusEvent: public Event
+{
+public:
+	StatusEvent(Class_base* c) : Event(c, "StatusEvent") {}
+	static void sinit(Class_base*);
+	static void buildTraits(ASObject* o) {}
+};
+
+class DataEvent: public Event
+{
+public:
+	DataEvent(Class_base* c) : Event(c, "DataEvent") {}
+	static void sinit(Class_base*);
+	static void buildTraits(ASObject* o) {}
+};
+
 class RootMovieClip;
 //Internal events from now on, used to pass data to the execution thread
 class BindClassEvent: public Event
@@ -310,11 +346,11 @@ class BindClassEvent: public Event
 friend class ABCVm;
 private:
 	_NR<RootMovieClip> base;
-	_NR<DictionaryTag> tag;
+	DictionaryTag* tag;
 	tiny_string class_name;
 public:
 	BindClassEvent(_R<RootMovieClip> b, const tiny_string& c);
-	BindClassEvent(_R<DictionaryTag> t, const tiny_string& c);
+	BindClassEvent(DictionaryTag* t, const tiny_string& c);
 	static void sinit(Class_base*);
 	EVENT_TYPE getEventType() const { return BIND_CLASS;}
 };
@@ -382,7 +418,7 @@ friend class ABCVm;
 private:
 	_NR<MovieClip> clip;
 public:
-	InitFrameEvent(_NR<MovieClip> m=NullRef) : Event("InitFrameEvent"),clip(m) {}
+	InitFrameEvent(_NR<MovieClip> m=NullRef) : Event(NULL, "InitFrameEvent"),clip(m) {}
 	EVENT_TYPE getEventType() const { return INIT_FRAME; }
 };
 
@@ -397,24 +433,19 @@ public:
 class FlushInvalidationQueueEvent: public Event
 {
 public:
-	FlushInvalidationQueueEvent():Event("FlushInvalidationQueueEvent"){};
+	FlushInvalidationQueueEvent():Event(NULL, "FlushInvalidationQueueEvent"){};
 	EVENT_TYPE getEventType() const { return FLUSH_INVALIDATION_QUEUE; };
 };
 
-class StatusEvent: public Event
+class ParseRPCMessageEvent: public Event
 {
 public:
-	StatusEvent() : Event("StatusEvent") {}
-	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o) {}
-};
-
-class DataEvent: public Event
-{
-public:
-	DataEvent() : Event("DataEvent") {}
-	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o) {}
+	_NR<ByteArray> message;
+	_NR<ASObject> client;
+	_NR<Responder> responder;
+	ParseRPCMessageEvent(_R<ByteArray> ba, _NR<ASObject> client, _R<Responder> responder);
+	EVENT_TYPE getEventType() const { return PARSE_RPC_MESSAGE; };
+	void finalize();
 };
 
 };
