@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -96,18 +96,22 @@ ASFUNCTIONBODY(ExternalInterface,call)
 	if(getSys()->extScriptObject == NULL)
 		throw Class<ASError>::getInstanceS("Container doesn't support callbacks");
 
-	assert_and_throw(argslen >= 1 && args[0]->getObjectType() == T_STRING);
+	assert_and_throw(argslen >= 1);
+	const tiny_string& arg0=args[0]->toString();
 
 	// TODO: Check security constraints & throw SecurityException
 
 	// Convert given arguments to ExtVariants
 	const ExtVariant** callArgs = g_newa(const ExtVariant*,argslen-1);
 	for(uint32_t i = 0; i < argslen-1; i++)
-		callArgs[i] = new ExtVariant(args[i+1]);
+	{
+		args[i+1]->incRef();
+		callArgs[i] = new ExtVariant(_MR(args[i+1]));
+	}
 
 	ASObject* asobjResult = NULL;
 	// Let the external script object call the external method
-	bool callSuccess = getSys()->extScriptObject->callExternal(args[0]->toString().raw_buf(), callArgs, argslen-1, &asobjResult);
+	bool callSuccess = getSys()->extScriptObject->callExternal(arg0.raw_buf(), callArgs, argslen-1, &asobjResult);
 
 	// Delete converted arguments
 	for(uint32_t i = 0; i < argslen-1; i++)
@@ -116,9 +120,9 @@ ASFUNCTIONBODY(ExternalInterface,call)
 	if(!callSuccess)
 	{
 		assert(asobjResult==NULL);
-		LOG(LOG_INFO, "External function failed, returning null: " << args[0]->toString().raw_buf());
+		LOG(LOG_INFO, "External function failed, returning null: " << arg0);
 		// If the call fails, return null
-		asobjResult = new Null;
+		asobjResult = getSys()->getNullRef();
 	}
 
 	return asobjResult;
