@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -28,21 +28,31 @@ namespace lightspark
 
 class BitmapFilter: public ASObject
 {
+private:
+	virtual BitmapFilter* cloneImpl() const;
 public:
+	BitmapFilter(Class_base* c):ASObject(c){}
 	static void sinit(Class_base* c);
 //	static void buildTraits(ASObject* o);
+	ASFUNCTION(clone);
 };
 
 class GlowFilter: public BitmapFilter
 {
+private:
+	virtual GlowFilter* cloneImpl() const;
 public:
+	GlowFilter(Class_base* c):BitmapFilter(c){}
 	static void sinit(Class_base* c);
 //	static void buildTraits(ASObject* o);
 };
 
 class DropShadowFilter: public BitmapFilter
 {
+private:
+	virtual DropShadowFilter* cloneImpl() const;
 public:
+	DropShadowFilter(Class_base* c):BitmapFilter(c){}
 	static void sinit(Class_base* c);
 //	static void buildTraits(ASObject* o);
 };
