@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,7 @@ namespace lightspark
 class Rectangle: public ASObject
 {
 public:
-	Rectangle():x(0),y(0),width(0),height(0){}
+	Rectangle(Class_base* c):ASObject(c),x(0),y(0),width(0),height(0){}
 	number_t x,y,width,height;
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
@@ -78,8 +78,9 @@ class Point: public ASObject
 {
 private:
 	number_t x,y;
+	static number_t lenImpl(number_t x, number_t y);
 public:
-	Point(number_t _x = 0, number_t _y = 0):x(_x),y(_y){}
+	Point(Class_base* c,number_t _x = 0, number_t _y = 0):ASObject(c),x(_x),y(_y){}
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -100,8 +101,8 @@ public:
 	ASFUNCTION(_toString);
 	
 	number_t len() const;
-	number_t getX() { return x; };
-	number_t getY() { return y; };
+	number_t getX() const { return x; };
+	number_t getY() const { return y; };
 };
 
 class ColorTransform: public ASObject
@@ -110,6 +111,7 @@ private:
 	number_t redMultiplier,greenMultiplier,blueMultiplier,alphaMultiplier;
 	number_t redOffset,greenOffset,blueOffset,alphaOffset;
 public:
+	ColorTransform(Class_base* c):ASObject(c){}
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -140,8 +142,11 @@ public:
 
 class Matrix: public ASObject
 {
+private:
+	MATRIX matrix;
 public:
-	number_t a, b, c, d, tx, ty;
+	Matrix(Class_base* c);
+	Matrix(Class_base* c,const MATRIX& m);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	void _createBox(number_t scaleX, number_t scaleY, number_t angle, number_t x, number_t y);
@@ -180,15 +185,23 @@ public:
 
 class Transform: public ASObject
 {
+private:
+	_NR<DisplayObject> owner;
 public:
+	Transform(Class_base* c):ASObject(c){};
+	Transform(Class_base* c, _R<DisplayObject> o);
+	ASFUNCTION(_constructor);
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
-	ASPROPERTY_GETTER_SETTER(_NR<Matrix>, matrix);
+	void finalize();
+	ASFUNCTION(_getMatrix);
+	ASFUNCTION(_setMatrix);
 };
 
 class Vector3D: public ASObject
 {
 public:
+	Vector3D(Class_base* c):ASObject(c){}
 	number_t w, x, y, z;
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
