@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2011-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -29,22 +29,26 @@
 namespace lightspark
 {
 
-class URLStream: public EventDispatcher, public IThreadJob, public IDataInput
+class URLStream;
+
+class URLStreamThread : public DownloaderThreadBase
+{
+private:
+	_R<URLStream> loader;
+	_R<ByteArray> data;
+	void execute();
+public:
+	URLStreamThread(_R<URLRequest> request, _R<URLStream> ldr, _R<ByteArray> bytes);
+};
+
+class URLStream: public EventDispatcher, public IDataInput, public IDownloaderThreadListener
 {
-CLASSBUILDABLE(URLStream);
 private:
 	URLInfo url;
 	_NR<ByteArray> data;
-	Spinlock downloaderLock;
-	Downloader* downloader;
-	std::vector<uint8_t> postData;
-	URLStream() : data(_MNR(Class<ByteArray>::getInstanceS())), downloader(NULL) {}
-	void execute();
-	void threadAbort();
-	void jobFence();
+	URLStreamThread *job;
+	Spinlock spinlock;
 	void finalize();
-	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_getEndian);
 	ASFUNCTION(_setEndian);
@@ -67,6 +71,11 @@ private:
 	ASFUNCTION(readUnsignedShort);
 	ASFUNCTION(readUTF);
 	ASFUNCTION(readUTFBytes);
+public:
+	URLStream(Class_base* c):EventDispatcher(c),data(_MNR(Class<ByteArray>::getInstanceS())) {}
+	static void sinit(Class_base*);
+	static void buildTraits(ASObject* o);
+	void threadFinished(IThreadJob *job);
 };
 
 }
