@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -42,7 +42,7 @@ private:
 	tiny_string validatedContentType() const;
 	ASPROPERTY_GETTER_SETTER(tiny_string,contentType);
 public:
-	URLRequest();
+	URLRequest(Class_base* c);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
@@ -60,6 +60,7 @@ public:
 class URLRequestMethod: public ASObject
 {
 public:
+	URLRequestMethod(Class_base* c):ASObject(c){}
 	static void sinit(Class_base*);
 };
 
@@ -69,8 +70,8 @@ private:
 	void decode(const tiny_string& s);
 	tiny_string toString_priv();
 public:
-	URLVariables(){}
-	URLVariables(const tiny_string& s);
+	URLVariables(Class_base* c):ASObject(c){}
+	URLVariables(Class_base* c,const tiny_string& s);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -82,39 +83,54 @@ public:
 class URLLoaderDataFormat: public ASObject
 {
 public:
+	URLLoaderDataFormat(Class_base* c):ASObject(c){}
 	static void sinit(Class_base*);
 };
 
 class SharedObject: public EventDispatcher
 {
 public:
+	SharedObject(Class_base* c);
 	static void sinit(Class_base*);
+	ASFUNCTION(getLocal);
+	ASPROPERTY_GETTER(_NR<ASObject>,data);
 };
 
 class ObjectEncoding: public ASObject
 {
 public:
+	ObjectEncoding(Class_base* c):ASObject(c){}
 	enum ENCODING { AMF0=0, AMF3=3, DEFAULT=3 };
 	static void sinit(Class_base*);
 };
 
-class URLLoader: public EventDispatcher, public IThreadJob
+class URLLoader;
+
+class URLLoaderThread : public DownloaderThreadBase
+{
+private:
+	_R<URLLoader> loader;
+	void execute();
+public:
+	URLLoaderThread(_R<URLRequest> _request, _R<URLLoader> _loader);
+};
+
+class URLLoader: public EventDispatcher, public IDownloaderThreadListener
 {
 private:
 	tiny_string dataFormat;
-	URLInfo url;
 	_NR<ASObject> data;
-	Spinlock downloaderLock;
-	Downloader* downloader;
-	std::vector<uint8_t> postData;
-	void execute();
-	void threadAbort();
-	void jobFence();
+	Spinlock spinlock;
+	URLLoaderThread *job;
 public:
-	URLLoader();
+	URLLoader(Class_base* c);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
+	void threadFinished(IThreadJob *job);
+	void setData(_NR<ASObject> data);
+	tiny_string getDataFormat();
+	void setDataFormat(const tiny_string& newFormat);
 	ASFUNCTION(_constructor);
 	ASFUNCTION(load);
 	ASFUNCTION(close);
@@ -129,13 +145,14 @@ private:
 	_NR<IFunction> result;
 	_NR<IFunction> status;
 public:
+	Responder(Class_base* c):ASObject(c){}
 	static void sinit(Class_base*);
 	void finalize();
 	ASFUNCTION(_constructor);
 	ASFUNCTION(onResult);
 };
 
-class NetConnection: public EventDispatcher
+class NetConnection: public EventDispatcher, public IThreadJob
 {
 friend class NetStream;
 private:
@@ -145,13 +162,25 @@ private:
 	ObjectEncoding::ENCODING objectEncoding;
 	tiny_string protocol;
 	URLInfo uri;
+	//Data for remoting support (NetConnection::call)
+	// The message data to be sent asynchronously
+	std::vector<uint8_t> messageData;
+	Spinlock downloaderLock;
+	Downloader* downloader;
+	_NR<Responder> responder;
+	uint32_t messageCount;
+	//IThreadJob interface
+	void execute();
+	void threadAbort();
+	void jobFence();
 public:
-	NetConnection();
+	NetConnection(Class_base* c);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
 	ASFUNCTION(connect);
+	ASFUNCTION(call);
 	ASFUNCTION(_getConnected);
 	ASFUNCTION(_getDefaultObjectEncoding);
 	ASFUNCTION(_setDefaultObjectEncoding);
@@ -183,6 +212,7 @@ private:
 	void jobFence();
 	//ITickJob interface to frame advance
 	void tick();
+	void tickFence();
 	bool isReady() const;
 
 	//Indicates whether the NetStream is paused
@@ -199,8 +229,12 @@ private:
 	bool rawAccessAllowed;
 	number_t oldVolume;
 	ASPROPERTY_GETTER_SETTER(NullableRef<SoundTransform>,soundTransform);
+
+	ASObject *createMetaDataObject(StreamDecoder* streamDecoder);
+	ASObject *createPlayStatusObject(const tiny_string& code);
+	void sendClientNotification(const tiny_string& name, ASObject *args);
 public:
-	NetStream();
+	NetStream(Class_base* c);
 	~NetStream();
 	void finalize();
 	static void sinit(Class_base*);
@@ -285,6 +319,8 @@ public:
 };
 
 ASObject* sendToURL(ASObject* obj,ASObject* const* args, const unsigned int argslen);
+ASObject* registerClassAlias(ASObject* obj,ASObject* const* args, const unsigned int argslen);
+ASObject* getClassByAlias(ASObject* obj,ASObject* const* args, const unsigned int argslen);
 
 };
 
