@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2011-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -32,7 +32,7 @@ namespace lightspark
 {
 class Accelerometer: public ASObject {
 	public:
-		Accelerometer();
+		Accelerometer(Class_base* c);
 		static void sinit(Class_base* c);
 		static void buildTraits(ASObject* o);
 		ASFUNCTION(_isSupported);
