@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -34,16 +34,19 @@ using namespace lightspark;
 SET_NAMESPACE("flash.system");
 
 REGISTER_CLASS_NAME(ApplicationDomain);
-REGISTER_CLASS_NAME(SecurityDomain);
 REGISTER_CLASS_NAME(Capabilities);
+REGISTER_CLASS_NAME(LoaderContext);
 REGISTER_CLASS_NAME(Security);
+REGISTER_CLASS_NAME(SecurityDomain);
+
 
 #ifdef _WIN32
-const char* Capabilities::EMULATED_VERSION = "WIN 11,1,0,"SHORTVERSION;
+const char* Capabilities::EMULATED_VERSION = "WIN 11,1,0," SHORTVERSION;
 #else
-const char* Capabilities::EMULATED_VERSION = "LNX 11,1,0,"SHORTVERSION;
+const char* Capabilities::EMULATED_VERSION = "LNX 11,1,0," SHORTVERSION;
 #endif
 
+
 void Capabilities::sinit(Class_base* c)
 {
 	c->setDeclaredMethodByQName("language","",Class<IFunction>::getFunction(_getLanguage),GETTER_METHOD,false);
@@ -104,18 +107,25 @@ ASFUNCTIONBODY(Capabilities,_getServerString)
 	return Class<ASString>::getInstanceS("");
 }
 
+ApplicationDomain::ApplicationDomain(Class_base* c, _NR<ApplicationDomain> p):ASObject(c),parentDomain(p)
+{
+}
+
 void ApplicationDomain::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	//Static
 	c->setDeclaredMethodByQName("currentDomain","",Class<IFunction>::getFunction(_getCurrentDomain),GETTER_METHOD,false);
+	c->setDeclaredMethodByQName("MIN_DOMAIN_MEMORY_LENGTH","",Class<IFunction>::getFunction(_getMinDomainMemoryLength),GETTER_METHOD,false);
 	//Instance
 	c->setDeclaredMethodByQName("hasDefinition","",Class<IFunction>::getFunction(hasDefinition),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("getDefinition","",Class<IFunction>::getFunction(getDefinition),NORMAL_METHOD,true);
 	REGISTER_GETTER_SETTER(c,domainMemory);
+	REGISTER_GETTER(c,parentDomain);
 }
 
 ASFUNCTIONBODY_GETTER_SETTER(ApplicationDomain,domainMemory);
+ASFUNCTIONBODY_GETTER(ApplicationDomain,parentDomain);
 
 void ApplicationDomain::buildTraits(ASObject* o)
 {
@@ -125,34 +135,56 @@ void ApplicationDomain::finalize()
 {
 	ASObject::finalize();
 	domainMemory.reset();
+	for(auto i = globalScopes.begin(); i != globalScopes.end(); ++i)
+		(*i)->decRef();
 }
 
 ASFUNCTIONBODY(ApplicationDomain,_constructor)
 {
+	ApplicationDomain* th = Class<ApplicationDomain>::cast(obj);
+	_NR<ApplicationDomain> parentDomain;
+	ARG_UNPACK (parentDomain, NullRef);
+	if(!th->parentDomain.isNull())
+		// Don't override parentDomain if it was set in the
+		// C++ constructor
+		return NULL;
+	else if(parentDomain.isNull())
+		th->parentDomain =  getSys()->systemDomain;
+	else
+		th->parentDomain = parentDomain;
 	return NULL;
 }
 
+ASFUNCTIONBODY(ApplicationDomain,_getMinDomainMemoryLength)
+{
+	return abstract_ui(1024);
+}
+
 ASFUNCTIONBODY(ApplicationDomain,_getCurrentDomain)
 {
-	ApplicationDomain* ret=getSys()->mainApplicationDomain.getPtr();
+	_NR<ApplicationDomain> ret=ABCVm::getCurrentApplicationDomain(getVm()->currentCallContext);
 	ret->incRef();
-	return ret;
+	return ret.getPtr();
 }
 
 ASFUNCTIONBODY(ApplicationDomain,hasDefinition)
 {
-	assert(args && argslen==1);
+	ApplicationDomain* th = obj->as<ApplicationDomain>();
+	assert(argslen==1);
 	const tiny_string& tmp=args[0]->toString();
 
-	multiname name;
+	multiname name(NULL);
 	name.name_type=multiname::NAME_STRING;
-	name.ns.push_back(nsNameAndKind("",NAMESPACE)); //TODO: set type
 
-	stringToQName(tmp,name.name_s,name.ns[0].name);
+	tiny_string nsName;
+	tiny_string tmpName;
+	stringToQName(tmp,tmpName,nsName);
+	name.name_s_id=getSys()->getUniqueStringId(tmpName);
+	name.ns.push_back(nsNameAndKind(nsName,NAMESPACE));
 
 	LOG(LOG_CALLS,_("Looking for definition of ") << name);
 	ASObject* target;
-	ASObject* o=getGlobal()->getVariableAndTargetByMultiname(name,target);
+	ASObject* o=th->getVariableAndTargetByMultiname(name,target);
 	if(o==NULL)
 		return abstract_b(false);
 	else
@@ -167,18 +199,22 @@ ASFUNCTIONBODY(ApplicationDomain,hasDefinition)
 
 ASFUNCTIONBODY(ApplicationDomain,getDefinition)
 {
-	assert(args && argslen==1);
+	ApplicationDomain* th = obj->as<ApplicationDomain>();
+	assert(argslen==1);
 	const tiny_string& tmp=args[0]->toString();
 
-	multiname name;
+	multiname name(NULL);
 	name.name_type=multiname::NAME_STRING;
-	name.ns.push_back(nsNameAndKind("",NAMESPACE)); //TODO: set type
 
-	stringToQName(tmp,name.name_s,name.ns[0].name);
+	tiny_string nsName;
+	tiny_string tmpName;
+	stringToQName(tmp,tmpName,nsName);
+	name.name_s_id=getSys()->getUniqueStringId(tmpName);
+	name.ns.push_back(nsNameAndKind(nsName,NAMESPACE));
 
 	LOG(LOG_CALLS,_("Looking for definition of ") << name);
 	ASObject* target;
-	ASObject* o=getGlobal()->getVariableAndTargetByMultiname(name,target);
+	ASObject* o=th->getVariableAndTargetByMultiname(name,target);
 	assert_and_throw(o);
 
 	//TODO: specs says that also namespaces and function may be returned
@@ -189,6 +225,82 @@ ASFUNCTIONBODY(ApplicationDomain,getDefinition)
 	return o;
 }
 
+void ApplicationDomain::registerGlobalScope(Global* scope)
+{
+	globalScopes.push_back(scope);
+}
+
+ASObject* ApplicationDomain::getVariableByString(const std::string& str, ASObject*& target)
+{
+	size_t index=str.rfind('.');
+	multiname name(NULL);
+	name.name_type=multiname::NAME_STRING;
+	if(index==str.npos) //No dot
+	{
+		name.name_s_id=getSys()->getUniqueStringId(str);
+		name.ns.push_back(nsNameAndKind("",NAMESPACE)); //TODO: use ns kind
+	}
+	else
+	{
+		name.name_s_id=getSys()->getUniqueStringId(str.substr(index+1));
+		name.ns.push_back(nsNameAndKind(str.substr(0,index),NAMESPACE));
+	}
+	return getVariableAndTargetByMultiname(name, target);
+}
+
+ASObject* ApplicationDomain::getVariableAndTargetByMultiname(const multiname& name, ASObject*& target)
+{
+	//Check in the parent first
+	if(!parentDomain.isNull())
+	{
+		ASObject* ret=parentDomain->getVariableAndTargetByMultiname(name, target);
+		if(ret)
+			return ret;
+	}
+
+	for(uint32_t i=0;i<globalScopes.size();i++)
+	{
+		_NR<ASObject> o=globalScopes[i]->getVariableByMultiname(name);
+		if(!o.isNull())
+		{
+			target=globalScopes[i];
+			// No incRef, return a reference borrowed from globalScopes
+			return o.getPtr();
+		}
+	}
+	return NULL;
+}
+
+void LoaderContext::sinit(Class_base* c)
+{
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
+	REGISTER_GETTER_SETTER(c, applicationDomain);
+	REGISTER_GETTER_SETTER(c, securityDomain);
+}
+
+void LoaderContext::finalize()
+{
+	ASObject::finalize();
+	applicationDomain.reset();
+	securityDomain.reset();
+}
+
+ASFUNCTIONBODY(LoaderContext,_constructor)
+{
+	LoaderContext* th=Class<LoaderContext>::cast(obj);
+	bool checkPolicy;
+	_NR<ApplicationDomain> appDomain;
+	_NR<SecurityDomain> secDomain;
+	ARG_UNPACK (checkPolicy, false) (appDomain, NullRef) (secDomain, NullRef);
+	//TODO: Support checkPolicyFile
+	th->applicationDomain=appDomain;
+	th->securityDomain=secDomain;
+	return NULL;
+}
+
+ASFUNCTIONBODY_GETTER_SETTER(LoaderContext, applicationDomain);
+ASFUNCTIONBODY_GETTER_SETTER(LoaderContext, securityDomain);
+
 void SecurityDomain::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -207,7 +319,9 @@ ASFUNCTIONBODY(SecurityDomain,_constructor)
 
 ASFUNCTIONBODY(SecurityDomain,_getCurrentDomain)
 {
-	return Class<SecurityDomain>::getInstanceS();
+	_NR<SecurityDomain> ret=ABCVm::getCurrentSecurityDomain(getVm()->currentCallContext);
+	ret->incRef();
+	return ret.getPtr();
 }
 
 void Security::sinit(Class_base* c)
