@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -27,11 +27,13 @@
 namespace lightspark
 {
 
+class SecurityDomain;
+
 class Capabilities: public ASObject
 {
 public:
 	DLL_PUBLIC static const char* EMULATED_VERSION;
-	Capabilities(){};
+	Capabilities(Class_base* c):ASObject(c){};
 	static void sinit(Class_base* c);
 	ASFUNCTION(_getLanguage);
 	ASFUNCTION(_getPlayerType);
@@ -46,16 +48,23 @@ public:
 
 class ApplicationDomain: public ASObject
 {
+private:
+	std::vector<Global*> globalScopes;
 public:
-	ApplicationDomain(){}
+	ApplicationDomain(Class_base* c, _NR<ApplicationDomain> p=NullRef);
 	void finalize();
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
+	void registerGlobalScope(Global* scope);
+	ASObject* getVariableByString(const std::string& name, ASObject*& target);
+	ASObject* getVariableAndTargetByMultiname(const multiname& name, ASObject*& target);
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_getCurrentDomain);
+	ASFUNCTION(_getMinDomainMemoryLength);
 	ASFUNCTION(hasDefinition);
 	ASFUNCTION(getDefinition);
 	ASPROPERTY_GETTER_SETTER(_NR<ByteArray>, domainMemory);
+	ASPROPERTY_GETTER(_NR<ApplicationDomain>, parentDomain);
 	template<class T>
 	T readFromDomainMemory(uint32_t addr) const
 	{
@@ -80,10 +89,21 @@ public:
 	}
 };
 
+class LoaderContext: public ASObject
+{
+public:
+	LoaderContext(Class_base* c):ASObject(c){};
+	static void sinit(Class_base* c);
+	ASFUNCTION(_constructor);
+	ASPROPERTY_GETTER_SETTER(_NR<ApplicationDomain>, applicationDomain);
+	ASPROPERTY_GETTER_SETTER(_NR<SecurityDomain>, securityDomain);
+	void finalize();
+};
+
 class SecurityDomain: public ASObject
 {
 public:
-	SecurityDomain(){}
+	SecurityDomain(Class_base* c):ASObject(c){}
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -93,6 +113,7 @@ public:
 class Security: public ASObject
 {
 public:
+	Security(Class_base* c):ASObject(c){}
 	static void sinit(Class_base* c);
 	ASFUNCTION(_getExactSettings);
 	ASFUNCTION(_setExactSettings);
