@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,8 @@ using namespace lightspark;
 
 SET_NAMESPACE("flash.text");
 
-REGISTER_CLASS_NAME2(lightspark::Font,"Font","flash.text");
+REGISTER_CLASS_NAME2(ASFont,"Font","flash.text");
+REGISTER_CLASS_NAME(AntiAliasType);
 REGISTER_CLASS_NAME(TextField);
 REGISTER_CLASS_NAME(TextFieldType);
 REGISTER_CLASS_NAME(TextFieldAutoSize);
@@ -38,16 +39,65 @@ REGISTER_CLASS_NAME(TextFormat);
 REGISTER_CLASS_NAME(StyleSheet);
 REGISTER_CLASS_NAME(StaticText);
 
-void lightspark::Font::sinit(Class_base* c)
+void lightspark::AntiAliasType::sinit(Class_base* c)
 {
-//	c->constructor=Class<IFunction>::getFunction(_constructor);
 	c->setConstructor(NULL);
-	c->setDeclaredMethodByQName("enumerateFonts","",Class<IFunction>::getFunction(enumerateFonts),NORMAL_METHOD,true);
+	c->setVariableByQName("ADVANCED","",Class<ASString>::getInstanceS("advanced"),DECLARED_TRAIT);
+	c->setVariableByQName("NORMAL","",Class<ASString>::getInstanceS("normal"),DECLARED_TRAIT);
 }
 
-ASFUNCTIONBODY(lightspark::Font,enumerateFonts)
+void ASFont::sinit(Class_base* c)
 {
-	return Class<Array>::getInstanceS();
+//	c->constructor=Class<IFunction>::getFunction(_constructor);
+	//c->setConstructor(NULL);
+	c->setSuper(Class<ASObject>::getRef());
+	c->setDeclaredMethodByQName("enumerateFonts","",Class<IFunction>::getFunction(enumerateFonts),NORMAL_METHOD,false);
+	c->setDeclaredMethodByQName("registerFont","",Class<IFunction>::getFunction(registerFont),NORMAL_METHOD,false);
+
+	REGISTER_GETTER(c,fontName);
+	REGISTER_GETTER(c,fontStyle);
+	REGISTER_GETTER(c,fontType);
+}
+void ASFont::SetFont(tiny_string& fontname,bool is_bold,bool is_italic, bool is_Embedded, bool is_EmbeddedCFF)
+{
+	fontName = fontname;
+	fontStyle = (is_bold ? 
+					 (is_italic ? "boldItalic" : "bold") :
+					 (is_italic ? "italic" : "regular")
+					 );
+	fontType = (is_Embedded ?
+					(is_EmbeddedCFF ? "embeddedCFF" : "embedded") :
+					"device");
+}
+
+std::vector<ASObject*>* ASFont::getFontList()
+{
+	static std::vector<ASObject*> fontlist;
+	return &fontlist;
+}
+ASFUNCTIONBODY_GETTER(ASFont, fontName);
+ASFUNCTIONBODY_GETTER(ASFont, fontStyle);
+ASFUNCTIONBODY_GETTER(ASFont, fontType);
+
+ASFUNCTIONBODY(ASFont,enumerateFonts)
+{
+	bool enumerateDeviceFonts=false;
+	ARG_UNPACK(enumerateDeviceFonts,false);
+
+	LOG(LOG_NOT_IMPLEMENTED,"Font::enumerateFonts: flag enumerateDeviceFonts is not handled");
+	Array* ret = Class<Array>::getInstanceS();
+	std::vector<ASObject*>* fontlist = getFontList();
+	for(auto i = fontlist->begin(); i != fontlist->end(); ++i)
+	{
+		(*i)->incRef();
+		ret->push(_MR(*i));
+	}
+	return ret;
+}
+ASFUNCTIONBODY(ASFont,registerFont)
+{
+	getFontList()->push_back(args[0]);
+	return NULL;
 }
 
 void TextField::sinit(Class_base* c)
@@ -67,7 +117,10 @@ void TextField::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("autoSize","",Class<IFunction>::getFunction(TextField::_setAutoSize),SETTER_METHOD,true);
 	c->setDeclaredMethodByQName("autoSize","",Class<IFunction>::getFunction(TextField::_getAutoSize),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("appendText","",Class<IFunction>::getFunction(TextField:: appendText),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("getTextFormat","",Class<IFunction>::getFunction(_getTextFormat),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("setTextFormat","",Class<IFunction>::getFunction(_setTextFormat),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("defaultTextFormat","",Class<IFunction>::getFunction(TextField::_getDefaultTextFormat),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("defaultTextFormat","",Class<IFunction>::getFunction(TextField::_setDefaultTextFormat),SETTER_METHOD,true);
 
 	REGISTER_GETTER_SETTER(c,textColor);
 }
@@ -110,7 +163,7 @@ ASFUNCTIONBODY(TextField,_setWordWrap)
 	assert_and_throw(argslen==1);
 	th->wordWrap=Boolean_concrete(args[0]);
 	if(th->onStage)
-		th->requestInvalidation();
+		th->requestInvalidation(getSys());
 	return NULL;
 }
 
@@ -147,7 +200,7 @@ ASFUNCTIONBODY(TextField,_setAutoSize)
 	else
 		throw Class<ArgumentError>::getInstanceS("Wrong argument in TextField.autoSize");
 	if(th->onStage)
-		th->requestInvalidation();//TODO:check if there was any change
+		th->requestInvalidation(getSys());//TODO:check if there was any change
 	return NULL;
 }
 
@@ -166,7 +219,7 @@ ASFUNCTIONBODY(TextField,_setWidth)
 	{
 		th->width=args[0]->toInt();
 		if(th->onStage)
-			th->requestInvalidation();
+			th->requestInvalidation(getSys());
 		else
 			th->updateSizes();
 	}
@@ -187,7 +240,7 @@ ASFUNCTIONBODY(TextField,_setHeight)
 	{
 		th->height=args[0]->toInt();
 		if(th->onStage)
-			th->requestInvalidation();
+			th->requestInvalidation(getSys());
 		else
 			th->updateSizes();
 	}
@@ -229,25 +282,68 @@ ASFUNCTIONBODY(TextField, appendText)
 	return NULL;
 }
 
+ASFUNCTIONBODY(TextField,_getTextFormat)
+{
+	TextField* th=Class<TextField>::cast(obj);
+	TextFormat *format=Class<TextFormat>::getInstanceS();
+
+	format->color=_MNR(abstract_ui(th->textColor.toUInt()));
+	format->font = th->font;
+	format->size = th->fontSize;
+
+	LOG(LOG_NOT_IMPLEMENTED, "getTextFormat is not fully implemeted");
+
+	return format;
+}
+
 ASFUNCTIONBODY(TextField,_setTextFormat)
 {
 	TextField* th=Class<TextField>::cast(obj);
-	if(argslen == 0)
-		throw RunTimeException("TextField.setTextFormat: Not enough parameters");
-	if(argslen > 1)
-		LOG(LOG_NOT_IMPLEMENTED,"setTextFormat with more than one parameter");
+	_NR<TextFormat> tf;
+	int beginIndex;
+	int endIndex;
 
-	TextFormat* tf = Class<TextFormat>::cast(args[0]);
-	if(tf->getObjectType() == T_NULL)
-		return NULL;
+	ARG_UNPACK(tf)(beginIndex, -1)(endIndex, -1);
+
+	if(beginIndex!=-1 || endIndex!=-1)
+		LOG(LOG_NOT_IMPLEMENTED,"setTextFormat with beginIndex or endIndex");
 
 	if(tf->color)
 		th->textColor = tf->color->toUInt();
+	if (tf->font != "")
+		th->font = tf->font;
+	th->fontSize = tf->size;
 
 	LOG(LOG_NOT_IMPLEMENTED,"setTextFormat does not read all fields of TextFormat");
 	return NULL;
 }
 
+ASFUNCTIONBODY(TextField,_getDefaultTextFormat)
+{
+	TextField* th=Class<TextField>::cast(obj);
+	
+	TextFormat* tf = Class<TextFormat>::getInstanceS();
+	tf->font = th->font;
+	LOG(LOG_NOT_IMPLEMENTED,"getDefaultTextFormat does not get all fields of TextFormat");
+	return tf;
+}
+
+ASFUNCTIONBODY(TextField,_setDefaultTextFormat)
+{
+	TextField* th=Class<TextField>::cast(obj);
+	_NR<TextFormat> tf;
+
+	ARG_UNPACK(tf);
+
+	if(tf->color)
+		th->textColor = tf->color->toUInt();
+	if (tf->font != "")
+		th->font = tf->font;
+	th->fontSize = tf->size;
+	LOG(LOG_NOT_IMPLEMENTED,"setDefaultTextFormat does not set all fields of TextFormat");
+	return NULL;
+}
+
 void TextField::updateSizes()
 {
 	uint32_t w,h,tw,th;
@@ -264,21 +360,20 @@ void TextField::updateSizes()
 void TextField::updateText(const tiny_string& new_text)
 {
 	text = new_text;
-	requestInvalidation();
 	if(onStage)
-		requestInvalidation();
+		requestInvalidation(getSys());
 	else
 		updateSizes();
 }
 
-void TextField::requestInvalidation()
+void TextField::requestInvalidation(InvalidateQueue* q)
 {
 	incRef();
 	updateSizes();
-	getSys()->addToInvalidateQueue(_MR(this));
+	q->addToInvalidateQueue(_MR(this));
 }
 
-void TextField::invalidate()
+IDrawable* TextField::invalidate(DisplayObject* target, const MATRIX& initialMatrix)
 {
 	int32_t x,y;
 	uint32_t width,height;
@@ -286,35 +381,34 @@ void TextField::invalidate()
 	if(boundsRect(bxmin,bxmax,bymin,bymax)==false)
 	{
 		//No contents, nothing to do
-		return;
+		return NULL;
 	}
 
-	computeDeviceBoundsForRect(bxmin,bxmax,bymin,bymax,x,y,width,height);
+	MATRIX totalMatrix(initialMatrix);
+	DisplayObject* cur=this;
+	while(cur!=target)
+	{
+		totalMatrix=cur->getMatrix().multiplyMatrix(totalMatrix);
+		cur=cur->getParent().getPtr();
+	}
+	computeBoundsForTransformedRect(bxmin,bxmax,bymin,bymax,x,y,width,height,totalMatrix);
 	if(width==0 || height==0)
-		return;
-	MATRIX mat = getConcatenatedMatrix();
-	if(mat.ScaleX != 1 || mat.ScaleY != 1)
+		return NULL;
+	if(totalMatrix.getScaleX() != 1 || totalMatrix.getScaleY() != 1)
 		LOG(LOG_NOT_IMPLEMENTED, "TextField when scaled is not correctly implemented");
 	/**  TODO: The scaling is done differently for textfields : height changes are applied directly
 		on the font size. In some cases, it can change the width (if autosize is on and wordwrap off).
 		Width changes do not change the font size, and do nothing when autosize is on and wordwrap off.
 		Currently, the TextField is stretched in case of scaling.
 	*/
-	CairoPangoRenderer* r=new CairoPangoRenderer(this, cachedSurface, *this,
-				getConcatenatedMatrix(), x, y, width, height, 1.0f,
+	return new CairoPangoRenderer(*this,
+				totalMatrix, x, y, width, height, 1.0f,
 				getConcatenatedAlpha());
-	getSys()->addJob(r);
 }
 
 void TextField::renderImpl(RenderContext& ctxt, bool maskEnabled, number_t t1, number_t t2, number_t t3, number_t t4) const
 {
-	//if(!isSimple())
-	//	rt->acquireTempBuffer(t1,t2,t3,t4);
-
 	defaultRender(ctxt, maskEnabled);
-
-	//if(!isSimple())
-	//	rt->blitTempBuffer(t1,t2,t3,t4);
 }
 
 void TextFieldAutoSize ::sinit(Class_base* c)
@@ -341,12 +435,31 @@ void TextFormatAlign ::sinit(Class_base* c)
 
 void TextFormat::sinit(Class_base* c)
 {
-	c->setConstructor(NULL);
+	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<ASObject>::getRef());
 	REGISTER_GETTER_SETTER(c,color);
+	REGISTER_GETTER_SETTER(c,font);
+	REGISTER_GETTER_SETTER(c,size);
+}
+ASFUNCTIONBODY(TextFormat,_constructor)
+{
+	TextFormat* th=static_cast<TextFormat*>(obj);
+	tiny_string font;
+	int32_t size;
+	_NR<ASObject> color;
+	ARG_UNPACK (font, "")(size, 12)(color,_MNR(getSys()->getNullRef()));
+	th->font = font;
+	th->size = size;
+	th->color = color;
+	LOG(LOG_NOT_IMPLEMENTED,"TextFormat: not all properties are set in constructor");
+	return NULL;
 }
 
+
+
 ASFUNCTIONBODY_GETTER_SETTER(TextFormat,color);
+ASFUNCTIONBODY_GETTER_SETTER(TextFormat,font);
+ASFUNCTIONBODY_GETTER_SETTER(TextFormat,size);
 
 void TextFormat::buildTraits(ASObject* o)
 {
@@ -399,7 +512,7 @@ ASFUNCTIONBODY(StyleSheet,getStyle)
 		return it->second.getPtr();
 	}
 	else
-		return new Null;
+		return getSys()->getNullRef();
 	return NULL;
 }
 
@@ -410,7 +523,7 @@ ASFUNCTIONBODY(StyleSheet,_getStyleNames)
 	Array* ret=Class<Array>::getInstanceS();
 	map<tiny_string, _R<ASObject>>::const_iterator it=th->styles.begin();
 	for(;it!=th->styles.end();++it)
-		ret->push(Class<ASString>::getInstanceS(it->first));
+		ret->push(_MR(Class<ASString>::getInstanceS(it->first)));
 	return ret;
 }
 
