@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,7 @@ using namespace lightspark;
 
 SET_NAMESPACE("flash.utils");
 
+REGISTER_CLASS_NAME(IExternalizable);
 REGISTER_CLASS_NAME(Endian);
 REGISTER_CLASS_NAME(IDataInput);
 REGISTER_CLASS_NAME(IDataOutput);
@@ -54,6 +55,12 @@ void Endian::sinit(Class_base* c)
 	c->setVariableByQName("BIG_ENDIAN","",Class<ASString>::getInstanceS(bigEndian),DECLARED_TRAIT);
 }
 
+void IExternalizable::linkTraits(Class_base* c)
+{
+	lookupAndLink(c,"readExternal","flash.utils:IExternalizable");
+	lookupAndLink(c,"writeExternal","flash.utils:IExternalizable");
+}
+
 void IDataInput::linkTraits(Class_base* c)
 {
 	lookupAndLink(c,"bytesAvailable","flash.utils:IDataInput");
@@ -93,22 +100,23 @@ void IDataOutput::linkTraits(Class_base* c)
 	lookupAndLink(c,"writeUTFBytes","flash.utils:IDataOutput");
 }
 
-ByteArray::ByteArray(uint8_t* b, uint32_t l):bytes(b),real_len(l),len(l),position(0),littleEndian(false),objectEncoding(ObjectEncoding::AMF3)
+ByteArray::ByteArray(Class_base* c, uint8_t* b, uint32_t l):ASObject(c),bytes(b),real_len(l),len(l),position(0),
+	littleEndian(false),objectEncoding(ObjectEncoding::AMF3)
 {
-}
-
-ByteArray::ByteArray(const ByteArray& b):ASObject(b),real_len(b.len),len(b.len),position(b.position),littleEndian(b.littleEndian),objectEncoding(b.objectEncoding)
-{
-	assert_and_throw(position==0);
-	bytes = (uint8_t*) malloc(len);
-	assert_and_throw(bytes);
-	memcpy(bytes,b.bytes,len);
+#ifdef MEMORY_USAGE_PROFILING
+	c->memoryAccount->addBytes(l);
+#endif
 }
 
 ByteArray::~ByteArray()
 {
 	if(bytes)
+	{
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->removeBytes(real_len);
+#endif
 		free(bytes);
+	}
 }
 
 void ByteArray::sinit(Class_base* c)
@@ -173,13 +181,22 @@ void ByteArray::buildTraits(ASObject* o)
 
 uint8_t* ByteArray::getBuffer(unsigned int size, bool enableResize)
 {
+	// the flash documentation doesn't tell how large ByteArrays are allowed to be
+	// so we simply don't allow bytearrays larger than 1GiB
+	// maybe we should set this smaller
+	if (size > 0x4000000) 
+		throw Class<ASError>::getInstanceS("Error #1000: out of memory.");
 	// The first allocation is exactly the size we need,
 	// the subsequent reallocations happen in increments of BA_CHUNK_SIZE bytes
+	uint32_t prevLen = len;
 	if(bytes==NULL)
 	{
 		len=size;
 		real_len=len;
 		bytes = (uint8_t*) malloc(len);
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->addBytes(len);
+#endif
 	}
 	else if(enableResize==false)
 	{
@@ -187,10 +204,14 @@ uint8_t* ByteArray::getBuffer(unsigned int size, bool enableResize)
 	}
 	else if(real_len<size) // && enableResize==true
 	{
+		uint32_t prev_real_len = real_len;
 		while(real_len < size)
 			real_len += BA_CHUNK_SIZE;
 		// Reallocate the buffer, in chunks of BA_CHUNK_SIZE bytes
 		uint8_t* bytes2 = (uint8_t*) realloc(bytes, real_len);
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->addBytes(real_len-prev_real_len);
+#endif
 		assert_and_throw(bytes2);
 		bytes = bytes2;
 		len=size;
@@ -200,6 +221,11 @@ uint8_t* ByteArray::getBuffer(unsigned int size, bool enableResize)
 	{
 		len=size;
 	}
+	if(prevLen<size)
+	{
+		//Extend
+		memset(bytes+prevLen,0,size-prevLen);
+	}
 	return bytes;
 }
 
@@ -291,6 +317,8 @@ ASFUNCTIONBODY(ByteArray,_setEndian)
 		th->littleEndian = true;
 	else if(args[0]->toString() == Endian::bigEndian)
 		th->littleEndian = false;
+	else
+		throw Class<ArgumentError>::getInstanceS("Error #2008: Parameter type must be one of the accepted values.");
 	return NULL;
 }
 
@@ -337,17 +365,25 @@ ASFUNCTIONBODY(ByteArray,_setLength)
 	uint32_t newLen=args[0]->toInt();
 	if(newLen==th->len) //Nothing to do
 		return NULL;
-	uint32_t prevLen = th->len;
-	uint8_t* newBytes= (uint8_t*) realloc(th->bytes, newLen);
-	assert_and_throw(newBytes);
-	th->bytes = newBytes;
-	th->len = newLen;
-	th->real_len = newLen;
-	if(prevLen<newLen)
+	if (newLen > 0)
 	{
-		//Extend
-		memset(th->bytes+prevLen,0,newLen-prevLen);
+		th->getBuffer(newLen,true);
 	}
+	else
+	{
+		if (th->bytes)
+		{
+#ifdef MEMORY_USAGE_PROFILING
+			th->getClass()->memoryAccount->removeBytes(th->real_len);
+#endif
+			free(th->bytes);
+		}
+		th->bytes = NULL;
+		th->len = newLen;
+		th->real_len = newLen;
+	}
+	if (th->position > th->len)
+		th->position = (th->len > 0 ? th->len-1 : 0);
 	return NULL;
 }
 
@@ -379,33 +415,29 @@ ASFUNCTIONBODY(ByteArray,readBoolean)
 ASFUNCTIONBODY(ByteArray,readBytes)
 {
 	ByteArray* th=static_cast<ByteArray*>(obj);
-	//Validate parameters
-	assert_and_throw(argslen>=1 && argslen<=3);
-	assert_and_throw(args[0]->getClass()->isSubClass(Class<ByteArray>::getClass()));
-
-	ByteArray* out=Class<ByteArray>::cast(args[0]);
-	uint32_t offset=0;
-	uint32_t length=0;
-	if(argslen>=2)
-		offset=args[1]->toInt();
-	if(argslen==3)
-		length=args[2]->toInt();
-	//TODO: Support offset (offset is in the destination!)
-	if(offset!=0)
+	_NR<ByteArray> out;
+	uint32_t offset;
+	uint32_t length;
+	ARG_UNPACK(out)(offset, 0)(length, 0);
+	
+	if(length == 0)
 	{
-		throw UnsupportedException("offset in ByteArray::readBytes");
+		assert(th->len >= th->position);
+		length = th->len - th->position;
 	}
 
-	if(length == 0)
-		length = th->len;
-
 	//Error checks
-	if(length > th->len)
+	if(th->position+length > th->len)
 	{
 		throw Class<EOFError>::getInstanceS("Error #2030: End of file was encountered.");
 	}
-	uint8_t* buf=out->getBuffer(length,true);
-	memcpy(buf,th->bytes+th->position,length);
+	if((uint64_t)length+offset > 0xFFFFFFFF)
+	{
+		throw Class<RangeError>::getInstanceS("length+offset");
+	}
+	
+	uint8_t* buf=out->getBuffer(length+offset,true);
+	memcpy(buf+offset,th->bytes+th->position,length);
 	th->position+=length;
 
 	return NULL;
@@ -457,6 +489,10 @@ ASFUNCTIONBODY(ByteArray,readUTFBytes)
 void ByteArray::writeUTF(const tiny_string& str)
 {
 	getBuffer(position+str.numBytes()+2,true);
+	if(str.numBytes() > 65535)
+	{
+		throw Class<RangeError>::getInstanceS("Error #2006: The supplied index is out of bounds.");
+	}
 	uint16_t numBytes=endianIn((uint16_t)str.numBytes());
 	memcpy(bytes+position,&numBytes,2);
 	memcpy(bytes+position+2,str.raw_buf(),str.numBytes());
@@ -514,8 +550,9 @@ uint32_t ByteArray::writeObject(ASObject* obj)
 	//TODO: support custom serialization
 	map<tiny_string, uint32_t> stringMap;
 	map<const ASObject*, uint32_t> objMap;
+	map<const Class_base*, uint32_t> traitsMap;
 	uint32_t oldPosition=position;
-	obj->serialize(this, stringMap, objMap);
+	obj->serialize(this, stringMap, objMap,traitsMap);
 	return position-oldPosition;
 }
 
@@ -617,7 +654,8 @@ ASFUNCTIONBODY(ByteArray,writeDouble)
 	assert_and_throw(argslen==1);
 
 	double value = args[0]->toNumber();
-	uint64_t value2=th->endianIn(*reinterpret_cast<uint64_t*>(&value));
+	uint64_t *intptr=reinterpret_cast<uint64_t*>(&value);
+	uint64_t value2=th->endianIn(*intptr);
 
 	th->getBuffer(th->position+8,true);
 	memcpy(th->bytes+th->position,&value2,8);
@@ -632,7 +670,8 @@ ASFUNCTIONBODY(ByteArray,writeFloat)
 	assert_and_throw(argslen==1);
 
 	float value = args[0]->toNumber();
-	uint32_t value2=th->endianIn(*reinterpret_cast<uint32_t*>(&value));
+	uint32_t *intptr=reinterpret_cast<uint32_t*>(&value);
+	uint32_t value2=th->endianIn(*intptr);
 
 	th->getBuffer(th->position+4,true);
 	memcpy(th->bytes+th->position,&value2,4);
@@ -683,6 +722,8 @@ bool ByteArray::readByte(uint8_t& b)
 
 bool ByteArray::readU29(uint32_t& ret)
 {
+	//Be careful! This is different from u32 parsing.
+	//Here the most significant bits appears before in the stream!
 	ret=0;
 	for(uint32_t i=0;i<4;i++)
 	{
@@ -690,15 +731,16 @@ bool ByteArray::readU29(uint32_t& ret)
 			return false;
 
 		uint8_t tmp=bytes[position++];
+		ret <<= i*7;
 		if(i<3)
 		{
-			ret|=((tmp&0x7f) << i*7);
+			ret |= tmp&0x7f;
 			if((tmp&0x80)==0)
 				break;
 		}
 		else
 		{
-			ret|=(tmp << 21);
+			ret |= tmp;
 			//Sign extend
 			if(tmp&0x80)
 				ret|=0xe0000000;
@@ -735,7 +777,8 @@ ASFUNCTIONBODY(ByteArray,readDouble)
 	th->position+=8;
 	ret = th->endianOut(ret);
 
-	return abstract_d(*reinterpret_cast<double*>(&ret));
+	double *doubleptr=reinterpret_cast<double*>(&ret);
+	return abstract_d(*doubleptr);
 }
 
 ASFUNCTIONBODY(ByteArray,readFloat)
@@ -753,7 +796,8 @@ ASFUNCTIONBODY(ByteArray,readFloat)
 	th->position+=4;
 	ret = th->endianOut(ret);
 
-	return abstract_d(*reinterpret_cast<float*>(&ret));
+	float *floatptr=reinterpret_cast<float*>(&ret);
+	return abstract_d(*floatptr);
 }
 
 ASFUNCTIONBODY(ByteArray,readInt)
@@ -876,11 +920,11 @@ ASFUNCTIONBODY(ByteArray,readObject)
 		throw Class<EOFError>::getInstanceS("Error #2030: End of file was encountered.");
 	}
 	assert_and_throw(th->objectEncoding==ObjectEncoding::AMF3);
-	std::vector<ASObject*> ret;
 	Amf3Deserializer d(th);
+	_NR<ASObject> ret(NullRef);
 	try
 	{
-		d.generateObjects(ret);
+		ret=d.readObject();
 	}
 	catch(LightsparkException& e)
 	{
@@ -888,18 +932,13 @@ ASFUNCTIONBODY(ByteArray,readObject)
 		//TODO: throw AS exception
 	}
 
-	if(ret.size()==0)
+	if(ret.isNull())
 	{
 		LOG(LOG_ERROR,"No objects in the AMF3 data. Returning Undefined");
-		return new Undefined;
+		return getSys()->getUndefinedRef();
 	}
-	if(ret.size()>1)
-	{
-		LOG(LOG_ERROR,"More than one object in the AMF3 data. Returning the first");
-		for(uint32_t i=1;i<ret.size();i++)
-			ret[i]->decRef();
-	}
-	return ret[0];
+	ret->incRef();
+	return ret.getPtr();
 }
 
 ASFUNCTIONBODY(ByteArray,_toString)
@@ -934,7 +973,7 @@ _NR<ASObject> ByteArray::getVariableByMultiname(const multiname& name, GET_VARIA
 		return _MNR(abstract_ui(static_cast<uint32_t>(value)));
 	}
 	else
-		return _MNR(new Undefined);
+		return _MNR(getSys()->getUndefinedRef());
 }
 
 int32_t ByteArray::getVariableByMultiname_i(const multiname& name)
@@ -950,15 +989,15 @@ int32_t ByteArray::getVariableByMultiname_i(const multiname& name)
 		return static_cast<uint32_t>(value);
 	}
 	else
-		return _MNR(new Undefined);
+		return _MNR(getSys()->getUndefinedRef());
 }
 
-void ByteArray::setVariableByMultiname(const multiname& name, ASObject* o)
+void ByteArray::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
 	if(!Array::isValidMultiname(name,index))
-		return ASObject::setVariableByMultiname(name,o);
+		return ASObject::setVariableByMultiname(name,o,allowConst);
 
 	if(index>=len)
 	{
@@ -977,16 +1016,24 @@ void ByteArray::setVariableByMultiname(const multiname& name, ASObject* o)
 
 void ByteArray::setVariableByMultiname_i(const multiname& name, int32_t value)
 {
-	setVariableByMultiname(name, abstract_i(value));
+	setVariableByMultiname(name, abstract_i(value),ASObject::CONST_NOT_ALLOWED);
 }
 
 void ByteArray::acquireBuffer(uint8_t* buf, int bufLen)
 {
 	if(bytes)
+	{
+#ifdef MEMORY_USAGE_PROFILING
+		getClass()->memoryAccount->removeBytes(real_len);
+#endif
 		free(bytes);
+	}
 	bytes=buf;
 	real_len=bufLen;
 	len=bufLen;
+#ifdef MEMORY_USAGE_PROFILING
+	getClass()->memoryAccount->addBytes(real_len);
+#endif
 	position=0;
 }
 
@@ -997,17 +1044,16 @@ void ByteArray::writeU29(uint32_t val)
 		uint8_t b;
 		if(i<3)
 		{
-			b=val&0x7f;
-			val>>=7;
-			if(val)
-				b|=0x80;
+			uint32_t tmp=(val >> ((3-i)*7));
+			if(tmp==0)
+				continue;
+
+			b=(tmp&0x7f)|0x80;
 		}
 		else
-			b=val&0xff;
+			b=val&0x7f;
 
 		writeByte(b);
-		if(val==0)
-			break;
 	}
 }
 
@@ -1099,6 +1145,9 @@ void ByteArray::uncompress_zlib()
 	inflateEnd(&strm);
 
 	len=strm.total_out;
+#ifdef MEMORY_USAGE_PROFILING
+	getClass()->memoryAccount->addBytes(len-real_len);
+#endif
 	real_len = len;
 	uint8_t* bytes2=(uint8_t*) realloc(bytes, len);
 	assert_and_throw(bytes2);
@@ -1110,26 +1159,20 @@ void ByteArray::uncompress_zlib()
 ASFUNCTIONBODY(ByteArray,_compress)
 {
 	ByteArray* th=static_cast<ByteArray*>(obj);
-	tiny_string algorithm;
-	ARG_UNPACK(algorithm, "zlib");
-	if(algorithm=="zlib")
-		th->compress_zlib();
-	else
-		throw Class<ASError>::getInstanceS("Unsupported algorithm");
-
+	// flash throws an error if compress is called with a compression algorithm,
+	// and always uses the zlib algorithm
+	// but tamarin tests do not catch it, so we simply ignore any parameters provided
+	th->compress_zlib();
 	return NULL;
 }
 
 ASFUNCTIONBODY(ByteArray,_uncompress)
 {
 	ByteArray* th=static_cast<ByteArray*>(obj);
-	tiny_string algorithm;
-	ARG_UNPACK(algorithm, "zlib");
-	if(algorithm=="zlib")
-		th->uncompress_zlib();
-	else
-		throw Class<ASError>::getInstanceS("Unsupported algorithm");
-
+	// flash throws an error if uncompress is called with a compression algorithm,
+	// and always uses the zlib algorithm
+	// but tamarin tests do not catch it, so we simply ignore any parameters provided
+	th->uncompress_zlib();
 	return NULL;
 }
 
@@ -1151,7 +1194,12 @@ ASFUNCTIONBODY(ByteArray,clear)
 {
 	ByteArray* th=static_cast<ByteArray*>(obj);
 	if(th->bytes)
+	{
+#ifdef MEMORY_USAGE_PROFILING
+		th->getClass()->memoryAccount->removeBytes(th->real_len);
+#endif
 		free(th->bytes);
+	}
 	th->bytes = NULL;
 	th->len=0;
 	th->real_len=0;
@@ -1166,9 +1214,9 @@ void Timer::tick()
 	this->incRef();
 	getVm()->addEvent(_MR(this),_MR(Class<TimerEvent>::getInstanceS("timer")));
 
+	currentCount++;
 	if(repeatCount!=0)
 	{
-		currentCount++;
 		if(currentCount==repeatCount)
 		{
 			this->incRef();
@@ -1179,6 +1227,63 @@ void Timer::tick()
 	}
 }
 
+void Timer::tickFence()
+{
+	this->decRef();
+}
+
+// this seems to be how AS3 handles generic pop calls in Array class
+ASFUNCTIONBODY(ByteArray,pop)
+{
+	ByteArray* th=static_cast<ByteArray*>(obj);
+	uint8_t res = 0;
+	if (th->readByte(res))
+	{
+		memmove(th->bytes,(th->bytes+1),th->getLength()-1);
+		th->len--;
+	}
+	return abstract_ui(res);
+	
+}
+
+// this seems to be how AS3 handles generic push calls in Array class
+ASFUNCTIONBODY(ByteArray,push)
+{
+	ByteArray* th=static_cast<ByteArray*>(obj);
+	th->getBuffer(th->len+argslen,true);
+	for (unsigned int i = 0; i < argslen; i++)
+	{
+		th->bytes[th->len+i] = (uint8_t)args[i]->toInt();
+	}
+	return abstract_ui(th->getLength());
+}
+
+// this seems to be how AS3 handles generic shift calls in Array class
+ASFUNCTIONBODY(ByteArray,shift)
+{
+	ByteArray* th=static_cast<ByteArray*>(obj);
+	uint8_t res = 0;
+	if (th->readByte(res))
+	{
+		memmove(th->bytes,(th->bytes+1),th->getLength()-1);
+		th->len--;
+	}
+	return abstract_ui(res);
+}
+
+// this seems to be how AS3 handles generic unshift calls in Array class
+ASFUNCTIONBODY(ByteArray,unshift)
+{
+	ByteArray* th=static_cast<ByteArray*>(obj);
+	th->getBuffer(th->len+argslen,true);
+	for (unsigned int i = 0; i < argslen; i++)
+	{
+		memmove((th->bytes+argslen),(th->bytes),th->len);
+		th->bytes[i] = (uint8_t)args[i]->toInt();
+	}
+	return abstract_ui(th->getLength());
+}
+
 void Timer::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -1265,6 +1370,7 @@ ASFUNCTIONBODY(Timer,start)
 		return NULL;
 	th->running=true;
 	th->stopMe=false;
+	//To be decReffed in tickFence
 	th->incRef();
 	if(th->repeatCount==1)
 		getSys()->addWait(th->delay,th);
@@ -1351,21 +1457,24 @@ ASFUNCTIONBODY(lightspark,getDefinitionByName)
 {
 	assert_and_throw(args && argslen==1);
 	const tiny_string& tmp=args[0]->toString();
-	multiname name;
+	multiname name(NULL);
 	name.name_type=multiname::NAME_STRING;
-	name.ns.push_back(nsNameAndKind("",NAMESPACE)); //TODO: set type
 
-	stringToQName(tmp,name.name_s,name.ns[0].name);
+	tiny_string nsName;
+	tiny_string tmpName;
+	stringToQName(tmp,tmpName,nsName);
+	name.name_s_id=getSys()->getUniqueStringId(tmpName);
+	name.ns.push_back(nsNameAndKind(nsName,NAMESPACE));
 
 	LOG(LOG_CALLS,_("Looking for definition of ") << name);
 	ASObject* target;
-	ASObject* o=getGlobal()->getVariableAndTargetByMultiname(name,target);
+	ASObject* o=ABCVm::getCurrentApplicationDomain(getVm()->currentCallContext)->getVariableAndTargetByMultiname(name,target);
 
 	//TODO: should raise an exception, for now just return undefined	
 	if(o==NULL)
 	{
 		LOG(LOG_ERROR,_("Definition for '") << name << _("' not found."));
-		return new Undefined;
+		return getSys()->getUndefinedRef();
 	}
 
 	assert_and_throw(o->getObjectType()==T_CLASS);
@@ -1387,6 +1496,11 @@ ASFUNCTIONBODY(lightspark,getTimer)
 	return abstract_i(ret);
 }
 
+Dictionary::Dictionary(Class_base* c):ASObject(c),
+	data(std::less<dictType::key_type>(), reporter_allocator<dictType::value_type>(c->memoryAccount))
+{
+}
+
 void Dictionary::finalize()
 {
 	ASObject::finalize();
@@ -1411,18 +1525,16 @@ ASFUNCTIONBODY(Dictionary,_constructor)
 void Dictionary::setVariableByMultiname_i(const multiname& name, int32_t value)
 {
 	assert_and_throw(implEnable);
-	Dictionary::setVariableByMultiname(name,abstract_i(value));
+	Dictionary::setVariableByMultiname(name,abstract_i(value),CONST_NOT_ALLOWED);
 }
 
-void Dictionary::setVariableByMultiname(const multiname& name, ASObject* o)
+void Dictionary::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
 {
 	assert_and_throw(implEnable);
 	if(name.name_type==multiname::NAME_OBJECT)
 	{
+		name.name_o->incRef();
 		_R<ASObject> name_o(name.name_o);
-		//This is ugly, but at least we are sure that we own name_o
-		multiname* tmp=const_cast<multiname*>(&name);
-		tmp->name_o=NULL;
 
 		map<_R<ASObject>, _R<ASObject> >::iterator it=data.find(name_o);
 		if(it!=data.end())
@@ -1437,7 +1549,7 @@ void Dictionary::setVariableByMultiname(const multiname& name, ASObject* o)
 		assert(name.name_type==multiname::NAME_STRING ||
 			name.name_type==multiname::NAME_INT ||
 			name.name_type==multiname::NAME_NUMBER);
-		ASObject::setVariableByMultiname(name, o);
+		ASObject::setVariableByMultiname(name, o, allowConst);
 	}
 }
 
@@ -1447,10 +1559,8 @@ bool Dictionary::deleteVariableByMultiname(const multiname& name)
 
 	if(name.name_type==multiname::NAME_OBJECT)
 	{
+		name.name_o->incRef();
 		_R<ASObject> name_o(name.name_o);
-		//This is ugly, but at least we are sure that we own name_o
-		multiname* tmp=const_cast<multiname*>(&name);
-		tmp->name_o=NULL;
 
 		map<_R<ASObject>, _R<ASObject> >::iterator it=data.find(name_o);
 		if(it != data.end())
@@ -1477,22 +1587,14 @@ _NR<ASObject> Dictionary::getVariableByMultiname(const multiname& name, GET_VARI
 	{
 		if(name.name_type==multiname::NAME_OBJECT)
 		{
+			name.name_o->incRef();
 			_R<ASObject> name_o(name.name_o);
 
 			map<_R<ASObject>, _R<ASObject> >::iterator it=data.find(name_o);
 			if(it != data.end())
-			{
-				//This is ugly, but at least we are sure that we own name_o
-				multiname* tmp=const_cast<multiname*>(&name);
-				tmp->name_o=NULL;
 				return it->second;
-			}
 			else
-			{
-				//Make sure name_o gets not destroyed, it's still owned by name
-				name_o->incRef();
 				return NullRef;
-			}
 		}
 		else
 		{
@@ -1515,6 +1617,7 @@ bool Dictionary::hasPropertyByMultiname(const multiname& name, bool considerDyna
 
 	if(name.name_type==multiname::NAME_OBJECT)
 	{
+		name.name_o->incRef();
 		_R<ASObject> name_o(name.name_o);
 
 		map<_R<ASObject>, _R<ASObject> >::iterator it=data.find(name_o);
@@ -1611,25 +1714,25 @@ void Proxy::buildTraits(ASObject* o)
 {
 }
 
-void Proxy::setVariableByMultiname(const multiname& name, ASObject* o)
+void Proxy::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
 {
 	//If a variable named like this already exist, use that
 	if(ASObject::hasPropertyByMultiname(name, true) || !implEnable)
 	{
-		ASObject::setVariableByMultiname(name,o);
+		ASObject::setVariableByMultiname(name,o,allowConst);
 		return;
 	}
 
 	//Check if there is a custom setter defined, skipping implementation to avoid recursive calls
-	multiname setPropertyName;
+	multiname setPropertyName(NULL);
 	setPropertyName.name_type=multiname::NAME_STRING;
-	setPropertyName.name_s="setProperty";
+	setPropertyName.name_s_id=getSys()->getUniqueStringId("setProperty");
 	setPropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> proxySetter=getVariableByMultiname(setPropertyName,ASObject::SKIP_IMPL);
 
 	if(proxySetter.isNull())
 	{
-		ASObject::setVariableByMultiname(name,o);
+		ASObject::setVariableByMultiname(name,o,allowConst);
 		return;
 	}
 
@@ -1639,7 +1742,7 @@ void Proxy::setVariableByMultiname(const multiname& name, ASObject* o)
 
 	//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
 	ASObject* args[2];
-	args[0]=Class<ASString>::getInstanceS(name.name_s);
+	args[0]=Class<ASString>::getInstanceS(name.normalizedName());
 	args[1]=o;
 	//We now suppress special handling
 	implEnable=false;
@@ -1654,13 +1757,13 @@ _NR<ASObject> Proxy::getVariableByMultiname(const multiname& name, GET_VARIABLE_
 {
 	//It seems that various kind of implementation works only with the empty namespace
 	assert_and_throw(name.ns.size()>0);
-	if(name.ns[0].name!="" || ASObject::hasPropertyByMultiname(name, true) || !implEnable || (opt & ASObject::SKIP_IMPL)!=0)
+	if(!name.ns[0].hasEmptyName() || ASObject::hasPropertyByMultiname(name, true) || !implEnable || (opt & ASObject::SKIP_IMPL)!=0)
 		return ASObject::getVariableByMultiname(name,opt);
 
 	//Check if there is a custom getter defined, skipping implementation to avoid recursive calls
-	multiname getPropertyName;
+	multiname getPropertyName(NULL);
 	getPropertyName.name_type=multiname::NAME_STRING;
-	getPropertyName.name_s="getProperty";
+	getPropertyName.name_s_id=getSys()->getUniqueStringId("getProperty");
 	getPropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> o=getVariableByMultiname(getPropertyName,ASObject::SKIP_IMPL);
 
@@ -1672,7 +1775,7 @@ _NR<ASObject> Proxy::getVariableByMultiname(const multiname& name, GET_VARIABLE_
 	IFunction* f=static_cast<IFunction*>(o.getPtr());
 
 	//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
-	ASObject* arg=Class<ASString>::getInstanceS(name.name_s);
+	ASObject* arg=Class<ASString>::getInstanceS(name.normalizedName());
 	//We now suppress special handling
 	implEnable=false;
 	LOG(LOG_CALLS,"Proxy::getProperty");
@@ -1685,15 +1788,14 @@ _NR<ASObject> Proxy::getVariableByMultiname(const multiname& name, GET_VARIABLE_
 bool Proxy::hasPropertyByMultiname(const multiname& name, bool considerDynamic)
 {
 	//If a variable named like this already exist, use that
-	if(ASObject::hasPropertyByMultiname(name, considerDynamic) || !implEnable)
-	{
-		return ASObject::deleteVariableByMultiname(name);
-	}
+	bool asobject_has_property=ASObject::hasPropertyByMultiname(name, considerDynamic);
+	if(asobject_has_property || !implEnable)
+		return asobject_has_property;
 
-	//Check if there is a custom deleter defined, skipping implementation to avoid recursive calls
-	multiname hasPropertyName;
+	//Check if there is a custom hasProperty defined, skipping implementation to avoid recursive calls
+	multiname hasPropertyName(NULL);
 	hasPropertyName.name_type=multiname::NAME_STRING;
-	hasPropertyName.name_s="hasProperty";
+	hasPropertyName.name_s_id=getSys()->getUniqueStringId("hasProperty");
 	hasPropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> proxyHasProperty=getVariableByMultiname(hasPropertyName,ASObject::SKIP_IMPL);
 
@@ -1707,7 +1809,7 @@ bool Proxy::hasPropertyByMultiname(const multiname& name, bool considerDynamic)
 	IFunction* f=static_cast<IFunction*>(proxyHasProperty.getPtr());
 
 	//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
-	ASObject* arg=Class<ASString>::getInstanceS(name.name_s);
+	ASObject* arg=Class<ASString>::getInstanceS(name.normalizedName());
 	//We now suppress special handling
 	implEnable=false;
 	LOG(LOG_CALLS,_("Proxy::hasProperty"));
@@ -1726,9 +1828,9 @@ bool Proxy::deleteVariableByMultiname(const multiname& name)
 	}
 
 	//Check if there is a custom deleter defined, skipping implementation to avoid recursive calls
-	multiname deletePropertyName;
+	multiname deletePropertyName(NULL);
 	deletePropertyName.name_type=multiname::NAME_STRING;
-	deletePropertyName.name_s="deleteProperty";
+	deletePropertyName.name_s_id=getSys()->getUniqueStringId("deleteProperty");
 	deletePropertyName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> proxyDeleter=getVariableByMultiname(deletePropertyName,ASObject::SKIP_IMPL);
 
@@ -1742,7 +1844,7 @@ bool Proxy::deleteVariableByMultiname(const multiname& name)
 	IFunction* f=static_cast<IFunction*>(proxyDeleter.getPtr());
 
 	//Well, I don't how to pass multiname to an as function. I'll just pass the name as a string
-	ASObject* arg=Class<ASString>::getInstanceS(name.name_s);
+	ASObject* arg=Class<ASString>::getInstanceS(name.normalizedName());
 	//We now suppress special handling
 	implEnable=false;
 	LOG(LOG_CALLS,_("Proxy::deleteProperty"));
@@ -1758,9 +1860,9 @@ uint32_t Proxy::nextNameIndex(uint32_t cur_index)
 	assert_and_throw(implEnable);
 	LOG(LOG_CALLS,"Proxy::nextNameIndex");
 	//Check if there is a custom enumerator, skipping implementation to avoid recursive calls
-	multiname nextNameIndexName;
+	multiname nextNameIndexName(NULL);
 	nextNameIndexName.name_type=multiname::NAME_STRING;
-	nextNameIndexName.name_s="nextNameIndex";
+	nextNameIndexName.name_s_id=getSys()->getUniqueStringId("nextNameIndex");
 	nextNameIndexName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> o=getVariableByMultiname(nextNameIndexName,ASObject::SKIP_IMPL);
 	assert_and_throw(!o.isNull() && o->getObjectType()==T_FUNCTION);
@@ -1778,9 +1880,9 @@ _R<ASObject> Proxy::nextName(uint32_t index)
 	assert_and_throw(implEnable);
 	LOG(LOG_CALLS, _("Proxy::nextName"));
 	//Check if there is a custom enumerator, skipping implementation to avoid recursive calls
-	multiname nextNameName;
+	multiname nextNameName(NULL);
 	nextNameName.name_type=multiname::NAME_STRING;
-	nextNameName.name_s="nextName";
+	nextNameName.name_s_id=getSys()->getUniqueStringId("nextName");
 	nextNameName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> o=getVariableByMultiname(nextNameName,ASObject::SKIP_IMPL);
 	assert_and_throw(!o.isNull() && o->getObjectType()==T_FUNCTION);
@@ -1795,9 +1897,9 @@ _R<ASObject> Proxy::nextValue(uint32_t index)
 	assert_and_throw(implEnable);
 	LOG(LOG_CALLS, _("Proxy::nextValue"));
 	//Check if there is a custom enumerator, skipping implementation to avoid recursive calls
-	multiname nextValueName;
+	multiname nextValueName(NULL);
 	nextValueName.name_type=multiname::NAME_STRING;
-	nextValueName.name_s="nextValue";
+	nextValueName.name_s_id=getSys()->getUniqueStringId("nextValue");
 	nextValueName.ns.push_back(nsNameAndKind(flash_proxy,NAMESPACE));
 	_NR<ASObject> o=getVariableByMultiname(nextValueName,ASObject::SKIP_IMPL);
 	assert_and_throw(!o.isNull() && o->getObjectType()==T_FUNCTION);
@@ -1825,7 +1927,8 @@ ASFUNCTIONBODY(lightspark,setInterval)
 	args[0]->incRef();
 	IFunction* callback=static_cast<IFunction*>(args[0]);
 	//Add interval through manager
-	uint32_t id = getSys()->intervalManager->setInterval(_MR(callback), callbackArgs, argslen-2, _MR(new Null), args[1]->toInt());
+	uint32_t id = getSys()->intervalManager->setInterval(_MR(callback), callbackArgs, argslen-2,
+			_MR(getSys()->getNullRef()), args[1]->toInt());
 	return abstract_i(id);
 }
 
@@ -1847,7 +1950,8 @@ ASFUNCTIONBODY(lightspark,setTimeout)
 	args[0]->incRef();
 	IFunction* callback=static_cast<IFunction*>(args[0]);
 	//Add timeout through manager
-	uint32_t id = getSys()->intervalManager->setTimeout(_MR(callback), callbackArgs, argslen-2, _MR(new Null), args[1]->toInt());
+	uint32_t id = getSys()->intervalManager->setTimeout(_MR(callback), callbackArgs, argslen-2,
+			_MR(getSys()->getNullRef()), args[1]->toInt());
 	return abstract_i(id);
 }
 
@@ -1867,7 +1971,7 @@ ASFUNCTIONBODY(lightspark,clearTimeout)
 
 IntervalRunner::IntervalRunner(IntervalRunner::INTERVALTYPE _type, uint32_t _id, _R<IFunction> _callback, ASObject** _args,
 		const unsigned int _argslen, _R<ASObject> _obj, uint32_t _interval):
-	type(_type), id(_id), callback(_callback),argslen(_argslen),obj(_obj),interval(_interval)
+	EventDispatcher(NULL),type(_type), id(_id), callback(_callback),argslen(_argslen),obj(_obj),interval(_interval)
 {
 	args = new ASObject*[argslen];
 	for(uint32_t i=0; i<argslen; i++)
@@ -1889,7 +1993,7 @@ void IntervalRunner::tick()
 	{
 		args[i]->incRef();
 	}
-	_R<FunctionEvent> event(new FunctionEvent(callback, obj, args, argslen));
+	_R<FunctionEvent> event(new (getSys()->unaccountedMemory) FunctionEvent(callback, obj, args, argslen));
 	getVm()->addEvent(NullRef,event);
 	if(type == TIMEOUT)
 	{
@@ -1900,6 +2004,11 @@ void IntervalRunner::tick()
 	}
 }
 
+void IntervalRunner::tickFence()
+{
+	delete this;
+}
+
 IntervalManager::IntervalManager() : currentID(1)
 {
 }
@@ -1911,7 +2020,6 @@ IntervalManager::~IntervalManager()
 	while(it != runners.end())
 	{
 		getSys()->removeJob((*it).second);
-		delete((*it).second);
 		runners.erase(it++);
 	}
 }
@@ -1921,7 +2029,8 @@ uint32_t IntervalManager::setInterval(_R<IFunction> callback, ASObject** args, c
 	Mutex::Lock l(mutex);
 
 	uint32_t id = getFreeID();
-	IntervalRunner* runner = new IntervalRunner(IntervalRunner::INTERVAL, id, callback, args, argslen, obj, interval);
+	IntervalRunner* runner = new (getSys()->unaccountedMemory)
+		IntervalRunner(IntervalRunner::INTERVAL, id, callback, args, argslen, obj, interval);
 
 	//Add runner as tickjob
 	getSys()->addTick(interval, runner);
@@ -1937,7 +2046,8 @@ uint32_t IntervalManager::setTimeout(_R<IFunction> callback, ASObject** args, co
 	Mutex::Lock l(mutex);
 
 	uint32_t id = getFreeID();
-	IntervalRunner* runner = new IntervalRunner(IntervalRunner::TIMEOUT, id, callback, args, argslen, obj, interval);
+	IntervalRunner* runner = new (getSys()->unaccountedMemory)
+		IntervalRunner(IntervalRunner::TIMEOUT, id, callback, args, argslen, obj, interval);
 
 	//Add runner as waitjob
 	getSys()->addWait(interval, runner);
@@ -1970,7 +2080,6 @@ void IntervalManager::clearInterval(uint32_t id, IntervalRunner::INTERVALTYPE ty
 		{
 			getSys()->removeJob((*it).second);
 		}
-		delete (*it).second;
 		runners.erase(it);
 	}
 }
