@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +23,7 @@
 #include "compat.h"
 #include "asobject.h"
 #include "smartrefs.h"
-#include <libxml++/parsers/domparser.h>
+#include "backends/xml_support.h"
 
 namespace lightspark
 {
@@ -37,8 +37,8 @@ protected:
 	_NR<XMLDocument> root;
 	xmlpp::Node* node;
 public:
-	XMLNode():root(NullRef),node(NULL){}
-	XMLNode(_R<XMLDocument> _r, xmlpp::Node* _n);
+	XMLNode(Class_base* c):ASObject(c),root(NullRef),node(NULL){}
+	XMLNode(Class_base* c, _R<XMLDocument> _r, xmlpp::Node* _n);
 	void finalize();
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
@@ -48,18 +48,17 @@ public:
 	ASFUNCTION(attributes);
 	ASFUNCTION(_getNodeType);
 	ASFUNCTION(_getNodeName);
+	ASFUNCTION(_getNodeValue);
 };
 
-class XMLDocument: public XMLNode
+class XMLDocument: public XMLNode, public XMLBase
 {
+friend class XMLNode;
 private:
-	//The parser will destroy the document and all the childs on destruction
-	xmlpp::DomParser parser;
-	bool ownsDocument;
-	xmlpp::Document* document;
-	void clear();
+	xmlpp::Node* rootNode;
 public:
-	XMLDocument():ownsDocument(false),document(NULL){}
+	XMLDocument(Class_base* c):XMLNode(c),rootNode(NULL){}
+	void parseXMLImpl(const std::string& str);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
@@ -67,7 +66,8 @@ public:
 	ASFUNCTION(firstChild);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 };
 
 };
