@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -23,38 +23,39 @@
 #include "compat.h"
 #include "argconv.h"
 #include "parsing/amf3_generator.h"
+#include "RegExp.h"
 
 using namespace std;
 using namespace lightspark;
 
 REGISTER_CLASS_NAME2(ASString, "String", "");
 
-ASString::ASString()
+ASString::ASString(Class_base* c):ASObject(c)
 {
 	type=T_STRING;
 }
 
-ASString::ASString(const string& s): data(s)
+ASString::ASString(Class_base* c,const string& s) : ASObject(c),data(s)
 {
 	type=T_STRING;
 }
 
-ASString::ASString(const tiny_string& s) : data(s)
+ASString::ASString(Class_base* c,const tiny_string& s) : ASObject(c),data(s)
 {
 	type=T_STRING;
 }
 
-ASString::ASString(const Glib::ustring& s) : data(s)
+ASString::ASString(Class_base* c,const Glib::ustring& s) : ASObject(c),data(s)
 {
 	type=T_STRING;
 }
 
-ASString::ASString(const char* s) : data(s, /*copy:*/true)
+ASString::ASString(Class_base* c,const char* s) : ASObject(c),data(s, /*copy:*/true)
 {
 	type=T_STRING;
 }
 
-ASString::ASString(const char* s, uint32_t len)
+ASString::ASString(Class_base* c,const char* s, uint32_t len) : ASObject(c)
 {
 	data = std::string(s,len);
 	type=T_STRING;
@@ -75,20 +76,21 @@ ASFUNCTIONBODY(ASString,_getLength)
 
 void ASString::sinit(Class_base* c)
 {
+	c->isFinal = true;
 	c->setSuper(Class<ASObject>::getRef());
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->setDeclaredMethodByQName("split",AS3,Class<IFunction>::getFunction(split),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("substr",AS3,Class<IFunction>::getFunction(substr),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("substring",AS3,Class<IFunction>::getFunction(substring),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("split",AS3,Class<IFunction>::getFunction(split,2),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("substr",AS3,Class<IFunction>::getFunction(substr,2),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("substring",AS3,Class<IFunction>::getFunction(substring,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("replace",AS3,Class<IFunction>::getFunction(replace),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("concat",AS3,Class<IFunction>::getFunction(concat),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("match",AS3,Class<IFunction>::getFunction(match),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("search",AS3,Class<IFunction>::getFunction(search),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("indexOf",AS3,Class<IFunction>::getFunction(indexOf),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("lastIndexOf",AS3,Class<IFunction>::getFunction(lastIndexOf),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("indexOf",AS3,Class<IFunction>::getFunction(indexOf,2),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("lastIndexOf",AS3,Class<IFunction>::getFunction(lastIndexOf,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("charCodeAt",AS3,Class<IFunction>::getFunction(charCodeAt),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("charAt",AS3,Class<IFunction>::getFunction(charAt),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("slice",AS3,Class<IFunction>::getFunction(slice),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("charAt",AS3,Class<IFunction>::getFunction(charAt,1),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("slice",AS3,Class<IFunction>::getFunction(slice,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toLocaleLowerCase",AS3,Class<IFunction>::getFunction(toLowerCase),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toLocaleUpperCase",AS3,Class<IFunction>::getFunction(toUpperCase),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toLowerCase",AS3,Class<IFunction>::getFunction(toLowerCase),NORMAL_METHOD,true);
@@ -98,17 +100,18 @@ void ASString::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(_toString),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(_toString),NORMAL_METHOD,true);
 
-	c->prototype->setVariableByQName("split","",Class<IFunction>::getFunction(split),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("substring","",Class<IFunction>::getFunction(substring),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("split","",Class<IFunction>::getFunction(split,2),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("substr","",Class<IFunction>::getFunction(substr,2),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("substring","",Class<IFunction>::getFunction(substring,2),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("replace","",Class<IFunction>::getFunction(replace),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("concat","",Class<IFunction>::getFunction(concat),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("match","",Class<IFunction>::getFunction(match),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("search","",Class<IFunction>::getFunction(search),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("indexOf","",Class<IFunction>::getFunction(indexOf),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("lastIndexOf","",Class<IFunction>::getFunction(lastIndexOf),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("indexOf","",Class<IFunction>::getFunction(indexOf,2),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("lastIndexOf","",Class<IFunction>::getFunction(lastIndexOf,2),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("charCodeAt","",Class<IFunction>::getFunction(charCodeAt),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("charAt","",Class<IFunction>::getFunction(charAt),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("slice","",Class<IFunction>::getFunction(slice),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("charAt","",Class<IFunction>::getFunction(charAt,1),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("slice","",Class<IFunction>::getFunction(slice,2),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("toLocaleLowerCase","",Class<IFunction>::getFunction(toLowerCase),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("toLocaleUpperCase","",Class<IFunction>::getFunction(toUpperCase),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("toLowerCase","",Class<IFunction>::getFunction(toLowerCase),DYNAMIC_TRAIT);
@@ -128,7 +131,7 @@ ASFUNCTIONBODY(ASString,search)
 	if(argslen == 0 || args[0]->getObjectType() == T_UNDEFINED)
 		return abstract_i(-1);
 
-	int options=PCRE_UTF8;
+	int options=PCRE_UTF8|PCRE_NEWLINE_ANY|PCRE_JAVASCRIPT_COMPAT;
 	tiny_string restr;
 	if(args[0]->getClass() && args[0]->getClass()==Class<RegExp>::getClass())
 	{
@@ -153,7 +156,6 @@ ASFUNCTIONBODY(ASString,search)
 	pcre* pcreRE=pcre_compile(restr.raw_buf(), options, &error, &errorOffset,NULL);
 	if(error)
 		return abstract_i(ret);
-	//Verify that 30 for ovector is ok, it must be at least (captGroups+1)*3
 	int capturingGroups;
 	int infoOk=pcre_fullinfo(pcreRE, NULL, PCRE_INFO_CAPTURECOUNT, &capturingGroups);
 	if(infoOk!=0)
@@ -161,11 +163,13 @@ ASFUNCTIONBODY(ASString,search)
 		pcre_free(pcreRE);
 		return abstract_i(ret);
 	}
-	assert_and_throw(capturingGroups<10);
-	int ovector[30];
+	pcre_extra extra;
+	extra.match_limit_recursion=200;
+	extra.flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
+	int ovector[(capturingGroups+1)*3];
 	int offset=0;
 	//Global is not used in search
-	int rc=pcre_exec(pcreRE, NULL, data.raw_buf(), data.numBytes(), offset, 0, ovector, 30);
+	int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
 	if(rc<0)
 	{
 		//No matches or error
@@ -176,6 +180,7 @@ ASFUNCTIONBODY(ASString,search)
 	// pcre_exec returns byte position, so we have to convert it to character position 
 	tiny_string tmp = data.substr_bytes(0, ret);
 	ret = tmp.numChars();
+	pcre_free(pcreRE);
 	return abstract_i(ret);
 }
 
@@ -183,7 +188,7 @@ ASFUNCTIONBODY(ASString,match)
 {
 	tiny_string data = obj->toString();
 	if(argslen == 0 || args[0]->getObjectType()==T_NULL || args[0]->getObjectType()==T_UNDEFINED)
-		return new Null;
+		return getSys()->getNullRef();
 	ASObject* ret=NULL;
 	RegExp* re;
 
@@ -227,7 +232,7 @@ ASFUNCTIONBODY(ASString,match)
 		if (resarr->size() == 0)
 		{
 			resarr->decRef();
-			ret = new Null;
+			ret = getSys()->getNullRef();
 		}
 		else
 			ret = resarr;
@@ -259,13 +264,16 @@ ASFUNCTIONBODY(ASString,split)
 {
 	tiny_string data = obj->toString();
 	Array* ret=Class<Array>::getInstanceS();
-	ASObject* delimiter=args[0];
-	if(argslen == 0 || delimiter->getObjectType()==T_UNDEFINED)
+	uint32_t limit = 0x7fffffff;
+	if(argslen == 0 )
 	{
-		ret->push(Class<ASString>::getInstanceS(data));
+		ret->push(_MR(Class<ASString>::getInstanceS(data)));
 		return ret;
 	}
-
+	if (argslen > 1 && !args[1]->is<Undefined>())
+		limit = args[1]->toUInt();
+	if (limit == 0)
+		return ret;
 	if(args[0]->getClass() && args[0]->getClass()==Class<RegExp>::getClass())
 	{
 		RegExp* re=static_cast<RegExp*>(args[0]);
@@ -274,25 +282,17 @@ ASFUNCTIONBODY(ASString,split)
 		{
 			//the RegExp is empty, so split every character
 			for(auto i=data.begin();i!=data.end();++i)
-				ret->push( Class<ASString>::getInstanceS( tiny_string::fromChar(*i) ) );
+			{
+				if (ret->size() >= limit)
+					break;
+				ret->push(_MR(Class<ASString>::getInstanceS( tiny_string::fromChar(*i) ) ));
+			}
 			return ret;
 		}
 
-		const char* error;
-		int offset;
-		int options=PCRE_UTF8;
-		if(re->ignoreCase)
-			options|=PCRE_CASELESS;
-		if(re->extended)
-			options|=PCRE_EXTENDED;
-		if(re->multiline)
-			options|=PCRE_MULTILINE;
-		if(re->dotall)
-			options|=PCRE_DOTALL;
-		pcre* pcreRE=pcre_compile(re->source.raw_buf(), options, &error, &offset,NULL);
-		if(error)
+		pcre* pcreRE = re->compile();
+		if (!pcreRE)
 			return ret;
-		//Verify that 30 for ovector is ok, it must be at least (captGroups+1)*3
 		int capturingGroups;
 		int infoOk=pcre_fullinfo(pcreRE, NULL, PCRE_INFO_CAPTURECOUNT, &capturingGroups);
 		if(infoOk!=0)
@@ -300,15 +300,17 @@ ASFUNCTIONBODY(ASString,split)
 			pcre_free(pcreRE);
 			return ret;
 		}
-		assert_and_throw(capturingGroups<10);
-		int ovector[30];
-		offset=0;
+		pcre_extra extra;
+		extra.match_limit_recursion=200;
+		extra.flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
+		int ovector[(capturingGroups+1)*3];
+		int offset=0;
 		unsigned int end;
 		uint32_t lastMatch = 0;
 		do
 		{
 			//offset is a byte offset that must point to the beginning of an utf8 character
-			int rc=pcre_exec(pcreRE, NULL, data.raw_buf(), data.numBytes(), offset, 0, ovector, 30);
+			int rc=pcre_exec(pcreRE, &extra, data.raw_buf(), data.numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
 			end=ovector[0];
 			if(rc<0)
 				break;
@@ -319,22 +321,26 @@ ASFUNCTIONBODY(ASString,split)
 			}
 			//Extract string from last match until the beginning of the current match
 			ASString* s=Class<ASString>::getInstanceS(data.substr_bytes(lastMatch,end-lastMatch));
-			ret->push(s);
+			if (ret->size() >= limit)
+				break;
+			ret->push(_MR(s));
 			lastMatch=offset=ovector[1];
 
 			//Insert capturing groups
 			for(int i=1;i<rc;i++)
 			{
+				if (ret->size() >= limit)
+					break;
 				//use string interface through raw(), because we index on bytes, not on UTF-8 characters
 				ASString* s=Class<ASString>::getInstanceS(data.substr_bytes(ovector[i*2],ovector[i*2+1]-ovector[i*2]));
-				ret->push(s);
+				ret->push(_MR(s));
 			}
 		}
-		while(end<data.numBytes());
-		if(lastMatch != data.numBytes()+1)
+		while(end<data.numBytes() && ret->size() < limit);
+		if(ret->size() < limit && lastMatch != data.numBytes()+1)
 		{
 			ASString* s=Class<ASString>::getInstanceS(data.substr_bytes(lastMatch,data.numBytes()-lastMatch));
-			ret->push(s);
+			ret->push(_MR(s));
 		}
 		pcre_free(pcreRE);
 	}
@@ -344,8 +350,19 @@ ASFUNCTIONBODY(ASString,split)
 		if(del.empty())
 		{
 			//the string is empty, so split every character
+
+			if (data.numChars() == 0)
+			{
+				ret->push(_MR(Class<ASString>::getInstanceS("")));
+			}
+			uint32_t j = 0;
 			for(auto i=data.begin();i!=data.end();++i)
-				ret->push( Class<ASString>::getInstanceS( tiny_string::fromChar(*i) ) );
+			{
+				if (j >= limit)
+					break;
+				j++;
+				ret->push(_MR(Class<ASString>::getInstanceS( tiny_string::fromChar(*i) ) ));
+			}
 			return ret;
 		}
 		unsigned int start=0;
@@ -357,10 +374,14 @@ ASFUNCTIONBODY(ASString,split)
 			if(match==-1)
 				match=data.numChars();
 			ASString* s=Class<ASString>::getInstanceS(data.substr(start,(match-start)));
-			ret->push(s);
+			if (ret->size() >= limit)
+				break;
+			ret->push(_MR(s));
 			start=match+del.numChars();
+			if (start == data.numChars())
+				ret->push(_MR(Class<ASString>::getInstanceS("")));
 		}
-		while(start<data.numChars());
+		while(start<data.numChars() && ret->size() < limit);
 	}
 
 	return ret;
@@ -371,7 +392,12 @@ ASFUNCTIONBODY(ASString,substr)
 	tiny_string data = obj->toString();
 	int start=0;
 	if(argslen>=1)
-		start=args[0]->toInt();
+	{
+		if (!std::isnan(args[0]->toNumber()))
+			start=args[0]->toInt();
+		if (start >= 0  && std::isinf(args[0]->toNumber()))
+			return Class<ASString>::getInstanceS("");
+	}
 	if(start<0) {
 		start=data.numChars()+start;
 		if(start<0)
@@ -381,33 +407,37 @@ ASFUNCTIONBODY(ASString,substr)
 		start=data.numChars();
 
 	int len=0x7fffffff;
-	if(argslen==2)
-		len=args[1]->toInt();
-
+	if (argslen==2 && !args[1]->is<Undefined>())
+	{
+		if (std::isinf(args[1]->toNumber()))
+		{
+			if (args[1]->toInt() < 0)
+				len = 0;
+		}
+		else
+			len=args[1]->toInt();
+	}
 	return Class<ASString>::getInstanceS(data.substr(start,len));
 }
 
 ASFUNCTIONBODY(ASString,substring)
 {
 	tiny_string data = obj->toString();
-	int start=0;
-	if (argslen>=1)
-		start=args[0]->toInt();
-	if(start<0)
+
+	number_t start, end;
+	ARG_UNPACK (start,0) (end,0x7fffffff);
+	if(start<0 || std::isnan(start))
 		start=0;
-	if(start>(int)data.numChars())
+	if(start>(int)data.numChars() || std::isinf(start))
 		start=data.numChars();
 
-	int end=0x7fffffff;
-	if(argslen>=2)
-		end=args[1]->toInt();
-	if(end<0)
+	if(end<0 || std::isnan(end))
 		end=0;
-	if(end>(int)data.numChars())
+	if(end>(int)data.numChars() || std::isinf(end))
 		end=data.numChars();
 
 	if(start>end) {
-		int tmp=start;
+		number_t tmp=start;
 		start=end;
 		end=tmp;
 	}
@@ -518,9 +548,10 @@ TRISTATE ASString::isLess(ASObject* r)
 }
 
 void ASString::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap)
 {
-	out->writeByte(amf3::string_marker);
+	out->writeByte(string_marker);
 	out->writeStringVR(stringMap, data);
 }
 
@@ -558,27 +589,27 @@ ASFUNCTIONBODY(ASString,slice)
 ASFUNCTIONBODY(ASString,charAt)
 {
 	tiny_string data = obj->toString();
-	int index;
+	number_t index;
 	ARG_UNPACK (index, 0);
 
 	int maxIndex=data.numChars();
-	if(index<0 || index>=maxIndex)
-		return Class<ASString>::getInstanceS();
+	if(index<0 || index>=maxIndex || std::isinf(index))
+		return Class<ASString>::getInstanceS("");
 	return Class<ASString>::getInstanceS( tiny_string::fromChar(data.charAt(index)) );
 }
 
 ASFUNCTIONBODY(ASString,charCodeAt)
 {
 	tiny_string data = obj->toString();
-	unsigned int index;
+	number_t index;
 	ARG_UNPACK (index, 0);
-	if(index<data.numChars())
+	if(index<0 || index>=data.numChars() || std::isinf(index) || std::isnan(index))
+		return abstract_d(Number::NaN);
+	else
 	{
 		//Character codes are expected to be positive
 		return abstract_i(data.charAt(index));
 	}
-	else
-		return abstract_d(Number::NaN);
 }
 
 ASFUNCTIONBODY(ASString,indexOf)
@@ -604,7 +635,7 @@ ASFUNCTIONBODY(ASString,lastIndexOf)
 	tiny_string data = obj->toString();
 	tiny_string val=args[0]->toString();
 	size_t startIndex=data.npos;
-	if(argslen > 1 && args[1]->getObjectType() != T_UNDEFINED && !std::isnan(args[1]->toNumber()))
+	if(argslen > 1 && args[1]->getObjectType() != T_UNDEFINED && !std::isnan(args[1]->toNumber()) && !(args[1]->toNumber() > 0 && std::isinf(args[1]->toNumber())))
 	{
 		int32_t i = args[1]->toInt();
 		if(i<0)
@@ -636,7 +667,7 @@ ASFUNCTIONBODY(ASString,fromCharCode)
 	ASString* ret=Class<ASString>::getInstanceS();
 	for(uint32_t i=0;i<argslen;i++)
 	{
-		ret->data += tiny_string::fromChar(args[i]->toUInt());
+		ret->data += tiny_string::fromChar(args[i]->toUInt16());
 	}
 	return ret;
 }
@@ -660,27 +691,18 @@ ASFUNCTIONBODY(ASString,replace)
 		replaceWith=args[1]->toString();
 	}
 	else
+	{
+		replaceWith=args[1]->toString();
 		type = FUNC;
-
+	}
 	if(args[0]->getClass()==Class<RegExp>::getClass())
 	{
 		RegExp* re=static_cast<RegExp*>(args[0]);
 
-		const char* error;
-		int errorOffset;
-		int options=PCRE_UTF8;
-		if(re->ignoreCase)
-			options|=PCRE_CASELESS;
-		if(re->extended)
-			options|=PCRE_EXTENDED;
-		if(re->multiline)
-			options|=PCRE_MULTILINE;
-		if(re->dotall)
-			options|=PCRE_DOTALL;
-		pcre* pcreRE=pcre_compile(re->source.raw_buf(), options, &error, &errorOffset,NULL);
-		if(error)
+		pcre* pcreRE = re->compile();
+		if (!pcreRE)
 			return ret;
-		//Verify that 30 for ovector is ok, it must be at least (captGroups+1)*3
+
 		int capturingGroups;
 		int infoOk=pcre_fullinfo(pcreRE, NULL, PCRE_INFO_CAPTURECOUNT, &capturingGroups);
 		if(infoOk!=0)
@@ -688,19 +710,24 @@ ASFUNCTIONBODY(ASString,replace)
 			pcre_free(pcreRE);
 			return ret;
 		}
-		assert_and_throw(capturingGroups<20);
-		int ovector[60];
+		pcre_extra extra;
+		extra.match_limit_recursion=200;
+		extra.flags = PCRE_EXTRA_MATCH_LIMIT_RECURSION;
+		int ovector[(capturingGroups+1)*3];
 		int offset=0;
 		int retDiff=0;
+		tiny_string prevsubstring = "";
 		do
 		{
-			int rc=pcre_exec(pcreRE, NULL, ret->data.raw_buf(), ret->data.numBytes(), offset, 0, ovector, 60);
+			tiny_string replaceWithTmp = replaceWith;
+			int rc=pcre_exec(pcreRE, &extra, ret->data.raw_buf(), ret->data.numBytes(), offset, 0, ovector, (capturingGroups+1)*3);
 			if(rc<0)
 			{
 				//No matches or error
 				pcre_free(pcreRE);
 				return ret;
 			}
+			prevsubstring += ret->data.substr_bytes(offset,ovector[0]-offset);
 			if(type==FUNC)
 			{
 				//Get the replace for this match
@@ -713,14 +740,14 @@ ASFUNCTIONBODY(ASString,replace)
 				subargs[capturingGroups+1]=abstract_i(ovector[0]-retDiff);
 				
 				subargs[capturingGroups+2]=Class<ASString>::getInstanceS(data);
-				ASObject* ret=f->call(new Null, subargs, 3+capturingGroups);
-				replaceWith=ret->toString().raw_buf();
+				ASObject* ret=f->call(getSys()->getNullRef(), subargs, 3+capturingGroups);
+				replaceWithTmp=ret->toString().raw_buf();
 				ret->decRef();
 			} else {
 					size_t pos, ipos = 0;
 					tiny_string group;
 					int i, j;
-					while((pos = replaceWith.find("$", ipos)) != tiny_string::npos) {
+					while((pos = replaceWithTmp.find("$", ipos)) != tiny_string::npos) {
 						i = 0;
 						ipos = pos;
 						/* docu is not clear what to do if the $nn value is higher 
@@ -729,21 +756,39 @@ ASFUNCTIONBODY(ASString,replace)
 						 * that we should take it as an $n value
 						 */
 						   
-						while (++ipos < replaceWith.numChars() && i<10) {
-						j = replaceWith.charAt(ipos)-'0';
+						while (++ipos < replaceWithTmp.numChars() && i<10) {
+						j = replaceWithTmp.charAt(ipos)-'0';
 							if (j <0 || j> 9 || rc < 10*i + j)
 								break;
 							i = 10*i + j;
 						}
 						if (i == 0)
+						{
+							switch (replaceWithTmp.charAt(ipos))
+							{
+								case '$':
+									replaceWithTmp.replace(ipos,1,"");
+									break;
+								case '&':
+									replaceWithTmp.replace(ipos-1,2,ret->data.substr_bytes(ovector[0], ovector[1]-ovector[0]));
+									break;
+								case '`':
+									replaceWithTmp.replace(ipos-1,2,prevsubstring);
+									break;
+								case '\'':
+									replaceWithTmp.replace(ipos-1,2,ret->data.substr_bytes(ovector[1],ret->data.numBytes()-ovector[1]));
+									break;
+							}
 							continue;
+						}
 						group = (i >= rc) ? "" : ret->data.substr_bytes(ovector[i*2], ovector[i*2+1]-ovector[i*2]);
-						replaceWith.replace(pos, ipos-pos, group);
+						replaceWithTmp.replace(pos, ipos-pos, group);
 					}
 			}
-			ret->data.replace_bytes(ovector[0],ovector[1]-ovector[0],replaceWith);
-			offset=ovector[0]+replaceWith.numBytes();
-			retDiff+=replaceWith.numBytes()-(ovector[1]-ovector[0]);
+			prevsubstring += ret->data.substr_bytes(ovector[0],ovector[1]-ovector[0]);
+			ret->data.replace_bytes(ovector[0],ovector[1]-ovector[0],replaceWithTmp);
+			offset=ovector[0]+replaceWithTmp.numBytes();
+			retDiff+=replaceWithTmp.numBytes()-(ovector[1]-ovector[0]);
 		}
 		while(re->global);
 
@@ -755,7 +800,6 @@ ASFUNCTIONBODY(ASString,replace)
 		int index=ret->data.find(s,0);
 		if(index==-1) //No result
 			return ret;
-		assert_and_throw(type==STRING);
 		ret->data.replace(index,s.numChars(),replaceWith);
 	}
 
