@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -32,16 +32,15 @@ namespace lightspark
  */
 class ASString: public ASObject
 {
-CLASSBUILDABLE(ASString);
 private:
 	tiny_string toString_priv() const;
-	ASString();
-	ASString(const std::string& s);
-	ASString(const tiny_string& s);
-	ASString(const Glib::ustring& s);
-	ASString(const char* s);
-	ASString(const char* s, uint32_t len);
 public:
+	ASString(Class_base* c);
+	ASString(Class_base* c, const std::string& s);
+	ASString(Class_base* c, const tiny_string& s);
+	ASString(Class_base* c, const Glib::ustring& s);
+	ASString(Class_base* c, const char* s);
+	ASString(Class_base* c, const char* s, uint32_t len);
 	tiny_string data;
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
@@ -71,7 +70,8 @@ public:
 	ASFUNCTION(generator);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 	std::string toDebugString() { return std::string("\"") + std::string(data) + "\""; }
 };
 
@@ -83,7 +83,7 @@ inline ASObject* Class<ASString>::coerce(ASObject* o) const
 	if(o->is<Undefined>())
 	{
 		o->decRef();
-		return new Null;
+		return getSys()->getNullRef();
 	}
 	tiny_string n = o->toString();
 	o->decRef();
