@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,9 @@
 #include "abc.h"
 #include "argconv.h"
 #include "parsing/amf3_generator.h"
+#include "Vector.h"
+#include "RegExp.h"
+#include "flash/utils/flashutils.h"
 
 using namespace std;
 using namespace lightspark;
@@ -28,7 +31,8 @@ using namespace lightspark;
 SET_NAMESPACE("");
 REGISTER_CLASS_NAME(Array);
 
-Array::Array()
+Array::Array(Class_base* c):ASObject(c),
+	data(std::less<arrayType::key_type>(), reporter_allocator<arrayType::value_type>(c->memoryAccount))
 {
 	currentsize=0;
 	type=T_ARRAY;
@@ -40,40 +44,51 @@ void Array::sinit(Class_base* c)
 	// public constants
 	c->setSuper(Class<ASObject>::getRef());
 
-	c->setVariableByQName("CASEINSENSITIVE","",abstract_d(CASEINSENSITIVE),DECLARED_TRAIT);
-	c->setVariableByQName("DESCENDING","",abstract_d(DESCENDING),DECLARED_TRAIT);
-	c->setVariableByQName("NUMERIC","",abstract_d(NUMERIC),DECLARED_TRAIT);
-	c->setVariableByQName("RETURNINDEXEDARRAY","",abstract_d(RETURNINDEXEDARRAY),DECLARED_TRAIT);
-	c->setVariableByQName("UNIQUESORT","",abstract_d(UNIQUESORT),DECLARED_TRAIT);
+	c->setVariableByQName("CASEINSENSITIVE","",abstract_d(CASEINSENSITIVE),CONSTANT_TRAIT);
+	c->setVariableByQName("DESCENDING","",abstract_d(DESCENDING),CONSTANT_TRAIT);
+	c->setVariableByQName("NUMERIC","",abstract_d(NUMERIC),CONSTANT_TRAIT);
+	c->setVariableByQName("RETURNINDEXEDARRAY","",abstract_d(RETURNINDEXEDARRAY),CONSTANT_TRAIT);
+	c->setVariableByQName("UNIQUESORT","",abstract_d(UNIQUESORT),CONSTANT_TRAIT);
 
 	// properties
 	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(_getLength),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(_setLength),SETTER_METHOD,true);
 
 	// public functions
-	c->setDeclaredMethodByQName("concat",AS3,Class<IFunction>::getFunction(_concat),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("concat",AS3,Class<IFunction>::getFunction(_concat,1),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("every",AS3,Class<IFunction>::getFunction(every),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("filter",AS3,Class<IFunction>::getFunction(filter),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("forEach",AS3,Class<IFunction>::getFunction(forEach),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("indexOf",AS3,Class<IFunction>::getFunction(indexOf),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("lastIndexOf",AS3,Class<IFunction>::getFunction(lastIndexOf),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("join",AS3,Class<IFunction>::getFunction(join),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("map",AS3,Class<IFunction>::getFunction(_map),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("join",AS3,Class<IFunction>::getFunction(join,1),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("map",AS3,Class<IFunction>::getFunction(_map,1),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("pop",AS3,Class<IFunction>::getFunction(_pop),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("push",AS3,Class<IFunction>::getFunction(_push),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("push",AS3,Class<IFunction>::getFunction(_push_as3),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("reverse",AS3,Class<IFunction>::getFunction(_reverse),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("shift",AS3,Class<IFunction>::getFunction(shift),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("slice",AS3,Class<IFunction>::getFunction(slice),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("slice",AS3,Class<IFunction>::getFunction(slice,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("some",AS3,Class<IFunction>::getFunction(some),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("sort",AS3,Class<IFunction>::getFunction(_sort),NORMAL_METHOD,true);
 	//c->setDeclaredMethodByQName("sortOn",AS3,Class<IFunction>::getFunction(sortOn),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("splice",AS3,Class<IFunction>::getFunction(splice),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("splice",AS3,Class<IFunction>::getFunction(splice,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toLocaleString",AS3,Class<IFunction>::getFunction(_toString),NORMAL_METHOD,true);
 	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(_toString),DYNAMIC_TRAIT);
 	c->setDeclaredMethodByQName("unshift",AS3,Class<IFunction>::getFunction(unshift),NORMAL_METHOD,true);
 
 	// workaround, pop was encountered not in the AS3 namespace before, need to investigate it further
 	c->setDeclaredMethodByQName("pop","",Class<IFunction>::getFunction(_pop),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("pop","",Class<IFunction>::getFunction(_pop),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("push","",Class<IFunction>::getFunction(_push),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("shift","",Class<IFunction>::getFunction(shift),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("unshift","",Class<IFunction>::getFunction(unshift),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("concat","",Class<IFunction>::getFunction(_concat,1),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("slice","",Class<IFunction>::getFunction(slice,2),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("splice","",Class<IFunction>::getFunction(splice,2),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("reverse","",Class<IFunction>::getFunction(_reverse),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("join","",Class<IFunction>::getFunction(join,1),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("map",AS3,Class<IFunction>::getFunction(_map,1),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("sort","",Class<IFunction>::getFunction(_sort),DYNAMIC_TRAIT);
 }
 
 void Array::buildTraits(ASObject* o)
@@ -98,8 +113,8 @@ ASFUNCTIONBODY(Array,_constructor)
 		th->resize(argslen);
 		for(unsigned int i=0;i<argslen;i++)
 		{
-			th->set(i,args[i]);
 			args[i]->incRef();
+			th->set(i,_MR(args[i]));
 		}
 	}
 	return NULL;
@@ -122,8 +137,8 @@ ASFUNCTIONBODY(Array,generator)
 		th->resize(argslen);
 		for(unsigned int i=0;i<argslen;i++)
 		{
-			th->set(i,args[i]);
 			args[i]->incRef();
+			th->set(i,_MR(args[i]));
 		}
 	}
 	return th;
@@ -140,6 +155,8 @@ ASFUNCTIONBODY(Array,_concat)
 	for(;it != th->data.end();++it)
 	{
 		ret->data[it->first]=it->second;
+		if(ret->data[it->first].type==DATA_OBJECT && ret->data[it->first].data)
+			ret->data[it->first].data->incRef();
 	}
 	
 	if(argslen==1 && args[0]->getObjectType()==T_ARRAY)
@@ -148,7 +165,10 @@ ASFUNCTIONBODY(Array,_concat)
 		std::map<uint32_t, data_slot>::iterator ittmp=tmp->data.begin();
 		for(;ittmp != tmp->data.end();++ittmp)
 		{
-			ret->data[ret->size()+ittmp->first]=ittmp->second;
+			uint32_t newIndex=ret->size()+ittmp->first;
+			ret->data[newIndex]=ittmp->second;
+			if(ret->data[newIndex].type==DATA_OBJECT && ret->data[newIndex].data)
+				ret->data[newIndex].data->incRef();
 		}
 		ret->resize(th->size()+tmp->size());
 	}
@@ -156,15 +176,10 @@ ASFUNCTIONBODY(Array,_concat)
 	{
 		//Insert the arguments in the array
 		for(unsigned int i=0;i<argslen;i++)
-			ret->push(args[i]);
-	}
-
-	//All the elements in the new array should be increffed, as args will be deleted and
-	//this array could die too
-	for(unsigned int i=0;i<ret->size();i++)
-	{
-		if(ret->data.count(i) && ret->data[i].type==DATA_OBJECT && ret->data[i].data)
-			ret->data[i].data->incRef();
+		{
+			args[i]->incRef();
+			ret->push(_MR(args[i]));
+		}
 	}
 
 	return ret;
@@ -179,20 +194,19 @@ ASFUNCTIONBODY(Array,filter)
 	Array* ret=Class<Array>::getInstanceS();
 	ASObject *funcRet;
 
-	for(unsigned int i=0;i<th->size();i++)
+	std::map<uint32_t, data_slot>::iterator it=th->data.begin();
+	for(;it != th->data.end();++it)
 	{
-		if (!th->data.count(i))
-			continue;
-		assert_and_throw(th->data[i].type==DATA_OBJECT);
-		params[0] = th->data[i].data;
-		th->data[i].data->incRef();
-		params[1] = abstract_i(i);
+		assert_and_throw(it->second.type==DATA_OBJECT);
+		params[0] = it->second.data;
+		it->second.data->incRef();
+		params[1] = abstract_i(it->first);
 		params[2] = th;
 		th->incRef();
 
 		if(argslen==1)
 		{
-			funcRet=f->call(new Null, params, 3);
+			funcRet=f->call(getSys()->getNullRef(), params, 3);
 		}
 		else
 		{
@@ -203,8 +217,8 @@ ASFUNCTIONBODY(Array,filter)
 		{
 			if(Boolean_concrete(funcRet))
 			{
-				th->data[i].data->incRef();
-				ret->push(th->data[i].data);
+				it->second.data->incRef();
+				ret->push(_MR(it->second.data));
 			}
 			funcRet->decRef();
 		}
@@ -220,20 +234,19 @@ ASFUNCTIONBODY(Array, some)
 	ASObject* params[3];
 	ASObject *funcRet;
 
-	for(unsigned int i=0; i < th->size(); i++)
+	std::map<uint32_t, data_slot>::iterator it=th->data.begin();
+	for(;it != th->data.end();++it)
 	{
-		if (!th->data.count(i))
-			continue;
-		assert_and_throw(th->data[i].type==DATA_OBJECT);
-		params[0] = th->data[i].data;
-		th->data[i].data->incRef();
-		params[1] = abstract_i(i);
+		assert_and_throw(it->second.type==DATA_OBJECT);
+		params[0] = it->second.data;
+		it->second.data->incRef();
+		params[1] = abstract_i(it->first);
 		params[2] = th;
 		th->incRef();
 
 		if(argslen==1)
 		{
-			funcRet=f->call(new Null, params, 3);
+			funcRet=f->call(getSys()->getNullRef(), params, 3);
 		}
 		else
 		{
@@ -260,20 +273,19 @@ ASFUNCTIONBODY(Array, every)
 	ASObject* params[3];
 	ASObject *funcRet;
 
-	for(unsigned int i=0; i < th->size(); i++)
+	std::map<uint32_t, data_slot>::iterator it=th->data.begin();
+	for(;it != th->data.end();++it)
 	{
-		if (!th->data.count(i))
-			continue;
-		assert_and_throw(th->data[i].type==DATA_OBJECT);
-		params[0] = th->data[i].data;
-		th->data[i].data->incRef();
-		params[1] = abstract_i(i);
+		assert_and_throw(it->second.type==DATA_OBJECT);
+		params[0] = it->second.data;
+		it->second.data->incRef();
+		params[1] = abstract_i(it->first);
 		params[2] = th;
 		th->incRef();
 
 		if(argslen==1)
 		{
-			funcRet=f->call(new Null, params, 3);
+			funcRet=f->call(getSys()->getNullRef(), params, 3);
 		}
 		else
 		{
@@ -317,21 +329,20 @@ ASFUNCTIONBODY(Array,forEach)
 	IFunction* f = static_cast<IFunction*>(args[0]);
 	ASObject* params[3];
 
-	for(unsigned int i=0; i < th->size(); i++)
+	std::map<uint32_t, data_slot>::iterator it=th->data.begin();
+	for(;it != th->data.end();++it)
 	{
-		if (!th->data.count(i))
-			continue;
-		assert_and_throw(th->data[i].type==DATA_OBJECT);
-		params[0] = th->data[i].data;
-		th->data[i].data->incRef();
-		params[1] = abstract_i(i);
+		assert_and_throw(it->second.type==DATA_OBJECT);
+		params[0] = it->second.data;
+		it->second.data->incRef();
+		params[1] = abstract_i(it->first);
 		params[2] = th;
 		th->incRef();
 
 		ASObject *funcret;
 		if( argslen == 1 )
 		{
-			funcret=f->call(new Null, params, 3);
+			funcret=f->call(getSys()->getNullRef(), params, 3);
 		}
 		else
 		{
@@ -349,7 +360,7 @@ ASFUNCTIONBODY(Array, _reverse)
 {
 	Array* th = static_cast<Array*>(obj);
 
-	std::map<uint32_t, data_slot> tmp = std::map<uint32_t, data_slot>(th->data);
+	std::map<uint32_t, data_slot> tmp = std::map<uint32_t, data_slot>(th->data.begin(),th->data.end());
 	uint32_t size = th->size();
 	th->data.clear();
 	std::map<uint32_t, data_slot>::iterator it=tmp.begin();
@@ -414,32 +425,50 @@ ASFUNCTIONBODY(Array,lastIndexOf)
 
 ASFUNCTIONBODY(Array,shift)
 {
+	if (!obj->is<Array>())
+	{
+		// this seems to be how Flash handles the generic shift calls
+		if (obj->is<Vector>())
+			return Vector::shift(obj,args,argslen);
+		if (obj->is<ByteArray>())
+			return ByteArray::shift(obj,args,argslen);
+		// for other objects we just decrease the length property
+		multiname lengthName(NULL);
+		lengthName.name_type=multiname::NAME_STRING;
+		lengthName.name_s_id=getSys()->getUniqueStringId("length");
+		lengthName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		lengthName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
+		lengthName.isAttribute = true;
+		_NR<ASObject> o=obj->getVariableByMultiname(lengthName,SKIP_IMPL);
+		uint32_t res = o->toUInt();
+		if (res > 0)
+			obj->setVariableByMultiname(lengthName,abstract_ui(res-1),CONST_NOT_ALLOWED);
+		return getSys()->getUndefinedRef();
+	}
 	Array* th=static_cast<Array*>(obj);
 	if(!th->size())
-		return new Undefined;
+		return getSys()->getUndefinedRef();
 	ASObject* ret;
 	if(!th->data.count(0))
-		ret = new Undefined;
+		ret = getSys()->getUndefinedRef();
 	else
 	{
 		if(th->data[0].type==DATA_OBJECT)
 			ret=th->data[0].data;
 		else
 			ret = abstract_i(th->data[0].data_i);
-		th->data.erase(0);
 	}
-	for(uint32_t i= 1;i< th->size();i++)
+	std::map<uint32_t,data_slot> tmp;
+	std::map<uint32_t,data_slot>::iterator it;
+	for ( it=th->data.begin(); it != th->data.end(); it++ )
 	{
-		if (th->data.count(i))
+		if(it->first)
 		{
-			th->data[i-1]=th->data[i];
-		}
-		else if (th->data.count(i-1))
-		{
-			th->data.erase(i-1);
+			tmp[it->first-1]=it->second;
 		}
 	}
-	th->data.erase(th->size()-1);// erase here to avoid decref
+	th->data.clear();
+	th->data.insert(tmp.begin(),tmp.end());
 	th->resize(th->size()-1);
 	return ret;
 }
@@ -483,8 +512,9 @@ ASFUNCTIONBODY(Array,slice)
 	{
 		if (th->data.count(i))
 		{
-			th->data[i].data->incRef();
-			ret->data[j] =th->data[i];
+			if(th->data[i].type == DATA_OBJECT)
+				th->data[i].data->incRef();
+			ret->data[j]=th->data[i];
 		}
 		j++;
 	}
@@ -546,15 +576,15 @@ ASFUNCTIONBODY(Array,splice)
 	for(unsigned int i=2;i<argslen;i++)
 	{
 		args[i]->incRef();
-		th->push(args[i]);
+		th->push(_MR(args[i]));
 	}
 	// move remembered items to new position
 	for(int i=0;i<totalSize- (startIndex+deleteCount);i++)
 	{
 		if (tmp[i].type != DATA_OBJECT || tmp[i].data != NULL)
-			th->data[startIndex+i] = tmp[i];
+			th->data[startIndex+i+(argslen > 2 ? argslen-2 : 0)] = tmp[i];
 	}
-	th->resize((totalSize-deleteCount)+(argslen-2));
+	th->resize((totalSize-deleteCount)+(argslen > 2 ? argslen-2 : 0));
 	return ret;
 }
 
@@ -568,7 +598,9 @@ ASFUNCTIONBODY(Array,join)
 	string ret;
 	for(uint32_t i=0;i<th->size();i++)
 	{
-		ret+=th->at(i)->toString().raw_buf();
+		_R<ASObject> o = th->at(i);
+		if (!o->is<Undefined>() && !o->is<Null>())
+			ret+= o->toString().raw_buf();
 		if(i!=th->size()-1)
 			ret+=del.raw_buf();
 	}
@@ -578,27 +610,26 @@ ASFUNCTIONBODY(Array,join)
 ASFUNCTIONBODY(Array,indexOf)
 {
 	Array* th=static_cast<Array*>(obj);
-	assert_and_throw(argslen==1 || argslen==2);
 	int ret=-1;
-	ASObject* arg0=args[0];
+	int32_t index=0;
+	ASObject* arg0 = args[0];
+	if (argslen > 1) 
+		index = args[1]->toInt();
 
-	int unsigned i = 0;
-	if(argslen == 2)
-	{
-		i = args[1]->toInt();
-	}
 
 	DATA_TYPE dtype;
-	for(;i<th->size();i++)
+	std::map<uint32_t,data_slot>::iterator it;
+	for ( it=th->data.begin() ; it != th->data.end(); it++ )
 	{
-		if (!th->data.count(i))
+		if (it->first < (uint32_t)index)
 			continue;
-		dtype = th->data[i].type;
+		data_slot sl = it->second;
+		dtype = sl.type;
 		assert_and_throw(dtype==DATA_OBJECT || dtype==DATA_INT);
-		if((dtype == DATA_OBJECT && ABCVm::strictEqualImpl(th->data[i].data,arg0)) ||
-			(dtype == DATA_INT && arg0->toInt() == th->data[i].data_i))
+		if((dtype == DATA_OBJECT && ABCVm::strictEqualImpl(sl.data,arg0)) ||
+			(dtype == DATA_INT && arg0->toInt() == sl.data_i))
 		{
-			ret=i;
+			ret=it->first;
 			break;
 		}
 	}
@@ -608,11 +639,32 @@ ASFUNCTIONBODY(Array,indexOf)
 
 ASFUNCTIONBODY(Array,_pop)
 {
+	if (!obj->is<Array>())
+	{
+		// this seems to be how Flash handles the generic pop calls
+		if (obj->is<Vector>())
+			return Vector::_pop(obj,args,argslen);
+		if (obj->is<ByteArray>())
+			return ByteArray::pop(obj,args,argslen);
+		// for other objects we just decrease the length property
+		multiname lengthName(NULL);
+		lengthName.name_type=multiname::NAME_STRING;
+		lengthName.name_s_id=getSys()->getUniqueStringId("length");
+		lengthName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		lengthName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
+		lengthName.isAttribute = true;
+		_NR<ASObject> o=obj->getVariableByMultiname(lengthName,SKIP_IMPL);
+		uint32_t res = o->toUInt();
+		if (res > 0)
+			obj->setVariableByMultiname(lengthName,abstract_ui(res-1),CONST_NOT_ALLOWED);
+		return getSys()->getUndefinedRef();
+	}
 	Array* th=static_cast<Array*>(obj);
 	uint32_t size =th->size();
 	if (size == 0)
-		return new Undefined;
+		return getSys()->getUndefinedRef();
 	ASObject* ret;
+	
 	if (th->data.count(size-1))
 	{
 		if(th->data[size-1].type==DATA_OBJECT)
@@ -622,7 +674,7 @@ ASFUNCTIONBODY(Array,_pop)
 		th->data.erase(size-1);
 	}
 	else
-		ret = new Undefined;
+		ret = getSys()->getUndefinedRef();
 
 	th->currentsize--;
 	return ret;
@@ -646,7 +698,10 @@ bool Array::sortComparatorDefault::operator()(const data_slot& d1, const data_sl
 
 		if(std::isnan(a) || std::isnan(b))
 			throw RunTimeException("Cannot sort non number with Array.NUMERIC option");
-		return a<b;
+		if(isDescending)
+			return b>a;
+		else
+			return a<b;
 	}
 	else
 	{
@@ -666,11 +721,22 @@ bool Array::sortComparatorDefault::operator()(const data_slot& d1, const data_sl
 		else
 			s2="undefined";
 
-		//TODO: unicode support
-		if(isCaseInsensitive)
-			return s1.strcasecmp(s2)<0;
+		if(isDescending)
+		{
+			//TODO: unicode support
+			if(isCaseInsensitive)
+				return s1.strcasecmp(s2)>0;
+			else
+				return s1>s2;
+		}
 		else
-			return s1<s2;
+		{
+			//TODO: unicode support
+			if(isCaseInsensitive)
+				return s1.strcasecmp(s2)<0;
+			else
+				return s1<s2;
+		}
 	}
 }
 
@@ -685,7 +751,7 @@ bool Array::sortComparatorWrapper::operator()(const data_slot& d1, const data_sl
 		objs[0]->incRef();
 	}
 	else
-		objs[0]=new Undefined;
+		objs[0]=getSys()->getUndefinedRef();
 
 	if(d2.type==DATA_INT)
 		objs[1]=abstract_i(d2.data_i);
@@ -695,10 +761,10 @@ bool Array::sortComparatorWrapper::operator()(const data_slot& d1, const data_sl
 		objs[1]->incRef();
 	}
 	else
-		objs[1]=new Undefined;
+		objs[1]=getSys()->getUndefinedRef();
 
 	assert(comparator);
-	_NR<ASObject> ret=_MNR(comparator->call(new Null, objs, 2));
+	_NR<ASObject> ret=_MNR(comparator->call(getSys()->getNullRef(), objs, 2));
 	assert_and_throw(ret);
 	return (ret->toInt()<0); //Less
 }
@@ -709,6 +775,7 @@ ASFUNCTIONBODY(Array,_sort)
 	IFunction* comp=NULL;
 	bool isNumeric=false;
 	bool isCaseInsensitive=false;
+	bool isDescending=false;
 	for(uint32_t i=0;i<argslen;i++)
 	{
 		if(args[i]->getObjectType()==T_FUNCTION) //Comparison func
@@ -723,7 +790,9 @@ ASFUNCTIONBODY(Array,_sort)
 				isNumeric=true;
 			if(options&CASEINSENSITIVE)
 				isCaseInsensitive=true;
-			if(options&(~(NUMERIC|CASEINSENSITIVE)))
+			if(options&DESCENDING)
+				isDescending=true;
+			if(options&(~(NUMERIC|CASEINSENSITIVE|DESCENDING)))
 				throw UnsupportedException("Array::sort not completely implemented");
 		}
 	}
@@ -738,7 +807,7 @@ ASFUNCTIONBODY(Array,_sort)
 	if(comp)
 		sort(tmp.begin(),tmp.end(),sortComparatorWrapper(comp));
 	else
-		sort(tmp.begin(),tmp.end(),sortComparatorDefault(isNumeric,isCaseInsensitive));
+		sort(tmp.begin(),tmp.end(),sortComparatorDefault(isNumeric,isCaseInsensitive,isDescending));
 
 	th->data.clear();
 	std::vector<data_slot>::iterator ittmp=tmp.begin();
@@ -763,36 +832,105 @@ ASFUNCTIONBODY(Array,sortOn)
 
 ASFUNCTIONBODY(Array,unshift)
 {
-	Array* th=static_cast<Array*>(obj);
-	th->resize(th->size()+argslen);
-	for(uint32_t i=th->size();i> 0;i--)
+	if (!obj->is<Array>())
 	{
-		if (th->data.count(i-1))
+		// this seems to be how Flash handles the generic unshift calls
+		if (obj->is<Vector>())
+			return Vector::unshift(obj,args,argslen);
+		if (obj->is<ByteArray>())
+			return ByteArray::unshift(obj,args,argslen);
+		// for other objects we just increase the length property
+		multiname lengthName(NULL);
+		lengthName.name_type=multiname::NAME_STRING;
+		lengthName.name_s_id=getSys()->getUniqueStringId("length");
+		lengthName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		lengthName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
+		lengthName.isAttribute = true;
+		_NR<ASObject> o=obj->getVariableByMultiname(lengthName,SKIP_IMPL);
+		uint32_t res = o->toUInt();
+		obj->setVariableByMultiname(lengthName,abstract_ui(res+argslen),CONST_NOT_ALLOWED);
+		return getSys()->getUndefinedRef();
+	}
+	Array* th=static_cast<Array*>(obj);
+	if (argslen > 0)
+	{
+		th->resize(th->size()+argslen);
+		std::map<uint32_t,data_slot> tmp;
+		std::map<uint32_t,data_slot>::reverse_iterator it;
+		for ( it=th->data.rbegin(); it != th->data.rend(); it++ )
 		{
-			th->data[(i-1)+argslen]=th->data[i-1];
-			th->data.erase(i-1);
+			tmp[it->first+argslen]=it->second;
 		}
 		
-	}
-
-	for(uint32_t i=0;i<argslen;i++)
-	{
-		th->data[i] = data_slot(args[i],DATA_OBJECT);
-		args[i]->incRef();
+		for(uint32_t i=0;i<argslen;i++)
+		{
+			tmp[i] = data_slot(args[i],DATA_OBJECT);
+			args[i]->incRef();
+		}
+		th->data.clear();
+		th->data.insert(tmp.begin(),tmp.end());
 	}
 	return abstract_i(th->size());
 }
 
 ASFUNCTIONBODY(Array,_push)
 {
+	if (!obj->is<Array>())
+	{
+		// this seems to be how Flash handles the generic push calls
+		if (obj->is<Vector>())
+			return Vector::push(obj,args,argslen);
+		if (obj->is<ByteArray>())
+			return ByteArray::push(obj,args,argslen);
+		// for other objects we just increase the length property
+		multiname lengthName(NULL);
+		lengthName.name_type=multiname::NAME_STRING;
+		lengthName.name_s_id=getSys()->getUniqueStringId("length");
+		lengthName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		lengthName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
+		lengthName.isAttribute = true;
+		_NR<ASObject> o=obj->getVariableByMultiname(lengthName,SKIP_IMPL);
+		uint32_t res = o->toUInt();
+		obj->setVariableByMultiname(lengthName,abstract_ui(res+argslen),CONST_NOT_ALLOWED);
+		return getSys()->getUndefinedRef();
+	}
 	Array* th=static_cast<Array*>(obj);
 	for(unsigned int i=0;i<argslen;i++)
 	{
-		if (th->size() > UINT32_MAX)
-			throw Class<RangeError>::getInstanceS("");
-			
-		th->push(args[i]);
 		args[i]->incRef();
+		th->push(_MR(args[i]));
+	}
+	return abstract_i(th->size());
+}
+// AS3 handles push on uint.MAX_VALUE differently than ECMA, so we need to push methods
+ASFUNCTIONBODY(Array,_push_as3)
+{
+	if (!obj->is<Array>())
+	{
+		// this seems to be how Flash handles the generic push calls
+		if (obj->is<Vector>())
+			return Vector::push(obj,args,argslen);
+		if (obj->is<ByteArray>())
+			return ByteArray::push(obj,args,argslen);
+		// for other objects we just increase the length property
+		multiname lengthName(NULL);
+		lengthName.name_type=multiname::NAME_STRING;
+		lengthName.name_s_id=getSys()->getUniqueStringId("length");
+		lengthName.ns.push_back(nsNameAndKind("",NAMESPACE));
+		lengthName.ns.push_back(nsNameAndKind(AS3,NAMESPACE));
+		lengthName.isAttribute = true;
+		_NR<ASObject> o=obj->getVariableByMultiname(lengthName,SKIP_IMPL);
+		uint32_t res = o->toUInt();
+		obj->setVariableByMultiname(lengthName,abstract_ui(res+argslen),CONST_NOT_ALLOWED);
+		return getSys()->getUndefinedRef();
+	}
+	Array* th=static_cast<Array*>(obj);
+	for(unsigned int i=0;i<argslen;i++)
+	{
+		if (th->size() >= UINT32_MAX)
+			break;
+		args[i]->incRef();
+		th->push(_MR(args[i]));
 	}
 	return abstract_i(th->size());
 }
@@ -800,16 +938,22 @@ ASFUNCTIONBODY(Array,_push)
 ASFUNCTIONBODY(Array,_map)
 {
 	Array* th=static_cast<Array*>(obj);
-	assert_and_throw(argslen==1 && args[0]->getObjectType()==T_FUNCTION);
-	IFunction* func=static_cast<IFunction*>(args[0]);
+
+	if(argslen < 1)
+		throw Class<ArgumentError>::getInstanceS("Error #1063: Non-optional argument missing");
+	IFunction* func= NULL;
+	if (!args[0]->is<RegExp>())
+	{
+		assert_and_throw(args[0]->getObjectType()==T_FUNCTION);
+		func=static_cast<IFunction*>(args[0]);
+	}
 	Array* arrayRet=Class<Array>::getInstanceS();
 
-	for(uint32_t i=0;i<th->size();i++)
+	uint32_t s = th->size();
+	for (uint32_t i=0; i < s; i++ )
 	{
 		ASObject* funcArgs[3];
-		if (!th->data.count(i))
-			funcArgs[0]=new Null;
-		else
+		if (th->data.count(i))
 		{
 			const data_slot& slot=th->data[i];
 			if(slot.type==DATA_INT)
@@ -820,14 +964,24 @@ ASFUNCTIONBODY(Array,_map)
 				funcArgs[0]->incRef();
 			}
 			else
-				funcArgs[0]=new Undefined;
+				funcArgs[0]=getSys()->getUndefinedRef();
 		}
+		else
+			funcArgs[0]=getSys()->getUndefinedRef();
 		funcArgs[1]=abstract_i(i);
 		funcArgs[2]=th;
 		funcArgs[2]->incRef();
-		ASObject* funcRet=func->call(new Null, funcArgs, 3);
+		ASObject* funcRet= NULL;
+		if (func)
+		{
+			if (argslen > 1)
+				args[1]->incRef();
+			funcRet = func->call(argslen > 1? args[1] : getSys()->getNullRef(), funcArgs, 3);
+		}
+		else
+			funcRet = RegExp::exec(args[0],funcArgs,1);
 		assert_and_throw(funcRet);
-		arrayRet->push(funcRet);
+		arrayRet->push(_MR(funcRet));
 	}
 
 	return arrayRet;
@@ -835,14 +989,22 @@ ASFUNCTIONBODY(Array,_map)
 
 ASFUNCTIONBODY(Array,_toString)
 {
-	Array* th=static_cast<Array*>(obj);
+	if(Class<Number>::getClass()->prototype == obj)
+		return Class<ASString>::getInstanceS("");
+	if(!obj->is<Array>())
+	{
+		LOG(LOG_NOT_IMPLEMENTED, "generic Array::toString");
+		return Class<ASString>::getInstanceS("");
+	}
+	
+	Array* th=obj->as<Array>();
 	return Class<ASString>::getInstanceS(th->toString_priv());
 }
 
 int32_t Array::getVariableByMultiname_i(const multiname& name)
 {
 	assert_and_throw(implEnable);
-	unsigned int index=0;
+	uint32_t index=0;
 	if(!isValidMultiname(name,index))
 		return ASObject::getVariableByMultiname_i(name);
 
@@ -850,26 +1012,27 @@ int32_t Array::getVariableByMultiname_i(const multiname& name)
 	{
 		if (!data.count(index))
 			return 0;
-		switch(data[index].type)
+		data_slot sl = data[index];
+		switch(sl.type)
 		{
 			case DATA_OBJECT:
 			{
-				assert(data[index].data!=NULL);
-				if(data[index].data->getObjectType()==T_INTEGER)
+				assert(sl.data!=NULL);
+				if(sl.data->getObjectType()==T_INTEGER)
 				{
-					Integer* i=static_cast<Integer*>(data[index].data);
+					Integer* i=static_cast<Integer*>(sl.data);
 					return i->toInt();
 				}
-				else if(data[index].data->getObjectType()==T_NUMBER)
+				else if(sl.data->getObjectType()==T_NUMBER)
 				{
-					Number* i=static_cast<Number*>(data[index].data);
+					Number* i=static_cast<Number*>(sl.data);
 					return i->toInt();
 				}
 				else
 					throw UnsupportedException("Array::getVariableByMultiname_i not completely implemented");
 			}
 			case DATA_INT:
-				return data[index].data_i;
+				return sl.data_i;
 		}
 	}
 
@@ -882,31 +1045,35 @@ _NR<ASObject> Array::getVariableByMultiname(const multiname& name, GET_VARIABLE_
 		return ASObject::getVariableByMultiname(name,opt);
 
 	assert_and_throw(name.ns.size()>0);
-	if(name.ns[0].name!="")
+	if(!name.ns[0].hasEmptyName())
 		return ASObject::getVariableByMultiname(name,opt);
 
-	unsigned int index=0;
+	uint32_t index=0;
 	if(!isValidMultiname(name,index))
 		return ASObject::getVariableByMultiname(name,opt);
 	if(index<size())
 	{
 		ASObject* ret=NULL;
 		if (!data.count(index))
-			ret = new Undefined;
-		switch(data[index].type)
+			ret = getSys()->getUndefinedRef();
+		else
 		{
-			case DATA_OBJECT:
-				ret=data[index].data;
-				if(ret==NULL)
-				{
-					ret=new Undefined;
-					data[index].data=ret;
-				}
-				ret->incRef();
-				break;
-			case DATA_INT:
-				ret=abstract_i(data[index].data_i);
-				break;
+			data_slot sl = data[index];
+			switch(sl.type)
+			{
+				case DATA_OBJECT:
+					ret=sl.data;
+					if(ret==NULL)
+					{
+						ret=getSys()->getUndefinedRef();
+						sl.data=ret;
+					}
+					ret->incRef();
+					break;
+				case DATA_INT:
+					ret=abstract_i(sl.data_i);
+					break;
+			}
 		}
 		return _MNR(ret);
 	}
@@ -952,51 +1119,18 @@ bool Array::isValidMultiname(const multiname& name, uint32_t& index)
 	//First of all the multiname has to contain the null namespace
 	//As the namespace vector is sorted, we check only the first one
 	assert_and_throw(name.ns.size()!=0);
-	if(name.ns[0].name!="")
+	if(!name.ns[0].hasEmptyName())
 		return false;
 
-	index=0;
-	switch(name.name_type)
-	{
-		//We try to convert this to an index, otherwise bail out
-		case multiname::NAME_STRING:
-			if(name.name_s.empty())
-				return false;
-			for(auto i=name.name_s.begin(); i!=name.name_s.end(); ++i)
-			{
-				if(!i.isdigit())
-					return false;
-
-				index*=10;
-				index+=i.digit_value();
-			}
-			break;
-		//This is already an int, so its good enough
-		case multiname::NAME_INT:
-			if(name.name_i < 0)
-				return false;
-			index=name.name_i;
-			break;
-		case multiname::NAME_NUMBER:
-			if(!Number::isInteger(name.name_d))
-				return false;
-			index = name.name_d;
-			break;
-		case multiname::NAME_OBJECT:
-			//TODO: should be use toPrimitive here?
-			return false;
-		default:
-			throw UnsupportedException("Array::isValidMultiname not completely implemented");
-	}
-	return true;
+	return name.toUInt(index);
 }
 
-void Array::setVariableByMultiname(const multiname& name, ASObject* o)
+void Array::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
 {
 	assert_and_throw(implEnable);
 	uint32_t index=0;
 	if(!isValidMultiname(name,index))
-		return ASObject::setVariableByMultiname(name,o);
+		return ASObject::setVariableByMultiname(name,o,allowConst);
 
 	if(index>=size())
 		resize((uint64_t)index+1);
@@ -1070,15 +1204,16 @@ tiny_string Array::toString_priv() const
 	{
 		if (data.count(i))
 		{
-			if(data.at(i).type==DATA_OBJECT)
+			data_slot sl = data.at(i);
+			if(sl.type==DATA_OBJECT)
 			{
-				if(data.at(i).data)
-					ret+=data.at(i).data->toString().raw_buf();
+				if(sl.data && !sl.data->is<Undefined>() && !sl.data->is<Null>())
+					ret+=sl.data->toString().raw_buf();
 			}
-			else if(data.at(i).type==DATA_INT)
+			else if(sl.type==DATA_INT)
 			{
 				char buf[20];
-				snprintf(buf,20,"%i",data.at(i).data_i);
+				snprintf(buf,20,"%i",sl.data_i);
 				ret+=buf;
 			}
 			else
@@ -1097,19 +1232,20 @@ _R<ASObject> Array::nextValue(uint32_t index)
 	{
 		index--;
 		if(!data.count(index))
-			return _MR(new Undefined);
-		if(data[index].type==DATA_OBJECT)
+			return _MR(getSys()->getUndefinedRef());
+		data_slot sl = data[index];
+		if(sl.type==DATA_OBJECT)
 		{
-			if(data[index].data==NULL)
-				return _MR(new Undefined);
+			if(sl.data==NULL)
+				return _MR(getSys()->getUndefinedRef());
 			else
 			{
-				data[index].data->incRef();
-				return _MR(data[index].data);
+				sl.data->incRef();
+				return _MR(sl.data);
 			}
 		}
-		else if(data[index].type==DATA_INT)
-			return _MR(abstract_i(data[index].data_i));
+		else if(sl.type==DATA_INT)
+			return _MR(abstract_i(sl.data_i));
 		else
 			throw UnsupportedException("Unexpected data type");
 	}
@@ -1158,26 +1294,30 @@ _R<ASObject> Array::nextName(uint32_t index)
 	}
 }
 
-ASObject* Array::at(unsigned int index) const
+_R<ASObject> Array::at(unsigned int index) const
 {
 	if(size()<=index)
 		outofbounds();
 
 	if (!data.count(index))
-		return new Undefined;
-	switch(data.at(index).type)
+		return _MR(getSys()->getUndefinedRef());
+	data_slot sl = data.at(index);
+	switch(sl.type)
 	{
 		case DATA_OBJECT:
 		{
-			if(data.at(index).data)
-				return data.at(index).data;
+			if(sl.data)
+			{
+				sl.data->incRef();
+				return _MR(sl.data);
+			}
 		}
 		case DATA_INT:
-			return abstract_i(data.at(index).data_i);
+			return _MR(abstract_i(sl.data_i));
 	}
 
 	//We should be here only if data is an object and is NULL
-	return new Undefined;
+	return _MR(getSys()->getUndefinedRef());
 }
 
 void Array::outofbounds() const
@@ -1185,11 +1325,31 @@ void Array::outofbounds() const
 	throw ParseException("Array access out of bounds");
 }
 
+void Array::resize(uint64_t n)
+{
+	std::map<uint32_t,data_slot>::reverse_iterator it;
+	std::map<uint32_t,data_slot>::iterator itstart = n ? data.end() : data.begin();
+	for ( it=data.rbegin() ; it != data.rend(); it++ )
+	{
+		if (it->first < n)
+		{
+			itstart = it.base();
+			break;
+		}
+		if (it->second.type==DATA_OBJECT && it->second.data)
+			it->second.data->decRef();
+	}
+	if (itstart != data.end())
+		data.erase(itstart,data.end());
+	currentsize = n;
+}
+
 void Array::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap)
 {
 	assert_and_throw(objMap.find(this)==objMap.end());
-	out->writeByte(amf3::array_marker);
+	out->writeByte(array_marker);
 	//Check if the array has been already serialized
 	auto it=objMap.find(this);
 	if(it!=objMap.end())
@@ -1206,7 +1366,7 @@ void Array::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap
 		assert_and_throw(denseCount<0x20000000);
 		uint32_t value = (denseCount << 1) | 1;
 		out->writeU29(value);
-		serializeDynamicProperties(out, stringMap, objMap);
+		serializeDynamicProperties(out, stringMap, objMap, traitsMap);
 		for(uint32_t i=0;i<denseCount;i++)
 		{
 			if (!data.count(i))
@@ -1216,19 +1376,25 @@ void Array::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap
 				case DATA_INT:
 					throw UnsupportedException("int not supported in Array::serialize");
 				case DATA_OBJECT:
-					data.at(i).data->serialize(out, stringMap, objMap);
+					data.at(i).data->serialize(out, stringMap, objMap, traitsMap);
 			}
 		}
 	}
 }
 
+Array::~Array()
+{
+	Array::finalize();
+}
+
 void Array::finalize()
 {
 	ASObject::finalize();
-	for(unsigned int i=0;i<size();i++)
+	std::map<uint32_t,data_slot>::iterator it;
+	for ( it=data.begin() ; it != data.end(); it++ )
 	{
-		if(data.count(i) && data[i].type==DATA_OBJECT && data[i].data)
-			data[i].data->decRef();
+		if(it->second.type==DATA_OBJECT && it->second.data)
+			it->second.data->decRef();
 	}
 	data.clear();
 }
