@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -43,12 +43,13 @@ struct data_slot
 class Array: public ASObject
 {
 friend class ABCVm;
-CLASSBUILDABLE(Array);
 protected:
 	uint64_t currentsize;
-	std::map<uint32_t,data_slot> data;
+	typedef std::map<uint32_t,data_slot,std::less<uint32_t>,
+		reporter_allocator<std::pair<const uint32_t, data_slot>>> arrayType;
+	arrayType data;
 	void outofbounds() const;
-	Array();
+	~Array();
 private:
 	enum SORTTYPE { CASEINSENSITIVE=1, DESCENDING=2, UNIQUESORT=4, RETURNINDEXEDARRAY=8, NUMERIC=16 };
 	class sortComparatorDefault
@@ -56,8 +57,9 @@ private:
 	private:
 		bool isNumeric;
 		bool isCaseInsensitive;
+		bool isDescending;
 	public:
-		sortComparatorDefault(bool n, bool ci):isNumeric(n),isCaseInsensitive(ci){}
+		sortComparatorDefault(bool n, bool ci, bool d):isNumeric(n),isCaseInsensitive(ci),isDescending(d){}
 		bool operator()(const data_slot& d1, const data_slot& d2);
 	};
 	class sortComparatorWrapper
@@ -71,6 +73,7 @@ private:
 	tiny_string toString_priv() const;
 	int capIndex(int i) const;
 public:
+	Array(Class_base* c);
 	void finalize();
 	//These utility methods are also used by ByteArray
 	static bool isValidMultiname(const multiname& name, uint32_t& index);
@@ -82,6 +85,7 @@ public:
 	ASFUNCTION(_constructor);
 	ASFUNCTION(generator);
 	ASFUNCTION(_push);
+	ASFUNCTION(_push_as3);
 	ASFUNCTION(_concat);
 	ASFUNCTION(_pop);
 	ASFUNCTION(join);
@@ -103,14 +107,15 @@ public:
 	ASFUNCTION(every);
 	ASFUNCTION(some);
 
-	ASObject* at(unsigned int index) const;
-	void set(unsigned int index, ASObject* o)
+	_R<ASObject> at(unsigned int index) const;
+	void set(unsigned int index, _R<ASObject> o)
 	{
 		if(index<currentsize)
 		{
 			if(!data.count(index))
 				data[index]=data_slot();
-			data[index].data=o;
+			o->incRef();
+			data[index].data=o.getPtr();
 			data[index].type=DATA_OBJECT;
 		}
 		else
@@ -120,27 +125,16 @@ public:
 	{
 		return currentsize;
 	}
-	void push(ASObject* o)
+	void push(_R<ASObject> o)
 	{
-		data[currentsize] = data_slot(o,DATA_OBJECT);
+		o->incRef();
+		data[currentsize] = data_slot(o.getPtr(),DATA_OBJECT);
 		currentsize++;
 	}
-	void resize(uint64_t n)
-	{
-		for (uint32_t i = n; i < currentsize; i++)
-		{
-			if(data.count(i))
-			{
-				if (data[i].type==DATA_OBJECT && data[i].data)
-					data[i].data->decRef();
-				data.erase(i);
-			}
-		}
-		currentsize = n;
-	}
+	void resize(uint64_t n);
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
 	int32_t getVariableByMultiname_i(const multiname& name);
-	void setVariableByMultiname(const multiname& name, ASObject* o);
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
 	bool deleteVariableByMultiname(const multiname& name);
 	void setVariableByMultiname_i(const multiname& name, int32_t value);
 	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
@@ -150,7 +144,8 @@ public:
 	_R<ASObject> nextValue(uint32_t index);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 };
 
 
