@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -71,6 +71,7 @@ ASFUNCTIONBODY(Boolean,generator)
 
 void Boolean::sinit(Class_base* c)
 {
+	c->isFinal=true;
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
 	c->setSuper(Class<ASObject>::getRef());
 	c->prototype->setVariableByQName("toString",AS3,Class<IFunction>::getFunction(_toString),DYNAMIC_TRAIT);
@@ -81,7 +82,7 @@ ASFUNCTIONBODY(Boolean,_constructor)
 {
 	Boolean* th=static_cast<Boolean*>(obj);
 	_NR<ASObject> o;
-	ARG_UNPACK (o,_MNR(new Undefined));
+	ARG_UNPACK (o,_MNR(getSys()->getUndefinedRef()));
 	th->val=Boolean_concrete(o.getPtr());
 	return NULL;
 }
@@ -112,12 +113,13 @@ ASFUNCTIONBODY(Boolean,_valueOf)
 }
 
 void Boolean::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap)
 {
 	if(val)
-		out->writeByte(amf3::true_marker);
+		out->writeByte(true_marker);
 	else
-		out->writeByte(amf3::false_marker);
+		out->writeByte(false_marker);
 }
 
 bool Boolean::isEqual(ASObject* r)
