@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -34,12 +34,10 @@ bool Boolean_concrete(const ASObject* obj);
 
 class Boolean: public ASObject
 {
-CLASSBUILDABLE(Boolean);
-private:
-	Boolean() {type=T_BOOLEAN;}
-	static void buildTraits(ASObject* o){};
-	static void sinit(Class_base*);
 public:
+	Boolean(Class_base* c):ASObject(c) {type=T_BOOLEAN;}
+	static void sinit(Class_base*);
+	static void buildTraits(ASObject* o){};
 	bool val;
 	int32_t toInt()
 	{
@@ -53,7 +51,8 @@ public:
 	ASFUNCTION(generator);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 };
 
 }
