@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -41,6 +41,11 @@ REGISTER_CLASS_NAME(URIError);
 REGISTER_CLASS_NAME(VerifyError);
 REGISTER_CLASS_NAME(UninitializedError);
 
+ASError::ASError(Class_base* c, const tiny_string& error_message, int id, const tiny_string& error_name):
+	ASObject(c),message(error_message), errorID(id), name(error_name)
+{
+}
+
 ASFUNCTIONBODY(ASError,getStackTrace)
 {
 	ASError* th=static_cast<ASError*>(obj);
@@ -77,19 +82,25 @@ ASFUNCTIONBODY(ASError,_constructor)
 	}
 	return NULL;
 }
+
 ASFUNCTIONBODY(ASError,generator)
 {
 	ASError* th=Class<ASError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
+void ASError::errorGenerator(ASError* obj, ASObject* const* args, const unsigned int argslen)
+{
 	assert_and_throw(argslen <= 2);
 	if(argslen >= 1)
 	{
-		th->message = args[0]->toString();
+		obj->message = args[0]->toString();
 	}
 	if(argslen == 2)
 	{
-		th->setErrorID(static_cast<Integer*>(args[1])->toInt());
+		obj->errorID = static_cast<Integer*>(args[1])->toInt();
 	}
-	return th;
 }
 
 void ASError::sinit(Class_base* c)
@@ -122,6 +133,13 @@ ASFUNCTIONBODY(SecurityError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(SecurityError,generator)
+{
+	ASError* th=Class<SecurityError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void SecurityError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -143,6 +161,13 @@ ASFUNCTIONBODY(ArgumentError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(ArgumentError,generator)
+{
+	ASError* th=Class<ArgumentError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void ArgumentError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -164,6 +189,13 @@ ASFUNCTIONBODY(DefinitionError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(DefinitionError,generator)
+{
+	ASError* th=Class<DefinitionError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void DefinitionError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -185,6 +217,13 @@ ASFUNCTIONBODY(EvalError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(EvalError,generator)
+{
+	ASError* th=Class<EvalError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void EvalError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -206,6 +245,13 @@ ASFUNCTIONBODY(RangeError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(RangeError,generator)
+{
+	ASError* th=Class<RangeError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void RangeError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -227,6 +273,13 @@ ASFUNCTIONBODY(ReferenceError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(ReferenceError,generator)
+{
+	ASError* th=Class<ReferenceError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void ReferenceError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -248,6 +301,13 @@ ASFUNCTIONBODY(SyntaxError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(SyntaxError,generator)
+{
+	ASError* th=Class<SyntaxError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void SyntaxError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -269,6 +329,13 @@ ASFUNCTIONBODY(TypeError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(TypeError,generator)
+{
+	ASError* th=Class<TypeError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void TypeError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -290,6 +357,13 @@ ASFUNCTIONBODY(URIError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(URIError,generator)
+{
+	ASError* th=Class<URIError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void URIError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -311,6 +385,13 @@ ASFUNCTIONBODY(VerifyError,_constructor)
 	return NULL;
 }
 
+ASFUNCTIONBODY(VerifyError,generator)
+{
+	ASError* th=Class<VerifyError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void VerifyError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
@@ -331,6 +412,14 @@ ASFUNCTIONBODY(UninitializedError,_constructor)
 	}
 	return NULL;
 }
+
+ASFUNCTIONBODY(UninitializedError,generator)
+{
+	ASError* th=Class<UninitializedError>::getInstanceS();
+	errorGenerator(th, args, argslen);
+	return th;
+}
+
 void UninitializedError::sinit(Class_base* c)
 {
 	c->setConstructor(Class<IFunction>::getFunction(_constructor));
