@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -28,8 +28,12 @@ namespace lightspark
 class Math: public ASObject
 {
 public:
+	Math(Class_base* c):ASObject(c){}
 	static void sinit(Class_base* c);
 
+	ASFUNCTION(_constructor);
+	ASFUNCTION(generator);
+
 	ASFUNCTION(abs);
 	ASFUNCTION(acos);
 	ASFUNCTION(asin);
