@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2011-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -30,9 +30,21 @@ class Vector: public ASObject
 {
 	Type* vec_type;
 	bool fixed;
-	std::vector<ASObject*> vec;
+	std::vector<ASObject*, reporter_allocator<ASObject*>> vec;
+	int capIndex(int i) const;
+	class sortComparatorWrapper
+	{
+	private:
+		IFunction* comparator;
+		Type* vec_type;
+	public:
+		sortComparatorWrapper(IFunction* c, Type* v):comparator(c),vec_type(v){}
+		bool operator()(ASObject* d1, ASObject* d2);
+	};
 public:
-	Vector() : vec_type(NULL) {}
+	Vector(Class_base* c);
+	~Vector();
+	void finalize();
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o) {};
 	static ASObject* generator(TemplatedClass<Vector>* o_class, ASObject* const* args, const unsigned int argslen);
@@ -41,9 +53,10 @@ public:
 
 	//Overloads
 	tiny_string toString(bool debugMsg=false);
-	void setVariableByMultiname(const multiname& name, ASObject* o);
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
 	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
+	static bool isValidMultiname(const multiname& name, uint32_t& index);
 
 	uint32_t nextNameIndex(uint32_t cur_index);
 	_R<ASObject> nextName(uint32_t index);
@@ -63,9 +76,29 @@ public:
 	ASFUNCTION(_applytype);
 
 	ASFUNCTION(push);
+	ASFUNCTION(_concat);
+	ASFUNCTION(_pop);
+	ASFUNCTION(join);
+	ASFUNCTION(shift);
+	ASFUNCTION(unshift);
+	ASFUNCTION(splice);
+	ASFUNCTION(_sort);
 	ASFUNCTION(getLength);
 	ASFUNCTION(setLength);
+	ASFUNCTION(filter);
+	ASFUNCTION(indexOf);
+	ASFUNCTION(_getLength);
+	ASFUNCTION(_setLength);
+	ASFUNCTION(getFixed);
+	ASFUNCTION(setFixed);
+	ASFUNCTION(forEach);
+	ASFUNCTION(_reverse);
+	ASFUNCTION(lastIndexOf);
+	ASFUNCTION(_map);
 	ASFUNCTION(_toString);
+	ASFUNCTION(slice);
+	ASFUNCTION(every);
+	ASFUNCTION(some);
 };
 
 
