@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -33,24 +33,25 @@ using namespace lightspark;
 SET_NAMESPACE("");
 REGISTER_CLASS_NAME(XML);
 
-XML::XML():node(NULL),constructed(false),ignoreComments(true)
+XML::XML(Class_base* c):ASObject(c),node(NULL),constructed(false),ignoreComments(true)
 {
 }
 
-XML::XML(const string& str):node(NULL),constructed(true)
+XML::XML(Class_base* c,const string& str):ASObject(c),node(NULL),constructed(true)
 {
-	buildFromString(str);
+	node=buildFromString(str);
 }
 
-XML::XML(_R<XML> _r, xmlpp::Node* _n):root(_r),node(_n),constructed(true)
+XML::XML(Class_base* c,_R<XML> _r, xmlpp::Node* _n):ASObject(c),root(_r),node(_n),constructed(true)
 {
 	assert(node);
 }
 
-XML::XML(xmlpp::Node* _n):constructed(true)
+XML::XML(Class_base* c,xmlpp::Node* _n):ASObject(c),constructed(true)
 {
 	assert(_n);
-	node=parser.get_document()->create_root_node_by_import(_n);
+	node=buildCopy(_n);
+	assert(node);
 }
 
 void XML::finalize()
@@ -69,9 +70,11 @@ void XML::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("valueOf",AS3,Class<IFunction>::getFunction(valueOf),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("toXMLString",AS3,Class<IFunction>::getFunction(toXMLString),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("nodeKind",AS3,Class<IFunction>::getFunction(nodeKind),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("child",AS3,Class<IFunction>::getFunction(child),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("children",AS3,Class<IFunction>::getFunction(children),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("attribute",AS3,Class<IFunction>::getFunction(attribute),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("attributes",AS3,Class<IFunction>::getFunction(attributes),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("length",AS3,Class<IFunction>::getFunction(length),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("localName",AS3,Class<IFunction>::getFunction(localName),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("name",AS3,Class<IFunction>::getFunction(name),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("descendants",AS3,Class<IFunction>::getFunction(descendants),NORMAL_METHOD,true);
@@ -82,96 +85,30 @@ void XML::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("elements",AS3,Class<IFunction>::getFunction(elements),NORMAL_METHOD,true);
 }
 
-void XML::buildFromString(const string& str)
-{
-	if(str.empty())
-	{
-		xmlpp::Element *el=parser.get_document()->create_root_node("");
-		node=el->add_child_text();
-		// TODO: node's parent (root) should be inaccessible from AS code
-	}
-	else
-	{
-		string buf = parserQuirks(str);
-		try
-		{
-			parser.parse_memory_raw((const unsigned char*)buf.c_str(), buf.size());
-		}
-		catch(const exception& e)
-		{
-			throw RunTimeException("Error while parsing XML");
-		}
-		node=parser.get_document()->get_root_node();
-	}
-}
-
-// Adobe player's XML parser accepts many strings which are not valid
-// XML according to the specs. This function attempts to massage
-// invalid-but-accepted-by-Adobe strings into valid XML so that
-// libxml++ parser doesn't throw an error.
-string XML::parserQuirks(const string& str)
-{
-	string buf = quirkCData(str);
-	buf = quirkXMLDeclarationInMiddle(buf);
-	return buf;
-}
-
-string XML::quirkCData(const string& str) {
-	//if this is a CDATA node replace CDATA tags to make it look like a text-node
-	//for compatibility with the Adobe player
-	if (str.compare(0, 9, "<![CDATA[") == 0) {
-		return "<a>"+str.substr(9, str.size()-12)+"</a>";
-	}
-	else
-		return str;
-}
-
-string XML::quirkXMLDeclarationInMiddle(const string& str) {
-	string buf(str);
-
-	// Adobe player ignores XML declarations in the middle of a
-	// string.
-	while (true)
-	{
-		size_t start = buf.find("<?xml ", 1);
-		if (start == buf.npos)
-			break;
-		
-		size_t end = buf.find("?>", start+5);
-		if (end == buf.npos)
-			break;
-		end += 2;
-		
-		buf.erase(start, end-start);
-	}
-
-	return buf;
-}
-
 ASFUNCTIONBODY(XML,generator)
 {
 	assert(obj==NULL);
 	assert_and_throw(argslen<=1);
-	if (argslen == 0)
+	if (argslen == 0 ||
+	    args[0]->is<Null>() ||
+	    args[0]->is<Undefined>())
 	{
 		return Class<XML>::getInstanceS("");
 	}
-	if(args[0]->getObjectType()==T_STRING)
+	else if(args[0]->is<ASString>() ||
+		args[0]->is<Number>() ||
+		args[0]->is<Integer>() ||
+		args[0]->is<UInteger>() ||
+		args[0]->is<Boolean>())
 	{
-		ASString* str=Class<ASString>::cast(args[0]);
-		return Class<XML>::getInstanceS(std::string(str->data));
+		return Class<XML>::getInstanceS(args[0]->toString());
 	}
-	else if(args[0]->getObjectType()==T_NULL ||
-		args[0]->getObjectType()==T_UNDEFINED)
-	{
-		return Class<XML>::getInstanceS("");
-	}
-	else if(args[0]->getClass()==Class<XML>::getClass())
+	else if(args[0]->is<XML>())
 	{
 		args[0]->incRef();
 		return args[0];
 	}
-	else if(args[0]->getClass()==Class<XMLList>::getClass())
+	else if(args[0]->is<XMLList>())
 	{
 		_R<XML> ret=args[0]->as<XMLList>()->reduceToXML();
 		ret->incRef();
@@ -193,15 +130,10 @@ ASFUNCTIONBODY(XML,_constructor)
 		return NULL;
 
 	if(argslen==0 ||
-	   args[0]->getObjectType()==T_NULL || 
-	   args[0]->getObjectType()==T_UNDEFINED)
+	   args[0]->is<Null>() || 
+	   args[0]->is<Undefined>())
 	{
-		th->buildFromString("");
-	}
-	else if(args[0]->getObjectType()==T_STRING)
-	{
-		ASString* str=Class<ASString>::cast(args[0]);
-		th->buildFromString(std::string(str->data));
+		th->node=th->buildFromString("");
 	}
 	else if(args[0]->getClass()->isSubClass(Class<ByteArray>::getClass()))
 	{
@@ -210,7 +142,27 @@ ASFUNCTIONBODY(XML,_constructor)
 		ByteArray* ba=Class<ByteArray>::cast(args[0]);
 		uint32_t len=ba->getLength();
 		const uint8_t* str=ba->getBuffer(len, false);
-		th->buildFromString(std::string((const char*)str,len));
+		th->node=th->buildFromString(std::string((const char*)str,len));
+	}
+	else if(args[0]->is<ASString>() ||
+		args[0]->is<Number>() ||
+		args[0]->is<Integer>() ||
+		args[0]->is<UInteger>() ||
+		args[0]->is<Boolean>())
+	{
+		//By specs, XML constructor will only convert to string Numbers or Booleans
+		//ints are not explicitly mentioned, but they seem to work
+		th->node=th->buildFromString(args[0]->toString());
+	}
+	else if(args[0]->is<XML>())
+	{
+		th->node=th->buildCopy(args[0]->as<XML>()->node);
+	}
+	else if(args[0]->is<XMLList>())
+	{
+		XMLList *list=args[0]->as<XMLList>();
+		_R<XML> reduced=list->reduceToXML();
+		th->node=th->buildCopy(reduced->node);
 	}
 	else
 		throw Class<TypeError>::getInstanceS("Unsupported type in XML conversion");
@@ -231,6 +183,10 @@ ASFUNCTIONBODY(XML,nodeKind)
 			return Class<ASString>::getInstanceS("element");
 		case XML_TEXT_NODE:
 			return Class<ASString>::getInstanceS("text");
+		case XML_COMMENT_NODE:
+			return Class<ASString>::getInstanceS("comment");
+		case XML_PI_NODE:
+			return Class<ASString>::getInstanceS("processing-instruction");
 		default:
 		{
 			LOG(LOG_ERROR,"Unsupported XML type " << libXml2Node->type);
@@ -239,6 +195,11 @@ ASFUNCTIONBODY(XML,nodeKind)
 	}
 }
 
+ASFUNCTIONBODY(XML,length)
+{
+	return abstract_i(1);
+}
+
 ASFUNCTIONBODY(XML,localName)
 {
 	XML* th=Class<XML>::cast(obj);
@@ -246,7 +207,7 @@ ASFUNCTIONBODY(XML,localName)
 	assert(th->node);
 	xmlElementType nodetype=th->node->cobj()->type;
 	if(nodetype==XML_TEXT_NODE || nodetype==XML_COMMENT_NODE)
-		return new Null;
+		return getSys()->getNullRef();
 	else
 		return Class<ASString>::getInstanceS(th->node->get_name());
 }
@@ -259,49 +220,52 @@ ASFUNCTIONBODY(XML,name)
 	xmlElementType nodetype=th->node->cobj()->type;
 	//TODO: add namespace
 	if(nodetype==XML_TEXT_NODE || nodetype==XML_COMMENT_NODE)
-		return new Null;
+		return getSys()->getNullRef();
 	else
-		return Class<ASString>::getInstanceS(th->node->get_name());
+	{
+		ASQName* ret = Class<ASQName>::getInstanceS();
+		ret->setByNode(th->node);
+		return ret;
+	}
 }
 
 ASFUNCTIONBODY(XML,descendants)
 {
 	XML* th=Class<XML>::cast(obj);
-	assert_and_throw(argslen==1);
-	assert_and_throw(args[0]->getObjectType()!=T_QNAME);
-	vector<_R<XML>> ret;
-	th->getDescendantsByQName(args[0]->toString(),"",ret);
-	return Class<XMLList>::getInstanceS(ret);
+	tiny_string name;
+	ARG_UNPACK(name,"*");
+ 	XMLVector ret;
+	th->getDescendantsByQName(name,"",ret);
+ 	return Class<XMLList>::getInstanceS(ret);
 }
 
 ASFUNCTIONBODY(XML,appendChild)
 {
 	XML* th=Class<XML>::cast(obj);
 	assert_and_throw(argslen==1);
-	XML* arg=NULL;
+	_NR<XML> arg;
 	if(args[0]->getClass()==Class<XML>::getClass())
-		arg=Class<XML>::cast(args[0]);
-	else if(args[0]->getClass()==Class<XMLList>::getClass())
-		arg=Class<XMLList>::cast(args[0])->convertToXML().getPtr();
-
-	if(arg==NULL)
-		throw RunTimeException("Invalid argument for XML::appendChild");
-	//Change the root of the appended XML node
-	_NR<XML> rootXML=NullRef;
-	if(th->root.isNull())
 	{
-		th->incRef();
-		rootXML=_MR(th);
+		args[0]->incRef();
+		arg=_MR(Class<XML>::cast(args[0]));
+	}
+	else if(args[0]->getClass()==Class<XMLList>::getClass())
+	{
+		XMLList* list=Class<XMLList>::cast(args[0]);
+		arg=list->convertToXML();
+		assert_and_throw(!arg.isNull());
 	}
 	else
-		rootXML=th->root;
+	{
+		//The appendChild specs says that any other type is converted to string
+		//NOTE: this is explicitly different from XML constructor, that will only convert to
+		//string Numbers and Booleans
+		arg=_MR(Class<XML>::getInstanceS(args[0]->toString()));
+	}
 
-	arg->root=rootXML;
-	xmlUnlinkNode(arg->node->cobj());
-	xmlNodePtr ret=xmlAddChild(th->node->cobj(),arg->node->cobj());
-	assert_and_throw(ret);
-	arg->incRef();
-	return arg;
+	th->node->import_node(arg->node, true);
+	th->incRef();
+	return th;
 }
 
 /* returns the named attribute in an XMLList */
@@ -332,7 +296,7 @@ ASFUNCTIONBODY(XML,attribute)
 	else
 		rootXML=th->root;
 
-	std::vector<_R<XML>> ret;
+	XMLVector ret;
 	ret.push_back(_MR(Class<XML>::getInstanceS(rootXML, attribute)));
 	return Class<XMLList>::getInstanceS(ret);
 }
@@ -352,7 +316,7 @@ XMLList* XML::getAllAttributes()
 		return Class<XMLList>::getInstanceS();
 	const xmlpp::Element::AttributeList& list=elem->get_attributes();
 	xmlpp::Element::AttributeList::const_iterator it=list.begin();
-	std::vector<_R<XML>> ret;
+	XMLVector ret;
 	_NR<XML> rootXML=NullRef;
 	if(root.isNull())
 	{
@@ -411,7 +375,15 @@ void XML::toXMLString_priv(xmlBufferPtr buf)
 		cNode->nsDef=&localNamespaces.front();
 	}
 
-	int retVal=xmlNodeDump(buf, xmlDoc, cNode, 0, 0);
+	int retVal;
+	if(cNode->type==XML_ATTRIBUTE_NODE)
+	{
+		retVal=xmlNodeBufGetContent(buf, cNode);
+	}
+	else
+	{
+		retVal=xmlNodeDump(buf, xmlDoc, cNode, 0, 0);
+	}
 	//Restore the previously defined namespaces
 	cNode->nsDef=oldNsDef;
 	if(retVal==-1)
@@ -431,29 +403,88 @@ ASFUNCTIONBODY(XML,toXMLString)
 	return ret;
 }
 
+void XML::childrenImpl(XMLVector& ret, const tiny_string& name)
+{
+	assert(node);
+	const xmlpp::Node::NodeList& list=node->get_children();
+	xmlpp::Node::NodeList::const_iterator it=list.begin();
+
+	_NR<XML> rootXML=NullRef;
+	if(root.isNull())
+	{
+		this->incRef();
+		rootXML=_MR(this);
+	}
+	else
+		rootXML=root;
+
+	for(;it!=list.end();it++)
+	{
+		if(name!="*" && (*it)->get_name() != name.raw_buf())
+			continue;
+		ret.push_back(_MR(Class<XML>::getInstanceS(rootXML, *it)));
+	}
+}
+
+void XML::childrenImpl(XMLVector& ret, uint32_t index)
+{
+	assert(node);
+	_NR<XML> rootXML=NullRef;
+	if(root.isNull())
+	{
+		this->incRef();
+		rootXML=_MR(this);
+	}
+	else
+		rootXML=root;
+
+	uint32_t i=0;
+	const xmlpp::Node::NodeList& list=node->get_children();
+	xmlpp::Node::NodeList::const_iterator it=list.begin();
+	while(it!=list.end())
+	{
+		xmlpp::TextNode* nodeText = dynamic_cast<xmlpp::TextNode*>(*it);
+		if(!(nodeText && nodeText->is_white_space()))
+		{
+			if(i==index)
+			{
+				ret.push_back(_MR(Class<XML>::getInstanceS(rootXML, *it)));
+				break;
+			}
+
+			i++;
+		}
+
+		it++;
+	}
+}
+
+ASFUNCTIONBODY(XML,child)
+{
+	XML* th=Class<XML>::cast(obj);
+	assert_and_throw(argslen==1);
+	const tiny_string& arg0=args[0]->toString();
+	XMLVector ret;
+	uint32_t index=0;
+	multiname mname(NULL);
+	mname.name_s_id=getSys()->getUniqueStringId(arg0);
+	mname.name_type=multiname::NAME_STRING;
+	mname.ns.push_back(nsNameAndKind("",NAMESPACE));
+	mname.isAttribute=false;
+	if(Array::isValidMultiname(mname, index))
+		th->childrenImpl(ret, index);
+	else
+		th->childrenImpl(ret, arg0);
+	XMLList* retObj=Class<XMLList>::getInstanceS(ret);
+	return retObj;
+}
+
 ASFUNCTIONBODY(XML,children)
 {
 	XML* th=Class<XML>::cast(obj);
 	assert_and_throw(argslen==0);
-	assert(th->node);
-	const xmlpp::Node::NodeList& list=th->node->get_children();
-	xmlpp::Node::NodeList::const_iterator it=list.begin();
-	std::vector<_R<XML>> ret;
-
-	_NR<XML> rootXML=NullRef;
-	if(th->root.isNull())
-	{
-		th->incRef();
-		rootXML=_MR(th);
-	}
-	else
-		rootXML=th->root;
-
-	for(;it!=list.end();it++)
-	{
-		rootXML->incRef();
-		ret.push_back(_MR(Class<XML>::getInstanceS(rootXML, *it)));
-	}
+	XMLVector ret;
+	th->childrenImpl(ret, "*");
 	XMLList* retObj=Class<XMLList>::getInstanceS(ret);
 	return retObj;
 }
@@ -476,7 +507,7 @@ ASFUNCTIONBODY(XML,valueOf)
 	return obj;
 }
 
-void XML::getText(vector<_R<XML>> &ret)
+void XML::getText(XMLVector& ret)
 {
 	xmlpp::Node::NodeList nl = node->get_children();
 	xmlpp::Node::NodeList::iterator i;
@@ -499,14 +530,14 @@ ASFUNCTIONBODY(XML,text)
 {
 	XML *th = obj->as<XML>();
 	ARG_UNPACK;
-	vector<_R<XML>> ret;
+	XMLVector ret;
 	th->getText(ret);
 	return Class<XMLList>::getInstanceS(ret);
 }
 
 ASFUNCTIONBODY(XML,elements)
 {
-	vector<_R<XML>> ret;
+	XMLVector ret;
 	XML *th = obj->as<XML>();
 	tiny_string name;
 	ARG_UNPACK (name, "");
@@ -528,10 +559,7 @@ ASFUNCTIONBODY(XML,elements)
 	{
 		xmlElementType nodetype=(*it)->cobj()->type;
 		if(nodetype==XML_ELEMENT_NODE && (name.empty() || name == (*it)->get_name()))
-		{
-			rootXML->incRef();
 			ret.push_back(_MR(Class<XML>::getInstanceS(rootXML, *it)));
-		}
 	}
 	return Class<XMLList>::getInstanceS(ret);
 }
@@ -572,14 +600,11 @@ xmlElementType XML::getNodeKind() const
 }
 
 void XML::recursiveGetDescendantsByQName(_R<XML> root, xmlpp::Node* node, const tiny_string& name, const tiny_string& ns,
-		std::vector<_R<XML>>& ret)
+		XMLVector& ret)
 {
 	//Check if this node is being requested. The empty string means ALL
 	if(name.empty() || name == node->get_name())
-	{
-		root->incRef();
 		ret.push_back(_MR(Class<XML>::getInstanceS(root, node)));
-	}
 	//NOTE: Creating a temporary list is quite a large overhead, but there is no way in libxml++ to access the first child
 	const xmlpp::Node::NodeList& list=node->get_children();
 	xmlpp::Node::NodeList::const_iterator it=list.begin();
@@ -587,7 +612,7 @@ void XML::recursiveGetDescendantsByQName(_R<XML> root, xmlpp::Node* node, const
 		recursiveGetDescendantsByQName(root, *it, name, ns, ret);
 }
 
-void XML::getDescendantsByQName(const tiny_string& name, const tiny_string& ns, std::vector<_R<XML> >& ret)
+void XML::getDescendantsByQName(const tiny_string& name, const tiny_string& ns, XMLVector& ret)
 {
 	assert(node);
 	assert_and_throw(ns=="");
@@ -615,7 +640,12 @@ _NR<ASObject> XML::getVariableByMultiname(const multiname& name, GET_VARIABLE_OP
 	}
 
 	bool isAttr=name.isAttribute;
+	unsigned int index=0;
+	//Normalize the name to the string form
 	const tiny_string normalizedName=name.normalizedName();
+	//TODO: support namespaces
+	assert_and_throw(name.ns.size()>0 && name.ns[0].hasEmptyName());
+
 	const char *buf=normalizedName.raw_buf();
 	if(!normalizedName.empty() && normalizedName.charAt(0)=='@')
 	{
@@ -625,13 +655,9 @@ _NR<ASObject> XML::getVariableByMultiname(const multiname& name, GET_VARIABLE_OP
 	if(isAttr)
 	{
 		//Lookup attribute
-		//TODO: support namespaces
-		assert_and_throw(name.ns.size()>0 && name.ns[0].name=="");
-
 		if(normalizedName.empty())
 			return _MR(getAllAttributes());
 
-		//Normalize the name to the string form
 		assert(node);
 		//To have attributes we must be an Element
 		xmlpp::Element* element=dynamic_cast<xmlpp::Element*>(node);
@@ -650,21 +676,30 @@ _NR<ASObject> XML::getVariableByMultiname(const multiname& name, GET_VARIABLE_OP
 		else
 			rootXML=root;
 
-		std::vector<_R<XML> > retnode;
+		XMLVector retnode;
 		retnode.push_back(_MR(Class<XML>::getInstanceS(rootXML, attr)));
 		return _MNR(Class<XMLList>::getInstanceS(retnode));
 	}
+	else if(Array::isValidMultiname(name,index))
+	{
+		// If the multiname is a valid array property, the XML
+		// object is treated as a single-item XMLList.
+		if(index==0)
+		{
+			incRef();
+			return _MNR(this);
+		}
+		else
+			return NullRef;
+	}
 	else
 	{
 		//Lookup children
-		//TODO: support namespaces
-		assert_and_throw(name.ns.size()>0 && name.ns[0].name=="");
-		//Normalize the name to the string form
 		assert(node);
 		const xmlpp::Node::NodeList& children=node->get_children(buf);
 		xmlpp::Node::NodeList::const_iterator it=children.begin();
 
-		std::vector<_R<XML>> ret;
+		XMLVector ret;
 
 		_NR<XML> rootXML=NullRef;
 		if(root.isNull())
@@ -685,6 +720,34 @@ _NR<ASObject> XML::getVariableByMultiname(const multiname& name, GET_VARIABLE_OP
 	}
 }
 
+void XML::setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
+{
+	bool isAttr=name.isAttribute;
+	//Normalize the name to the string form
+	const tiny_string normalizedName=name.normalizedName();
+	//TODO: support namespaces
+	assert_and_throw(name.ns.size()>0 && name.ns[0].hasEmptyName());
+
+	const char *buf=normalizedName.raw_buf();
+	if(!normalizedName.empty() && normalizedName.charAt(0)=='@')
+	{
+		isAttr=true;
+		buf+=1;
+	}
+	if(isAttr)
+	{
+		//To have attributes we must be an Element
+		xmlpp::Element* element=dynamic_cast<xmlpp::Element*>(node);
+		assert_and_throw(element);
+		element->set_attribute(getSys()->getStringFromUniqueId(name.name_s_id), o->toString());
+	}
+	else
+	{
+		xmlpp::Element* child=node->add_child(getSys()->getStringFromUniqueId(name.name_s_id));
+		child->add_child_text(o->toString());
+	}
+}
+
 bool XML::hasPropertyByMultiname(const multiname& name, bool considerDynamic)
 {
 	if(node==NULL || considerDynamic == false)
@@ -702,7 +765,7 @@ bool XML::hasPropertyByMultiname(const multiname& name, bool considerDynamic)
 	{
 		//Lookup attribute
 		//TODO: support namespaces
-		assert_and_throw(name.ns.size()>0 && name.ns[0].name=="");
+		assert_and_throw(name.ns.size()>0 && name.ns[0].hasEmptyName());
 		//Normalize the name to the string form
 		assert(node);
 		//To have attributes we must be an Element
@@ -718,7 +781,7 @@ bool XML::hasPropertyByMultiname(const multiname& name, bool considerDynamic)
 	{
 		//Lookup children
 		//TODO: support namespaces
-		assert_and_throw(name.ns.size()>0 && name.ns[0].name=="");
+		assert_and_throw(name.ns.size()>0 && name.ns[0].hasEmptyName());
 		//Normalize the name to the string form
 		assert(node);
 		const xmlpp::Node::NodeList& children=node->get_children(buf);
@@ -885,7 +948,8 @@ bool XML::isEqual(ASObject* r)
 }
 
 void XML::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-				std::map<const ASObject*, uint32_t>& objMap) const
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap)
 {
 	throw UnsupportedException("XML::serialize not implemented");
 }
