@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -20,52 +20,55 @@
 #ifndef XML_H
 #define XML_H
 #include "asobject.h"
-#include <libxml/tree.h>
-#include <libxml++/parsers/domparser.h>
+#include "backends/xml_support.h"
 
 namespace lightspark
 {
 class XMLList;
-class XML: public ASObject
+class XML: public ASObject, public XMLBase
 {
 friend class XMLList;
+public:
+	typedef std::vector<_R<XML>> XMLVector;
 private:
-	//The parser will destroy the document and all the childs on destruction
-	xmlpp::DomParser parser;
 	//Pointer to the root XML element, the one that owns the parser that created this node
 	_NR<XML> root;
 	//The node this object represent
 	xmlpp::Node* node;
 	static void recursiveGetDescendantsByQName(_R<XML> root, xmlpp::Node* node, const tiny_string& name, const tiny_string& ns, 
-			std::vector<_R<XML>>& ret);
+			XMLVector& ret);
 	tiny_string toString_priv();
-	void buildFromString(const std::string& str);
 	bool constructed;
 	bool nodesEqual(xmlpp::Node *a, xmlpp::Node *b) const;
 	XMLList* getAllAttributes();
-	void getText(std::vector<_R<XML>> &ret);
-	std::string parserQuirks(const std::string& str);
-	std::string quirkCData(const std::string& str);
-	std::string quirkXMLDeclarationInMiddle(const std::string& str);
+	void getText(XMLVector& ret);
 	bool ignoreComments;
 	bool ignoreProcessingInstructions;
 	bool ignoreWhitespace;
 	uint32_t prettyIndent;
 	bool prettyPrinting;
+	/*
+	 * @param name The name of the wanted children, "*" for all children
+	 *
+	 */
+	void childrenImpl(XMLVector& ret, const tiny_string& name);
+	void childrenImpl(XMLVector& ret, uint32_t index);
 public:
-	XML();
-	XML(const std::string& str);
-	XML(_R<XML> _r, xmlpp::Node* _n);
-	XML(xmlpp::Node* _n);
+	XML(Class_base* c);
+	XML(Class_base* c,const std::string& str);
+	XML(Class_base* c,_R<XML> _r, xmlpp::Node* _n);
+	XML(Class_base* c,xmlpp::Node* _n);
 	void finalize();
 	ASFUNCTION(_constructor);
 	ASFUNCTION(_toString);
 	ASFUNCTION(toXMLString);
 	ASFUNCTION(nodeKind);
+	ASFUNCTION(child);
 	ASFUNCTION(children);
 	ASFUNCTION(attributes);
 	ASFUNCTION(attribute);
 	ASFUNCTION(appendChild);
+	ASFUNCTION(length);
 	ASFUNCTION(localName);
 	ASFUNCTION(name);
 	ASFUNCTION(descendants);
@@ -77,9 +80,10 @@ public:
 	ASFUNCTION(elements);
 	static void buildTraits(ASObject* o){};
 	static void sinit(Class_base* c);
-	void getDescendantsByQName(const tiny_string& name, const tiny_string& ns, std::vector<_R<XML> >& ret);
+	void getDescendantsByQName(const tiny_string& name, const tiny_string& ns, XMLVector& ret);
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
 	bool hasPropertyByMultiname(const multiname& name, bool considerDynamic);
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
 	tiny_string toString();
 	void toXMLString_priv(xmlBufferPtr buf);
 	bool hasSimpleContent() const;
@@ -91,7 +95,8 @@ public:
 	_R<ASObject> nextValue(uint32_t index);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 };
 }
-#endif
\ No newline at end of file
+#endif
