@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -29,6 +29,7 @@
 #include "Boolean.h"
 #include "Error.h"
 #include "XML.h"
+#include "memory_support.h"
 
 namespace lightspark
 {
@@ -41,6 +42,7 @@ class ASString;
 class method_info;
 struct call_context;
 struct traits_info;
+class namespace_info;
 class Any;
 class Void;
 /* This abstract class represents a type, i.e. something that a value can be coerced to.
@@ -64,7 +66,7 @@ public:
 	 * then an exception is thrown.
 	 * The caller does not own the object returned.
 	 */
-	static const Type* getTypeFromMultiname(const multiname* mn);
+	static const Type* getTypeFromMultiname(const multiname* mn, const ABCContext* context);
 	/*
 	 * Checks if the type is already in sys->classes
 	 */
@@ -107,14 +109,16 @@ template<class T> friend class Template;
 private:
 	mutable std::vector<multiname> interfaces;
 	mutable std::vector<Class_base*> interfaces_added;
+	//TODO: move in Class_inherit
 	bool use_protected;
 	nsNameAndKind protected_ns;
+	void initializeProtectedNamespace(const tiny_string& name, const namespace_info& ns);
 	void recursiveBuild(ASObject* target);
 	IFunction* constructor;
 	void describeTraits(xmlpp::Element* root, std::vector<traits_info>& traits) const;
 	//Naive garbage collection until reference cycles are detected
 	Mutex referencedObjectsMutex;
-	std::set<ASObject*> referencedObjects;
+	std::set<ASObject*, std::less<ASObject*>, reporter_allocator<ASObject*>> referencedObjects;
 	void finalizeObjects() const;
 protected:
 	void copyBorrowedTraitsFromSuper();
@@ -127,19 +131,21 @@ public:
 	_NR<Class_base> super;
 	//We need to know what is the context we are referring to
 	ABCContext* context;
-	QName class_name;
+	const QName class_name;
 	int class_index;
+	//Memory reporter to keep track of used bytes
+	MemoryAccount* memoryAccount;
 	void handleConstruction(ASObject* target, ASObject* const* args, unsigned int argslen, bool buildAndLink);
 	void setConstructor(IFunction* c);
-	Class_base(const QName& name);
+	Class_base(const QName& name, MemoryAccount* m);
 	~Class_base();
 	void finalize();
-	virtual ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen)=0;
+	virtual ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass=NULL)=0;
 	void addImplementedInterface(const multiname& i);
 	void addImplementedInterface(Class_base* i);
 	virtual void buildInstanceTraits(ASObject* o) const=0;
 	const std::vector<Class_base*>& getInterfaces() const;
-	void linkInterface(Class_base* c) const;
+	virtual void linkInterface(Class_base* c) const;
 	/*
 	 * Returns true when 'this' is a subclass of 'cls',
 	 * i.e. this == cls or cls equals some super of this.
@@ -182,8 +188,8 @@ public:
 class Class_object: public Class_base
 {
 private:
-	Class_object():Class_base(QName("Class","")){}
-	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen)
+	Class_object():Class_base(QName("Class",""),NULL){}
+	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass)
 	{
 		throw RunTimeException("Class_object::getInstance");
 		return NULL;
@@ -207,7 +213,7 @@ public:
 class Class_function: public Class_base
 {
 private:
-	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen)
+	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass)
 	{
 		throw RunTimeException("Class_function::getInstance");
 		return NULL;
@@ -218,7 +224,7 @@ private:
 	}
 public:
 	//Name is 'Object' because trace(new f()) gives "[object Object]"
-	Class_function() : Class_base(QName("Object","")) {}
+	Class_function() : Class_base(QName("Object",""),NULL) {}
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt=NONE)
 	{
 		return NullRef;
@@ -232,7 +238,7 @@ public:
 	{
 		throw UnsupportedException("Class_function::setVariableByMultiname_i");
 	}
-	void setVariableByMultiname(const multiname& name, ASObject* o)
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst)
 	{
 		throw UnsupportedException("Class_function::setVariableByMultiname");
 	}
@@ -244,9 +250,8 @@ public:
 
 class IFunction: public ASObject
 {
-CLASSBUILDABLE(IFunction);
 protected:
-	IFunction();
+	IFunction(Class_base *c);
 	virtual IFunction* clone()=0;
 	_NR<ASObject> closure_this;
 public:
@@ -284,7 +289,6 @@ public:
 			{
 				//Generate a copy
 				ret=clone();
-				ret->classdef=NULL; //Drop the classdef and set it ex novo
 				ret->setClass(getClass());
 			}
 			ret->closure_this=c;
@@ -308,11 +312,10 @@ public:
 	typedef ASObject* (*as_function)(ASObject*, ASObject* const *, const unsigned int);
 private:
 	as_function val;
-	Function(){}
-	Function(as_function v):val(v){}
+	Function(Class_base* c, as_function v=NULL):IFunction(c),val(v){}
 	Function* clone()
 	{
-		return new Function(*this);
+		return new (getClass()->memoryAccount) Function(*this);
 	}
 	method_info* getMethodInfo() const { return NULL; }
 public:
@@ -336,13 +339,14 @@ private:
 	int hit_count;
 	method_info* mi;
 	synt_function val;
-	SyntheticFunction(method_info* m);
+	SyntheticFunction(Class_base* c,method_info* m);
 	SyntheticFunction* clone()
 	{
-		return new SyntheticFunction(*this);
+		return new (getClass()->memoryAccount) SyntheticFunction(*this);
 	}
 	method_info* getMethodInfo() const { return mi; }
 public:
+	~SyntheticFunction();
 	ASObject* call(ASObject* obj, ASObject* const* args, uint32_t num_args);
 	void finalize();
 	std::vector<scope_entry> func_scope;
@@ -369,8 +373,8 @@ template<>
 class Class<IFunction>: public Class_base
 {
 private:
-	Class<IFunction>():Class_base(QName("Function","")){}
-	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen);
+	Class<IFunction>(MemoryAccount* m):Class_base(QName("Function",""),m){}
+	ASObject* getInstance(bool construct, ASObject* const* args, const unsigned int argslen, Class_base* realClass);
 public:
 	static Class<IFunction>* getClass();
 	static _R<Class<IFunction>> getRef()
@@ -382,15 +386,20 @@ public:
 	static Function* getFunction(Function::as_function v)
 	{
 		Class<IFunction>* c=Class<IFunction>::getClass();
-		Function* ret=new Function(v);
-		ret->setClass(c);
+		Function* ret=new (c->memoryAccount) Function(c, v);
+		return ret;
+	}
+	static Function* getFunction(Function::as_function v, int len)
+	{
+		Class<IFunction>* c=Class<IFunction>::getClass();
+		Function* ret=new (c->memoryAccount) Function(c, v);
+		ret->length = len;
 		return ret;
 	}
 	static SyntheticFunction* getSyntheticFunction(method_info* m)
 	{
 		Class<IFunction>* c=Class<IFunction>::getClass();
-		SyntheticFunction* ret=new SyntheticFunction(m);
-		ret->setClass(c);
+		SyntheticFunction* ret=new (c->memoryAccount) SyntheticFunction(c, m);
 		c->handleConstruction(ret,NULL,0,true);
 		return ret;
 	}
@@ -410,34 +419,42 @@ public:
 	ASObject *describeType() const;
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
 };
 
 class Null: public ASObject
 {
 public:
-	Null(){type=T_NULL;}
+	Null();
 	bool isEqual(ASObject* r);
 	TRISTATE isLess(ASObject* r);
 	int32_t toInt();
+	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt);
+	int32_t getVariableByMultiname_i(const multiname& name);
+	void setVariableByMultiname(const multiname& name, ASObject* o, CONST_ALLOWED_FLAG allowConst);
+
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
+				std::map<const ASObject*, uint32_t>& objMap,
+				std::map<const Class_base*, uint32_t>& traitsMap);
 };
 
 class ASQName: public ASObject
 {
 friend class multiname;
 friend class Namespace;
-CLASSBUILDABLE(ASQName);
 private:
 	bool uri_is_null;
 	tiny_string uri;
 	tiny_string local_name;
-	ASQName(){type=T_QNAME; uri_is_null=false;}
 public:
+	ASQName(Class_base* c);
+	void setByNode(xmlpp::Node* node);
 	static void sinit(Class_base*);
 	ASFUNCTION(_constructor);
+	ASFUNCTION(generator);
 	ASFUNCTION(_getURI);
 	ASFUNCTION(_getLocalName);
 	ASFUNCTION(_toString);
@@ -451,17 +468,17 @@ class Namespace: public ASObject
 {
 friend class ASQName;
 friend class ABCContext;
-CLASSBUILDABLE(Namespace);
 private:
 	bool prefix_is_undefined;
 	tiny_string uri;
 	tiny_string prefix;
-	Namespace(){type=T_NAMESPACE; prefix_is_undefined=false;}
-	Namespace(const tiny_string& _uri):uri(_uri){type=T_NAMESPACE; prefix_is_undefined=false;}
 public:
+	Namespace(Class_base* c);
+	Namespace(Class_base* c, const tiny_string& _uri);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	ASFUNCTION(_constructor);
+	ASFUNCTION(generator);
 	ASFUNCTION(_getURI);
 	ASFUNCTION(_setURI);
 	ASFUNCTION(_getPrefix);
@@ -472,156 +489,21 @@ public:
 	bool isEqual(ASObject* o);
 };
 
-class Integer : public ASObject
-{
-friend class Number;
-friend class Array;
-friend class ABCVm;
-friend class ABCContext;
-friend ASObject* abstract_i(int32_t i);
-CLASSBUILDABLE(Integer);
-private:
-	Integer(int32_t v=0):val(v){type=T_INTEGER;}
-public:
-	int32_t val;
-	static void buildTraits(ASObject* o){};
-	static void sinit(Class_base* c);
-	ASFUNCTION(_toString);
-	tiny_string toString();
-	static tiny_string toString(int32_t val);
-	int32_t toInt()
-	{
-		return val;
-	}
-	TRISTATE isLess(ASObject* r);
-	bool isEqual(ASObject* o);
-	ASFUNCTION(generator);
-	std::string toDebugString() { return toString()+"i"; }
-	//Serialization interface
-	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
-};
-
-class UInteger: public ASObject
-{
-friend ASObject* abstract_ui(uint32_t i);
-CLASSBUILDABLE(UInteger);
-private:
-public:
-	uint32_t val;
-	UInteger(uint32_t v=0):val(v){type=T_UINTEGER;}
-
-	static void sinit(Class_base* c);
-	tiny_string toString();
-	static tiny_string toString(uint32_t val);
-	int32_t toInt()
-	{
-		return val;
-	}
-	uint32_t toUInt()
-	{
-		return val;
-	}
-	TRISTATE isLess(ASObject* r);
-	bool isEqual(ASObject* o);
-	ASFUNCTION(generator);
-	ASFUNCTION(_toString);
-	std::string toDebugString() { return toString()+"ui"; }
-	//CHECK: should this have a special serialization?
-};
-
-class Number : public ASObject
-{
-friend ASObject* abstract_d(number_t i);
-friend class ABCContext;
-friend class ABCVm;
-CLASSBUILDABLE(Number);
-private:
-	Number():val(0) {type=T_NUMBER;}
-	Number(double v):val(v){type=T_NUMBER;}
-	static void purgeTrailingZeroes(char* buf);
-public:
-	static const number_t NaN;
-	double val;
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_toString);
-	tiny_string toString();
-	static tiny_string toString(number_t val);
-	static bool isInteger(number_t val)
-	{
-		return floor(val) == val;
-	}
-	unsigned int toUInt()
-	{
-		return (unsigned int)(val);
-	}
-	int32_t toInt()
-	{
-		if(val<0)
-			return int(val);
-		else
-		{
-			uint32_t ret=val;
-			return ret;
-		}
-	}
-	TRISTATE isLess(ASObject* o);
-	bool isEqual(ASObject* o);
-	static void buildTraits(ASObject* o){};
-	static void sinit(Class_base* c);
-	ASFUNCTION(generator);
-	std::string toDebugString() { return toString()+"d"; }
-	//Serialization interface
-	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
-			std::map<const ASObject*, uint32_t>& objMap) const;
-};
-
-class RegExp: public ASObject
-{
-CLASSBUILDABLE(RegExp);
-friend class ASString;
-private:
-	RegExp();
-	RegExp(const tiny_string& _re);
-public:
-	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o);
-	ASObject *match(const tiny_string& str);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(generator);
-	ASFUNCTION(exec);
-	ASFUNCTION(test);
-	ASFUNCTION(_toString);
-	ASPROPERTY_GETTER(bool, dotall);
-	ASPROPERTY_GETTER(bool, global);
-	ASPROPERTY_GETTER(bool, ignoreCase);
-	ASPROPERTY_GETTER(bool, extended);
-	ASPROPERTY_GETTER(bool, multiline);
-	ASPROPERTY_GETTER_SETTER(int, lastIndex);
-	ASPROPERTY_GETTER(tiny_string, source);
-};
 
 class Global : public ASObject
 {
-CLASSBUILDABLE(Global);
 private:
-	Global(ABCContext* c, int s): context(c), scriptId(s) {}
-	static void sinit(Class_base* c);
-	static void buildTraits(ASObject* o) {};
 	ABCContext* context;
 	int scriptId;
 public:
+	Global(Class_base* cb, ABCContext* c, int s);
+	static void sinit(Class_base* c);
+	static void buildTraits(ASObject* o) {};
 	_NR<ASObject> getVariableByMultiname(const multiname& name, GET_VARIABLE_OPTION opt=NONE);
-};
-
-class GlobalObject
-{
-public:
-	void registerGlobalScope(Global* scope);
-	std::vector<Global*> globalScopes;
-	ASObject* getVariableByString(const std::string& name, ASObject*& target);
-	ASObject* getVariableAndTargetByMultiname(const multiname& name, ASObject*& target);
-	~GlobalObject();
+	/*
+	 * Utility method to register builtin methods and classes
+	 */
+	void registerBuiltin(const char* name, const char* ns, _R<ASObject> o);
 };
 
 ASObject* eval(ASObject* obj,ASObject* const* args, const unsigned int argslen);
@@ -643,7 +525,7 @@ ASObject* _isXMLName(ASObject* obj,ASObject* const* args, const unsigned int arg
 
 inline void Manager::put(ASObject* o)
 {
-	if(available.size()>maxCache)
+	if(available.size()>=maxCache)
 		delete o;
 	else
 	{
