@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2008-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -34,6 +34,7 @@
 #include "backends/security.h"
 #include "backends/image.h"
 #include "backends/extscriptobject.h"
+#include "memory_support.h"
 
 #ifdef ENABLE_CURL
 #include <curl/curl.h>
@@ -70,18 +71,21 @@ ParseThread* lightspark::getParseThread()
 	return pt;
 }
 
-RootMovieClip::RootMovieClip(LoaderInfo* li, bool isSys):parsingIsFailed(false),frameRate(0),
-	toBind(false), finishedLoading(false)
+RootMovieClip::RootMovieClip(LoaderInfo* li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c):
+	MovieClip(c),
+	parsingIsFailed(false),frameRate(0),
+	toBind(false),finishedLoading(false),applicationDomain(appDomain),securityDomain(secDomain)
 {
 	if(li)
 		li->incRef();
 	loaderInfo=_MNR(li);
-
-	//We set the protoype to a generic MovieClip
-	if(!isSys)
-		setClass(Class<MovieClip>::getClass());
 }
 
+RootMovieClip::~RootMovieClip()
+{
+	for(DictionaryTag* it: dictionary)
+		delete it;
+}
 
 void RootMovieClip::parsingFailed()
 {
@@ -131,10 +135,10 @@ void RootMovieClip::setOnStage(bool staged)
 	MovieClip::setOnStage(staged);
 }
 
-RootMovieClip* RootMovieClip::getInstance(LoaderInfo* li)
+RootMovieClip* RootMovieClip::getInstance(LoaderInfo* li, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain)
 {
-	RootMovieClip* ret=new RootMovieClip(li);
-	ret->setClass(Class<MovieClip>::getClass());
+	Class_base* movieClipClass = Class<MovieClip>::getClass();
+	RootMovieClip* ret=new (movieClipClass->memoryAccount) RootMovieClip(li, appDomain, secDomain, movieClipClass);
 	return ret;
 }
 
@@ -162,37 +166,64 @@ void SystemState::staticDeinit()
 #endif
 }
 
-SystemState::SystemState(ParseThread* parseThread, uint32_t fileSize):
-	RootMovieClip(NULL,true),terminated(0),renderRate(0),error(false),shutdown(false),
+//See BUILTIN_STRINGS enum
+static const char* builtinStrings[] = {"", "any", "void", "prototype" };
+
+SystemState::SystemState(uint32_t fileSize, FLASH_MODE mode):
+	RootMovieClip(NULL,NullRef,NullRef,NULL),terminated(0),renderRate(0),error(false),shutdown(false),
 	renderThread(NULL),inputThread(NULL),engineData(NULL),mainThread(0),dumpedSWFPathAvailable(0),
 	vmVersion(VMNONE),childPid(0),
 	parameters(NullRef),
-	invalidateQueueHead(NullRef),invalidateQueueTail(NullRef),showProfilingData(false),
-	currentVm(NULL),useInterpreter(true),useJit(false),exitOnError(false),downloadManager(NULL),
-	extScriptObject(NULL),scaleMode(SHOW_ALL),mainApplicationDomain(NullRef)
+	invalidateQueueHead(NullRef),invalidateQueueTail(NullRef),lastUsedStringId(0),lastUsedNamespaceId(0x7fffffff),
+	showProfilingData(false),flashMode(mode),
+	currentVm(NULL),useInterpreter(true),useJit(false),exitOnError(ERROR_NONE),downloadManager(NULL),
+	extScriptObject(NULL),scaleMode(SHOW_ALL),unaccountedMemory(NULL),tagsMemory(NULL),stringMemory(NULL)
 {
+	//Forge the builtin strings
+	for(uint32_t i=0;i<LAST_BUILTIN_STRING;i++)
+	{
+		uint32_t tmp=getUniqueStringId(builtinStrings[i]);
+		assert(tmp==i);
+	}
+	//Forge the empty namespace and make sure it gets id 0
+	nsNameAndKindImpl emptyNs("", NAMESPACE);
+	uint32_t nsId;
+	uint32_t baseId;
+	getUniqueNamespaceId(emptyNs, 0, nsId, baseId);
+	assert(nsId==0 && baseId==0);
+
 	cookiesFileName = NULL;
 
 	setTLSSys(this);
 
 	mainThread = Thread::self();
-	//Get starting time
-	if(parseThread) //ParseThread may be null in tightspark
-		parseThread->setRootMovie(this);
+
+	unaccountedMemory = allocateMemoryAccount("Unaccounted");
+	tagsMemory = allocateMemoryAccount("Tags");
+	stringMemory = allocateMemoryAccount("Tiny_string");
+
+	null=_MR(new (unaccountedMemory) Null);
+	undefined=_MR(new (unaccountedMemory) Undefined);
+	systemDomain = _MR(Class<ApplicationDomain>::getInstanceS());
+	applicationDomain=_MR(Class<ApplicationDomain>::getInstanceS(systemDomain));
+	securityDomain = _MR(Class<SecurityDomain>::getInstanceS());
+
 	threadPool=new ThreadPool(this);
 	timerThread=new TimerThread(this);
 	pluginManager = new PluginManager;
 	audioManager=new AudioManager(pluginManager);
 	intervalManager=new IntervalManager();
 	securityManager=new SecurityManager();
+
 	loaderInfo=_MR(Class<LoaderInfo>::getInstanceS());
+	loaderInfo->applicationDomain = applicationDomain;
 	//If the size is not known those will stay at zero
 	loaderInfo->setBytesLoaded(fileSize);
 	loaderInfo->setBytesTotal(fileSize);
 	stage=Class<Stage>::getInstanceS();
-	mainApplicationDomain=_MR(Class<ApplicationDomain>::getInstanceS());
 	this->incRef();
 	stage->_addChildAt(_MR(this),0);
+	//Get starting time
 	startTime=compat_msectiming();
 	
 	setClass(Class<MovieClip>::getClass());
@@ -373,14 +404,55 @@ void SystemState::saveProfilingInformation()
 }
 #endif
 
+MemoryAccount* SystemState::allocateMemoryAccount(const tiny_string& name)
+{
+#ifdef MEMORY_USAGE_PROFILING
+	Locker l(memoryAccountsMutex);
+	memoryAccounts.emplace_back(name);
+	return &memoryAccounts.back();
+#else
+	return NULL;
+#endif
+}
+
+#ifdef MEMORY_USAGE_PROFILING
+void SystemState::saveMemoryUsageInformation(ofstream& out, int snapshotCount) const
+{
+	out << "#-----------\nsnapshot=" << snapshotCount << "\n#-----------\ntime=" << snapshotCount << endl;
+	uint32_t totalMem=0;
+	uint32_t totalCount=0;
+	Locker l(memoryAccountsMutex);
+	auto it=memoryAccounts.begin();
+	for(;it!=memoryAccounts.end();++it)
+	{
+		if(it->bytes>0)
+		{
+			totalMem+=it->bytes;
+			totalCount++;
+		}
+	}
+
+	out << "mem_heap_B=" << totalMem << "\nmem_heap_extra_B=0\nmem_stacks_B=0\nheap_tree=detailed" << endl;
+	out << "n" << totalCount << ": " << totalMem << " ActionScript_objects" << endl;
+	it=memoryAccounts.begin();
+	for(;it!=memoryAccounts.end();++it)
+	{
+		if(it->bytes>0)
+			out << " n0: " << it->bytes << " " << it->name << endl;
+	}
+}
+#endif
+
 void SystemState::finalize()
 {
 	RootMovieClip::finalize();
 	invalidateQueueHead.reset();
 	invalidateQueueTail.reset();
-	mainApplicationDomain.reset();
 	parameters.reset();
 	frameListeners.clear();
+	null.reset();
+	undefined.reset();
+	systemDomain.reset();
 }
 
 SystemState::~SystemState()
@@ -394,7 +466,7 @@ void SystemState::destroy()
 #endif
 	terminated.wait();
 	//Acquire the mutex to sure that the engines are not being started right now
-	Locker l(mutex);
+	Locker l(rootMutex);
 	renderThread->wait();
 	inputThread->wait();
 	if(currentVm)
@@ -449,8 +521,11 @@ void SystemState::destroy()
 	 * 'classes' and 'templates' maps.
 	 */
 
-	std::map<QName, Class_base*>::iterator it=classes.begin();
-	for(;it!=classes.end();++it)
+	for(auto it = builtinClasses.begin(); it != builtinClasses.end(); ++it)
+		it->second->finalize();
+	for(auto it = customClasses.begin(); it != customClasses.end(); ++it)
+		(*it)->finalize();
+	for(auto it = templates.begin(); it != templates.end(); ++it)
 		it->second->finalize();
 
 	//Here we clean the events queue
@@ -458,8 +533,10 @@ void SystemState::destroy()
 		currentVm->finalize();
 
 	//Free classes by decRef'ing them
-	for(auto i = classes.begin(); i != classes.end(); ++i)
+	for(auto i = builtinClasses.begin(); i != builtinClasses.end(); ++i)
 		i->second->decRef();
+	for(auto i = customClasses.begin(); i != customClasses.end(); ++i)
+		(*i)->decRef();
 
 	//Free templates by decRef'ing them
 	for(auto i = templates.begin(); i != templates.end(); ++i)
@@ -502,10 +579,11 @@ bool SystemState::shouldTerminate() const
 	return shutdown || error;
 }
 
-void SystemState::setError(const string& c)
+void SystemState::setError(const string& c, ERROR_TYPE type)
 {
-	if(exitOnError)
+	if((exitOnError & type) != 0)
 		exit(1);
+
 	//We record only the first error for easier fix and reporting
 	if(!error)
 	{
@@ -520,10 +598,10 @@ void SystemState::setError(const string& c)
 
 void SystemState::setShutdownFlag()
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 	if(currentVm)
 	{
-		_R<ShutdownEvent> e(new ShutdownEvent);
+		_R<ShutdownEvent> e(new (unaccountedMemory) ShutdownEvent);
 		currentVm->addEvent(NullRef,e);
 	}
 	shutdown=true;
@@ -602,7 +680,7 @@ void SystemState::delayedStopping()
 
 void SystemState::createEngines()
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 	if(shutdown)
 	{
 		//A shutdown request has arrived before the creation of engines
@@ -644,7 +722,7 @@ void SystemState::createEngines()
 
 void SystemState::launchGnash()
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 	if(Config::getConfig()->getGnashPath().empty())
 	{
 		LOG(LOG_INFO, "Unsupported flash file (AVM1), and no gnash found");
@@ -797,7 +875,7 @@ void SystemState::launchGnash()
 
 void SystemState::needsAVM2(bool avm2)
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 
 	/* Check if we already loaded another swf. If not, then
 	 * vmVersion is VMNONE.
@@ -816,7 +894,8 @@ void SystemState::needsAVM2(bool avm2)
 		assert(!currentVm);
 		vmVersion=AVM2;
 		LOG(LOG_INFO,_("Creating VM"));
-		currentVm=new ABCVm(this);
+		MemoryAccount* vmDataMemory=this->allocateMemoryAccount("VM_Data");
+		currentVm=new ABCVm(this, vmDataMemory);
 	}
 	else
 		vmVersion=AVM1;
@@ -827,7 +906,7 @@ void SystemState::needsAVM2(bool avm2)
 
 void SystemState::setParamsAndEngine(EngineData* e, bool s)
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 	engineData=e;
 	standalone=s;
 	if(vmVersion)
@@ -836,7 +915,7 @@ void SystemState::setParamsAndEngine(EngineData* e, bool s)
 
 void SystemState::setRenderRate(float rate)
 {
-	Locker l(mutex);
+	Locker l(rootMutex);
 	if(renderRate>=rate)
 		return;
 	
@@ -880,16 +959,7 @@ void SystemState::addToInvalidateQueue(_R<DisplayObject> d)
 	if(!d->invalidateQueueNext.isNull() || d==invalidateQueueTail)
 		return;
 	if(!invalidateQueueHead)
-	{
 		invalidateQueueHead=invalidateQueueTail=d;
-		//This is the first object added to the invalidation queue
-		//TODO: support the case without the VM
-		if(currentVm)
-		{
-			//Let's ask the VM to invalidate the queue ater all events already queued are run
-			currentVm->addEvent(NullRef,_MR(new FlushInvalidationQueueEvent));
-		}
-	}
 	else
 	{
 		d->invalidateQueueNext=invalidateQueueHead;
@@ -903,7 +973,14 @@ void SystemState::flushInvalidationQueue()
 	_NR<DisplayObject> cur=invalidateQueueHead;
 	while(!cur.isNull())
 	{
-		cur->invalidate();
+		if(cur->isOnStage())
+		{
+			IDrawable* d=cur->invalidate(stage, MATRIX());
+			//Check if the drawable is valid and forge a new job to
+			//render it and upload it to GPU
+			if(d)
+				addJob(new AsyncDrawJob(d,cur));
+		}
 		_NR<DisplayObject> next=cur->invalidateQueueNext;
 		cur->invalidateQueueNext=NullRef;
 		cur=next;
@@ -912,7 +989,7 @@ void SystemState::flushInvalidationQueue()
 	invalidateQueueTail=NullRef;
 }
 
-_NR<Stage> SystemState::getStage() const
+_NR<Stage> SystemState::getStage()
 {
 	stage->incRef();
 	return _MR(stage);
@@ -1036,8 +1113,8 @@ void ThreadProfile::plot(uint32_t maxTime, cairo_t *cr)
 	}
 }
 
-ParseThread::ParseThread(istream& in, Loader *_loader, tiny_string srcurl)
-  : version(0),
+ParseThread::ParseThread(istream& in, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain, Loader *_loader, tiny_string srcurl)
+  : version(0),applicationDomain(appDomain),securityDomain(secDomain),
     f(in),zlibFilter(NULL),backend(NULL),loader(_loader),
     parsedObject(NullRef),url(srcurl),fileType(FT_UNKNOWN)
 {
@@ -1045,7 +1122,7 @@ ParseThread::ParseThread(istream& in, Loader *_loader, tiny_string srcurl)
 }
 
 ParseThread::ParseThread(std::istream& in, RootMovieClip *root)
-  : version(0),
+  : version(0),applicationDomain(NullRef),securityDomain(NullRef), //The domains are not needed since the system state create them itself
     f(in),zlibFilter(NULL),backend(NULL),loader(NULL),
     parsedObject(NullRef),url(),fileType(FT_UNKNOWN)
 {
@@ -1139,7 +1216,7 @@ void ParseThread::execute()
 	catch(LightsparkException& e)
 	{
 		LOG(LOG_ERROR,_("Exception in ParseThread ") << e.cause);
-		getSys()->setError(e.cause);
+		getSys()->setError(e.cause, SystemState::ERROR_PARSING);
 	}
 	catch(std::exception& e)
 	{
@@ -1149,9 +1226,21 @@ void ParseThread::execute()
 
 void ParseThread::parseSWF(UI8 ver)
 {
-	RootMovieClip* root=getRootMovie();
-	assert_and_throw(root);
+	objectSpinlock.lock();
+	RootMovieClip* root=NULL;
+	if(parsedObject.isNull())
+	{
+		LoaderInfo *li=loader?loader->getContentLoaderInfo().getPtr():NULL;
+		root=RootMovieClip::getInstance(li, applicationDomain, securityDomain);
+		parsedObject=_MNR(root);
+		if(!url.empty())
+			root->setOrigin(url, "");
+	}
+	else
+		root=getRootMovie();
+	objectSpinlock.unlock();
 
+	std::queue<const ControlTag*> symbolClassTags;
 	try
 	{
 		parseSWFHeader(root, ver);
@@ -1164,9 +1253,9 @@ void ParseThread::parseSWF(UI8 ver)
 		}
 
 		TagFactory factory(f, true);
-		_NR<Tag> tag=factory.readTag();
+		Tag* tag=factory.readTag(root);
 
-		FileAttributesTag* fat = dynamic_cast<FileAttributesTag*>(tag.getPtr());
+		FileAttributesTag* fat = dynamic_cast<FileAttributesTag*>(tag);
 		if(!fat)
 		{
 			LOG(LOG_ERROR,"Invalid SWF - First tag must be a FileAttributesTag!");
@@ -1190,7 +1279,7 @@ void ParseThread::parseSWF(UI8 ver)
 		bool empty=true;
 		while(!done)
 		{
-			tag=factory.readTag();
+			tag=factory.readTag(root);
 			switch(tag->getType())
 			{
 				case END_TAG:
@@ -1199,7 +1288,9 @@ void ParseThread::parseSWF(UI8 ver)
 					// in the order in which they appeared in the file.
 					while(!symbolClassTags.empty())
 					{
-						symbolClassTags.front()->execute(root);
+						const ControlTag* t=symbolClassTags.front();
+						t->execute(root);
+						delete t;
 						symbolClassTags.pop();
 					}
 
@@ -1210,17 +1301,17 @@ void ParseThread::parseSWF(UI8 ver)
 					RELEASE_WRITE(root->finishedLoading,true);
 					done=true;
 					root->check();
+					delete tag;
 					break;
 				}
 				case DICT_TAG:
 				{
-					_R<DictionaryTag> d=tag.cast<DictionaryTag>();
-					d->setLoadedFrom(root);
+					DictionaryTag* d=static_cast<DictionaryTag*>(tag);
 					root->addToDictionary(d);
 					break;
 				}
 				case DISPLAY_LIST_TAG:
-					root->addToFrame(tag.cast<DisplayListTag>());
+					root->addToFrame(static_cast<const DisplayListTag*>(tag));
 					empty=false;
 					break;
 				case SHOW_TAG:
@@ -1228,12 +1319,15 @@ void ParseThread::parseSWF(UI8 ver)
 					// in the order in which they appeared in the file.
 					while(!symbolClassTags.empty())
 					{
-						symbolClassTags.front()->execute(root);
+						const ControlTag* t=symbolClassTags.front();
+						t->execute(root);
+						delete t;
 						symbolClassTags.pop();
 					}
 
 					root->commitFrame(true);
 					empty=true;
+					delete tag;
 					break;
 				case SYMBOL_CLASS_TAG:
 				{
@@ -1241,7 +1335,7 @@ void ParseThread::parseSWF(UI8 ver)
 					// frame has been parsed. This is to handle invalid SWF files that define ID's
 					// used in the SymbolClass tag only after the tag, which would otherwise result
 					// in "undefined dictionary ID" errors.
-					_R<ControlTag> stag = tag.cast<ControlTag>();
+					const ControlTag* stag = static_cast<const ControlTag*>(tag);
 					symbolClassTags.push(stag);
 					break;
 				}
@@ -1258,19 +1352,22 @@ void ParseThread::parseSWF(UI8 ver)
 					//fall through
 				case ABC_TAG:
 				{
-					_R<ControlTag> ctag = tag.cast<ControlTag>();
+					const ControlTag* ctag = static_cast<const ControlTag*>(tag);
 					ctag->execute(root);
+					delete tag;
 					break;
 				}
 				case FRAMELABEL_TAG:
 					/* No locking required, as the last frames is not
 					 * commited to the vm yet.
 					 */
-					root->addFrameLabel(root->frames.size()-1,static_cast<FrameLabelTag*>(tag.getPtr())->Name);
+					root->addFrameLabel(root->frames.size()-1,static_cast<const FrameLabelTag*>(tag)->Name);
 					empty=false;
+					delete tag;
 					break;
 				case TAG:
 					//Not yet implemented tag, ignore it
+					delete tag;
 					break;
 			}
 			if(getSys()->shouldTerminate() || threadAborting)
@@ -1282,7 +1379,7 @@ void ParseThread::parseSWF(UI8 ver)
 		root->parsingFailed();
 		throw;
 	}
-	LOG(LOG_INFO,_("End of parsing"));
+	LOG(LOG_TRACE,_("End of parsing"));
 }
 
 void ParseThread::parseBitmap()
@@ -1309,34 +1406,20 @@ void ParseThread::setRootMovie(RootMovieClip *root)
 	parsedObject=_MNR(root);
 }
 
-RootMovieClip *ParseThread::getRootMovie()
+RootMovieClip* ParseThread::getRootMovie() const
 {
-	objectSpinlock.lock();
-	RootMovieClip *root=dynamic_cast<RootMovieClip*>(parsedObject.getPtr());
-	objectSpinlock.unlock();
-	if(root)
-		return root;
-	else if(fileType==FT_SWF || fileType==FT_COMPRESSED_SWF)
-	{
-		LoaderInfo *li=loader?loader->getContentLoaderInfo().getPtr():NULL;
-		root=RootMovieClip::getInstance(li);
-		objectSpinlock.lock();
-		parsedObject=_MNR(root);
-		objectSpinlock.unlock();
-		if(!url.empty())
-			root->setOrigin(url, "");
-		return root;
-	}
-	else
-		return NULL;
+	return dynamic_cast<RootMovieClip*>(parsedObject.getPtr());
 }
 
 void ParseThread::threadAbort()
 {
-	//Tell the our RootMovieClip that the parsing is ending
-	RootMovieClip *root=getRootMovie();
-	if(root)
-		root->parsingFailed();
+	SpinlockLocker l(objectSpinlock);
+	if(parsedObject.isNull())
+		return;
+	RootMovieClip* root=getRootMovie();
+	if(root==NULL)
+		return;
+	root->parsingFailed();
 }
 
 bool RootMovieClip::boundsRect(number_t& xmin, number_t& xmax, number_t& ymin, number_t& ymax) const
@@ -1388,7 +1471,7 @@ void RootMovieClip::commitFrame(bool another)
 		else
 		{
 			this->incRef();
-			sys->currentVm->addEvent(NullRef, _MR(new InitFrameEvent(_MNR(this))));
+			sys->currentVm->addEvent(NullRef, _MR(new (sys->unaccountedMemory) InitFrameEvent(_MNR(this))));
 		}
 	}
 }
@@ -1411,14 +1494,14 @@ void RootMovieClip::setBackground(const RGB& bg)
 }
 
 /* called in parser's thread context */
-void RootMovieClip::addToDictionary(_R<DictionaryTag> r)
+void RootMovieClip::addToDictionary(DictionaryTag* r)
 {
 	SpinlockLocker l(dictSpinlock);
 	dictionary.push_back(r);
 }
 
 /* called in vm's thread context */
-_R<DictionaryTag> RootMovieClip::dictionaryLookup(int id)
+DictionaryTag* RootMovieClip::dictionaryLookup(int id)
 {
 	SpinlockLocker l(dictSpinlock);
 	auto it = dictionary.begin();
@@ -1429,7 +1512,7 @@ _R<DictionaryTag> RootMovieClip::dictionaryLookup(int id)
 	}
 	if(it==dictionary.end())
 	{
-		LOG(LOG_ERROR,_("No such Id on dictionary ") << id);
+		LOG(LOG_ERROR,_("No such Id on dictionary ") << id << " for " << origin);
 		throw RunTimeException("Could not find an object on the dictionary");
 	}
 	return *it;
@@ -1529,7 +1612,7 @@ void SystemState::tick()
 	/* Step 3: create legacy objects, which are new in this frame (top-down),
 	 * run their constructors (bottom-up)
 	 * and their frameScripts (Step 5) (bottom-up) */
-	getSys()->currentVm->addEvent(NullRef, _MR(new InitFrameEvent()));
+	getSys()->currentVm->addEvent(NullRef, _MR(new (unaccountedMemory) InitFrameEvent()));
 
 	/* Step 4: dispatch frameConstructed events */
 	/* (TODO: should be run between step 3 and 5 */
@@ -1557,11 +1640,15 @@ void SystemState::tick()
 	/* TODO: Step 7: dispatch render event (Assuming stage.invalidate() has been called) */
 
 	/* Step 0: Set current frame number to the next frame */
-	_R<AdvanceFrameEvent> advFrame = _MR(new AdvanceFrameEvent());
+	_R<AdvanceFrameEvent> advFrame = _MR(new (unaccountedMemory) AdvanceFrameEvent());
 	if(getSys()->currentVm->addEvent(NullRef, advFrame))
 		advFrame->done.wait();
 }
 
+void SystemState::tickFence()
+{
+}
+
 void SystemState::resizeCompleted() const
 {
 	if(currentVm && scaleMode==NO_SCALE)
@@ -1571,6 +1658,71 @@ void SystemState::resizeCompleted() const
 	}
 }
 
+Null* SystemState::getNullRef() const
+{
+	Null* ret=null.getPtr();
+	ret->incRef();
+	return ret;
+}
+
+Undefined* SystemState::getUndefinedRef() const
+{
+	Undefined* ret=undefined.getPtr();
+	ret->incRef();
+	return ret;
+}
+
+const tiny_string& SystemState::getStringFromUniqueId(uint32_t id) const
+{
+	Locker l(poolMutex);
+	auto it=uniqueStringMap.right.find(id);
+	assert(it!=uniqueStringMap.right.end());
+	return it->second;
+}
+
+uint32_t SystemState::getUniqueStringId(const tiny_string& s)
+{
+	Locker l(poolMutex);
+	auto it=uniqueStringMap.left.find(s);
+	if(it==uniqueStringMap.left.end())
+	{
+		auto ret=uniqueStringMap.left.insert(make_pair(s,lastUsedStringId));
+		assert(ret.second);
+		it=ret.first;
+		lastUsedStringId++;
+	}
+	return it->second;
+}
+
+const nsNameAndKindImpl& SystemState::getNamespaceFromUniqueId(uint32_t id) const
+{
+	Locker l(poolMutex);
+	auto it=uniqueNamespaceMap.right.find(id);
+	assert(it!=uniqueNamespaceMap.right.end());
+	return it->second;
+}
+
+void SystemState::getUniqueNamespaceId(const nsNameAndKindImpl& s, uint32_t& nsId, uint32_t& baseId)
+{
+	int32_t hintedId=ATOMIC_DECREMENT(lastUsedNamespaceId);
+	getUniqueNamespaceId(s, hintedId, nsId, baseId);
+}
+
+void SystemState::getUniqueNamespaceId(const nsNameAndKindImpl& s, uint32_t hintedId, uint32_t& nsId, uint32_t& baseId)
+{
+	Locker l(poolMutex);
+	auto it=uniqueNamespaceMap.left.find(s);
+	if(it==uniqueNamespaceMap.left.end())
+	{
+		auto ret=uniqueNamespaceMap.left.insert(make_pair(s,hintedId));
+		assert(ret.second);
+		it=ret.first;
+	}
+
+	nsId=it->second;
+	baseId=(it->first.baseId==0xffffffff)?nsId:it->first.baseId;
+}
+
 /* This is run in vm's thread context */
 void RootMovieClip::initFrame()
 {
@@ -1600,3 +1752,10 @@ void RootMovieClip::constructionComplete()
 	if(this==getSys())
 		loaderInfo->sendInit();
 }
+
+void RootMovieClip::finalize()
+{
+	MovieClip::finalize();
+	applicationDomain.reset();
+	securityDomain.reset();
+}
