@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2008-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -25,13 +25,13 @@
 #include <list>
 #include <queue>
 #include <map>
+#include <boost/bimap.hpp>
 #include <string>
 #include "swftypes.h"
 #include "scripting/flash/display/flashdisplay.h"
 #include "scripting/flash/net/flashnet.h"
-#include "scripting/flash/system/flashsystem.h"
 #include "timer.h"
-
+#include "memory_support.h"
 #include "platforms/engineutils.h"
 
 namespace lightspark
@@ -51,19 +51,20 @@ class PluginManager;
 class RenderThread;
 class SecurityManager;
 class Tag;
+class ApplicationDomain;
+class SecurityDomain;
 
-//RootMovieClip is used as a ThreadJob for timed rendering purpose
 class RootMovieClip: public MovieClip
 {
 friend class ParseThread;
 protected:
-	Mutex mutex;
+	Mutex rootMutex;
 	URLInfo origin;
 private:
 	bool parsingIsFailed;
 	RGB Background;
 	Spinlock dictSpinlock;
-	std::list < _R<DictionaryTag> > dictionary;
+	std::list < DictionaryTag* > dictionary;
 	//frameSize and frameRate are valid only after the header has been parsed
 	RECT frameSize;
 	float frameRate;
@@ -78,7 +79,9 @@ private:
 	void setOnStage(bool staged);
 	ACQUIRE_RELEASE_FLAG(finishedLoading);
 public:
-	RootMovieClip(LoaderInfo* li, bool isSys=false);
+	RootMovieClip(LoaderInfo* li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c);
+	~RootMovieClip();
+	void finalize();
 	bool hasFinishedLoading() { return ACQUIRE_READ(finishedLoading); }
 	uint32_t version;
 	uint32_t fileLength;
@@ -88,8 +91,8 @@ public:
 	RECT getFrameSize() const;
 	float getFrameRate() const;
 	void setFrameRate(float f);
-	void addToDictionary(_R<DictionaryTag> r);
-	_R<DictionaryTag> dictionaryLookup(int id);
+	void addToDictionary(DictionaryTag* r);
+	DictionaryTag* dictionaryLookup(int id);
 	void labelCurrentFrame(const STRING& name);
 	void commitFrame(bool another);
 	void revertFrame();
@@ -102,7 +105,15 @@ public:
 	void setVariableByQName(const tiny_string& name, const tiny_string& ns, ASObject* o);
 	void setVariableByMultiname(multiname& name, ASObject* o);
 	void setVariableByString(const std::string& s, ASObject* o);*/
-	static RootMovieClip* getInstance(LoaderInfo* li);
+	static RootMovieClip* getInstance(LoaderInfo* li, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain);
+	/*
+	 * The application domain for this clip
+	 */
+	_NR<ApplicationDomain> applicationDomain;
+	/*
+	 * The security domain for this clip
+	 */
+	_NR<SecurityDomain> securityDomain;
 	//DisplayObject interface
 	_NR<RootMovieClip> getRoot();
 };
@@ -133,7 +144,10 @@ public:
 	void plot(uint32_t max, cairo_t *cr);
 };
 
-class SystemState: public RootMovieClip, public ITickJob
+enum BUILTIN_STRINGS { EMPTY=0, ANY, VOID, PROTOTYPE, LAST_BUILTIN_STRING };
+enum BUILTIN_NAMESPACES { EMPTY_NS=0 };
+
+class SystemState: public RootMovieClip, public ITickJob, public InvalidateQueue
 {
 private:
 	class EngineCreator: public IThreadJob
@@ -184,6 +198,10 @@ private:
 	GPid childPid;
 #endif
 
+	//shared null and undefined instances
+	_NR<Null> null;
+	_NR<Undefined> undefined;
+
 	//Parameters/FlashVars
 	_NR<ASObject> parameters;
 	void setParameters(_R<ASObject> p);
@@ -219,6 +237,19 @@ private:
 	*/
 	tiny_string profOut;
 #endif
+#ifdef MEMORY_USAGE_PROFILING
+	mutable Mutex memoryAccountsMutex;
+	std::list<MemoryAccount> memoryAccounts;
+#endif
+	/*
+	 * Pooling support
+	 */
+	mutable Mutex poolMutex;
+	boost::bimap<tiny_string, uint32_t> uniqueStringMap;
+	uint32_t lastUsedStringId;
+	boost::bimap<nsNameAndKindImpl, uint32_t> uniqueNamespaceMap;
+	//This needs to be atomic because it's decremented without the mutex held
+	ATOMIC_INT32(lastUsedNamespaceId);
 protected:
 	~SystemState();
 public:
@@ -227,9 +258,17 @@ public:
 	//Interative analysis flags
 	bool showProfilingData;
 	bool standalone;
+	//Flash for execution mode
+	enum FLASH_MODE { FLASH=0, AIR };
+	const FLASH_MODE flashMode;
 	
+	// Error types used to decide when to exit, extend as a bitmap
+	enum ERROR_TYPE { ERROR_NONE    = 0x0000,
+			  ERROR_PARSING = 0x0001,
+			  ERROR_OTHER   = 0x8000,
+			  ERROR_ANY     = 0xFFFF };
 	std::string errorCause;
-	void setError(const std::string& c);
+	void setError(const std::string& c, ERROR_TYPE type=ERROR_OTHER);
 	bool hasError() { return error; }
 	std::string& getErrorCause() { return errorCause; }
 	bool shouldTerminate() const;
@@ -237,17 +276,22 @@ public:
 	bool isOnError() const;
 	void setShutdownFlag() DLL_PUBLIC;
 	void tick();
+	void tickFence();
 	RenderThread* getRenderThread() const { return renderThread; }
 	InputThread* getInputThread() const { return inputThread; }
 	void setParamsAndEngine(EngineData* e, bool s) DLL_PUBLIC;
 	void setDownloadedPath(const tiny_string& p) DLL_PUBLIC;
 	void needsAVM2(bool n);
 	//DisplayObject interface
-	_NR<Stage> getStage() const;
+	_NR<Stage> getStage();
 
-	//Be careful, SystemState constructor does some global initialization that must be done
-	//before any other thread gets started
-	SystemState(ParseThread* p, uint32_t fileSize) DLL_PUBLIC;
+	/**
+	 * Be careful, SystemState constructor does some global initialization that must be done
+	 * before any other thread gets started
+	 * \param fileSize The size of the SWF being parsed, if known
+	 * \param mode FLASH or AIR
+	 */
+	SystemState(uint32_t fileSize, FLASH_MODE mode) DLL_PUBLIC;
 	void finalize();
 	/* Stop engines, threads and free classes and objects.
 	 * This call will decRef this object in the end,
@@ -259,6 +303,9 @@ public:
 	ThreadProfile* allocateProfiler(const RGB& color);
 	std::list<ThreadProfile*> profilingData;
 	
+	Null* getNullRef() const;
+	Undefined* getUndefinedRef() const;
+
 	Stage* stage;
 	ABCVm* currentVm;
 
@@ -268,14 +315,15 @@ public:
 	//Application starting time in milliseconds
 	uint64_t startTime;
 
-	//Class/Template map. They own one reference to each class/template
-	std::map<QName, Class_base*> classes;
+	//Classes set. They own one reference to each class/template
+	std::set<Class_base*> customClasses;
+	std::map<QName, Class_base*> builtinClasses;
 	std::map<QName, Template_base*> templates;
 
 	//Flags for command line options
 	bool useInterpreter;
 	bool useJit;
-	bool exitOnError;
+	ERROR_TYPE exitOnError;
 
 	//Parameters/FlashVars
 	void parseParametersFromFile(const char* f) DLL_PUBLIC;
@@ -300,6 +348,11 @@ public:
 	*/
 	LoaderInfo* getLoaderInfo() const { return loaderInfo.getPtr(); }
 
+	/*
+	 * The application domain for the system
+	 */
+	_NR<ApplicationDomain> systemDomain;
+
 	//Stuff to be done once for process and not for plugin instance
 	static void staticInit() DLL_PUBLIC;
 	static void staticDeinit() DLL_PUBLIC;
@@ -313,6 +366,7 @@ public:
 	SCALE_MODE scaleMode;
 	
 	//Static AS class properties
+	//TODO: Those should be different for each security domain
 	//NAMING: static$CLASSNAME$$PROPERTYNAME$
 	//	NetConnection
 	ObjectEncoding::ENCODING staticNetConnectionDefaultObjectEncoding;
@@ -333,16 +387,40 @@ public:
 	void resizeCompleted() const;
 
 	/*
-	 * The application domain for the main code
+	 * Support for class aliases in AMF3 serialization
 	 */
-	_NR<ApplicationDomain> mainApplicationDomain;
-
+	std::map<tiny_string, _R<Class_base> > aliasMap;
 #ifdef PROFILING_SUPPORT
 	void setProfilingOutput(const tiny_string& t) DLL_PUBLIC;
 	const tiny_string& getProfilingOutput() const;
 	std::vector<ABCContext*> contextes;
 	void saveProfilingInformation();
 #endif
+	MemoryAccount* allocateMemoryAccount(const tiny_string& name) DLL_PUBLIC;
+	MemoryAccount* unaccountedMemory;
+	MemoryAccount* tagsMemory;
+	MemoryAccount* stringMemory;
+#ifdef MEMORY_USAGE_PROFILING
+	void saveMemoryUsageInformation(std::ofstream& out, int snapshotCount) const;
+#endif
+	/*
+	 * Pooling support
+	 */
+	uint32_t getUniqueStringId(const tiny_string& s);
+	const tiny_string& getStringFromUniqueId(uint32_t id) const;
+	/*
+	 * Looks for the given nsNameAndKindImpl in the map.
+	 * If not present it will be created with hintedId as it's id.
+	 * The namespace id and the baseId are returned by reference.
+	 */
+	void getUniqueNamespaceId(const nsNameAndKindImpl& s, uint32_t hintedId, uint32_t& nsId, uint32_t& baseId);
+	/*
+	 * Looks for the given nsNameAndKindImpl in the map.
+	 * If not present it will be create with an id chosen internally.
+	 * The namespace id and the baseId are returned by reference.
+	 */
+	void getUniqueNamespaceId(const nsNameAndKindImpl& s, uint32_t& nsId, uint32_t& baseId);
+	const nsNameAndKindImpl& getNamespaceFromUniqueId(uint32_t id) const;
 };
 
 class ParseThread: public IThreadJob
@@ -352,17 +430,18 @@ public:
 	// Parse an object from stream. The type is detected
 	// automatically. After parsing the new object is available
 	// from getParsedObject().
-	ParseThread(std::istream& in, Loader *loader=NULL, tiny_string url="") DLL_PUBLIC;
+	ParseThread(std::istream& in, _R<ApplicationDomain> appDomain, _R<SecurityDomain> secDomain, Loader *loader, tiny_string url) DLL_PUBLIC;
 	// Parse a clip from stream into root. The stream must be an
 	// SWF file.
 	ParseThread(std::istream& in, RootMovieClip *root) DLL_PUBLIC;
 	~ParseThread();
 	FILE_TYPE getFileType() const { return fileType; }
         _NR<DisplayObject> getParsedObject();
-	void setRootMovie(RootMovieClip *root);
-	RootMovieClip *getRootMovie();
+	RootMovieClip* getRootMovie() const;
 	static FILE_TYPE recognizeFile(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
 	void execute();
+	_NR<ApplicationDomain> applicationDomain;
+	_NR<SecurityDomain> securityDomain;
 private:
 	std::istream& f;
 	std::streambuf* zlibFilter;
@@ -372,12 +451,12 @@ private:
 	Spinlock objectSpinlock;
 	tiny_string url;
 	FILE_TYPE fileType;
-	std::queue<_R<ControlTag>> symbolClassTags;
 	void threadAbort();
 	void jobFence() {};
 	void parseSWFHeader(RootMovieClip *root, UI8 ver);
 	void parseSWF(UI8 ver);
 	void parseBitmap();
+	void setRootMovie(RootMovieClip *root);
 };
 
 /* Returns the thread-specific SystemState */
