@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2008-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -22,26 +22,23 @@
 
 #include "compat.h"
 #include <iostream>
-#include <fstream>
 #include <vector>
 #include <list>
+#include <cairo.h>
 
 #include "logger.h"
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+#include <cstdlib>
+#include <cassert>
 #include "exceptions.h"
+#include "smartrefs.h"
+#include "tiny_string.h"
+#include "memory_support.h"
+#include "scripting/flash/display/BitmapContainer.h"
 
 #ifdef BIG_ENDIAN
 #include <algorithm>
 #endif
 
-/* for utf8 handling */
-#include <glib.h>
-
-/* forward declare for tiny_string conversion */
-namespace Glib { class ustring; }
-
 namespace lightspark
 {
 
@@ -84,386 +81,8 @@ enum FILE_TYPE { FT_UNKNOWN=0, FT_SWF, FT_COMPRESSED_SWF, FT_PNG, FT_JPEG, FT_GI
 typedef double number_t;
 
 class ASObject;
-class BitmapData;
-
-/* Iterates over utf8 characters */
-class CharIterator /*: public forward_iterator<uint32_t>*/
-{
-friend class tiny_string;
-private:
-	char* buf_ptr;
-public:
-	CharIterator(char* buf) : buf_ptr(buf) {}
-	/* Return the utf8-character value */
-	uint32_t operator*() const
-	{
-		return g_utf8_get_char(buf_ptr);
-	}
-	CharIterator& operator++() //prefix
-	{
-		buf_ptr = g_utf8_next_char(buf_ptr);
-		return *this;
-	}
-	CharIterator operator++(int) // postfix
-	{
-		CharIterator result = *this;
-	    ++(*this);
-	    return result;
-	}
-	bool operator==(const CharIterator& o) const
-	{
-		return buf_ptr == o.buf_ptr;
-	}
-	bool operator!=(const CharIterator& o) const { return !(*this == o); }
-	/* returns true if the current character is a digit */
-	bool isdigit() const
-	{
-		return g_unichar_isdigit(g_utf8_get_char(buf_ptr));
-	}
-	/* return the value of the current character interpreted as decimal digit,
-	 * i.e. '7' -> 7
-	 */
-	int32_t digit_value() const
-	{
-		return g_unichar_digit_value(g_utf8_get_char(buf_ptr));
-	}
-};
-
-/*
- * String class.
- * The string can contain '\0's, so don't use raw_buf().
- * Use len() to determine actual size.
- */
-class tiny_string
-{
-friend std::ostream& operator<<(std::ostream& s, const tiny_string& r);
-private:
-	enum TYPE { READONLY=0, STATIC, DYNAMIC };
-	/*must be at least 6 bytes for tiny_string(uint32_t c) constructor */
-	#define STATIC_SIZE 64
-	char _buf_static[STATIC_SIZE];
-	char* buf;
-	/*
-	   stringSize includes the trailing \0
-	*/
-	uint32_t stringSize;
-	TYPE type;
-	//TODO: use static buffer again if reassigning to short string
-	void makePrivateCopy(const char* s)
-	{
-		resetToStatic();
-		stringSize=strlen(s)+1;
-		if(stringSize > STATIC_SIZE)
-			createBuffer(stringSize);
-		strcpy(buf,s);
-	}
-	void createBuffer(uint32_t s)
-	{
-		type=DYNAMIC;
-		buf=new char[s];
-	}
-	void resizeBuffer(uint32_t s)
-	{
-		assert(type==DYNAMIC);
-		char* oldBuf=buf;
-		buf=new char[s];
-		assert(s >= stringSize);
-		memcpy(buf,oldBuf,stringSize);
-		delete[] oldBuf;
-	}
-	void resetToStatic()
-	{
-		if(type==DYNAMIC)
-			delete[] buf;
-		stringSize=1;
-		buf=_buf_static;
-		buf[0] = '\0';
-		type=STATIC;
-	}
-public:
-	static const uint32_t npos = (uint32_t)(-1);
-
-	tiny_string():buf(_buf_static),stringSize(1),type(STATIC){buf[0]=0;}
-	/* construct from utf character */
-	static tiny_string fromChar(uint32_t c)
-	{
-		tiny_string ret;
-		ret.buf = ret._buf_static;
-		ret.type = STATIC;
-		ret.stringSize = g_unichar_to_utf8(c,ret.buf) + 1;
-		ret.buf[ret.stringSize-1] = '\0';
-		return ret;
-	}
-	tiny_string(const char* s,bool copy=false):buf(_buf_static),type(READONLY)
-	{
-		if(copy)
-			makePrivateCopy(s);
-		else
-		{
-			stringSize=strlen(s)+1;
-			buf=(char*)s; //This is an unsafe conversion, we have to take care of the RO data
-		}
-	}
-	tiny_string(const tiny_string& r):buf(_buf_static),stringSize(r.stringSize),type(STATIC)
-	{
-		//Fast path for static read-only strings
-		if(r.type==READONLY)
-		{
-			type=READONLY;
-			buf=r.buf;
-			return;
-		}
-		if(stringSize > STATIC_SIZE)
-			createBuffer(stringSize);
-		memcpy(buf,r.buf,stringSize);
-	}
-	tiny_string(const std::string& r):buf(_buf_static),stringSize(r.size()+1),type(STATIC)
-	{
-		if(stringSize > STATIC_SIZE)
-			createBuffer(stringSize);
-		memcpy(buf,r.c_str(),stringSize);
-	}
-	tiny_string(const Glib::ustring& r);
-	~tiny_string()
-	{
-		resetToStatic();
-	}
-	tiny_string& operator=(const tiny_string& s)
-	{
-		resetToStatic();
-		stringSize=s.stringSize;
-		//Fast path for static read-only strings
-		if(s.type==READONLY)
-		{
-			type=READONLY;
-			buf=s.buf;
-		}
-		else
-		{
-			if(stringSize > STATIC_SIZE)
-				createBuffer(stringSize);
-			memcpy(buf,s.buf,stringSize);
-		}
-		return *this;
-	}
-	tiny_string& operator=(const std::string& s)
-	{
-		resetToStatic();
-		stringSize=s.size()+1;
-		if(stringSize > STATIC_SIZE)
-			createBuffer(stringSize);
-		memcpy(buf,s.c_str(),stringSize);
-		return *this;
-	}
-	tiny_string& operator=(const char* s)
-	{
-		makePrivateCopy(s);
-		return *this;
-	}
-	tiny_string& operator=(const Glib::ustring& s);
-	tiny_string& operator+=(const char* s);
-	tiny_string& operator+=(const tiny_string& r);
-	tiny_string& operator+=(const std::string& s)
-	{
-		//TODO: optimize
-		return *this += tiny_string(s);
-	}
-	tiny_string& operator+=(const Glib::ustring& s);
-	tiny_string& operator+=(uint32_t c)
-	{
-		return (*this += tiny_string::fromChar(c));
-	}
-	const tiny_string operator+(const tiny_string& r) const;
-	const tiny_string operator+(const char* s) const
-	{
-		return *this + tiny_string(s);
-	}
-	const tiny_string operator+(const std::string& r) const
-	{
-		return *this + tiny_string(r);
-	}
-	const tiny_string operator+(const Glib::ustring& r) const;
-	bool operator<(const tiny_string& r) const
-	{
-		//don't check trailing \0
-		return memcmp(buf,r.buf,std::min(stringSize,r.stringSize))<0;
-	}
-	bool operator>(const tiny_string& r) const
-	{
-		//don't check trailing \0
-		return memcmp(buf,r.buf,std::min(stringSize,r.stringSize))>0;
-	}
-	bool operator==(const tiny_string& r) const
-	{
-		//The length is checked as an optimization before checking the contents
-		if(stringSize != r.stringSize)
-			return false;
-		//don't check trailing \0
-		return memcmp(buf,r.buf,stringSize-1)==0;
-	}
-	bool operator==(const std::string& r) const
-	{
-		//The length is checked as an optimization before checking the contents
-		if(stringSize != r.size()+1)
-			return false;
-		//don't check trailing \0
-		return memcmp(buf,r.c_str(),stringSize-1)==0;
-	}
-	bool operator!=(const std::string& r) const
-	{
-		if(stringSize != r.size()+1)
-			return true;
-		//don't check trailing \0
-		return memcmp(buf,r.c_str(),stringSize-1)!=0;
-	}
-	bool operator!=(const tiny_string& r) const
-	{
-		return !(*this==r);
-	}
-	bool operator==(const char* r) const
-	{
-		return strcmp(buf,r)==0;
-	}
-	bool operator!=(const char* r) const
-	{
-		return !(*this==r);
-	}
-	bool operator==(const Glib::ustring&) const;
-	bool operator!=(const Glib::ustring&) const;
-	const char* raw_buf() const
-	{
-		return buf;
-	}
-	bool empty() const
-	{
-		return stringSize == 1;
-	}
-	/* returns the length in bytes, not counting the trailing \0 */
-	uint32_t numBytes() const
-	{
-		return stringSize-1;
-	}
-	/* returns the length in utf-8 characters, not counting the trailing \0 */
-	uint32_t numChars() const
-	{
-		//we cannot use g_utf8_strlen, as we may have '\0' inside our string
-		uint32_t len = 0;
-		char* end = buf+numBytes();
-		char* p = buf;
-		while(p < end)
-		{
-			p = g_utf8_next_char(p);
-			++len;
-		}
-		return len;
-	}
-	/* start and len are indices of utf8-characters */
-	tiny_string substr(uint32_t start, uint32_t len) const;
-	tiny_string substr(uint32_t start, const CharIterator& end) const;
-	/* start and len are indices of bytes */
-	tiny_string substr_bytes(uint32_t start, uint32_t len) const;
-	/* finds the first occurence of char in the utf-8 string
-	 * Return NULL if not found, else ptr to beginning of first occurence of c */
-	char* strchr(char c) const
-	{
-		//TODO: does this handle '\0' in middle of buf gracefully?
-		return g_utf8_strchr(buf, numBytes(), c);
-	}
-	char* strchrr(char c) const
-	{
-		//TODO: does this handle '\0' in middle of buf gracefully?
-		return g_utf8_strrchr(buf, numBytes(), c);
-	}
-	/*explicit*/ operator std::string() const
-	{
-		return std::string(buf,stringSize-1);
-	}
-	operator Glib::ustring() const;
-	bool startsWith(const char* o) const
-	{
-		return strncmp(buf,o,strlen(o)) == 0;
-	}
-	/* idx is an index of utf-8 characters */
-	uint32_t charAt(uint32_t idx) const
-	{
-		return g_utf8_get_char(g_utf8_offset_to_pointer(buf,idx));
-	}
-	/* start is an index of characters.
-	 * returns index of character */
-	uint32_t find(const tiny_string& needle, uint32_t start = 0) const
-	{
-		//TODO: omit copy into std::string
-		size_t bytestart = g_utf8_offset_to_pointer(buf,start) - buf;
-		size_t bytepos = std::string(*this).find(needle.raw_buf(),bytestart,needle.numBytes());
-		if(bytepos == std::string::npos)
-			return npos;
-		else
-			return g_utf8_pointer_to_offset(buf,buf+bytepos);
-	}
-	uint32_t rfind(const tiny_string& needle, uint32_t start = npos) const
-	{
-		//TODO: omit copy into std::string
-		size_t bytestart;
-		if(start == npos)
-			bytestart = std::string::npos;
-		else
-			bytestart = g_utf8_offset_to_pointer(buf,start) - buf;
-
-		size_t bytepos = std::string(*this).rfind(needle.raw_buf(),bytestart,needle.numBytes());
-		if(bytepos == std::string::npos)
-			return npos;
-		else
-			return g_utf8_pointer_to_offset(buf,buf+bytepos);
-	}
-	tiny_string& replace(uint32_t pos1, uint32_t n1, const tiny_string& o);
-	tiny_string& replace_bytes(uint32_t bytestart, uint32_t bytenum, const tiny_string& o);
-	tiny_string lowercase() const
-	{
-		//TODO: omit copy, handle \0 in string
-		char *strdown = g_utf8_strdown(buf,numBytes());
-		tiny_string ret(strdown,/*copy:*/true);
-		g_free(strdown);
-		return ret;
-	}
-	tiny_string uppercase() const
-	{
-		//TODO: omit copy, handle \0 in string
-		char *strup = g_utf8_strup(buf,numBytes());
-		tiny_string ret(strup,/*copy:*/true);
-		g_free(strup);
-		return ret;
-	}
-	/* like strcasecmp(s1.raw_buf(),s2.raw_buf()) but for unicode
-	 * TODO: slow! */
-	int strcasecmp(tiny_string& s2) const
-	{
-		char* str1 = g_utf8_casefold(this->raw_buf(),this->numBytes());
-		char* str2 = g_utf8_casefold(s2.raw_buf(),s2.numBytes());
-		int ret = g_utf8_collate(str1,str2);
-		g_free(str1);
-		g_free(str2);
-		return ret;
-	}
-	/* iterate over utf8 characters */
-	CharIterator begin()
-	{
-		return CharIterator(buf);
-	}
-	CharIterator begin() const
-	{
-		return CharIterator(buf);
-	}
-	CharIterator end()
-	{
-		//points to the trailing '\0' byte
-		return CharIterator(buf+numBytes());
-	}
-	CharIterator end() const
-	{
-		//points to the trailing '\0' byte
-		return CharIterator(buf+numBytes());
-	}
-};
+class ABCContext;
+class namespace_info;
 
 struct multiname;
 class QName
@@ -665,59 +284,98 @@ public:
 enum NS_KIND { NAMESPACE=0x08, PACKAGE_NAMESPACE=0x16, PACKAGE_INTERNAL_NAMESPACE=0x17, PROTECTED_NAMESPACE=0x18, 
 			EXPLICIT_NAMESPACE=0x19, STATIC_PROTECTED_NAMESPACE=0x1A, PRIVATE_NAMESPACE=0x05 };
 
-struct nsNameAndKind
+struct nsNameAndKindImpl
 {
 	tiny_string name;
 	NS_KIND kind;
-	nsNameAndKind(const tiny_string& _name, NS_KIND _kind):name(_name),kind(_kind){}
-	nsNameAndKind(const char* _name, NS_KIND _kind):name(_name),kind(_kind){}
-	bool operator<(const nsNameAndKind& r) const
+	uint32_t baseId;
+	nsNameAndKindImpl(const tiny_string& _name, NS_KIND _kind, uint32_t b=-1):name(_name),kind(_kind),baseId(b){}
+	nsNameAndKindImpl(const char* _name, NS_KIND _kind, uint32_t b=-1):name(_name),kind(_kind),baseId(b){}
+	bool operator<(const nsNameAndKindImpl& r) const
 	{
-		return name < r.name;
+		if(kind==r.kind)
+			return name < r.name;
+		else
+			return kind < r.kind;
 	}
-	bool operator>(const nsNameAndKind& r) const
+	bool operator>(const nsNameAndKindImpl& r) const
 	{
-		return name > r.name;
-	}
-	bool operator==(const nsNameAndKind& r) const
-	{
-		return /*kind==r.kind &&*/ name==r.name;
+		if(kind==r.kind)
+			return name > r.name;
+		else
+			return kind > r.kind;
 	}
 };
 
-struct multiname
+struct nsNameAndKind
 {
+	uint32_t nsId;
+	uint32_t nsRealId;
+	bool nameIsEmpty;
+	nsNameAndKind(const tiny_string& _name, NS_KIND _kind);
+	nsNameAndKind(const char* _name, NS_KIND _kind);
+	nsNameAndKind(ABCContext * c, uint32_t nsContextIndex);
+	/*
+	 * Special constructor for protected namespace, which have
+	 * different representationId
+	 */
+	nsNameAndKind(const tiny_string& _name, uint32_t _baseId, NS_KIND _kind);
+	/*
+	 * Special version to create the empty bultin namespace
+	 */
+	nsNameAndKind(uint32_t id):nsId(id),nsRealId(id),nameIsEmpty(true)
+	{
+		assert(nsId==0);
+	}
+	bool operator<(const nsNameAndKind& r) const
+	{
+		return nsId < r.nsId;
+	}
+	bool operator>(const nsNameAndKind& r) const
+	{
+		return nsId > r.nsId;
+	}
+	bool operator==(const nsNameAndKind& r) const
+	{
+		return nsId==r.nsId;
+	}
+	const nsNameAndKindImpl& getImpl() const;
+	bool hasEmptyName() const
+	{
+		return nameIsEmpty;
+	}
+};
+
+struct multiname: public memory_reporter
+{
+	multiname(MemoryAccount* m);
 	enum NAME_TYPE {NAME_STRING,NAME_INT,NAME_NUMBER,NAME_OBJECT};
 	NAME_TYPE name_type;
-	tiny_string name_s;
 	union
 	{
+		uint32_t name_s_id;
 		int32_t name_i;
 		number_t name_d;
 		ASObject* name_o;
 	};
-	std::vector<nsNameAndKind> ns;
+	std::vector<nsNameAndKind, reporter_allocator<nsNameAndKind>> ns;
 	bool isAttribute;
 	/*
 		Returns a string name whatever is the name type
 	*/
 	tiny_string normalizedName() const;
+	/*
+	 * 	Return a string id whatever is the name type
+	 */
+	uint32_t normalizedNameId() const;
 	tiny_string qualifiedString() const;
 	/* sets name_type, name_s/name_d based on the object n */
 	void setName(ASObject* n);
+	void resetNameIfObject();
 	bool isQName() const { return ns.size() == 1; }
+	bool toUInt(uint32_t& out) const;
 };
 
-inline QName::operator multiname() const
-{
-	multiname ret;
-	ret.name_type = multiname::NAME_STRING;
-	ret.name_s = name;
-	ret.ns.push_back( nsNameAndKind(ns, PACKAGE_NAMESPACE) );
-	ret.isAttribute = false;
-	return ret;
-}
-
 class FLOAT 
 {
 friend std::istream& operator>>(std::istream& s, FLOAT& v);
@@ -1102,19 +760,12 @@ public:
 template<class T> class Vector2Tmpl;
 typedef Vector2Tmpl<double> Vector2f;
 
-class MATRIX
+class MATRIX: public cairo_matrix_t
 {
 	friend std::istream& operator>>(std::istream& stream, MATRIX& v);
 	friend std::ostream& operator<<(std::ostream& s, const MATRIX& r);
 public:
-	number_t ScaleX;
-	number_t ScaleY;
-	number_t RotateSkew0;
-	number_t RotateSkew1;
-	int TranslateX;
-	int TranslateY;
-public:
-	MATRIX():ScaleX(1),ScaleY(1),RotateSkew0(0),RotateSkew1(0),TranslateX(0),TranslateY(0){}
+	MATRIX(number_t sx=1, number_t sy=1, number_t sk0=0, number_t sk1=0, number_t tx=0, number_t ty=0);
 	void get4DMatrix(float matrix[16]) const;
 	void multiply2D(number_t xin, number_t yin, number_t& xout, number_t& yout) const;
 	Vector2f multiply2D(const Vector2f& in) const;
@@ -1122,6 +773,55 @@ public:
 	bool operator!=(const MATRIX& r) const;
 	MATRIX getInverted() const;
 	bool isInvertible() const;
+	number_t getTranslateX() const
+	{
+		return x0;
+	}
+	number_t getTranslateY() const
+	{
+		return y0;
+	}
+	number_t getScaleX() const
+	{
+		number_t ret=sqrt(xx*xx + yx*yx);
+		if(xx>0)
+			return ret;
+		else
+			return -ret;
+	}
+	number_t getScaleY() const
+	{
+		number_t ret=sqrt(yy*yy + xy*xy);
+		if(yy>0)
+			return ret;
+		else
+			return -ret;
+	}
+	number_t getRotation() const
+	{
+		return atan(xx/yx)*180/M_PI;
+	}
+	/*
+	 * Implement flash style premultiply matrix operators
+	 */
+	void rotate(number_t angle)
+	{
+		cairo_matrix_t tmp;
+		cairo_matrix_init_rotate(&tmp,angle);
+		cairo_matrix_multiply(this,this,&tmp);
+	}
+	void scale(number_t sx, number_t sy)
+	{
+		cairo_matrix_t tmp;
+		cairo_matrix_init_scale(&tmp,sx,sy);
+		cairo_matrix_multiply(this,this,&tmp);
+	}
+	void translate(number_t dx, number_t dy)
+	{
+		cairo_matrix_t tmp;
+		cairo_matrix_init_translate(&tmp,dx,dy);
+		cairo_matrix_multiply(this,this,&tmp);
+	}
 };
 
 class GRADRECORD
@@ -1171,15 +871,16 @@ enum FILL_STYLE_TYPE { SOLID_FILL=0x00, LINEAR_GRADIENT=0x10, RADIAL_GRADIENT=0x
 class FILLSTYLE
 {
 public:
-	FILLSTYLE(int v):version(v),Gradient(v){}
+	FILLSTYLE(int v);
+	FILLSTYLE(const FILLSTYLE& r);
+	virtual ~FILLSTYLE();
 	int version;
 	FILL_STYLE_TYPE FillStyleType;
 	RGBA Color;
 	MATRIX Matrix;
 	GRADIENT Gradient;
 	FOCALGRADIENT FocalGradient;
-	BitmapData* bitmap;
-	virtual ~FILLSTYLE(){}
+	BitmapContainer bitmap;
 };
 
 class MORPHFILLSTYLE:public FILLSTYLE
@@ -1190,6 +891,8 @@ public:
 	RGBA EndColor;
 	MATRIX StartGradientMatrix;
 	MATRIX EndGradientMatrix;
+	MATRIX StartBitmapMatrix;
+	MATRIX EndBitmapMatrix;
 	UI8 NumGradients;
 	std::vector<UI8> StartRatios;
 	std::vector<UI8> EndRatios;
@@ -1235,6 +938,21 @@ public:
 	RGBA EndColor;
 };
 
+class MORPHLINESTYLE2: public MORPHLINESTYLE
+{
+public:
+	UB StartCapStyle;
+	UB JoinStyle;
+	UB HasFillFlag;
+	UB NoHScaleFlag;
+	UB NoVScaleFlag;
+	UB PixelHintingFlag;
+	UB NoClose;
+	UB EndCapStyle;
+	UI16_SWF MiterLimitFactor;
+	MORPHFILLSTYLE FillType;
+};
+
 class LINESTYLEARRAY
 {
 public:
@@ -1249,8 +967,11 @@ public:
 class MORPHLINESTYLEARRAY
 {
 public:
+	MORPHLINESTYLEARRAY(int v):version(v){}
+	const int version;
 	UI8 LineStyleCount;
-	MORPHLINESTYLE* LineStyles;
+	std::list<MORPHLINESTYLE> LineStyles;
+	std::list<MORPHLINESTYLE2> LineStyles2;
 };
 
 class FILLSTYLEARRAY
@@ -1267,7 +988,7 @@ class MORPHFILLSTYLEARRAY
 {
 public:
 	UI8 FillStyleCount;
-	MORPHFILLSTYLE* FillStyles;
+	std::list<MORPHFILLSTYLE> FillStyles;
 };
 
 class SHAPE;
@@ -1603,6 +1324,7 @@ std::istream& operator>>(std::istream& stream, MORPHLINESTYLEARRAY& v);
 std::istream& operator>>(std::istream& stream, LINESTYLE& v);
 std::istream& operator>>(std::istream& stream, LINESTYLE2& v);
 std::istream& operator>>(std::istream& stream, MORPHLINESTYLE& v);
+std::istream& operator>>(std::istream& stream, MORPHLINESTYLE2& v);
 std::istream& operator>>(std::istream& stream, FILLSTYLE& v);
 std::istream& operator>>(std::istream& stream, MORPHFILLSTYLE& v);
 std::istream& operator>>(std::istream& stream, SHAPERECORD& v);
