@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -21,22 +21,35 @@
 #define _THREADING_H
 
 #include "compat.h"
-#include <stdlib.h>
-#include <assert.h>
+#include <cstdlib>
+#include <cassert>
 #include <vector>
+
+#ifdef HAVE_NEW_GLIBMM_THREAD_API
+#include <glibmm/threads.h>
+#else
 #include <glibmm/thread.h>
+#endif
 
 namespace lightspark
 {
 
 /* Be aware that on win32, both Mutex and RecMutex are recursive! */
+#ifdef HAVE_NEW_GLIBMM_THREAD_API
+using Glib::Threads::Mutex;
+using Glib::Threads::RecMutex;
+using Glib::Threads::Cond;
+using Glib::Threads::Thread;
+typedef Mutex StaticMutex; // GLib::Threads::Mutex can be static
+#else
 using Glib::Mutex;
 using Glib::RecMutex;
 using Glib::StaticMutex;
 using Glib::Cond;
 using Glib::Thread;
+#endif
 
-typedef Glib::Mutex::Lock Locker;
+typedef Mutex::Lock Locker;
 typedef Mutex Spinlock;
 typedef Mutex::Lock SpinlockLocker;
 
@@ -183,6 +196,25 @@ public:
 	}
 
 };
+
+// This class represents the end time when waiting on a conditional
+// variable. It encapsulates the differences between new and old
+// glibmm API.
+class CondTime {
+private:
+#ifdef HAVE_NEW_GLIBMM_THREAD_API
+	gint64 timepoint;
+#else
+        Glib::TimeVal timepoint;
+#endif
+public:
+	CondTime(long milliseconds);
+	bool operator<(CondTime& c) const;
+	bool operator>(CondTime& c) const;
+	bool isInTheFuture() const;
+	void addMilliseconds(long ms);
+	bool wait(Mutex& mutex, Cond& cond);
+};
 };
 
 #endif
