@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -81,7 +81,12 @@ int main(int argc, char* argv[])
 	Log::setLogLevel(log_level);
 	SystemState::staticInit();
 	//NOTE: see SystemState declaration
-	SystemState* sys=new SystemState(NULL,0);
+#ifdef MEMORY_USAGE_PROFILING
+	MemoryAccount sysAccount("sysAccount");
+	SystemState* sys=new (&sysAccount) SystemState(0, SystemState::FLASH);
+#else
+	SystemState* sys=new ((MemoryAccount*)NULL) SystemState(0, SystemState::FLASH);
+#endif
 	setTLSSys(sys);
 
 	//Set a bit of SystemState using parameters
@@ -104,7 +109,8 @@ int main(int argc, char* argv[])
 	//setrlimit(RLIMIT_AS,&rl);
 #endif
 
-	ABCVm* vm=new ABCVm(sys);
+	MemoryAccount* vmDataMemory=sys->allocateMemoryAccount("VM_Data");
+	ABCVm* vm=new ABCVm(sys, vmDataMemory);
 	sys->currentVm=vm;
 	vector<ABCContext*> contexts;
 	for(unsigned int i=0;i<fileNames.size();i++)
@@ -112,10 +118,11 @@ int main(int argc, char* argv[])
 		ifstream f(fileNames[i]);
 		if(f.is_open())
 		{
-			ABCContext* context=new ABCContext(f);
+			sys->incRef();
+			ABCContext* context=new ABCContext(_MR(sys), f, vm);
 			contexts.push_back(context);
 			f.close();
-			vm->addEvent(NullRef,_MR(new ABCContextInitEvent(context,false)));
+			vm->addEvent(NullRef,_MR(new (sys->unaccountedMemory) ABCContextInitEvent(context,false)));
 		}
 		else
 		{
