@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2011  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2010-2012  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -22,7 +22,7 @@
 
 #include "compat.h"
 #include <list>
-#include <time.h>
+#include <ctime>
 #include "threading.h"
 
 namespace lightspark
@@ -45,6 +45,8 @@ public:
 	virtual void tick()=0;
 	ITickJob():stopMe(false){}
 	virtual ~ITickJob(){};
+	//Last method to be called when no more ticks will be sent
+	virtual void tickFence() = 0;
 };
 
 class TimerThread
@@ -53,10 +55,11 @@ private:
 	class TimingEvent
 	{
 	public:
+		TimingEvent(ITickJob* _job, bool _isTick, uint32_t _tickTime, uint32_t _waitTime) 
+			: isTick(_isTick),job(_job),wakeUpTime(_isTick ? _tickTime : _waitTime), tickTime(_tickTime) {};
 		bool isTick;
 		ITickJob* job;
-		//Timing are in milliseconds
-		Glib::TimeVal timing;
+		CondTime wakeUpTime;
 		uint32_t tickTime;
 	};
 	Mutex mutex;
